// winmain.cpp

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <mbctype.h>
#include <limits.h>
#include <errno.h>
#include "msc.h"
#include "winmain.h"



// memo -D
//   NOCRT     Do not use C-Runtime Library
//   NOARG     Declared main without argc, argv
//   NT_ONLY   Cannot run on 95/98/Me platforms.



// select UNICODE entry point compulsorily
#if defined _MSC_VER && ! defined _UNICODE
#define _UNICODE
#endif



#ifdef _UNICODE
static int SetArgumentA ( void ) ;
#else
static int SetArgumentW ( void ) ;
#endif



static int DisplayError ( const char *szMessage ) ;

static int InitProc ( void ) ;
static void __cdecl ExitProc ( void ) ;



#define MEMORY_ERROR_MESSAGE        "Critical error - not enough memory.\n"
#define INIT_ERROR_MESSAGE          "Critical error - unable to initialize environment.\n"
#define ALERT_ON_CLOSE_MESSAGE      "Press any key to close..."



#if defined _WINDOWS && ! defined _USRDLL
#define MessageBoxA MessageBoxA_
typedef int ( WINAPI *MESSAGEBOXA ) ( HWND hWnd, LPCSTR lpText, LPCSTR lpCaption, UINT uType ) ;
static HINSTANCE hUser32 ;
static MESSAGEBOXA MessageBoxA ;
#endif



////////////////////////////////////////////
//                   CRT                  //
////////////////////////////////////////////



#ifdef __cplusplus
extern "C" {
#endif


#ifdef _MSC_VER
int __cdecl _setenvp ( void ) { return 0 ; }
int __cdecl _wsetenvp ( void ) { return 0 ; }
char *__cdecl __crtGetEnvironmentStringsA ( void ) { return NULL ; }
wchar_t *__cdecl __crtGetEnvironmentStringsW ( void ) { return NULL ; }
#endif // _MSC_VER



#if defined _WINDOWS && defined _USRDLL || defined NOARG
#ifdef _MSC_VER
int __cdecl _setargv ( void ) { return 0 ; }
int __cdecl _wsetargv ( void ) { return 0 ; }
char *__cdecl __crtGetCommandLineA ( void ) { return NULL ; }
wchar_t *__cdecl __crtGetCommandLineW ( void ) { return NULL ; }
#endif // _MSC_VER
#endif // defined _WINDOWS && defined _USRDLL



#if defined _WINDOWS && ! defined _USRDLL && ! defined NOCRT
#ifdef _MSC_VER
char *__cdecl _wincmdln ( void ) { return NULL ; }
wchar_t *__cdecl _wwincmdln ( void ) { return NULL ; }
#endif // _MSC_VER
#endif // defined _WINDOWS && ! defined _USRDLL && ! defined NOCRT



#if defined NOCRT
#define NOARG
#ifdef _MSC_VER
#pragma comment ( linker, "/nodefaultlib" )
#ifdef _USRDLL
#pragma comment ( linker, "/entry:DllMain" )
#else // _USRDLL
#pragma comment ( linker, "/entry:main" )
#undef main
int main ( void ) { return winmain_ () ; }
#endif // else of _USRDLL
#endif // _MSC_VER
#endif // defined NOCRT



#ifdef __cplusplus
}
#endif



////////////////////////////////////////////
//              WINDOWS MAIN              //
////////////////////////////////////////////



#if defined _WINDOWS && ! defined _USRDLL && ! defined NOCRT



#ifdef _UNICODE
int WINAPI wWinMain ( HINSTANCE hInstance,
                      HINSTANCE hPrevInstance,
                      LPWSTR    lpCmdLine,
                      int       nCmdShow )
{

   atexit ( ExitProc ) ;

   if ( InitProc () ) return DisplayError ( INIT_ERROR_MESSAGE ) ;

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentA () ) return DisplayError ( MEMORY_ERROR_MESSAGE ) ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#else
int WINAPI WinMain ( HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow )
{

   atexit ( ExitProc ) ;

   if ( InitProc () ) return DisplayError ( INIT_ERROR_MESSAGE ) ;

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentW () ) return DisplayError ( MEMORY_ERROR_MESSAGE ) ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#endif



static int DisplayError ( const char *szMessage ) {
   if ( ! hUser32 ) {
      if ( IsNT () ) hUser32 = LoadLibraryW ( L"USER32.DLL" ) ;
      else           hUser32 = LoadLibraryA (  "USER32.DLL" ) ;
   }
   if ( GETPROCADDRESS ( hUser32, MESSAGEBOXA, MessageBoxA ) ) return 255 ;
   MessageBoxA ( NULL, szMessage, "", MB_OK | MB_ICONHAND | MB_SETFOREGROUND ) ;
   return 255 ;
}



#endif // defined _WINDOWS && ! defined _USRDLL && ! defined NOCRT



////////////////////////////////////////////
//              CONSOLE MAIN              //
////////////////////////////////////////////



#if defined _CONSOLE && ! defined NOCRT



#ifdef _UNICODE
#undef main
int wmain ( int argc, wchar_t *argv [] ) {

   atexit ( ExitProc ) ;

   if ( InitProc () ) return DisplayError ( INIT_ERROR_MESSAGE ) ;

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentA () ) return DisplayError ( MEMORY_ERROR_MESSAGE ) ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#else
#undef main
int main ( int argc, char *argv [] ) {

   atexit ( ExitProc ) ;

   if ( InitProc () ) return DisplayError ( INIT_ERROR_MESSAGE ) ;

#if defined NOARG
   return winmain_ () ;
#else
   if ( SetArgumentW () ) return DisplayError ( MEMORY_ERROR_MESSAGE ) ;
   return winmain_ ( __argc, __argv ) ;
#endif

}
#endif



static int DisplayError ( const char *szMessage ) {
   unsigned long dwCharWritten ;
   WriteFile ( GetStdHandle ( STD_ERROR_HANDLE ), szMessage, (unsigned long) strlen ( szMessage ), & dwCharWritten, NULL ) ;
   return 255 ;
}



#endif // defined _CONSOLE && ! defined NOCRT



////////////////////////////////////////////
//             EXIT/INIT PROC             //
////////////////////////////////////////////



#if ! defined NOCRT



static int InitializeLoadDllSection ( void ) ;
static void DeleteLoadDllSection ( void ) ;

static int AllocTlsIndex ( void ) ;
static void FreeTlsIndex ( void ) ;

static void AlertOnClose ( void ) ;



//  0 As 0 ȊOԂ
static int InitProc ( void ) {

#if defined _MT
   if ( InitializeLoadDllSection () ) return 1 ;
   if ( AllocTlsIndex () ) return 1 ;
#endif

#if ! defined _USRDLL
   if ( InitializeThreadData ( AllocThreadData () ) ) return 1 ;
#endif

   return 0 ;
}



static void __cdecl ExitProc ( void ) {

#if defined _CONSOLE
   AlertOnClose () ;
#endif

   FreeThreadData ( GetThreadData () ) ;

#if defined _MT
   FreeTlsIndex () ;
   DeleteLoadDllSection () ;
#endif

   return ;
}



#endif // ! defined NOCRT



////////////////////////////////////////////
//                 ALERT                  //
////////////////////////////////////////////



#if defined _CONSOLE && ! defined NOCRT



static int IsEnabled ;
static int IsDisabled ;


static void WaitKey ( void ) ;



void EnableAlertOnClose ( void ) {
   IsEnabled = 1 ;
   return ;
}



void DisableAlertOnClose ( void ) {
   IsDisabled = 1 ;
   return ;
}



// GNXv[璼ڌĂяoꂽAOɃL[͂𑣂
static void AlertOnClose ( void ) {

   if ( ( IsEnabled || ! IsDisabled && ! IsConsole () ) && fisatty ( stdout ) && fisatty ( stderr ) ) {
      fflush ( NULL ) ;
      DisplayError ( ALERT_ON_CLOSE_MESSAGE ) ;
      WaitKey () ;
   }

   return ;
}



// R\[L[͂琧Ԃ
// W͂_CNgĂÂ܂ܐԂ
static void WaitKey ( void ) {

   HANDLE hConsole = fget_osfhandle ( stdin ) ;
   if ( hConsole == INVALID_HANDLE_VALUE ) return ;

   unsigned long dwDummy ;
   if ( ! GetConsoleMode ( hConsole, & dwDummy ) ) return ;

   while ( 1 ) {
      INPUT_RECORD InputRecord ;
      unsigned long nReadLength ;
      if ( ! ReadConsoleInput ( hConsole, & InputRecord, 1, & nReadLength ) || ! nReadLength ) break ;
      if ( InputRecord.EventType == KEY_EVENT && InputRecord.Event.KeyEvent.bKeyDown ) break ;
   }

   return ;
}



// R\[NꂽȂ 0 ȊOԂ
int IsConsole ( void ) {

   STARTUPINFOA StartupInfo = { 0 } ;
   StartupInfo.cb = sizeof(STARTUPINFOA) ;
   GetStartupInfoA ( & StartupInfo ) ;

   if ( StartupInfo.dwFlags & STARTF_USESHOWWINDOW && StartupInfo.wShowWindow != SW_HIDE ) return 0 ;
   else                                                                                    return 1 ;

}



#endif // defined _CONSOLE && ! defined NOCRT



////////////////////////////////////////////
//              GETARGUMENT               //
////////////////////////////////////////////



#if ! defined NOARG && ! defined NOCRT && ( defined _CONSOLE || defined _WINDOWS && ! defined _USRDLL )



#ifndef _MSC_VER
static wchar_t **wargv_dummy_ ;
#endif



// 擾iANSIŁj
char *GetArgumentA ( int nCount ) {
   return __argv [ nCount ] ;
}



// 擾iUNICODEŁj
wchar_t *GetArgumentW ( int nCount ) {
   return __wargv [ nCount ] ;
}



#ifdef _UNICODE
// iANSIŁj
//  0 As 0 ȊOԂ
static int SetArgumentA ( void ) {

   if ( ! ( __argv = (char**) malloc ( ( __argc + 1 ) * sizeof(char*) ) ) ) return 1 ;

   for ( int nCount = 0 ; nCount <= __argc ; nCount ++ ) {
      if ( ! __wargv [ nCount ] ) __argv [ nCount ] = NULL ;
      else {
         char *string = w2adup ( __wargv [ nCount ] ) ;
         if ( ! string ) return 1 ;
         __argv [ nCount ] = string ;
      }
   }

   return 0 ;
}
#else
// iUNICODEŁj
//  0 As 0 ȊOԂ
static int SetArgumentW ( void ) {

   if ( ! ( __wargv = (wchar_t**) malloc ( ( __argc + 1 ) * sizeof(wchar_t*) ) ) ) return 1 ;

   for ( int nCount = 0 ; nCount <= __argc ; nCount ++ ) {
      if ( ! __argv [ nCount ] ) __wargv [ nCount ] = NULL ;
      else {
         wchar_t *string = a2wdup ( __argv [ nCount ] ) ;
         if ( ! string ) return 1 ;
         __wargv [ nCount ] = string ;
      }
   }

   return 0 ;
}
#endif



#endif // ! defined NOARG && ! defined NOCRT && ( defined _CONSOLE || defined _WINDOWS && ! defined _USRDLL )



////////////////////////////////////////////
//           GET WINMAIN CMDLINE          //
////////////////////////////////////////////



#if ( defined NOARG || defined NOCRT ) && ! defined _USRDLL



#ifdef NOCRT
#undef ismbblead
#define ismbblead IsDBCSLeadByte
#endif



// WinMain `̈擾iANSIŁj
char *GetWinMainCmdLineA ( void ) {

   char *szCommandLine ;
   static char szDummy [] = "" ;
   int IsInQuote = 0 ;

   szCommandLine = GetCommandLineA () ;
   if ( ! szCommandLine ) szCommandLine = szDummy ;

   while ( *szCommandLine > 0x20 || *szCommandLine && IsInQuote ) {
      if ( *szCommandLine == '\"' ) IsInQuote = ! IsInQuote ;
      if ( ismbblead ( (unsigned char) *szCommandLine ) && *( szCommandLine + 1 ) ) szCommandLine ++ ;
      szCommandLine ++ ;
   }

   while ( *szCommandLine && *szCommandLine <= 0x20 ) szCommandLine ++ ;

   return szCommandLine ;
}



// WinMain `̈擾iUNICODEŁj
wchar_t *GetWinMainCmdLineW ( void ) {

   wchar_t *szCommandLine ;
   static wchar_t szDummy [] = L"" ;
   int IsInQuote = 0 ;

   szCommandLine = GetCommandLineW () ;
   if ( ! szCommandLine ) szCommandLine = szDummy ;

   while ( *szCommandLine > 0x20 || *szCommandLine && IsInQuote ) {
      if ( *szCommandLine == '\"' ) IsInQuote = ! IsInQuote ;
      szCommandLine ++ ;
   }

   while ( *szCommandLine && *szCommandLine <= 0x20 ) szCommandLine ++ ;

   return szCommandLine ;
}



#endif // ( defined NOARG || defined NOCRT ) && ! defined _USRDLL



////////////////////////////////////////////
//                DLL MAIN                //
////////////////////////////////////////////



#if defined _USRDLL && ! defined NOCRT



#undef DllMain
int WINAPI DllMain ( HINSTANCE hInst, unsigned long dwReason, void *pReserved ) {

   int nRetValue = TRUE ;

   if ( dwReason == DLL_PROCESS_ATTACH ) {
      if ( InitProc () ) nRetValue = FALSE ;
   }

   if ( nRetValue ) nRetValue = DllMain_ ( hInst, dwReason, pReserved ) ;

#if _MSC_VER < 1400
   // Â CRT ł attach s detach Ă΂Ȃ
   if ( dwReason == DLL_PROCESS_ATTACH && ! nRetValue ) {
      DllMain ( hInst, DLL_PROCESS_DETACH, pReserved ) ;
   }
#endif

#if defined _MT
   if ( dwReason == DLL_THREAD_DETACH ) {
      FreeThreadData ( GetThreadData () ) ;
   }
#endif

   if ( dwReason == DLL_PROCESS_DETACH ) {
      ExitProc () ;
   }

   return nRetValue ;
}



#endif // defined _USRDLL && ! defined NOCRT



////////////////////////////////////////////
//            CRITICAL SECTION            //
////////////////////////////////////////////



#if ! defined NOCRT


#ifndef NTFUNC_IMPORT_IMPLICITLY
#define InitializeCriticalSectionAndSpinCount InitializeCriticalSectionAndSpinCount_
static BOOL InitializeCriticalSectionAndSpinCount ( LPCRITICAL_SECTION lpCriticalSection, DWORD dwSpinCount ) ;
#endif


#define SPINCOUNT 4000



//  0 As 0 ȊOԂ
int InitializeCriticalSectionEx ( PCRITICAL_SECTION pCriticalSection ) {

   if ( IsWin2000orLater () ) return ! InitializeCriticalSectionAndSpinCount ( pCriticalSection, SPINCOUNT ) ;

   __try {
      InitializeCriticalSection ( pCriticalSection ) ;
   }
   __except ( EXCEPTION_EXECUTE_HANDLER ) { return 1 ; }

   return 0 ;
}



#ifndef NTFUNC_IMPORT_IMPLICITLY


typedef BOOL ( WINAPI *INITIALIZECRITICALSECTIONANDSPINCOUNT ) ( LPCRITICAL_SECTION lpCriticalSection, DWORD dwSpinCount ) ;


static BOOL InitializeCriticalSectionAndSpinCount ( LPCRITICAL_SECTION lpCriticalSection, DWORD dwSpinCount ) {
#undef InitializeCriticalSectionAndSpinCount

   static HINSTANCE hKernel32 ;
   static INITIALIZECRITICALSECTIONANDSPINCOUNT InitializeCriticalSectionAndSpinCount ;

   if ( ! hKernel32 ) hKernel32 = GetModuleHandleW ( L"KERNEL32.DLL" ) ;
   if ( ! GETPROCADDRESS ( hKernel32, INITIALIZECRITICALSECTIONANDSPINCOUNT, InitializeCriticalSectionAndSpinCount ) ) {
      return InitializeCriticalSectionAndSpinCount ( lpCriticalSection, dwSpinCount ) ;
   }

   return 0 ;
}


#endif // ! NTFUNC_IMPORT_IMPLICITLY



#endif // ! defined NOCRT



////////////////////////////////////////////
//      CRITICAL SECTION  (LOAD DLL)      //
////////////////////////////////////////////



#if ! defined NOCRT && defined _MT



CRITICAL_SECTION LoadDllSection ;
static int IsLoadDllSectionInitialized ;


//  0 As 0 ȊOԂ
static int InitializeLoadDllSection ( void ) {
   if ( InitializeCriticalSectionEx ( & LoadDllSection ) ) return 1 ;
   IsLoadDllSectionInitialized = 1 ;
   return 0 ;
}


static void DeleteLoadDllSection ( void ) {
   if ( IsLoadDllSectionInitialized ) DeleteCriticalSection ( & LoadDllSection ) ;
}



#endif // ! defined NOCRT && defined _MT



////////////////////////////////////////////
//               MULTITHREAD              //
////////////////////////////////////////////



#if ! defined NOCRT


// }`Xbhł TLS ɂAhXۑ
// VOXbhł͐ÓIϐɂAhXۑ
// DLL ł͗vƂɃ[mۂ


#if defined _MT
static unsigned long nTlsIndex ;
#else
static THREAD_DATA *pPrimaryThreadData ;
#endif



#if defined _MT
// TLS CfbNX擾AXbh̃f[^Zbg
//  0 As 0 ȊOԂ
static int AllocTlsIndex ( void ) {
   if ( ( nTlsIndex = TlsAlloc () ) == TLS_OUT_OF_INDEXES ) return 1 ;
   return 0 ;
}
#endif



#if defined _MT
// TLS CfbNX
static void FreeTlsIndex ( void ) {
   if ( ( nTlsIndex != TLS_OUT_OF_INDEXES ) ) TlsFree ( nTlsIndex ) ;
}
#endif



// Xbhf[^̃AhX擾
THREAD_DATA *GetThreadData ( void ) {
#if defined _MT
   THREAD_DATA *pThreadData = (THREAD_DATA*) TlsGetValue ( nTlsIndex ) ;
#else
   THREAD_DATA *pThreadData = pPrimaryThreadData ;
#endif
#if defined _USRDLL
   if ( ! pThreadData ) {
      pThreadData = AllocThreadData () ;
      InitializeThreadData ( pThreadData ) ;
   }
#endif
   return pThreadData ;
}



// Xbhf[^mۂ
THREAD_DATA *AllocThreadData ( void ) {

   THREAD_DATA *pThreadData = (THREAD_DATA*) malloc ( sizeof(THREAD_DATA) ) ;
   if ( pThreadData ) memzero ( pThreadData, sizeof(THREAD_DATA) ) ;

   return pThreadData ;
}



// Xbhf[^
int InitializeThreadData ( THREAD_DATA *pThreadData ) {
#if defined _MT
   if ( ! TlsSetValue ( nTlsIndex, pThreadData ) ) return 1 ;
#else
   pPrimaryThreadData = pThreadData ;
#endif
   return 0 ;
}



// Xbhf[^
void FreeThreadData ( THREAD_DATA *pThreadData ) {

   if ( ! pThreadData ) return ;

   // res.cpp
   free ( pThreadData->szGetStringA ) ;
   free ( pThreadData->szGetStringW ) ;
   free ( pThreadData->szCopyrightA ) ;

   // environ.cpp
   free ( pThreadData->szEnvironA ) ;
   free ( pThreadData->szEnvironW ) ;

   free ( pThreadData ) ;

   return ;
}



#endif // ! defined NOCRT



////////////////////////////////////////////
//          INTERLOCKED EXCHANGE          //
////////////////////////////////////////////



#if defined _MSC_VER && ! defined NT_ONLY



#pragma warning ( push )
#pragma warning ( disable: 4035 )   // no return value



long InterlockedExchangeAdd_95 ( long volatile *Addend, long Increment ) {
   __asm {
      mov   eax, Increment
      mov   ecx, Addend
      lock xadd   [ecx], eax        // 80486 or later
   }
}



long InterlockedCompareExchange_95 ( long volatile *Destination, long Exchange, long Comperand ) {
   __asm {
      mov   eax, Comperand
      mov   ecx, Destination
      mov   edx, Exchange
      lock cmpxchg   [ecx], edx     // 80486 or later
   }
}



#pragma warning ( pop )



#endif



////////////////////////////////////////////
//               strlcpy ()               //
////////////////////////////////////////////



#if ! defined NOCRT



// w肳ꂽ܂ŕRs[iANSIŁj
//  (size_t) -1 ȂkI[ƌȂ
// bufsiz  dst Ɏ߂邱Ƃ̂łő̕iI[k܂ށj
// src ̕ԂiI[k܂܂Ȃj
size_t strlcpylen ( char *dst, const char *src, size_t srclen, size_t bufsiz ) {

   if ( srclen == (size_t) -1 ) srclen = strlen ( src ) ;

   if ( bufsiz ) {
      size_t cpylen = ( srclen < bufsiz ) ? srclen : bufsiz - 1 ;
      memmove ( dst, src, cpylen * sizeof(char) ) ;
      dst [ cpylen ] = 0 ;
   }

   return srclen ;
}



// w肳ꂽ܂ŕRs[iUNICODEŁj
//  (size_t) -1 ȂkI[ƌȂ
// bufsiz  dst Ɏ߂邱Ƃ̂łő̕iI[k܂ށj
// src ̕ԂiI[k܂܂Ȃj
size_t wcslcpylen ( wchar_t *dst, const wchar_t *src, size_t srclen, size_t bufsiz ) {

   if ( srclen == (size_t) -1 ) srclen = wcslen ( src ) ;

   if ( bufsiz ) {
      size_t cpylen = ( srclen < bufsiz ) ? srclen : bufsiz - 1 ;
      memmove ( dst, src, cpylen * sizeof(wchar_t) ) ;
      dst [ cpylen ] = 0 ;
   }

   return srclen ;
}



#endif // ! defined NOCRT



////////////////////////////////////////////
//               w/alcpy ()               //
////////////////////////////////////////////



#if ! defined NOCRT



// w肳ꂽ܂ŕRs[iANSIUNICODEŁj
//  (size_t) -1 ȂkI[ƌȂ
// bufsiz  dst Ɏ߂邱Ƃ̂łő̕iI[k܂ށj
// { dst ̕ԂiI[k܂܂Ȃj
// s (size_t) -1 Ԃ
size_t a2wlcpylen_cp ( wchar_t *dst, const char *src, size_t srclen, size_t bufsiz, int codepage ) {

   if ( CP_ACP && ! codepage ) codepage = CP_ACP ;    // Ô

   if ( bufsiz > INT_MAX ) bufsiz = INT_MAX ;
   if ( bufsiz ) *dst = 0 ;

   if ( srclen == (size_t) -1 ) srclen = strlen ( src ) ;
   if ( srclen > INT_MAX ) return (size_t) -1 ;

   size_t dstlen ;

   if ( bufsiz && ( dstlen = MultiByteToWideChar ( codepage, 0, src, (int) srclen, dst, (int) bufsiz - 1 ) ) ) {
      dst [ dstlen ] = 0 ;
   }
   else {
      // obt@Ɏ܂Ȃꍇ

      dstlen = MultiByteToWideChar ( codepage, 0, src, (int) srclen, NULL, 0 ) ;
      if ( ! dstlen ) return (size_t) -1 ;

      if ( bufsiz ) {
         size_t cpylen = ( dstlen < bufsiz ) ? dstlen : bufsiz - 1 ;
         MultiByteToWideChar ( codepage, 0, src, (int) srclen, dst, (int) cpylen ) ;
         dst [ cpylen ] = 0 ;
      }
   }

   return dstlen ;
}



// w肳ꂽ܂ŕRs[iUNICODEANSIŁj
//  (size_t) -1 ȂkI[ƌȂ
// bufsiz  dst Ɏ߂邱Ƃ̂łő̕iI[k܂ށj
// ϊłȂ errno  EILSEQ Zbg
// { dst ̕ԂiI[k܂܂Ȃj
// s (size_t) -1 Ԃ
size_t w2alcpylen_cp ( char *dst, const wchar_t *src, size_t srclen, size_t bufsiz, int codepage ) {

   int IsUsedDefaultChar = 0 ;

   if ( CP_ACP && ! codepage ) codepage = CP_ACP ;    // Ô

   if ( bufsiz > INT_MAX ) bufsiz = INT_MAX ;
   if ( bufsiz ) *dst = 0 ;

   if ( srclen == (size_t) -1 ) srclen = wcslen ( src ) ;
   if ( srclen > INT_MAX ) return (size_t) -1 ;

   size_t dstlen ;

   if ( bufsiz && ( dstlen = WideCharToMultiByte ( codepage, 0, src, (int) srclen, dst, (int) bufsiz - 1, NULL, & IsUsedDefaultChar ) ) ) {
      dst [ dstlen ] = 0 ;
   }
   else {
      // obt@Ɏ܂Ȃꍇ

      dstlen = WideCharToMultiByte ( codepage, 0, src, (int) srclen, NULL, 0, NULL, NULL ) ;
      if ( ! dstlen ) return (size_t) -1 ;

      if ( bufsiz ) {
         size_t cpylen = ( dstlen < bufsiz ) ? dstlen : bufsiz - 1 ;
         if ( cpylen ) dst [ cpylen - 1 ] = 0 ;
         WideCharToMultiByte ( codepage, 0, src, (int) srclen, dst, (int) cpylen, NULL, & IsUsedDefaultChar ) ;
         dst [ cpylen ] = 0 ;
      }
   }

   if ( IsUsedDefaultChar ) errno = EILSEQ ;

   return dstlen ;
}



#endif // ! defined NOCRT



////////////////////////////////////////////
//         strduplen_max_expand ()        //
////////////////////////////////////////////



#if ! defined NOCRT



// w肵ĕ𕡐iANSIŁj
//  (size_t) -1 ȂkI[ƌȂ
// string ̒ maxlen ȏȂ NULL Ԃ
// o̖͂ɃkI[
// w肳ꂽ]Ɋmۂ
// s NULL Ԃ
char *strduplen_max_expand ( const char *string, size_t srclen, size_t maxlen, size_t addlen ) {

   if ( srclen == (size_t) -1 ) srclen = strlen ( string ) ;
   if ( srclen >= maxlen ) return NULL ;

   char *buffer = (char*) malloc ( ( srclen + addlen + 1 ) * sizeof(char) ) ;
   if ( buffer ) {
      memmove ( buffer, string, srclen * sizeof(char) ) ;
      buffer [ srclen ] = 0 ;
   }

   return buffer ;
}



// w肵ĕ𕡐iUNICODEŁj
//  (size_t) -1 ȂkI[ƌȂ
// string ̒ maxlen ȏȂ NULL Ԃ
// o̖͂ɃkI[
// w肳ꂽ]Ɋmۂ
// s NULL Ԃ
wchar_t *wcsduplen_max_expand ( const wchar_t *string, size_t srclen, size_t maxlen, size_t addlen ) {

   if ( srclen == (size_t) -1 ) srclen = wcslen ( string ) ;
   if ( srclen >= maxlen ) return NULL ;

   wchar_t *buffer = (wchar_t*) malloc ( ( srclen + addlen + 1 ) * sizeof(wchar_t) ) ;
   if ( buffer ) {
      memmove ( buffer, string, srclen * sizeof(wchar_t) ) ;
      buffer [ srclen ] = 0 ;
   }

   return buffer ;
}



#endif // ! defined NOCRT



////////////////////////////////////////////
//         a/wduplen_max_expand ()        //
////////////////////////////////////////////



#if ! defined NOCRT



// ANSIChɕϊĕ
//  (size_t) -1 ȂkI[ƌȂ
// string ̒ maxlen ȏȂ NULL Ԃ
// o̖͂ɃkI[
// o͂ꂽ p_dstlen Ɋi[ikI[܂܂ȂAKvłȂ NULL łj
// w肳ꂽ]Ɋmۂ
// s NULL Ԃ
wchar_t *a2wduplen_max_expand_cp ( const char *string, size_t srclen, size_t maxlen, size_t addlen, size_t *p_dstlen, int codepage ) {

   int dstlen ;
   wchar_t *buffer ;

   if ( CP_ACP && ! codepage ) codepage = CP_ACP ;    // Ô

   if ( srclen == (size_t) -1 ) srclen = strlen ( string ) ;
   if ( srclen >= maxlen ) return NULL ;
   if ( srclen > INT_MAX ) return NULL ;

   if ( ! ( dstlen = MultiByteToWideChar ( codepage, 0, string, (int) srclen, NULL, 0 ) ) ) return NULL ;
   if ( ! ( buffer = (wchar_t*) malloc ( ( dstlen + addlen + 1 ) * sizeof(wchar_t) ) ) ) return NULL ;
   if ( ! ( dstlen = MultiByteToWideChar ( codepage, 0, string, (int) srclen, buffer, dstlen ) ) ) { free ( buffer ) ; return NULL ; }

   buffer [ dstlen ] = 0 ;

   if ( p_dstlen ) *p_dstlen = dstlen ;

   return buffer ;
}



// ChANSIɕϊĕ
//  (size_t) -1 ȂkI[ƌȂ
// string ̒ maxlen ȏȂ NULL Ԃ
// o̖͂ɃkI[
// o͂ꂽ p_dstlen Ɋi[ikI[܂܂ȂAKvłȂ NULL łj
// ϊłȂ errno  EILSEQ Zbg
// w肳ꂽ]Ɋmۂ
// s NULL Ԃ
char *w2aduplen_max_expand_cp ( const wchar_t *string, size_t srclen, size_t maxlen, size_t addlen, size_t *p_dstlen, int codepage ) {

   int IsUsedDefaultChar = 0 ;

   int dstlen ;
   char *buffer ;

   if ( CP_ACP && ! codepage ) codepage = CP_ACP ;    // Ô

   if ( srclen == (size_t) -1 ) srclen = wcslen ( string ) ;
   if ( srclen >= maxlen ) return NULL ;
   if ( srclen > INT_MAX ) return NULL ;

   if ( ! ( dstlen = WideCharToMultiByte ( codepage, 0, string, (int) srclen, NULL, 0, NULL, NULL ) ) ) return NULL ;
   if ( ! ( buffer = (char*) malloc ( ( dstlen + addlen + 1 ) * sizeof(char) ) ) ) return NULL ;
   if ( ! ( dstlen = WideCharToMultiByte ( codepage, 0, string, (int) srclen, buffer, dstlen, NULL, & IsUsedDefaultChar ) ) ) { free ( buffer ) ; return NULL ; }

   buffer [ dstlen ] = 0 ;

   if ( p_dstlen ) *p_dstlen = dstlen ;

   if ( IsUsedDefaultChar ) errno = EILSEQ ;

   return buffer ;
}



#endif // ! defined NOCRT



////////////////////////////////////////////
//             osver / winver             //
////////////////////////////////////////////



#if ! defined _MSC_VER || _MSC_VER >= 1500 || defined NOCRT || defined OSVER_WINVER_TEST


typedef struct {
   volatile long osplatform ;
   volatile long osver ;
   volatile long winver ;
} WINVER_INFO ;



static int InitOsverWinverA ( WINVER_INFO *p_winver_info ) ;
static int InitOsverWinverW ( WINVER_INFO *p_winver_info ) ;



unsigned int GetOsverWinver_ ( int mode ) {

   static WINVER_INFO WinverInfo ;

   if ( ! WinverInfo.winver ) {
      if ( IS_COMPILER_NT_ONLY ) InitOsverWinverW ( & WinverInfo ) ;
      else                       InitOsverWinverA ( & WinverInfo ) ;
   }

   if      ( mode < 0 ) return WinverInfo.osplatform ;
   else if ( mode > 0 ) return WinverInfo.winver ;
   else                 return WinverInfo.osver ;
}



static int InitOsverWinverA ( WINVER_INFO *p ) {

#if defined NOCRT && ( ! defined _MSC_VER || _MSC_VER >= 1400 )
   HANDLE hProcessHeap = GetProcessHeap () ;
   OSVERSIONINFOA *pOsVersionInfo = (OSVERSIONINFOA*) HeapAlloc ( hProcessHeap, HEAP_ZERO_MEMORY, sizeof(OSVERSIONINFOA) ) ;
#else
   OSVERSIONINFOA OsVersionInfo = { 0 } ;
   OSVERSIONINFOA *pOsVersionInfo = & OsVersionInfo ;
#endif

   if ( pOsVersionInfo ) {

      pOsVersionInfo->dwOSVersionInfoSize = sizeof(OSVERSIONINFOA) ;
      GetVersionExA ( pOsVersionInfo ) ;

      InterlockedExchange ( & p->osplatform, pOsVersionInfo->dwPlatformId ) ;

      if ( pOsVersionInfo->dwPlatformId != VER_PLATFORM_WIN32_NT ) InterlockedExchange ( & p->osver, ( pOsVersionInfo->dwBuildNumber & 0x7FFF ) | 0x8000 ) ;
      else                                                         InterlockedExchange ( & p->osver, ( pOsVersionInfo->dwBuildNumber & 0x7FFF ) ) ;

      InterlockedExchange ( & p->winver, ( pOsVersionInfo->dwMajorVersion << 8 ) + pOsVersionInfo->dwMinorVersion ) ;

#if defined NOCRT && ( ! defined _MSC_VER || _MSC_VER >= 1400 )
      HeapFree ( hProcessHeap, 0, pOsVersionInfo ) ;
#endif
   }

   return 0 ;
}



static int InitOsverWinverW ( WINVER_INFO *p ) {

#if defined NOCRT && ( ! defined _MSC_VER || _MSC_VER >= 1400 )
   HANDLE hProcessHeap = GetProcessHeap () ;
   OSVERSIONINFOW *pOsVersionInfo = (OSVERSIONINFOW*) HeapAlloc ( hProcessHeap, HEAP_ZERO_MEMORY, sizeof(OSVERSIONINFOW) ) ;
#else
   OSVERSIONINFOW OsVersionInfo = { 0 } ;
   OSVERSIONINFOW *pOsVersionInfo = & OsVersionInfo ;
#endif

   if ( pOsVersionInfo ) {

      pOsVersionInfo->dwOSVersionInfoSize = sizeof(OSVERSIONINFOW) ;
      GetVersionExW ( pOsVersionInfo ) ;

      InterlockedExchange ( & p->osplatform, pOsVersionInfo->dwPlatformId ) ;

      if ( pOsVersionInfo->dwPlatformId != VER_PLATFORM_WIN32_NT ) InterlockedExchange ( & p->osver, ( pOsVersionInfo->dwBuildNumber & 0x7FFF ) | 0x8000 ) ;
      else                                                         InterlockedExchange ( & p->osver, ( pOsVersionInfo->dwBuildNumber & 0x7FFF ) ) ;

      InterlockedExchange ( & p->winver, ( pOsVersionInfo->dwMajorVersion << 8 ) + pOsVersionInfo->dwMinorVersion ) ;

#if defined NOCRT && ( ! defined _MSC_VER || _MSC_VER >= 1400 )
      HeapFree ( hProcessHeap, 0, pOsVersionInfo ) ;
#endif
   }

   return 0 ;
}



#endif // ! defined _MSC_VER || _MSC_VER >= 1500 || defined NOCRT || defined OSVER_WINVER_TEST



////////////////////////////////////////////
//                  WIN64                 //
////////////////////////////////////////////



#if ! defined _WIN64



typedef VOID ( WINAPI *GETNATIVESYSTEMINFO ) ( LPSYSTEM_INFO lpSystemInfo ) ;



// OS  64-bit Ȃ 0 ȊOԂ
int IsWin64 ( void ) {

   static volatile long nRetValueSaved = -1 ;
   if ( nRetValueSaved >= 0 ) return nRetValueSaved ;

   int nRetValue = 0 ;

   if ( IsNT () ) {

      HINSTANCE hKernel32 = GetModuleHandleW ( L"KERNEL32.DLL" ) ;

      GETNATIVESYSTEMINFO GetNativeSystemInfo = NULL ;
      if ( ! GETPROCADDRESS ( hKernel32, GETNATIVESYSTEMINFO, GetNativeSystemInfo ) ) {

         SYSTEM_INFO SystemInfo ;
         GetNativeSystemInfo ( & SystemInfo ) ;

         if ( SystemInfo.wProcessorArchitecture != PROCESSOR_ARCHITECTURE_INTEL ) nRetValue = 1 ;
      }
   }

   InterlockedExchange ( & nRetValueSaved, nRetValue ) ;

   return nRetValue ;
}



#endif
