// rtfconv_com.cpp : R\[p Rtfconv

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <conio.h>
#include <signal.h>
#include "msc.h"

#include "access.h"
#include "ask.h"
#include "codepage.h"
#include "fileconv.h"
#include "filename.h"
#include "nls.h"
#include "oemio.h"
#include "res.h"
#include "resource.h"
#include "shfolder.h"
#include "stringex.h"
#include "uniblock.h"
#include "wildcard.h"
#include "winmain.h"

#include "rtfconv.h"
#include "rtfconv_opt.h"
#include "rtfconv_com.h"
#include "conv_err.h"
#include "conv_opt.h"
#include "conv_loaddll.h"



static volatile int IsAborted ;


static int IsCalledByOldCommand ;
#define CALLED_BY_TXT2TXT  1
#define CALLED_BY_RTF2TXT  2
#define CALLED_BY_TXT2RTF  3


int DisplayErrorMessageA ( int nErrorCode, const char *szFileName, const char *szExtension ) ;
int DisplayErrorMessageW ( int nErrorCode, const wchar_t *szFileName, const wchar_t *szExtension ) ;
int CALLBACK WildcardCallbackA ( const char *szSearchPath, WIN32_FIND_DATAA *pFindData, int nDepth, LPARAM lParam ) ;
int CALLBACK WildcardCallbackW ( const wchar_t *szSearchPath, WIN32_FIND_DATAW *pFindData, int nDepth, LPARAM lParam ) ;
int WINAPI CallRtfconvExA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, void *pInfo ) ;
int WINAPI CallRtfconvExW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, void *pInfo ) ;
int PrintFileStart ( int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
int PrintFileEnd ( int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
void __cdecl AbortProc ( int nSignal ) ;
int Help ( void ) ;



int main ( int argc, char *argv [] ) {

   int nExitCode = 0 ;           // IR[h
   int nError = 0 ;              // G[
   int nSearchPath = 0 ;         // pX
   int nSearchDepth = 0 ;        // [
   int nExtension = 0 ;          // gq

   // for wildcard.cpp
   unsigned long dwSearchMode = WILD_SEARCH_FILE | WILD_SORT ;

   int nSrcCodePage = -1 ;          // ̓R[hy[W
   int nDstCodePage = -1 ;          // o̓R[hy[W
   char *szSingleByteFont = NULL ;  // PoCgptHg
   char *szMultiByteFont = NULL ;   // QoCgptHg

   int IsSpecifiedCodePage = 0 ;    // R[hy[Ww肵
   int IsDisplayHelp = 0 ;          // wv\邩
   int IsWriteInUnicode = 0 ;       // G[o͂ Unicode 
   int IsReadOption = 0 ;           // t@Cǂݍނ
   int IsCatchErrorStrictly = 0 ;   // G[o~
   unsigned long dwFileconvMode = 0 ;  // t@Cϊ̃IvV

   char *szConfigFileNameA = NULL ;
   wchar_t *szConfigFileNameW = NULL ;


   // ̏iOj
   for ( int nCountArgv = 1 ; nCountArgv < argc ; nCountArgv ++ ) {

      if ( nError ) break ;

      char *szArgv = argv [ nCountArgv ] ;

      if ( *szArgv == '/' || *szArgv == '-' ) {
         switch ( *( szArgv + 1 ) ) {
            case 'W' :
            case 'w' :
               IsWriteInUnicode = 1 ;
               break ;
            case '?' :
               IsDisplayHelp = 1 ;
               break ;
            case '@' :
               IsReadOption = 1 ;
               if ( *( szArgv + 2 ) ) {
                  szConfigFileNameA = GetArgumentA ( nCountArgv ) + 2 ;
                  szConfigFileNameW = GetArgumentW ( nCountArgv ) + 2 ;
                  continue ;
               }
               break ;
            default :
               continue ;
         }
         if ( *( szArgv + 2 ) ) nError = nCountArgv ;
         continue ;
      }
   }


   // Wo͂ƃG[o͂ UNICODE 
   fwide ( stdout, ( IsWriteInUnicode ) ? 1 : -1 ) ;
   fwide ( stderr, ( IsWriteInUnicode ) ? 1 : -1 ) ;


   // IvVǂݍ
   if ( ! IsDisplayHelp && IsReadOption ) {
      int nResult ;
      if ( IsNT () ) nResult = ReadOptionW ( szConfigFileNameW ) ;
      else           nResult = ReadOptionA ( szConfigFileNameA ) ;
      if ( nResult > 0 ) {
         fputws ( GetStringW ( IDS_CANNOT_READ_OPTION ), stderr ) ;
         return 1 ;
      }
      if ( nResult < 0 ) {
         fputws ( GetStringW ( IDS_NOT_ENOUGH_MEMORY ), stderr ) ;
         return 1 ;
      }
   }
   if ( ( UnmappableCharFormatList [ nUnmappableCharFormat ] ).szFormat ) {
      szUnmappableCharFormat = ( UnmappableCharFormatList [ nUnmappableCharFormat ] ).szFormat ;
      IsOutputNumber = 1 ;
   }


   // ̏
   for ( int nCountArgv = 1 ; nCountArgv < argc ; nCountArgv ++ ) {

      if ( nError ) break ;

      char *szArgv = argv [ nCountArgv ] ;

      char *szString ;
      int *pCodePage ;
      int nName ;       // R[hy[WJEgpϐ
      int nNameMax ;    // R[hy[WJEgpϐ̍ől

      // IvV̎擾
      if ( *szArgv == '/' || *szArgv == '-' ) {
         switch ( *( szArgv + 1 ) ) {
            case 'b' :
            case 'B' :
               if ( isascii ( *( szArgv + 2 ) ) && isdigit ( *( szArgv + 2 ) ) ) {
                  errno = 0 ;
                  int nResult = strtoul ( szArgv + 2, & szString, 10 ) ;
                  if ( nResult == 0 )      IsUseArray = 0, IsNotUseArray = 0 ;
                  else if ( nResult == 1 ) IsUseArray = 0, IsNotUseArray = 1 ;
                  else if ( nResult == 2 ) IsUseArray = 1, IsNotUseArray = 0 ;
                  else                     nError = nCountArgv ;
                  if ( errno == ERANGE || *szString ) nError = nCountArgv ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "AUTO" ) ) {
                  IsUseArray = 0, IsNotUseArray = 0 ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "SYS" ) ) {
                  IsUseArray = 0, IsNotUseArray = 1 ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "DLL" ) ) {
                  IsUseArray = 1, IsNotUseArray = 0 ;
                  continue ;
               }
               nError = nCountArgv ;
               continue ;
            case 'c' :
            case 'C' :
               if ( IsSpecifiedCodePage >= 2 ) {
                  nError = nCountArgv ;
                  continue ;
               }
               if ( IsCalledByOldCommand == 0 || IsCalledByOldCommand == CALLED_BY_TXT2TXT ) {
                  if ( IsSpecifiedCodePage == 0 ) pCodePage = & nSrcCodePage ;
                  if ( IsSpecifiedCodePage == 1 ) pCodePage = & nDstCodePage ;
                  IsSpecifiedCodePage ++ ;
               }
               else if ( IsCalledByOldCommand == CALLED_BY_RTF2TXT ) {
                  pCodePage = & nDstCodePage ;
                  IsSpecifiedCodePage += 2 ;
               }
               else if ( IsCalledByOldCommand == CALLED_BY_TXT2RTF ) {
                  pCodePage = & nSrcCodePage ;
                  IsSpecifiedCodePage += 2 ;
               }

               if ( *( szArgv + 2 ) == 0 ) {
                  if ( pCodePage ) *pCodePage = ( IsOsCodePageHkscs () ) ? CP_CHINESE_HKSCS : GetACP () ;
                  continue ;
               }
               if ( isascii ( *( szArgv + 2 ) ) && isdigit ( *( szArgv + 2 ) ) ) {
                  errno = 0 ;
                  if ( pCodePage ) *pCodePage = strtoul ( szArgv + 2, & szString, 10 ) ;
                  if ( errno == ERANGE || *szString ) nError = nCountArgv ;
                  continue ;
               }
               for ( nName = 0, nNameMax = CODEPAGE_NAME_LIST_NELEM ; nName < nNameMax ; nName ++ ) {
                  if ( ! stricmp ( szArgv + 2, CodePageNameList [ nName ].szCodeName ) ) {
                     if ( pCodePage ) *pCodePage = CodePageNameList [ nName ].nCodePage ;
                     break ;
                  }
               }
               if ( ! ( nName < nNameMax ) ) nError = nCountArgv ;
               continue ;
            case 'e' :
            case 'E' :
               if ( ! stricmp ( szArgv + 1, "error" ) ) {
                  IsCatchErrorStrictly = 1 ;
                  continue ;
               }
               nError = nCountArgv ;
               break ;
            case 'F' :
               szString = strchr ( szArgv, '\"' ) ;
               if ( szString ) *szString = 0 ;
               if ( ! *( szArgv + 2 ) ) nError = nCountArgv ;
               szMultiByteFont = szArgv + 2 ;
               continue ;
            case 'f' :
               szString = strchr ( szArgv, '\"' ) ;
               if ( szString ) *szString = 0 ;
               if ( ! *( szArgv + 2 ) ) {
                  IsUseSameFont = 1 ;
                  continue ;
               }
               szSingleByteFont = szArgv + 2 ;
               continue ;
            case 'K' :
            case 'k' :
               if ( isascii ( *( szArgv + 2 ) ) && isdigit ( *( szArgv + 2 ) ) ) {
                  errno = 0 ;
                  nDefaultCjkCodePage = strtoul ( szArgv + 2, & szString, 10 ) ;
                  if ( errno == ERANGE || *szString ) nError = nCountArgv ;
                  if ( ! IsMultiByteCodePage ( nDefaultCjkCodePage ) ) nError = nCountArgv ;
                  continue ;
               }
               for ( nName = 0, nNameMax = CODEPAGE_NAME_LIST_NELEM ; nName < nNameMax ; nName ++ ) {
                  if ( ! stricmp ( szArgv + 2, CodePageNameList [ nName ].szCodeName ) ) {
                     nDefaultCjkCodePage = CodePageNameList [ nName ].nCodePage ;
                     break ;
                  }
               }
               if ( ! ( nName < nNameMax ) ) nError = nCountArgv ;
               if ( ! IsMultiByteCodePage ( nDefaultCjkCodePage ) ) nError = nCountArgv ;
               continue ;
            case 'L' :
            case 'l' :
               if ( *( szArgv + 2 ) == '-' && *( szArgv + 3 ) == 0 ) IsUseSameFont = 0 ;
               else if ( *( szArgv + 2 ) == 0 ) IsUseSameFont = 1 ;
               else nError = nCountArgv ;
               continue ;
            case 'M' :
            case 'm' :
               for ( nName = 0, nNameMax = MODE_OPTION_NAME_LIST_NELEM ; nName < nNameMax ; nName ++ ) {

                  const char *szName = ( ModeOptionNameList [ nName ] ).szName ;
                  int *pValue = ( ModeOptionNameList [ nName ] ).nValue ;

                  if ( ! strheadicmp ( szArgv + 2, szName ) ) {

                     int cFlag = *( szArgv + 2 + strlen ( szName ) ) ;

                     if ( pValue == & IsOutputNumber ) szUnmappableCharFormat = NULL ;

                     if ( cFlag == 0 )        *pValue = 1 ;
                     else if ( cFlag == '-' ) *pValue = 0 ;
                     else if ( pValue == & IsOutputNumber && cFlag == ':' ) {
                        *pValue = 1 ;
                        char *szFormat = szArgv + 2 + strlen ( szName ) + 1 ;
                        if ( *szFormat ) szUnmappableCharFormat = szFormat ;
                     }
                     else nError = nCountArgv ;

                     if ( pValue == & IsOutputNumber && *pValue ) IsOutputNumberHtml = IsOneQuestionMark = IsOutputNumberMsc = 0 ;
                     if ( pValue == & IsOutputNumberHtml && *pValue ) IsOutputNumber = IsOneQuestionMark = IsOutputNumberMsc = 0 ;
                     if ( pValue == & IsOneQuestionMark && *pValue ) IsOutputNumber = IsOutputNumberHtml = IsOutputNumberMsc = 0 ;
                     if ( pValue == & IsOutputNumberMsc && *pValue ) IsOutputNumber = IsOutputNumberHtml = IsOneQuestionMark = 0 ;

                     if ( pValue == & IsNoJis83New && *pValue ) IsNoJis83Old = 0 ;
                     if ( pValue == & IsNoJis83Old && *pValue ) IsNoJis83New = 0 ;

                     if ( IsNoGbk ) IsGbOutput = 0 ;
                     if ( IsGb18030 ) IsGbOutput = 2 ;

                     break ;
                  }

               }
               if ( ! ( nName < nNameMax ) ) nError = nCountArgv ;
               continue ;
            case 'H' :
            case 'h' :
               if ( ! stricmp ( szArgv + 1, "hide" ) ) {
                  dwFileconvMode |= FILECONV_TO_HIDDEN ;
                  continue ;
               }
               dwFileconvMode |= FILECONV_NO_ECHO ;
               break ;
            case 'p' :
            case 'P' :
               if ( isascii ( *( szArgv + 2 ) ) && isdigit ( *( szArgv + 2 ) ) ) {
                  errno = 0 ;
                  int nResult = strtoul ( szArgv + 2, & szString, 10 ) ;
                  if ( nResult == 0 )      IsOutputCR = 1, IsOutputLF = 1 ;
                  else if ( nResult == 1 ) IsOutputCR = 0, IsOutputLF = 1 ;
                  else if ( nResult == 2 ) IsOutputCR = 1, IsOutputLF = 0 ;
                  else                     nError = nCountArgv ;
                  if ( errno == ERANGE || *szString ) nError = nCountArgv ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "CRLF" ) ) {
                  IsOutputCR = 1 ;
                  IsOutputLF = 1 ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "LF" ) ) {
                  IsOutputCR = 0 ;
                  IsOutputLF = 1 ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "CR" ) ) {
                  IsOutputCR = 1 ;
                  IsOutputLF = 0 ;
                  continue ;
               }
               nError = nCountArgv ;
               continue ;
            case 'S' :
            case 's' :
               dwSearchMode |= WILD_SEARCH_SUB ;
               if ( *( szArgv + 2 ) == ':' && isascii ( *( szArgv + 3 ) ) && isdigit ( *( szArgv + 3 ) ) ) {
                  errno = 0 ;
                  nSearchDepth = strtoul ( szArgv + 3, & szString, 10 ) ;
                  if ( errno == ERANGE || *szString ) nError = nCountArgv ;
                  if ( ! nSearchDepth ) dwSearchMode &= ~ WILD_SEARCH_SUB ;
                  continue ;
               }
               break ;
            case 'T' :
            case 't' :
               dwFileconvMode |= FILECONV_SAVE_TIMESTAMP ;
               break ;
            case 'W' :
            case 'w' :
               // IsWriteInUnicode = 1 ;
               break ;
            case 'Z' :
            case 'z' :
               if ( isascii ( *( szArgv + 2 ) ) && isdigit ( *( szArgv + 2 ) ) ) {
                  szString = szArgv + 2 ;
                  errno = 0 ;
                  nFontSize = strtoul ( szArgv + 2, & szString, 10 ) ;
                  if ( errno == ERANGE || *szString ) nError = nCountArgv ;
                  continue ;
               }
               nError = nCountArgv ;
               break ;
            case '?' :
               // IsDisplayHelp = 1 ;
               continue ;
            case '@' :
               // ReadOption () ;
               continue ;
            case '\\' :
               // ݊̂
               if ( ! stricmp ( szArgv + 2, "T2T" ) ) {
                  IsCalledByOldCommand = CALLED_BY_TXT2TXT ;
                  nSrcCodePage = nDstCodePage = ( IsOsCodePageHkscs () ) ? CP_CHINESE_HKSCS : GetACP () ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "R2T" ) ) {
                  IsCalledByOldCommand = CALLED_BY_RTF2TXT ;
                  nSrcCodePage = 0 ;
                  nDstCodePage = ( IsOsCodePageHkscs () ) ? CP_CHINESE_HKSCS : GetACP () ;
                  continue ;
               }
               if ( ! stricmp ( szArgv + 2, "T2R" ) ) {
                  IsCalledByOldCommand = CALLED_BY_TXT2RTF ;
                  nSrcCodePage = ( IsOsCodePageHkscs () ) ? CP_CHINESE_HKSCS : GetACP () ;
                  nDstCodePage = 0 ;
                  continue ;
               }
               nError = nCountArgv ;
               continue ;
            default :
               nError = nCountArgv ;
               break ;
         }
         if ( *( szArgv + 2 ) ) nError = nCountArgv ;
         continue ;
      }

      // t@C̎擾
      if ( ! nSearchPath ) {
         nSearchPath = nCountArgv ;
         continue ;
      }

      // gq̎擾
      if ( ! nExtension ) {
         nExtension = nCountArgv ;
         continue ;
      }

      nError = nCountArgv ;
   }


   // _CNǧ
   if ( nSearchPath && ! fisatty ( stdin ) || nExtension && ! fisatty ( stdout ) ) {
      fputws ( GetStringW ( IDS_CANNOT_REDIRECT ), stderr ) ;
      return 1 ;
   }

   // wv̕\
   if ( IsDisplayHelp || ! nSearchPath && fisatty ( stdin ) && ! nError ) {
      Help () ;
      return 1 ;
   }

   // G[\
   if ( nError ) {
      fwprintf ( stderr, L"%s%s\n", GetStringW ( IDS_PARAM_ERROR ), GetArgumentW ( nError ) ) ;
      return 1 ;
   }

   // R\[w
   if ( nSearchPath ) {
      wchar_t *szFileName = GetFileNameW ( GetArgumentW ( nSearchPath ) ) ;
      if ( szFileName && ! wcsicmp ( szFileName, L"CON" ) ) nSearchPath = 0 ;
   }

   // TufBNg\H
   if ( ! nSearchPath && ( dwSearchMode & WILD_SEARCH_SUB ) ) {
      fputws ( GetStringW ( IDS_CANNOT_SEARCH_SUBDIR ), stderr ) ;
      return 1 ;
   }

   // R[hy[Ww肳ꂽH
   if ( IsSpecifiedCodePage != 2 && ! IsCalledByOldCommand ) {
      fputws ( GetStringW ( IDS_NOT_SPECIFIED_CODEPAGE ), stderr ) ;
      return 1 ;
   }

   // ^CX^vۑ\H
   if ( ( dwFileconvMode & FILECONV_SAVE_TIMESTAMP ) && ! nExtension ) {
      fputws ( GetStringW ( IDS_CANNOT_SAVE_TIMESTAMP ), stderr ) ;
   }


   // DLL̃[h
   if ( LoadRtfconvDll () ) {
      fputws ( GetStringW ( IDS_DLL_ERROR ), stderr ) ;
      return 1 ;
   }


   RTFCONVEXINFO Info = { sizeof(RTFCONVEXINFO) } ;
   RTFFONTINFO FontInfo = { sizeof(RTFFONTINFO) } ;
   RTFFONTEXINFO FontExInfo [ UNICODE_BLOCK_NELEM ] ;
   RTFLANGIDINFO LangIdInfo [ UNICODE_BLOCK_NELEM ] ;

   Info.pFontInfo = & FontInfo ;
   Info.pFontExInfo = FontExInfo ;
   Info.pLangIdInfo = LangIdInfo ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   Info.pReserved = (void*) & ReservedInfo ;


   // IvVݒ
   Info.dwConversionMode = TranslateConversionMode ( nSrcCodePage, nDstCodePage ) ;
   Info.dwAdditionalMode = TranslateAdditionalMode ( nSrcCodePage, nDstCodePage ) ;


   // TXTo͎̎w
   if ( nDstCodePage ) {

      // o̓tH[}bgݒ
      if ( IsOutputNumber && szUnmappableCharFormat ) Info.pUnmappableCharFormat = szUnmappableCharFormat ;
      if ( IsOneQuestionMark ) Info.pUnmappableCharFormat = "?" ;
      if ( IsOutputNumberMsc ) Info.pUnmappableCharFormat = "\\x%04X" ;

   }
   // RTFo͎̎w
   else {

      // Unicode̎w
      Info.nDefaultCjkCodePage = nDefaultCjkCodePage ;

      // tHgTCY
      Info.nFontSize = nFontSize ;
      if ( ! Info.nFontSize ) Info.nFontSize = -1 ;

      // PoCgo̓tHgݒ
      if ( szSingleByteFont ) strlcpy ( szEuropeanFont, szSingleByteFont, FONTNAME_MAX ) ;
      if ( *szEuropeanFont ) {
         // }`oCgR[h͎Ƀ}`oCgƃVOoCg̃tHgȂ
         if ( szMultiByteFont && ! strcmp ( szEuropeanFont, szMultiByteFont ) ) Info.dwConversionMode |= CONVMODE_USE_LOCAL_FONT ;
         else                                                                   FontInfo.pEuropean = szEuropeanFont ;
      }

      // QoCgo̓tHgݒ
      if ( szMultiByteFont ) {
         int nCodePage = GetWinCodePage ( GetModelessCodePage ( nSrcCodePage ) ) ;
         if ( nCodePage == CP_UTF16 ) nCodePage = nDefaultCjkCodePage ;
         if ( ! nCodePage || nCodePage == CP_JAPANESE )  strlcpy ( szJapaneseFont, szMultiByteFont, FONTNAME_MAX ) ;
         else if ( nCodePage == CP_KOREAN )              strlcpy ( szKoreanFont, szMultiByteFont, FONTNAME_MAX ) ;
         else if ( nCodePage == CP_CHINESE_SIMPLIFIED )  strlcpy ( szChineseSimplifiedFont, szMultiByteFont, FONTNAME_MAX ) ;
         else if ( nCodePage == CP_CHINESE_TRADITIONAL ) strlcpy ( szChineseTraditionalFont, szMultiByteFont, FONTNAME_MAX ) ;
         else if ( nCodePage == CP_THAI )                strlcpy ( szThaiFont, szMultiByteFont, FONTNAME_MAX ) ;
      }
      if ( *szJapaneseFont )           FontInfo.pJapanese = szJapaneseFont ;
      if ( *szChineseSimplifiedFont )  FontInfo.pChineseSimplified = szChineseSimplifiedFont ;
      if ( *szChineseTraditionalFont ) FontInfo.pChineseTraditional = szChineseTraditionalFont ;
      if ( *szKoreanFont )             FontInfo.pKorean = szKoreanFont ;
      if ( *szThaiFont )               FontInfo.pThai = szThaiFont ;

      // tHgڍׂݒ
      if ( FontNameArray ) {
         for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
            char *szFontName = FontNameArray + nIndex * LF_FACESIZE ;
            if ( *szFontName ) {
               int C ;
               if ( GetUnicodeBlock ( 0, 0, nIndex, & C, NULL, NULL ) < 0 ) continue ;
               ( FontExInfo [ Info.nNumberOfFontExInfo ] ).C = C ;
               ( FontExInfo [ Info.nNumberOfFontExInfo ] ).pFontName = szFontName ;
               Info.nNumberOfFontExInfo ++ ;
            }
         }
      }

      // ꎯʎqݒ
      if ( IsLangIdUserDefined ) {
         for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
            if ( LangIdArray [ nIndex ] ) {
               int C ;
               if ( GetUnicodeBlock ( 0, 0, nIndex, & C, NULL, NULL ) < 0 ) continue ;
               ( LangIdInfo [ Info.nNumberOfLangIdInfo ] ).C = C ;
               ( LangIdInfo [ Info.nNumberOfLangIdInfo ] ).nLangId = LangIdArray [ nIndex ] ;
               Info.nNumberOfLangIdInfo ++ ;
            }
         }
      }
   }



   // VOi֐w
   signal ( SIGINT, AbortProc ) ;


   if ( IsNT () ) {

      // t@Cw肵Ȃꍇ
      if ( ! nSearchPath ) {
         // s
         int nResult = CallRtfconvExW ( NULL, NULL, nSrcCodePage, nDstCodePage, & Info ) ;
         // G[𕪐
         if ( nResult && DisplayErrorMessageW ( nResult, NULL, NULL ) ) nExitCode = 1 ;
      }
      // t@Cw肵ꍇ
      else {

         // ChJ[h̓WJ
         WILDCARDW *Wildcard = SearchWildcardW ( NULL, GetArgumentW ( nSearchPath ), dwSearchMode, WildcardCallbackW, nSearchDepth ) ;

         if ( Wildcard == WILDCARD_ERROR ) {
            if ( IsAborted ) fputws ( GetStringW ( IDS_ABORTED ), stderr ) ;
            else             fputws ( GetStringW ( IDS_TOO_MANY_FILES ), stderr ) ;
            return 1 ;
         }

         if ( Wildcard == NULL ) {
            fputws ( GetStringW ( IDS_FILE_NOT_FOUND ), stderr ) ;
            return 1 ;
         }

         wchar_t *szExtension = ( nExtension ) ? GetArgumentW ( nExtension ) : NULL ;
         wchar_t *szDirName = NULL ;

         nExitCode = 0 ;
         if ( szExtension && *szExtension == '.' ) szExtension ++ ;  // gq '.' 폜

         int IsPrintStartEnd = ( ! szExtension && ( ( nDstCodePage == CP_UTF8 ) ? ( Info.dwConversionMode & CONVMODE_UTF8_WITH_BOM ) : ( ! ( Info.dwConversionMode & CONVMODE_NO_OUTPUT_BOM ) ) ) ) ;
         if ( IsPrintStartEnd ) ReservedInfo.dwMode = RESERVEDMODE_NO_START_END ;
         if ( IsPrintStartEnd ) PrintFileStart ( nDstCodePage, & Info ) ;

         // t@C݂ꍇ
         for ( const wchar_t *szString = GetFirstFileW ( Wildcard ) ; szString ; szString = GetNextFileW ( Wildcard ) ) {

            // ~
            if ( IsAborted && DisplayErrorMessageW ( CONV_ERROR_ABORTED, NULL, NULL ) < 0 ) { nExitCode = 1 ; break ; }

            // tH_̎擾
            if ( ! GetFileNameW ( szString ) ) {
               free ( szDirName ) ;
               szDirName = wcsdup_expand ( szString, (size_t) -1, MAX_PATH ) ;
               if ( ! szDirName && DisplayErrorMessageW ( FILECONV_ERROR_MEMORY, szString, NULL ) < 0 ) { nExitCode = 1 ; break ; }
               continue ;
            }

            // t@C̎擾
            const wchar_t *szFileName ;
            if ( szDirName ) szFileName = ChangeFileNameW ( szDirName, szString ) ;
            else             szFileName = szString ;

            // s
            int nResult = FileConvertW ( szFileName, szExtension, CallRtfconvExW, nSrcCodePage, nDstCodePage, & Info, dwFileconvMode ) ;

            // G[𕪐
            if ( nResult ) {
               if ( DisplayErrorMessageW ( nResult, szFileName, szExtension ) < 0 ) { nExitCode = 1 ; break ; }
               if ( IsCatchErrorStrictly ) { nExitCode = 1 ; break ; }
               continue ;
            }
         }

         free ( szDirName ) ;

         if ( IsPrintStartEnd ) PrintFileEnd ( nDstCodePage, & Info ) ;

         // ChJ[hnh̉
         CloseWildcardW ( Wildcard ) ;
      }
   }
   else {

      // t@Cw肵Ȃꍇ
      if ( ! nSearchPath ) {
         // s
         int nResult = CallRtfconvExA ( NULL, NULL, nSrcCodePage, nDstCodePage, & Info ) ;
         // G[𕪐
         if ( nResult && DisplayErrorMessageA ( nResult, NULL, NULL ) < 0 ) nExitCode = 1 ;
      }
      // t@Cw肵ꍇ
      else {

         // ChJ[h̓WJ
         WILDCARDA *Wildcard = SearchWildcardA ( NULL, GetArgumentA ( nSearchPath ), dwSearchMode, WildcardCallbackA, nSearchDepth ) ;

         if ( Wildcard == WILDCARD_ERROR ) {
            if ( IsAborted ) fputws ( GetStringW ( IDS_ABORTED ), stderr ) ;
            else             fputws ( GetStringW ( IDS_TOO_MANY_FILES ), stderr ) ;
            return 1 ;
         }

         if ( Wildcard == NULL ) {
            fputws ( GetStringW ( IDS_FILE_NOT_FOUND ), stderr ) ;
            return 1 ;
         }

         char *szExtension = ( nExtension ) ? GetArgumentA ( nExtension ) : NULL ;
         char *szDirName = NULL ;

         nExitCode = 0 ;
         if ( szExtension && *szExtension == '.' ) szExtension ++ ;  // gq '.' 폜

         int IsPrintStartEnd = ( ! szExtension && ( ( nDstCodePage == CP_UTF8 ) ? ( Info.dwConversionMode & CONVMODE_UTF8_WITH_BOM ) : ( ! ( Info.dwConversionMode & CONVMODE_NO_OUTPUT_BOM ) ) ) ) ;
         if ( IsPrintStartEnd ) ReservedInfo.dwMode = RESERVEDMODE_NO_START_END ;
         if ( IsPrintStartEnd ) PrintFileStart ( nDstCodePage, & Info ) ;

         // t@C݂ꍇ
         for ( const char *szString = GetFirstFileA ( Wildcard ) ; szString ; szString = GetNextFileA ( Wildcard ) ) {

            // ~
            if ( IsAborted && DisplayErrorMessageA ( CONV_ERROR_ABORTED, NULL, NULL ) < 0 ) { nExitCode = 1 ; break ; }

            // tH_̎擾
            if ( ! GetFileNameA ( szString ) ) {
               free ( szDirName ) ;
               szDirName = strdup_expand ( szString, (size_t) -1, MAX_PATH ) ;
               if ( ! szDirName && DisplayErrorMessageA ( FILECONV_ERROR_MEMORY, szString, NULL ) < 0 ) { nExitCode = 1 ; break ; }
               continue ;
            }

            // t@C̎擾
            const char *szFileName ;
            if ( szDirName ) szFileName = ChangeFileNameA ( szDirName, szString ) ;
            else             szFileName = szString ;

            // s
            int nResult = FileConvertA ( szFileName, szExtension, CallRtfconvExA, nSrcCodePage, nDstCodePage, & Info, dwFileconvMode ) ;

            // G[𕪐
            if ( nResult ) {
               if ( DisplayErrorMessageA ( nResult, szFileName, szExtension ) < 0 ) { nExitCode = 1 ; break ; }
               if ( IsCatchErrorStrictly ) { nExitCode = 1 ; break ; }
               continue ;
            }
         }

         free ( szDirName ) ;

         if ( IsPrintStartEnd ) PrintFileEnd ( nDstCodePage, & Info ) ;

         // ChJ[hnh̉
         CloseWildcardA ( Wildcard ) ;
      }
   }

   return nExitCode ;
}



// Ro[^̃G[bZ[Wo
// ߂ĺAȂ 0 AsȂ 1 Aps\ȎsȂ -1 Ԃ
int DisplayErrorMessageA ( int nErrorCode, const char *szFileName, const char *szExtension ) {

   if ( ! szFileName ) szFileName = "" ;
   if ( ! szExtension ) szExtension = "" ;

   wchar_t szFileNameW [ MAX_PATH ] ;
   wchar_t szExtensionW [ MAX_PATH ] ;

   a2wlcpy ( szFileNameW, szFileName, MAX_PATH ) ;
   a2wlcpy ( szExtensionW, szExtension, MAX_PATH ) ;

   return DisplayErrorMessageW ( nErrorCode, szFileNameW, szExtensionW ) ;
}



// Ro[^̃G[bZ[Wo
// ߂ĺAȂ 0 AsȂ 1 Aps\ȎsȂ -1 Ԃ
int DisplayErrorMessageW ( int nErrorCode, const wchar_t *szFileName, const wchar_t *szExtension ) {

   if ( ! szFileName ) szFileName = L"" ;
   if ( ! szExtension ) szExtension = L"" ;

   switch ( nErrorCode ) {
      case CONV_SUCCESS :
         return 0 ;
      case CONV_ERROR_WRITE :
         fwprintf ( stderr, L"\r%s%s", szFileName, GetStringW ( IDS_WRITE_ERROR ) ) ;
         return 1 ;
      case CONV_ERROR_READ :
         fwprintf ( stderr, L"\r%s%s", szFileName, GetStringW ( IDS_ACCESS_DENIED ) ) ;
         return 1 ;
      case CONV_ERROR_FORMAT :
         fwprintf ( stderr, L"\r%s%s", szFileName, GetStringW ( IDS_INVALID_FORMAT ) ) ;
         return 1 ;
      case CONV_ERROR_BRACE :
         fwprintf ( stderr, L"\r%s%s", szFileName, GetStringW ( IDS_BRACE_ERROR ) ) ;
         return 1 ;
      case CONV_ERROR_MEMORY :
         fwprintf ( stderr, L"\r%s%s", szFileName, GetStringW ( IDS_NOT_ENOUGH_MEMORY ) ) ;
         return -1 ;
      case CONV_ERROR_CODEPAGE :
         fputws ( GetStringW ( IDS_INVALID_CODEPAGE ), stderr ) ;
         return -1 ;
      case CONV_ERROR_ABORTED :
         fwprintf ( stderr, L"\r%s%s", szFileName, GetStringW ( IDS_ABORTED ) ) ;
         return -1 ;
      case FILECONV_ERROR_ACCESS :
         fwprintf ( stderr, L"%s%s", szFileName, GetStringW ( IDS_ACCESS_DENIED ) ) ;
         return 1 ;
      case FILECONV_ERROR_EXTENSION :
         fwprintf ( stderr, L"\r%s%s", szExtension, GetStringW ( IDS_INVALID_EXTENSION ) ) ;
         return -1 ;
      case FILECONV_ERROR_MEMORY :
         fwprintf ( stderr, L"%s%s", szFileName, GetStringW ( IDS_NOT_ENOUGH_MEMORY ) ) ;
         return -1 ;
      default :
         fputws ( GetStringW ( IDS_UNKNOWN_ERROR ), stderr ) ;
         return -1 ;
   }
}



// ChJ[h̃R[obN֐
// TCY 0 ̃t@CrATufBNg̐[𐧌䂷
int CALLBACK WildcardCallbackA ( const char *szSearchPath, WIN32_FIND_DATAA *pFindData, int nDepth, LPARAM lParam ) {
   if ( ! pFindData->nFileSizeLow && ! pFindData->nFileSizeHigh ) return 0 ;
   if ( lParam ) if ( nDepth > lParam ) return -1 ;
   return 1 ;
}



// ChJ[h̃R[obN֐
// TCY 0 ̃t@CrATufBNg̐[𐧌䂷
int CALLBACK WildcardCallbackW ( const wchar_t *szSearchPath, WIN32_FIND_DATAW *pFindData, int nDepth, LPARAM lParam ) {
   if ( ! pFindData->nFileSizeLow && ! pFindData->nFileSizeHigh ) return 0 ;
   if ( lParam ) if ( nDepth > lParam ) return -1 ;
   return 1 ;
}



// I֐
void __cdecl AbortProc ( int nSignal ) {
   IsAborted = 1 ;
   AbortWildcard () ;
   if ( AbortRtfconv ) AbortRtfconv () ;
   else {
      fputws ( GetStringW ( IDS_ABORTED ), stderr ) ;
      DisableAlertOnClose () ;
      exit ( 1 ) ;
   }
   signal ( nSignal, AbortProc ) ;
}



////////////////////////////////////////////
//           ϊ֐̌Ăяo           //
////////////////////////////////////////////



int WINAPI CallRtfconvExA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, void *pInfo ) {
   return RtfconvExA ( szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, (RTFCONVEXINFO*) pInfo ) ;
}



int WINAPI CallRtfconvExW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, void *pInfo ) {
   return RtfconvExW ( szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, (RTFCONVEXINFO*) pInfo ) ;
}



int PrintFileStart ( int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   RTFCONVEXINFO Info = *pInfo ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   Info.pReserved = (void*) & ReservedInfo ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;

   if ( IsNT () ) {
      int nResult = RtfconvExW ( NULL, NULL, 0, nDstCodePage, (RTFCONVEXINFO*) & Info ) ;
      if ( nResult ) { DisplayErrorMessageW ( nResult, NULL, NULL ) ; exit ( 1 ) ; }
   }
   else {
      int nResult = RtfconvExA ( NULL, NULL, 0, nDstCodePage, (RTFCONVEXINFO*) & Info ) ;
      if ( nResult ) { DisplayErrorMessageA ( nResult, NULL, NULL ) ; exit ( 1 ) ; }
   }

   return 0 ;
}



int PrintFileEnd ( int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   RTFCONVEXINFO Info = *pInfo ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   Info.pReserved = (void*) & ReservedInfo ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;

   if ( IsNT () ) {
      int nResult = RtfconvExW ( NULL, NULL, 0, nDstCodePage, (RTFCONVEXINFO*) & Info ) ;
      if ( nResult ) { DisplayErrorMessageW ( nResult, NULL, NULL ) ; exit ( 1 ) ; }
   }
   else {
      int nResult = RtfconvExA ( NULL, NULL, 0, nDstCodePage, (RTFCONVEXINFO*) & Info ) ;
      if ( nResult ) { DisplayErrorMessageA ( nResult, NULL, NULL ) ; exit ( 1 ) ; }
   }

   return 0 ;
}



////////////////////////////////////////////
//                 wv                 //
////////////////////////////////////////////



// wv\֐
int Help ( void ) {


   // R}h
   static const wchar_t szCommandName [] = L"RTFCONV" ;


   int nVersion = 0 ;


   // ĨL[vvg𖳌
   DisableAlertOnClose () ;


   // o[W擾
   if ( ! hRtfconv && LoadRtfconvDll () ) {
      fputws ( GetStringW ( IDS_DLL_ERROR ), stderr ) ;
      if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_ASK_DISPLAY_HELP ), MB_YESNO ) ) return 1 ;
      FILEVERSION FileVersion ;
      if ( ! GetFileVersion ( & FileVersion ) ) nVersion = TranslateVersionInfo ( & FileVersion ) ;
   }
   else {
      nVersion = GetDllVersion () ;
   }


   // T

   fwprintf ( stdout, L" %s   version %d.%02d  %s  %s\n", szCommandName, nVersion / 100, nVersion % 100, GetVersionCopyrightW (), WIN3264BIT_SUFFIX_W ) ;
   fwprintf ( stdout, GetStringW ( IDS_HELP_100 ), szCommandName ) ;
   fwprintf ( stdout, GetStringW ( IDS_HELP_101 ), szCommandName ) ;
   fwprintf ( stdout, GetStringW ( IDS_HELP_102 ), szCommandName ) ;

   fputws ( GetStringW ( IDS_HELP_103 ), stdout ) ;
   fputws ( GetStringW ( IDS_HELP_104 ), stdout ) ;
   fputws ( GetStringW ( IDS_HELP_105 ), stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_HELP_107 ), MB_YESNO ) ) return 1 ;


   // R[hy[Ww̐

   fputws ( GetStringW ( IDS_HELP_200 ), stdout ) ;
   fputws ( GetStringW ( IDS_HELP_201 ), stdout ) ;

   fputws ( szHelpLangList1, stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( szHelpLangList2, stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( szHelpLangList3, stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( szHelpLangList4, stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( szHelpLangList5, stdout ) ;
   fputws ( GetStringW ( IDS_HELP_202 ), stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( GetStringW ( IDS_HELP_203 ), stdout ) ;
   fputws ( GetStringW ( IDS_HELP_204 ), stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;


   // IvV̐

   fputws ( GetStringW ( IDS_HELP_300 ), stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( GetStringW ( IDS_HELP_301 ), stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   fputws ( GetStringW ( IDS_HELP_302 ), stdout ) ;
   if ( fisatty ( stdout ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   // 

   fputws ( GetStringW ( IDS_HELP_303 ), stdout ) ;
   if ( ! IsConsole () && fisatty ( stdout ) && fisatty ( stderr ) && AskW ( GetStringW ( IDS_PRESS_ANY_KEY ), MB_OK ) ) return 1 ;

   return 1 ;
}



