// rtfconv.h

#ifndef RTFCONV_H
#define RTFCONV_H


#ifdef __cplusplus
extern "C" {
#endif



////////////////////////////////////////////
//            C^[tFCX            //
////////////////////////////////////////////



typedef struct {
   size_t cbSize ;
   const char *pEuropean ;
   const char *pJapanese ;
   const char *pChineseSimplified ;
   const char *pChineseTraditional ;
   const char *pKorean ;
   const char *pThai ;
} RTFFONTINFO ;



typedef struct {
   int C ;
   const char *pFontName ;
} RTFFONTEXINFO ;



typedef struct {
   int C ;
   int nLangId ;
} RTFLANGIDINFO ;



typedef struct {

   size_t cbSize ;
   unsigned long dwConversionMode ;
   unsigned long dwAdditionalMode ;
   const char *pUnmappableCharFormat ;

// Writing RTF
   int nDefaultCjkCodePage ;
   int nFontSize ;
   const RTFFONTINFO *pFontInfo ;

   const RTFFONTEXINFO *pFontExInfo ;
   size_t nNumberOfFontExInfo ;

   const RTFLANGIDINFO *pLangIdInfo ;
   size_t nNumberOfLangIdInfo ;

// Reserved
   const void *pReserved ;

} RTFCONVEXINFO ;



// ϊ[h
#define CONVMODE_OUTPUT_CODE_NUMBER                   1
#define CONVMODE_NO_VARIANT_KANJI                     2
#define CONVMODE_NO_VARIANT_EUROPEAN                  4
#define CONVMODE_CONVERT_PUA                          8
#define CONVMODE_NO_HANKANA                        0x10
#define CONVMODE_NO_ZENALPHA                       0x20
#define CONVMODE_NO_ZENSPACE                       0x40
#define CONVMODE_NO_ZENSYMBOL                      0x80
#define CONVMODE_USE_LOCAL_FONT                   0x100
#define CONVMODE_NO_GBK                           0x200
#define CONVMODE_NO_CW                            0x400
#define CONVMODE_OUTPUT_GB18030                   0x800
#define CONVMODE_OUTPUT_CR                       0x1000
#define CONVMODE_OUTPUT_LF                       0x2000
#define CONVMODE_OUTPUT_HIDDEN                   0x4000
#define CONVMODE_EUC_HOJO                        0x8000
#define CONVMODE_USE_DLL_TABLE                  0x10000
#define CONVMODE_NO_OUTPUT_BOM                  0x20000
#define CONVMODE_ENDIAN_AUTO                    0x40000
#define CONVMODE_OUTPUT_LINK                    0x80000
#define CONVMODE_OUTPUT_RUBY                   0x100000
#define CONVMODE_OUTPUT_CODE_NUMBER_HTML       0x200000
#define CONVMODE_INPUT_CODE_NUMBER_HTML        0x400000
#define CONVMODE_USE_SYSTEM_TABLE              0x800000
#define CONVMODE_YEN_NO_BACKSLASH             0x1000000
#define CONVMODE_UTF8_WITH_BOM                0x2000000
#define CONVMODE_OUTPUT_NUMBER_SURROGATE      0x4000000
#define CONVMODE_VARIANT_ONLY_ROUNDTRIP       0x8000000
#define CONVMODE_DELETE_EOF                  0x10000000
#define CONVMODE_INPUT_CODE_NUMBER_C         0x20000000
#define CONVMODE_PEEKMESSAGE                 0x80000000

#define CONVMODE_NO_VARIANT         ( CONVMODE_NO_VARIANT_KANJI | CONVMODE_NO_VARIANT_EUROPEAN )

#if defined RTFCONV_DLL
#define CONVMODE_FILENAME_UNICODE            0x40000000
#endif



// gϊ[h
#define ADDITIONAL_MODE_PUA_TO_HKSCS                  1
#define ADDITIONAL_MODE_HKSCS_CJKB_TO_PUA             2
#define ADDITIONAL_MODE_HKSCS_CJKA_TO_PUA             4
#define ADDITIONAL_MODE_HKSCS_ALL_TO_PUA              8
#define ADDITIONAL_MODE_HKSCS_CJK41_TO_PUA         0x10
#define ADDITIONAL_MODE_GBK_TO_LINSHIDAIMA         0x20
#define ADDITIONAL_MODE_LINSHIDAIMA_TO_GBK         0x40
#define ADDITIONAL_MODE_GBK_OUTPUT_FANTIZI         0x80
#define ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR      0x100
#define ADDITIONAL_MODE_JIS_INPUT_MAC             0x200
#define ADDITIONAL_MODE_JIS_OUTPUT_MAC            0x400
#define ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR       0x800
#define ADDITIONAL_MODE_JIS_NO_JIS83_NEW_STYLE   0x1000
#define ADDITIONAL_MODE_JIS_NO_JIS83_OLD_STYLE   0x2000
#define ADDITIONAL_MODE_JIS_INPUT_2004           0x4000
#define ADDITIONAL_MODE_JIS_OUTPUT_2004          0x8000
#define ADDITIONAL_MODE_KS_NO_UHC               0x10000
#define ADDITIONAL_MODE_KS_PUNC                 0x20000
#define ADDITIONAL_MODE_OVERLINE_NO_TILDE       0x40000
#define ADDITIONAL_MODE_RTFFONT_XP             0x100000
#define ADDITIONAL_MODE_RTFFONT_VISTA          0x200000
#define ADDITIONAL_MODE_HKSCS_1999           0x20000000
#define ADDITIONAL_MODE_HKSCS_2001           0x40000000
#define ADDITIONAL_MODE_HKSCS_2004           0x80000000



#ifndef RESERVEDMODE_NO_START

typedef struct {
   size_t cbSize ;
   unsigned long dwMode ;
} RESERVEDINFO ;

// pReserved->dwMode, dwReserved
#define RESERVEDMODE_NO_START                         1
#define RESERVEDMODE_NO_END                           2
#define RESERVEDMODE_ONLY_START                       4
#define RESERVEDMODE_ONLY_END                         8
#define RESERVEDMODE_NO_START_END      ( RESERVEDMODE_NO_START | RESERVEDMODE_NO_END )

#endif



#ifdef RTFCONV_DLL


// V֐
int WINAPI RtfconvExA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
int WINAPI RtfconvExW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
intptr_t WINAPI RtfconvStringEx ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo, size_t nMaxLen ) ;
int WINAPI RtfconvCharEx ( int C, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
int WINAPI IsRtfconvValidCodePage ( int nCodePage ) ;
int WINAPI GetRtfconvLastError ( void ) ;
int WINAPI AbortRtfconv ( void ) ;
int WINAPI GetDllVersion ( void ) ;

// Â֐
int WINAPI RtfconvA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
int WINAPI RtfconvW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
intptr_t WINAPI RtfconvString ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode, size_t nMaxLen ) ;
int WINAPI RtfconvChar ( int C, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
int WINAPI IsAvailableCodePage ( int nCodePage ) ;
int WINAPI SetUnicodeFormat ( const char *szFormat ) ;
int WINAPI SetFontName ( int nCodePage, const char *szFontName ) ;
int WINAPI SetFontNameEx ( int C, const char *szFontName ) ;
int WINAPI SetDefaultCodePage ( int nCodePage ) ;
int WINAPI SetFontSize ( int nSize ) ;
int WINAPI SetLangId ( int C, int nLangId ) ;
int WINAPI SetAdditionalMode ( unsigned long dwMode ) ;
int WINAPI PrintFileStartA ( int nDstCodePage, const char *szDstFileName ) ;
int WINAPI PrintFileStartW ( int nDstCodePage, const wchar_t *szDstFileName ) ;
int WINAPI PrintFileEndA ( int nDstCodePage, const char *szDstFileName ) ;
int WINAPI PrintFileEndW ( int nDstCodePage, const wchar_t *szDstFileName ) ;
intptr_t WINAPI PrintFileStartString ( int nDstCodePage, void *szDstBuffer, size_t nMaxLen ) ;
intptr_t WINAPI PrintFileEndString ( int nDstCodePage, void *szDstBuffer, size_t nMaxLen ) ;
int WINAPI SetRtfconvWindow ( HWND hWnd, int IsDialog ) ;
int WINAPI Rtfconv ( const void *szSrcFileName, const void *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
int WINAPI PrintFileStart ( int nDstCodePage, void *pReserved ) ;
int WINAPI PrintFileEnd ( int nDstCodePage, void *pReserved ) ;


#else


// V֐
typedef int ( WINAPI *RTFCONVEXA  ) ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
typedef int ( WINAPI *RTFCONVEXW  ) ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
typedef intptr_t ( WINAPI *RTFCONVSTRINGEX  ) ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo, size_t nMaxLen ) ;
typedef intptr_t ( WINAPI *RTFCONVSTRINGEX2  ) ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo, size_t nSrcLen, size_t nMaxLen ) ;
typedef int ( WINAPI *RTFCONVCHAREX  ) ( int C, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
typedef int ( WINAPI *ISRTFCONVVALIDCODEPAGE ) ( int nCodePage ) ;
typedef int ( WINAPI *GETRTFCONVLASTERROR  ) ( void ) ;
typedef int ( WINAPI *ABORTRTFCONV  ) ( void ) ;
typedef int ( WINAPI *GETDLLVERSION  ) ( void ) ;

// Â֐
typedef int ( WINAPI *RTFCONVA  ) ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
typedef int ( WINAPI *RTFCONVW  ) ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
typedef intptr_t ( WINAPI *RTFCONVSTRING  ) ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode, size_t nMaxLen ) ;
typedef int ( WINAPI *RTFCONVCHAR  ) ( int C, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) ;
typedef int ( WINAPI *ISAVAILABLECODEPAGE  ) ( int nCodePage ) ;
typedef int ( WINAPI *SETUNICODEFORMAT  ) ( const char *szFormat ) ;
typedef int ( WINAPI *SETFONTNAME  ) ( int nCodePage, const char *szFontName ) ;
typedef int ( WINAPI *SETFONTNAMEEX  ) ( int C, const char *szFontName ) ;
typedef int ( WINAPI *SETDEFAULTCODEPAGE  ) ( int nCodePage ) ;
typedef int ( WINAPI *SETFONTSIZE  ) ( int nSize ) ;
typedef int ( WINAPI *SETLANGID  ) ( int C, int nLangId ) ;
typedef int ( WINAPI *SETADDITIONALMODE  ) ( unsigned long dwMode ) ;
typedef int ( WINAPI *PRINTFILESTARTA  ) ( int nDstCodePage, const char *szDstFileName ) ;
typedef int ( WINAPI *PRINTFILESTARTW  ) ( int nDstCodePage, const wchar_t *szDstFileName ) ;
typedef int ( WINAPI *PRINTFILEENDA  ) ( int nDstCodePage, const char *szDstFileName ) ;
typedef int ( WINAPI *PRINTFILEENDW  ) ( int nDstCodePage, const wchar_t *szDstFileName ) ;
typedef int ( WINAPI *SETRTFCONVWINDOW  ) ( HWND hWnd, int IsDialog ) ;


#endif // else of #ifdef RTFCONV_DLL



////////////////////////////////////////////
//              ̍\              //
////////////////////////////////////////////
#ifdef RTFCONV_DLL



typedef struct {

// ConvertTxt
   STOREDCHARSA *CharRefBuffer ;
   STOREDCHARSW32 *CharRefUnreadBuffer ;
   STOREDCHARSA *ReverseBuffer ;
   int cUnicodeLead ;

// GetcModal
   short nEncodeMode ;
   short IsIsoKrEscFound ;

// DecodeUtf7
   short nUtf7Shift ;
   short cUtf7Next ;

// FlushReverseBuffer
   short C1 ;        // = EOF
   short C2 ;        // = EOF

// DecodeVietnameseViqr
   short nViqrMode ;

// DecodeIscii
   unsigned short nIsciiCodePage ;

} READINGTEXTDATA ;



typedef struct {

// PutcTxt
   STOREDCHARSA *ReverseBuffer ;
   short IsReverse ;
   short nEncodeMode ;

// EncodeTamil, EncodeVietnamese, EncodeUtf7, EncodeSjis2004Composition
   int U1 ;       // = EOF    EncodeTamil, EncodeVietnamese, EncodeUtf7, EncodeSjis2004Composition
   int U2 ;       // = EOF    EncodeTamil, EncodeVietnamese, EncodeUtf7
   int U3 ;       // = EOF    EncodeTamil
   int U4 ;       // = EOF    EncodeTamil
   int C1 ;       // = EOF    EncodeTamil, EncodeSjis2004Composition
   int C2 ;       // = EOF    EncodeTamil
   int C3 ;       // = EOF    EncodeTamil
   int C4 ;       // = EOF    EncodeTamil
   short IsAvoidLoop ;     // EncodeTamil, EncodeVietnamese, EncodeSjis2004Composition

// EncodeUtf7
   short nUtf7Shift ;

// EncodeIscii
   short IsPrevHalant ;
   short IsPrevZwj ;
   unsigned short nIsciiCodePage ;

} WRITINGTEXTDATA ;



typedef struct {

   int nDefaultCodePage ;
   int nFontSize ;

// PutcRtf
   int nFontPrev ;         // = -1 ;
   int nLowByteFontPrev ;  // = -1 ;
   int nLangPrev ;
   int nLangFePrev ;
   short IsReversePrev ;
   short IsBackgroundMultiByte ;
   short IsInsideQuote ;
   short IsBidiFlushBuffer ;
   short IsReverseOnFlushBuffer ;
   STOREDCHARSW32 *ReverseBuffer ;

// WriteFontTable
   const RTFFONTINFO *pFontInfo ;
   const char **FontNameArray ;        // tHgւ̃|C^iUNICODE BLOCKj
   unsigned short *FontNumberArray ;   // tHg  NULL łȂ΃tHgԍiUNICODE BLOCKj
   unsigned short *CharsetArray ;      // ZbgiUNICODE BLOCKtHgԍj
   unsigned short *LangIdArray ;       // ꎯʎqiUNICODE BLOCKj

} WRITINGRTFDATA ;



typedef struct {

   STREAM *StreamIn ;
   STREAM *StreamOut ;
   int nSrcCodePage ;
   int nDstCodePage ;
   unsigned long dwConversionMode ;
   unsigned long dwAdditionalMode ;
   volatile const short *pAbortFlag ;
   int nError ;

// cache
   int nSrcPseudoCodePage ;
   int nDstPseudoCodePage ;
   int nSrcWinCodePage ;
   int nDstWinCodePage ;

// Putc
   int cHanKanaPrev ;
   short IsPrevCr ;
// PutUnmappableChar
   short IsInPutUnmmapleChar ;
   wchar_t *szUnmappableCharFormat ;

   READINGTEXTDATA *rt ;      // ConvertTxt

   union {
      WRITINGTEXTDATA *wt ;   // PutTxt
      WRITINGRTFDATA *wr ;    // PutRtf
   } ;

// PeekMessageProc
   HWND hDialog ;
   int IsMessageUnicode ;

// Reserved
   unsigned long dwReserved ;

} RTFCONVDATA ;



int SetRtfconvLastError ( int nError ) ;



#endif // #ifdef RTFCONV_DLL



////////////////////////////////////////////
//                 ̑                 //
////////////////////////////////////////////



#if defined RTFCONV_DLL || defined RTFCONV_EXE || defined RTFCONV_COM || defined SETUP_R

// obt@̒
#define FONTNAME_MAX                   LF_FACESIZE
#define UNMAPPABLE_CHAR_FORMAT_MAX     0x100

// W̏o̓tHgTCY
#define DEFAULT_FONT_SIZE              21

// W̏o̓tHg
#define DEFAULT_EUROPEAN_FONT             "Times New Roman"
#define DEFAULT_JAPANESE_FONT             "MS Mincho"
#define DEFAULT_CHINESE_SIMPLIFIED_FONT   "SimSun"
#define DEFAULT_CHINESE_TRADITIONAL_FONT  "MingLiU"
#define DEFAULT_KOREAN_FONT               "Gulim"
#define DEFAULT_THAI_FONT                 "Tahoma"
#define DEFAULT_GB18030_FONT              "Simsun (Founder Extended)"

// W̏o̓tHgi[Jj
#define DEFAULT_JAPANESE_FONT_L              "lr "
#define DEFAULT_CHINESE_SIMPLIFIED_FONT_L    "\xCB\xCE\xCC\xE5"
#define DEFAULT_CHINESE_TRADITIONAL_FONT_L   "\xB2\xD3\xA9\xFA\xC5\xE9"
#define DEFAULT_KOREAN_FONT_L                "\xB1\xBC\xB8\xB2"

#endif // #if defined RTFCONV_DLL || defined RTFCONV_EXE || defined RTFCONV_COM || defined SETUP_R



#ifdef __cplusplus
}
#endif

#endif


