// rtfconv_dll.h


#ifndef RTFCONV_DLL_H
#define RTFCONV_DLL_H



////////////////////////////////////////////
//              Reading RTF               //
////////////////////////////////////////////



#define ESCAPE       '\\'
#define BRACE_START  '{'
#define BRACE_END    '}'
#define SEMICOLON    ';'



static const char *const HiddenTagList [] = {
   "*",
   "v",
   "info",
   "colortbl",
   "stylesheet",
   "pict",
   "shppict",
   "nonshppict",
   "datafield",
   "header",
   "footer",
   "headerl",
   "headerr",
   "headerf",
   "footerl",
   "footerr",
   "footerf",
} ;

static const int HIDDEN_TAG_LIST_NELEM = NELEMOF ( HiddenTagList ) ;



static const wchar_t szCwHead       [] = L"fa" ;
static const wchar_t szCwFantiHead  [] = L"jf" ;
static const wchar_t szCw7Head      [] = L"CW-GB" ;
static const wchar_t szCw7FantiHead [] = L"CW-B5" ;

static const wchar_t szKwHead       [] = L"jv" ;
static const wchar_t szKwName       [] = L"؍" ;
static const wchar_t szKwHeadOld    [] = L"jr" ;
static const wchar_t szKwCompat     [] = L"KW݊nO" ;

static const wchar_t szCwnHead      [] = L"OMGB" ;
static const wchar_t szCwnFantiHead [] = L"OMɑ" ;
static const wchar_t szKwnHead      [] = L"OMKS" ;

static const wchar_t szSymbolFont      [] = L"Symbol" ;
static const wchar_t szWingdingsFont   [] = L"Wingdings" ;
static const wchar_t szSil       [] = L"SIL" ;
static const wchar_t szIpa       [] = L"IPA" ;
static const wchar_t sz93        [] = L"93" ;
static const wchar_t szUnicode   [] = L"Unicode" ;
static const wchar_t szGalatia         [] = L"Galatia" ;
static const wchar_t szGalatiaExtras   [] = L"Galatia Extras" ;
static const wchar_t szGreekTrans      [] = L"Greek Trans" ;
//static const wchar_t szEzra          [] = L"Ezra" ;
static const wchar_t szHebTrans        [] = L"Heb Trans" ;
static const wchar_t szHebTransCaps    [] = L"Heb Trans Caps" ;
static const wchar_t szDaiBanna        [] = L"Dai Banna" ;
static const wchar_t szApparatus       [] = L"Apparatus" ;
static const wchar_t szIpaphon      [] = L"IPAPhon" ;
static const wchar_t szIpasam       [] = L"Ipa-sam" ;
static const wchar_t szTimesNewRomanPhonetic [] = L"Times New Roman Phonetic" ;
static const wchar_t szSgreek       [] = L"Sgreek" ;
static const wchar_t szSGkClassic   [] = L"SGkClassic" ;
//static const wchar_t szShebrew    [] = L"Shebrew" ;
static const wchar_t szSPIonic      [] = L"SPIonic" ;
static const wchar_t szGreek           [] = L"Greek" ;
static const wchar_t szStandardGreek   [] = L"Standard Greek" ;
static const wchar_t szMilanGreek      [] = L"Milan Greek" ;
static const wchar_t szAngaros         [] = L"Angaros" ;
static const wchar_t szKorinthus       [] = L"Korinthus" ;
static const wchar_t szGrecsDuRoiWg    [] = L"Grecs du roi WG" ;
static const wchar_t szGrammata        [] = L"Grammata" ;
static const wchar_t szAthenian        [] = L"Athenian" ;
static const wchar_t szMounce          [] = L"Mounce" ;
static const wchar_t szTekniaGreek     [] = L"TekniaGreek" ;
static const wchar_t szMidEastTimes    [] = L"MidEastTimes" ;

static const wchar_t szHkscsName       [] = L"HKSCS" ;

static const wchar_t szVni          [] = L"VNI-" ;
static const wchar_t szVps          [] = L"VPS " ;
static const wchar_t szTcvn         [] = L".VN" ;
static const wchar_t szViscii1      [] = L"VN-TienGiang" ;
static const wchar_t szViscii2      [] = L"VI " ;
static const wchar_t szVnTimes      [] = L"VnTimes" ;
static const wchar_t szCirilica     [] = L" Cirilica" ;

static const wchar_t *const VisciiTrichloList [] = {
#if defined _MSC_VER && _MSC_VER < 1300   // Cannot compile wide char 0x0080-0x00FF with VC6
#pragma setlocale ( "C" )
#endif
   L"\x00C1nhMinh",
   L"MinhQu\x00E2n",
   L"UHo\x00E0i",
#if defined _MSC_VER && _MSC_VER < 1300   // Cannot compile wide char 0x0080-0x00FF with VC6
#include "msc.h"
#endif
   L"Heo May",
   L"HoangYen",
   L"PhuongThao",
   L"ThaHuong",
} ;

static const int VISCII_TRICHLO_LIST_NELEM = NELEMOF ( VisciiTrichloList ) ;



////////////////////////////////////////////
//              Writing RTF               //
////////////////////////////////////////////



static const char szRtfStartString  [] =  "{\\rtf1\\ansi\\deff0" ;
static const char szRtfEndString    [] = "\r\n}" ;
static const char szCharFormat      [] = "\\\'%02x" ;
static const char szFontNameFormat  [] = "{\\f%d\\fcharset%d %s%s;}" ;

static const char szFontSuffixGreek             [] = " Greek" ;
static const char szFontSuffixTurkish           [] = " Tur" ;
static const char szFontSuffixBaltic            [] = " Baltic" ;
static const char szFontSuffixCyrillic          [] = " Cyr" ;
static const char szFontSuffixCentralEuropean   [] = " CE" ;
static const char szFontSuffixHebrew            [] = " (Hebrew)" ;
static const char szFontSuffixArabic            [] = " (Arabic)" ;


// for PutCodeNumber()
static const wchar_t szUnmappableCharFormatDefault [] = L"[UNICODE:%04X]" ;



////////////////////////////////////////////
//          Reading/Writing RTF           //
////////////////////////////////////////////



typedef struct {
   int nCode ;
   const char *szName ;
} CHARACTER_NAME ;



static const CHARACTER_NAME RtfEscapeList [] = {
   { L'\\',    "\\" },
   { L'{',     "{" },
   { L'}',     "}" },
   { L'\n',    "par" },
   { L'\t',    "tab" },
   { 0x00A0,   "~" },
   { 0x00AD,   "-" },
   { 0x2018,   "lquote" },
   { 0x2019,   "rquote" },
   { 0x201C,   "ldblquote" },
   { 0x201D,   "rdblquote" },
   { 0x2022,   "bullet" },
   { 0x2013,   "endash" },
   { 0x2014,   "emdash" },
} ;

static const int RTF_ESCAPE_LIST_NELEM = NELEMOF ( RtfEscapeList ) ;



static const CHARACTER_NAME RtfEscapeCompatibleList [] = {
   { L'\n',    "line" },
   { L'\n',    "cell" },
   { 0x2002,   "enspace" },
   { 0x2003,   "emspace" },
   { 0x2009,   "qmspace" },
   { 0x2011,   "_" },
   { L'\n',    "\n" },
   { L'\n',    "\r" },
} ;

static const int RTF_ESCAPE_COMP_LIST_NELEM = NELEMOF ( RtfEscapeCompatibleList ) ;



#endif

