// rtfconv_dll_old.cpp

#include <windows.h>
#include <stdio.h>
#include "msc.h"
#include "codepage.h"
#include "storechr.h"
#include "stream.h"
#include "stringex.h"
#include "uniblock.h"

#include "conv_err.h"
#include "rtfconv.h"



static volatile long nSemaphore ;            // NĂ 1
#define LockSemaphore()    InterlockedExchange ( & nSemaphore, 1 )   // if success, return 0
#define UnlockSemaphore()  InterlockedExchange ( & nSemaphore, 0 )



////////////////////////////////////////////
//               ֐ (1)               //
////////////////////////////////////////////



static RTFCONVEXINFO Info ;



int WINAPI RtfconvA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = ( dwConversionMode & CONVMODE_NO_OUTPUT_BOM ) ? RESERVEDMODE_NO_START_END : 0 ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = RtfconvExA ( szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI RtfconvW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = ( dwConversionMode & CONVMODE_NO_OUTPUT_BOM ) ? RESERVEDMODE_NO_START_END : 0 ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = RtfconvExW ( szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



intptr_t WINAPI RtfconvString ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode, size_t nMaxLen ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = ( dwConversionMode & CONVMODE_NO_OUTPUT_BOM ) ? RESERVEDMODE_NO_START_END : 0 ;
   Info.pReserved = (void*) & ReservedInfo ;

   intptr_t nResult = RtfconvStringEx ( szSrcBuffer, szDstBuffer, nSrcCodePage, nDstCodePage, & Info, nMaxLen ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI RtfconvChar ( int C, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   Info.pReserved = NULL ;

   int nResult = RtfconvCharEx ( C, nSrcCodePage, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI IsAvailableCodePage ( int nCodePage ) {
   return IsRtfconvValidCodePage ( nCodePage ) ;
}



////////////////////////////////////////////
//               ֐ (2)               //
////////////////////////////////////////////



static RTFFONTINFO FontInfo ;
static char *pUnmappableCharFormat ;

static volatile long IsFreeBufferProcRegistered ;
void __cdecl FreeBuffer ( void ) ;



int WINAPI SetAdditionalMode ( unsigned long dwMode ) {

   if ( LockSemaphore () ) return 1 ;

   Info.dwAdditionalMode = dwMode ;

   UnlockSemaphore () ;
   return 0 ;
}



int WINAPI SetUnicodeFormat ( const char *szFormat ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = 0 ;

   free ( pUnmappableCharFormat ) ;
   pUnmappableCharFormat = NULL ;

   if ( szFormat && *szFormat ) {
      pUnmappableCharFormat = strdup ( szFormat ) ;
      if ( ! pUnmappableCharFormat ) nResult = 1 ;
   }

   if ( ! nResult ) Info.pUnmappableCharFormat = pUnmappableCharFormat ;

   if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI SetDefaultCodePage ( int nCodePage ) {

   if ( LockSemaphore () ) return 1 ;

   Info.nDefaultCjkCodePage = nCodePage ;

   UnlockSemaphore () ;
   return 0 ;
}



int WINAPI SetFontSize ( int nSize ) {

   if ( LockSemaphore () ) return 1 ;

   if ( nSize > 0 )    Info.nFontSize = nSize ;
   else if ( ! nSize ) Info.nFontSize = -1 ;

   UnlockSemaphore () ;
   return 0 ;
}



int WINAPI SetFontName ( int nCodePage, const char *szFontName ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = 0 ;
   const char **pDst ;

   switch ( nCodePage ) {
      case CP_ANSI_EUROPEAN :          pDst = & FontInfo.pEuropean ; break ;
      case CP_JAPANESE :               pDst = & FontInfo.pJapanese ; break ;
      case CP_CHINESE_SIMPLIFIED :     pDst = & FontInfo.pChineseSimplified ; break ;
      case CP_CHINESE_TRADITIONAL :    pDst = & FontInfo.pChineseTraditional ; break ;
      case CP_KOREAN :                 pDst = & FontInfo.pKorean ; break ;
      case CP_THAI :                   pDst = & FontInfo.pThai ; break ;
      default :                        pDst = NULL ; nResult = 1 ; break ;
   }

   if ( pDst ) {

      free ( (char*) *pDst ) ;
      *pDst = NULL ;

      if ( szFontName ) {
         *pDst = strdup ( szFontName ) ;
         if ( ! *pDst ) nResult = 1 ;
      }
   }

   FontInfo.cbSize = sizeof(RTFFONTINFO) ;
   Info.pFontInfo = & FontInfo ;

   if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

   UnlockSemaphore () ;
   return nResult ;
}



////////////////////////////////////////////
//               ֐ (3)               //
////////////////////////////////////////////



static int SetFontNameExInternal ( int C, const char *szFontName ) ;
static int SetLangIdInternal ( int C, int nLangId ) ;
static int FreeFontExArray ( void ) ;

static RTFFONTEXINFO *pFontExInfo ;
static int nNumberOfFontExInfo ;

static RTFLANGIDINFO *pLangIdInfo ;
static int nNumberOfLangIdInfo ;



int WINAPI SetFontNameEx ( int C, const char *szFontName ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = SetFontNameExInternal ( C, szFontName ) ;
   Info.pFontExInfo = pFontExInfo ;
   Info.nNumberOfFontExInfo = nNumberOfFontExInfo ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI SetLangId ( int C, int nLangId ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = SetLangIdInternal ( C, nLangId ) ;
   Info.pLangIdInfo = pLangIdInfo ;
   Info.nNumberOfLangIdInfo = nNumberOfLangIdInfo ;

   UnlockSemaphore () ;
   return nResult ;
}



static int SetFontNameExInternal ( int C, const char *szFontName ) {

   if ( C == -1 && ! szFontName ) {
      FreeFontExArray () ;
      pFontExInfo = NULL ;
      nNumberOfFontExInfo = 0 ;
   }
   else {

      if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

      if ( C >= 0 ) {
         int nIndex = GetUnicodeBlock ( C, 0, 0, & C, NULL, NULL ) ;
         if ( nIndex < 0 ) return 1 ;
      }

      if ( szFontName ) {

         for ( int nCount = 0 ; nCount < nNumberOfFontExInfo ; nCount ++ ) {
            if ( ( pFontExInfo [ nCount ] ).C == C ) {

               char *szTmp = strdup ( szFontName ) ;
               if ( ! szTmp ) return 1 ;

               free ( (char*) ( pFontExInfo [ nCount ] ).pFontName ) ;
               ( pFontExInfo [ nCount ] ).pFontName = szTmp ;

               return 0 ;
            }
         }


         RTFFONTEXINFO *pNew = (RTFFONTEXINFO*) realloc ( pFontExInfo, ( nNumberOfFontExInfo + 1 ) * sizeof(RTFFONTEXINFO) ) ;
         if ( ! pNew ) return 1 ;

         pFontExInfo = pNew ;

         ( pFontExInfo [ nNumberOfFontExInfo ] ).C = C ;
         ( pFontExInfo [ nNumberOfFontExInfo ] ).pFontName = strdup ( szFontName ) ;

         if ( ! ( pFontExInfo [ nNumberOfFontExInfo ] ).pFontName ) return 1 ;

         nNumberOfFontExInfo ++ ;


      }
      else if ( nNumberOfFontExInfo ) {

         for ( int nCount = 0 ; nCount < nNumberOfFontExInfo ; nCount ++ ) {
            if ( ( pFontExInfo [ nCount ] ).C == C ) {

               free ( (char*) ( pFontExInfo [ nCount ] ).pFontName ) ;
               memmove ( pFontExInfo + nCount, pFontExInfo + nCount + 1, ( nNumberOfFontExInfo - nCount - 1 ) * sizeof(RTFFONTEXINFO) ) ;
               nNumberOfFontExInfo -- ;

            }
         }

      }
   }

   return 0 ;
}



static int SetLangIdInternal ( int C, int nLangId ) {

   if ( C == -1 && ! nLangId ) {
      free ( pLangIdInfo ) ;
      pLangIdInfo = NULL ;
      nNumberOfLangIdInfo = 0 ;
   }
   else {

      if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

      if ( C >= 0 ) {
         int nIndex = GetUnicodeBlock ( C, 0, 0, & C, NULL, NULL ) ;
         if ( nIndex < 0 ) return 1 ;
      }

      if ( nLangId ) {

         for ( int nCount = 0 ; nCount < nNumberOfLangIdInfo ; nCount ++ ) {
            if ( ( pLangIdInfo [ nCount ] ).C == C ) {

               ( pLangIdInfo [ nCount ] ).nLangId = nLangId ;

               return 0 ;
            }
         }


         RTFLANGIDINFO *pNew = (RTFLANGIDINFO*) realloc ( pLangIdInfo, ( nNumberOfLangIdInfo + 1 ) * sizeof(RTFLANGIDINFO) ) ;
         if ( ! pNew ) return 1 ;

         pLangIdInfo = pNew ;

         ( pLangIdInfo [ nNumberOfLangIdInfo ] ).C = C ;
         ( pLangIdInfo [ nNumberOfLangIdInfo ] ).nLangId = nLangId ;

         nNumberOfLangIdInfo ++ ;


      }
      else if ( nNumberOfLangIdInfo ) {

         for ( int nCount = 0 ; nCount < nNumberOfLangIdInfo ; nCount ++ ) {
            if ( ( pLangIdInfo [ nCount ] ).C == C ) {

               memmove ( pLangIdInfo + nCount, pLangIdInfo + nCount + 1, ( nNumberOfLangIdInfo - nCount - 1 ) * sizeof(RTFLANGIDINFO) ) ;
               nNumberOfLangIdInfo -- ;

            }
         }

      }
   }

   return 0 ;
}



static int FreeFontExArray ( void ) {
   if ( pFontExInfo ) {
      for ( int nCount = 0 ; nCount < nNumberOfFontExInfo ; nCount ++ ) free ( (char*) ( pFontExInfo [ nCount ] ).pFontName ) ;
   }
   free ( pFontExInfo ) ;
   return 0 ;
}



void __cdecl FreeBuffer ( void ) {

   free ( (char*) FontInfo.pEuropean ) ;
   free ( (char*) FontInfo.pJapanese ) ;
   free ( (char*) FontInfo.pChineseSimplified ) ;
   free ( (char*) FontInfo.pChineseTraditional ) ;
   free ( (char*) FontInfo.pKorean ) ;
   free ( (char*) FontInfo.pThai ) ;

   free ( pUnmappableCharFormat ) ;

   free ( pLangIdInfo ) ;
   FreeFontExArray () ;
}



////////////////////////////////////////////
//               ֐ (4)               //
////////////////////////////////////////////



int WINAPI PrintFileStartA ( int nDstCodePage, const char *szDstFileName ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = RtfconvExA ( NULL, szDstFileName, CP_UTF16, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI PrintFileStartW ( int nDstCodePage, const wchar_t *szDstFileName ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = RtfconvExW ( NULL, szDstFileName, CP_UTF16, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI PrintFileEndA ( int nDstCodePage, const char *szDstFileName ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = RtfconvExA ( NULL, szDstFileName, CP_UTF16, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI PrintFileEndW ( int nDstCodePage, const wchar_t *szDstFileName ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = RtfconvExW ( NULL, szDstFileName, CP_UTF16, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



intptr_t WINAPI PrintFileStartString ( int nDstCodePage, void *szDstBuffer, size_t nMaxLen ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;
   Info.pReserved = (void*) & ReservedInfo ;

   intptr_t nResult = RtfconvStringEx ( NULL, szDstBuffer, CP_UTF16, nDstCodePage, & Info, nMaxLen ) ;

   UnlockSemaphore () ;
   return nResult ;
}



intptr_t WINAPI PrintFileEndString ( int nDstCodePage, void *szDstBuffer, size_t nMaxLen ) {

   if ( LockSemaphore () ) return SetRtfconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(RTFCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;
   Info.pReserved = (void*) & ReservedInfo ;

   intptr_t nResult = RtfconvStringEx ( NULL, szDstBuffer, CP_UTF16, nDstCodePage, & Info, nMaxLen ) ;

   UnlockSemaphore () ;
   return nResult ;
}



////////////////////////////////////////////
//               ֐ (5)               //
////////////////////////////////////////////



int WINAPI SetRtfconvWindow ( HWND hWnd, int IsDialog ) {
   return 0 ;
}



////////////////////////////////////////////
//               ֐ (6)               //
////////////////////////////////////////////



int WINAPI Rtfconv ( const void *szSrcFileName, const void *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {
   if ( dwConversionMode & CONVMODE_FILENAME_UNICODE ) return RtfconvW ( (const wchar_t*) szSrcFileName, (const wchar_t*) szDstFileName, nSrcCodePage, nDstCodePage, dwConversionMode & ~ CONVMODE_FILENAME_UNICODE ) ;
   else                                                return RtfconvA ( (const char*)    szSrcFileName, (const char*)    szDstFileName, nSrcCodePage, nDstCodePage, dwConversionMode & ~ CONVMODE_FILENAME_UNICODE ) ;
}



int WINAPI PrintFileStart ( int nDstCodePage, void *pReserved ) {
   return PrintFileStartA ( nDstCodePage, (char*) pReserved ) ;
}



int WINAPI PrintFileEnd ( int nDstCodePage, void *pReserved ) {
   return PrintFileEndA ( nDstCodePage, (char*) pReserved ) ;
}



