// tamil.cpp

#include <string.h>
#include "msc.h"
#include "codepage.h"
#include "stringex.h"
#include "tamil.h"
#include "tamiltable.h"



#define IsValidFirstChar(C)   ( C == 0x20 || C == 0xA0 || C >= 0x0B80 && C <= 0x0BFF )



const TAMIL_EQUIVALENT_TABLE *UnicodeToTamil ( const wchar_t *szSrc ) {

   if ( ! IsValidFirstChar ( *szSrc ) ) return NULL ;

   size_t nMaxLength = 0 ;
   int nCountSaved = -1 ;

   for ( int nCount = 0 ; nCount < TAMIL_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {

      const wchar_t *szDst = ( TamilEquivalentTable [ nCount ] ).szUnicode ;
      if ( ! szDst ) continue ;

      if ( ! wcsheadcmp ( szSrc, szDst ) ) {
         size_t nLength = wcslen ( szDst ) ;
         if ( nMaxLength < nLength ) {
            nCountSaved = nCount ;
            nMaxLength = nLength ;
         }
      }
   }

   if ( nCountSaved >= 0 ) return & TamilEquivalentTable [ nCountSaved ] ;

   return NULL ;
}



const TAMIL_EQUIVALENT_TABLE *TamilToUnicode ( const char *szSrc, int nSrcCodePage ) {

   size_t nMaxLength = 0 ;
   int nCountSaved = -1 ;

   for ( int nCount = 0 ; nCount < TAMIL_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {

      const char *szDst = NULL ;
      switch ( nSrcCodePage ) {
         case CP_TAMIL_TSCII :   szDst = ( TamilEquivalentTable [ nCount ] ).szTscii ; break ;
         case CP_TAMIL_TAB :     szDst = ( TamilEquivalentTable [ nCount ] ).szTab ; break ;
         case CP_TAMIL_TAM :     szDst = ( TamilEquivalentTable [ nCount ] ).szTam ; break ;
      }
      if ( ! szDst ) continue ;

      if ( ! strheadcmp ( szSrc, szDst ) ) {
         size_t nLength = strlen ( szDst ) ;
         if ( nMaxLength < nLength ) {
            nCountSaved = nCount ;
            nMaxLength = nLength ;
         }
      }
   }

   if ( nCountSaved >= 0 ) return & TamilEquivalentTable [ nCountSaved ] ;

   return NULL ;
}



