// viet.cpp

#include <string.h>
#include <ctype.h>
#include "msc.h"
#include "stringex.h"
#include "viet.h"
#include "viettable.h"



#define IsValidFirstChar(C)         ( ( isascii ( C ) ) ? ( isalpha ( C ) ) : ( (unsigned) C <= 0x01FF || (unsigned) C >= 0x1EA0 && (unsigned) C <= 0x1EFF ) )
#define IsValidFirstCharViqr(C)     ( isascii ( C ) && isalpha ( C ) )
#define IsValidFirstCharViqrAlt(C)  ( ! isascii ( C ) )



const VIET_EQUIVALENT_TABLE *UnicodeToViet ( const wchar_t *szSrc, size_t *pLength ) {

   size_t nLength = wcslen ( szSrc ) ;
   int C = *szSrc ;

   if ( ! IsValidFirstChar ( C ) ) return NULL ;

   *pLength = nLength ;

   if ( nLength == 1 ) {
      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {
         int cUnicode = ( VietEquivalentTable [ nCount ] ).cUnicode ;
         if ( cUnicode == C ) return & VietEquivalentTable [ nCount ] ;
      }
   }

   if ( nLength == 2 ) {
      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {
         const wchar_t *szDst = ( VietEquivalentTable [ nCount ] ).szUnicode ;
         if ( szDst && ! wcscmp ( szDst, szSrc ) ) return & VietEquivalentTable [ nCount ] ;
      }
   }

   if ( nLength >= 2 ) {
      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {
         const wchar_t *szDst = ( VietEquivalentTable [ nCount ] ).szUnicodeDecomp ;
         if ( szDst && ! wcscmp ( szDst, szSrc ) ) return & VietEquivalentTable [ nCount ] ;
      }
   }

   if ( nLength == 2 ) {
      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {
         const wchar_t *szDst = ( VietEquivalentTable [ nCount ] ).szUnicodeAlt ;
         if ( szDst && ! wcscmp ( szDst, szSrc ) ) return & VietEquivalentTable [ nCount ] ;
      }
   }

   // ւĔr

   if ( nLength == 3 ) {

      wchar_t szBuffer [] = {
         szSrc [ 0 ],
         szSrc [ 2 ],
         szSrc [ 1 ],
      0 } ;

      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {
         const wchar_t *szDst = ( VietEquivalentTable [ nCount ] ).szUnicodeDecomp ;
         if ( szDst && ! wcscmp ( szDst, szBuffer ) ) return & VietEquivalentTable [ nCount ] ;
      }
   }

   // k߂Ĕr

   if ( nLength == 3 ) {

      wchar_t szBuffer [] = {
         szSrc [ 0 ],
         szSrc [ 1 ],
      0 } ;

      const VIET_EQUIVALENT_TABLE *pResult = UnicodeToViet ( szBuffer, pLength ) ;
      if ( pResult ) return pResult ;
   }

   if ( nLength >= 2 ) {

      wchar_t szBuffer [] = {
         szSrc [ 0 ],
      0 } ;

      const VIET_EQUIVALENT_TABLE *pResult = UnicodeToViet ( szBuffer, pLength ) ;
      if ( pResult ) return pResult ;
   }

   *pLength = 0 ;

   return NULL ;
}



const wchar_t *GetVniToUnicodeString ( int C ) {

   if ( C >= 0xC0 && C <= 0xFF ) return VniToUnicode_C0_FF [ C - 0xC0 ] ;

   return NULL ;
}



const VIET_EQUIVALENT_TABLE *ViqrStringToUnicode ( const char *szSrc, size_t *pLength ) {


   if ( IsValidFirstCharViqr ( *szSrc ) ) {

      if ( ! strheadcmp ( szSrc, "dD" ) || ! strheadcmp ( szSrc, "Dd" ) ) szSrc = "DD" ;

      size_t nMaxLength = 0 ;
      int nCountSaved = -1 ;

      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {

         const char *szDst = ( VietEquivalentTable [ nCount ] ).szViqr ;
         if ( ! szDst ) continue ;

         if ( ! strheadcmp ( szSrc, szDst ) ) {
            size_t nLength = strlen ( szDst ) ;
            if ( nMaxLength < nLength ) {
               nCountSaved = nCount ;
               nMaxLength = nLength ;
            }
         }
      }

      if ( nCountSaved >= 0 ) {
         *pLength = nMaxLength ;
         return & VietEquivalentTable [ nCountSaved ] ;
      }
   }


   if ( IsValidFirstCharViqrAlt ( *szSrc ) ) {

      size_t nMaxLength = 0 ;
      int nCountSaved = -1 ;

      for ( int nCount = 0 ; nCount < VIET_EQUIVALENT_TABLE_NELEM ; nCount ++ ) {

         const char *szDst = ( VietEquivalentTable [ nCount ] ).szViqrAlt ;
         if ( ! szDst ) continue ;

         if ( ! strheadcmp ( szSrc, szDst ) ) {
            size_t nLength = strlen ( szDst ) ;
            if ( nMaxLength < nLength ) {
               nCountSaved = nCount ;
               nMaxLength = nLength ;
            }
         }
      }

      if ( nCountSaved >= 0 ) {
         *pLength = nMaxLength ;
         return & VietEquivalentTable [ nCountSaved ] ;
      }
   }


   return NULL ;
}



int IsViqrDiacritics ( int cUnicode ) {

   if ( cUnicode && isascii ( cUnicode ) && wcschr ( ViqrDiacritics, cUnicode ) ) return 1 ;

   return 0 ;
}



int GetUnicodeForTcvn2 ( int cUnicode, const wchar_t *szUnicode ) {

   const wchar_t *NoTcvn2Char = ( cUnicode < 0x1000 ) ? NoTcvn2CharLatin : NoTcvn2CharLatinEx ;

   if ( cUnicode && wcschr ( NoTcvn2Char, cUnicode ) ) return *szUnicode ;

   return cUnicode ;
}



int GetUnicodeForTcvn3 ( int cUnicode, const wchar_t *szUnicode ) {

   const wchar_t *NoTcvn3Char = ( cUnicode < 0x1000 ) ? NoTcvn3CharLatin : NoTcvn3CharLatinEx ;

   if ( cUnicode && wcschr ( NoTcvn3Char, cUnicode ) ) return *szUnicode ;

   return cUnicode ;
}



