// rtfconv_exe.cpp : RTFRo[^̃EBhEC^[tFCX

#include <windows.h>
#include <stdio.h>
#include <ctype.h>
#include "msc.h"

#include "access.h"
#include "brsfile.h"
#include "cjkctype.h"
#include "codepage.h"
#include "filename.h"
#include "guifont.h"
#include "langid.h"
#include "mui.h"
#include "mvwnd.h"
#include "nls.h"
#include "res.h"
#include "resource.h"
#include "sendmsg.h"
#include "shfileop.h"
#include "shfolder.h"
#include "shortcut.h"
#include "stringex.h"
#include "thread.h"
#include "tmpfile.h"
#include "uniblock.h"
#include "winmain.h"

#include "rtfconv.h"
#include "rtfconv_opt.h"

#include "conv_err.h"
#include "conv_opt.h"
#include "conv_loaddll.h"
#include "conv_icon.h"
#include "unics.h"



// for debug
#define DEBUG_LOCAL_FONT   0     // [JȃtHg
#define DEBUG_TAB_CONTROL  0     // ^uRg[
#define DEBUG_DRAW_ICON    0     // 󋵕\ACR̃TCY
#define DEBUG_SEMAPHORE    0     // ϊ̃_CAO`


static int nVersion ;
static int nOsCodePage ;         // OS̃R[hy[W

static char szCommandNameA [ SHORTSTRING_MAX ] ;
static wchar_t szCommandNameW [ SHORTSTRING_MAX ] ;

static char *szConfigFileNameA ;
static wchar_t *szConfigFileNameW ;

#define TEMPFILE_HEADER_A      "rtf"
#define TEMPFILE_HEADER_W     L"rtf"

INT_PTR CALLBACK MainDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
const char *GetDefaultFontName ( int nIndex ) ;
int IsZeroMemory ( const void *pMemory, size_t nSize ) ;
int CountPrintfArgumentsA ( const char *szFormat ) ;


// for XP Style
static HINSTANCE hUxTheme ;
typedef HRESULT ( WINAPI *ENABLETHEMEDIALOGTEXTURE ) ( HWND hwnd, DWORD dwFlags ) ;
static ENABLETHEMEDIALOGTEXTURE EnableThemeDialogTexture ;
#define ETDT_ENABLETAB 0x06



int main ( int argc, char *argv [] ) {


   int nError = 0 ;     // G[
   int nMuiLangIdSpecified = 0 ;


   if ( DEBUG_LOCAL_FONT || DEBUG_TAB_CONTROL || DEBUG_DRAW_ICON || DEBUG_SEMAPHORE ) {
      MessageBoxA ( NULL, "Debug Version", "RTFCONV.EXE", MB_OK ) ;
   }

   // for XP Visual Style
   InitCommonControls () ;

   // for XP Visual Style
   if ( IsWinXPorLater () ) {
      hUxTheme = LoadLibraryW ( L"UxTheme.dll" ) ;
      GETPROCADDRESS ( hUxTheme, ENABLETHEMEDIALOGTEXTURE, EnableThemeDialogTexture ) ;
   }

   // VXe擾
   nOsCodePage = GetACP () ;

   // \[X̌
   if ( IsNT () ) InitMuiLibraryW ( RT_DIALOG, IDD_DIALOG_MAIN, NULL, MUILANG_PREFIX_W ) ;
   else           InitMuiLibraryA ( RT_DIALOG, IDD_DIALOG_MAIN, NULL, MUILANG_PREFIX_A ) ;

   // o[W
   FILEVERSION FileVersion ;
   if ( ! GetFileVersion ( & FileVersion ) ) nVersion = TranslateVersionInfo ( & FileVersion ) ;


   // ̎擾
   for ( int nCountArgv = 1 ; nCountArgv < argc ; nCountArgv ++ ) {
      char *szArgv = argv [ nCountArgv ] ;
      if ( *szArgv == '/' || *szArgv == '-' ) {
         switch ( *( szArgv + 1 ) ) {
            case 'E' :
            case 'e' :
               nMuiLangIdSpecified = LANGID_ENGLISH ;
               break ;
            case 'J' :
            case 'j' :
               nMuiLangIdSpecified = LANGID_JAPANESE ;
               break ;
            case 'L' :
            case 'l' :
               if ( *( szArgv + 2 ) ) {
                  char *szString ;
                  errno = 0 ;
                  int nLangId = strtoul ( szArgv + 2, & szString, 0x10 ) ;
                  if ( errno || *szString ) { nError = nCountArgv ; break ; }
                  nMuiLangIdSpecified = nLangId ;
                  continue ;
               }
               nError = nCountArgv ;
               break ;
            case 'W' :
            case 'w' :
               IsIgnoreSaveWindow = 1 ;
               break ;
            case '@' :
               if ( *( szArgv + 2 ) ) {
                  szConfigFileNameA = GetArgumentA ( nCountArgv ) + 2 ;
                  szConfigFileNameW = GetArgumentW ( nCountArgv ) + 2 ;
                  continue ;
               }
               break ;
            default :
               nError = nCountArgv ;
               break ;
         }
         if ( *( szArgv + 2 ) ) nError = nCountArgv ;
         continue ;
      }
      nError = nCountArgv ;
   }


   // \̐ݒiR}hC̎wj
   if ( nMuiLangIdSpecified ) LoadMuiLibrary ( nMuiLangIdSpecified ) ;
   // 
   LoadStringExA ( IDS_COMMANDNAME, szCommandNameA, SHORTSTRING_MAX ) ;
   if ( IsNT () ) LoadStringExW ( IDS_COMMANDNAME, szCommandNameW, SHORTSTRING_MAX ) ;


   // G[\
   if ( nError ) {
      if ( IsNT () ) MessageBoxFormatW ( NULL, szCommandNameW, MB_OK, L"%s%s\n", GetStringW ( IDS_PARAM_ERROR ), GetArgumentW ( nError ) ) ;
      else           MessageBoxFormatA ( NULL, szCommandNameA, MB_OK,  "%s%s\n", GetStringA ( IDS_PARAM_ERROR ), GetArgumentA ( nError ) ) ;
      return 1 ;
   }


   // tHg
   if ( nOsCodePage == CP_JAPANESE )            szDefaultJapaneseFont            = DEFAULT_JAPANESE_FONT_L ;
   if ( nOsCodePage == CP_CHINESE_SIMPLIFIED )  szDefaultChineseSimplifiedFont   = DEFAULT_CHINESE_SIMPLIFIED_FONT_L ;
   if ( nOsCodePage == CP_CHINESE_TRADITIONAL ) szDefaultChineseTraditionalFont  = DEFAULT_CHINESE_TRADITIONAL_FONT_L ;
   if ( nOsCodePage == CP_KOREAN )              szDefaultKoreanFont              = DEFAULT_KOREAN_FONT ;

   // test of localized font name
   if ( DEBUG_LOCAL_FONT   ) {
      szDefaultJapaneseFont            = DEFAULT_JAPANESE_FONT_L ;
      szDefaultChineseSimplifiedFont   = DEFAULT_CHINESE_SIMPLIFIED_FONT_L ;
      szDefaultChineseTraditionalFont  = DEFAULT_CHINESE_TRADITIONAL_FONT_L ;
      szDefaultKoreanFont              = DEFAULT_KOREAN_FONT ;
   }


   // tHg̏lݒ
   strcpy ( szEuropeanFont, szDefaultAsciiFont ) ;
   strcpy ( szJapaneseFont, szDefaultJapaneseFont ) ;
   strcpy ( szKoreanFont, szDefaultKoreanFont ) ;
   strcpy ( szChineseSimplifiedFont, szDefaultChineseSimplifiedFont ) ;
   strcpy ( szChineseTraditionalFont, szDefaultChineseTraditionalFont ) ;
   strcpy ( szThaiFont, szDefaultThaiFont ) ;


   // VXe擾
   int nOsCodePageTmp = nOsCodePage ;
   if ( IsOsCodePageHkscs () ) nOsCodePageTmp = CP_CHINESE_HKSCS ;
   for ( int nCount = 0 ; nCount < AVAILABLE_CODE_LIST_NELEM ; nCount ++ ) {
      if ( CP_ANSI_EUROPEAN == ( AvailableCodeList [ nCount ] ).nCodePage ) {
         nSrcCode = nCount ;
         nDstCode = nCount ;
      }
      if ( nOsCodePageTmp == ( AvailableCodeList [ nCount ] ).nCodePage ) {
         nSrcCode = nCount ;
         nDstCode = nCount ;
         break ;
      }
   }
   nDefaultCjkCodePage = nOsCodePage ;
   if ( ! IsMultiByteCodePage ( nDefaultCjkCodePage ) ) nDefaultCjkCodePage = CP_JAPANESE ;


   // IvV擾
   int IsConfigFileNotFound ;
   if ( IsNT () ) IsConfigFileNotFound = ReadOptionW ( szConfigFileNameW ) ;
   else           IsConfigFileNotFound = ReadOptionA ( szConfigFileNameA ) ;
   if ( IsConfigFileNotFound < 0 ) {
      if ( IsNT () ) MessageBoxW ( NULL, GetStringW ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameW, MB_OK ) ;
      else           MessageBoxA ( NULL, GetStringA ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameA, MB_OK ) ;
      return 1 ;
   }
   if ( IsConfigFileNotFound && ( szConfigFileNameA || szConfigFileNameW ) ) {
      if ( IsNT () ) MessageBoxFormatW ( NULL, szCommandNameW, MB_OK, L"%s\n\n%s\n", GetStringW ( IDS_CANNOT_READ_OPTION ), szConfigFileNameW ) ;
      else           MessageBoxFormatA ( NULL, szCommandNameA, MB_OK,  "%s\n\n%s\n", GetStringA ( IDS_CANNOT_READ_OPTION ), szConfigFileNameA ) ;
      return 1 ;
   }


   // \̐ݒit@C̎wj
   if ( ! nMuiLangIdSpecified && nSavedMuiLangId ) {
      LoadMuiLibrary ( nSavedMuiLangId ) ;
      // Ǖ
      LoadStringExA ( IDS_COMMANDNAME, szCommandNameA, SHORTSTRING_MAX ) ;
      if ( IsNT () ) LoadStringExW ( IDS_COMMANDNAME, szCommandNameW, SHORTSTRING_MAX ) ;
   }


   // tH_擾
   if ( IsNT () ) {
      if ( ! *szBrowsePathW ) {
         if ( *szBrowsePathA ) {
            a2wlcpy ( szBrowsePathW, szBrowsePathA, MAX_PATH ) ;
         }
         else {
            GetSpecialFolderPathW ( NULL, szBrowsePathW, CSIDL_PERSONAL, 0 ) ;
            if ( ! *szBrowsePathW ) wcscpy ( szBrowsePathW, L"C:\\" ) ;
         }
      }
   }
   else {
      if ( ! *szBrowsePathA ) {
         if ( *szBrowsePathW ) {
            w2alcpy ( szBrowsePathA, szBrowsePathW, MAX_PATH ) ;
         }
         else {
            GetSpecialFolderPathA ( NULL, szBrowsePathA, CSIDL_PERSONAL, 0 ) ;
            if ( ! *szBrowsePathA ) strcpy ( szBrowsePathA, "C:\\" ) ;
         }
      }
   }

   // _CAÕACe̐`FbN
   if ( ! IsDlgItemIdConformable () ) {
      if ( IsNT () ) MessageBoxW ( NULL, GetStringW ( IDS_UNKNOWN_ERROR ), szCommandNameW, MB_OK ) ;
      else           MessageBoxA ( NULL, GetStringA ( IDS_UNKNOWN_ERROR ), szCommandNameA, MB_OK ) ;
      return 0 ;
   }


   // _CAO\
   if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_MAIN, NULL, MainDialogProc ) ;
   else           DialogBoxExA ( IDD_DIALOG_MAIN, NULL, MainDialogProc ) ;

   // IvVۑ
   if ( IsOptionChanged || IsSavePath || IsSaveCode || IsSaveWindow || IsConfigFileNotFound ) {
      if ( IsNT () ) WriteOptionW ( szConfigFileNameW ) ;
      else           WriteOptionA ( szConfigFileNameA ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//            C_CAO            //
////////////////////////////////////////////



int SetSemaphore ( int nValue ) ;
#define LockSemaphore()       SetSemaphore ( 1 )   // if success, return 0
#define UnlockSemaphore()     SetSemaphore ( 0 )
#define IsLockedSemaphore()   ( SetSemaphore ( -1 ) != 0 )


typedef struct {
   HWND           hWnd ;
   volatile int   *pIsAborted ;
} CONVERSIONTHREADINFO ;

int ConversionThread ( void *pConversionThreadInfo ) ;


int SetOption ( HWND hWnd ) ;
int DisplayHelp ( HWND hWnd ) ;
int SetMainDialogTitle ( HWND hWnd, int nMode ) ;

int CallBrowseForFile ( HWND hWnd, int nFileNumber, int IsDestination ) ;
#define BrowseSrcFileName(hWnd,nFileNumber)  CallBrowseForFile(hWnd,nFileNumber,0)
#define BrowseDstFileName(hWnd,nFileNumber)  CallBrowseForFile(hWnd,nFileNumber,1)

static WNDPROC OldEditCtrlProc ;
LRESULT CALLBACK NewEditCtrlProcA ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
LRESULT CALLBACK NewEditCtrlProcW ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;



// C_CAO
INT_PTR CALLBACK MainDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   static HFONT hFont ;
   static HICON hIcon ;
   static volatile int IsAborted ;
   int nId ;

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // EBhE\ʒu
         if ( IsSaveWindow && ! IsIgnoreSaveWindow ) SetWindowPos ( hWnd, NULL, nSavedWindowX, nSavedWindowY, 0, 0, SWP_NOSIZE | SWP_NOZORDER ) ;
         else                                        MoveWindowToCenter ( hWnd ) ;
         ShiftWindow ( hWnd ) ;
         MoveWindowTitleIntoDesktop ( hWnd ) ;

         // ^Cgݒ
         hIcon = GetIcon ( IDI_ICON_2 ) ;
         if ( hIcon ) Wm_SetIcon ( hWnd, ICON_BIG, hIcon ) ;   // ICON_BIGłȂ ALT+TAB ŃACR\łȂ
         SetMainDialogTitle ( hWnd, 0 ) ;

         // bZ[W̏
         for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) SetStatusIcon ( hWnd, nCount, STATUS_ICON_READY ) ;

         if ( DEBUG_DRAW_ICON ) DisplayControlSize ( hWnd, IDC_STATIC_STATUS_MIN ) ;

         // WI{^ݒ
         CheckDlgButton ( hWnd, nSrcType + IDC_CHECK_SRC_TYPE_MIN, BST_CHECKED ) ;
         CheckDlgButton ( hWnd, nDstType + IDC_CHECK_DST_TYPE_MIN, BST_CHECKED ) ;

         // R{{bNX̐ݒ
         Dlg_ComboBox_ResetContent ( hWnd, IDC_COMBO_SRC_CODE ) ;
         for ( int nCount = 0 ; nCount < AVAILABLE_CODE_LIST_NELEM ; nCount ++ ) {
            if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_SRC_CODE, -1, GetStringW ( ( AvailableCodeList [ nCount ] ).nNameId ) ) ;
            else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_SRC_CODE, -1, GetStringA ( ( AvailableCodeList [ nCount ] ).nNameId ) ) ;
         }
         Dlg_ComboBox_ResetContent ( hWnd, IDC_COMBO_DST_CODE ) ;
         for ( int nCount = 0 ; nCount < AVAILABLE_CODE_LIST_NELEM ; nCount ++ ) {
            if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_DST_CODE, -1, GetStringW ( ( AvailableCodeList [ nCount ] ).nNameId ) ) ;
            else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_DST_CODE, -1, GetStringA ( ( AvailableCodeList [ nCount ] ).nNameId ) ) ;
         }
         Dlg_ComboBox_SetMaxHeight ( hWnd, IDC_COMBO_SRC_CODE, 16 ) ;
         Dlg_ComboBox_SetMaxHeight ( hWnd, IDC_COMBO_DST_CODE, 16 ) ;

         if ( nSrcType == TYPE_TXT ) {
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_SRC_CODE, nSrcCode ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_SRC_CODE, TRUE ) ;
         }
         else {
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_SRC_CODE, -1 ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_SRC_CODE, FALSE ) ;
         }

         if ( nDstType == TYPE_TXT ) {
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_DST_CODE, nDstCode ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_DST_CODE, TRUE ) ;
         }
         else {
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_DST_CODE, -1 ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_DST_CODE, FALSE ) ;
         }

         // tHg쐬
         hFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_DIALOG_MAIN ), 0 ) ;
         if ( hFont ) {
            for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
               Dlg_Wm_SetFont ( hWnd, IDC_EDIT_SRC_MIN + nCount, hFont, TRUE ) ;
               Dlg_Wm_SetFont ( hWnd, IDC_EDIT_DST_MIN + nCount, hFont, TRUE ) ;
            }
         }

         // GfBbgRg[̃}[Wݒ (Win95΍)
         if ( IsWin95 () ) {
            for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
               Dlg_Edit_SetMargins ( hWnd, IDC_EDIT_SRC_MIN + nCount, 1, 1 ) ;
               Dlg_Edit_SetMargins ( hWnd, IDC_EDIT_DST_MIN + nCount, 1, 1 ) ;
            }
         }

         // GfBbgRg[͂̒̐
         for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
            Dlg_Edit_LimitText ( hWnd, IDC_EDIT_SRC_MIN + nCount, MAX_PATH - 1 ) ;
            Dlg_Edit_LimitText ( hWnd, IDC_EDIT_DST_MIN + nCount, MAX_PATH - 1 ) ;
         }

         // GfBbgRg[tbN
         if ( IsNT () ) OldEditCtrlProc = GetWindowProcW ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN ) ) ;
         else           OldEditCtrlProc = GetWindowProcA ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN ) ) ;
         for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
            if ( IsNT () ) {
               SetWindowProcW ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN + nCount ), NewEditCtrlProcW ) ;
               SetWindowProcW ( GetDlgItem ( hWnd, IDC_EDIT_DST_MIN + nCount ), NewEditCtrlProcW ) ;
            }
            else {
               SetWindowProcA ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN + nCount ), NewEditCtrlProcA ) ;
               SetWindowProcA ( GetDlgItem ( hWnd, IDC_EDIT_DST_MIN + nCount ), NewEditCtrlProcA ) ;
            }
            DragAcceptFiles ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN + nCount ), TRUE ) ;
            DragAcceptFiles ( GetDlgItem ( hWnd, IDC_EDIT_DST_MIN + nCount ), TRUE ) ;
         }

         return TRUE ;


      case WM_COMMAND :

         // ϊ͖
         if ( IsLockedSemaphore () ) {
            if ( LOWORD ( wParam ) == IDCANCEL ) {
               if ( AbortRtfconv ) AbortRtfconv () ;
               IsAborted = 1 ;
               return TRUE ;
            }
            if ( HIWORD ( wParam ) == CBN_SETFOCUS ) {
               Dlg_SetFocus ( hWnd, IDOK ) ;
               return TRUE ;
            }
            if ( HIWORD ( wParam ) == EN_SETFOCUS ) {
               Dlg_SetFocus ( hWnd, IDOK ) ;
               return TRUE ;
            }
            if ( HIWORD ( wParam ) == BN_CLICKED ) {
               Dlg_SetFocus ( hWnd, IDOK ) ;
               CheckRadioButton ( hWnd, IDC_CHECK_SRC_TYPE_MIN, IDC_CHECK_SRC_TYPE_MIN + SRC_TYPE_NELEM - 1, nSrcType + IDC_CHECK_SRC_TYPE_MIN ) ;
               CheckRadioButton ( hWnd, IDC_CHECK_DST_TYPE_MIN, IDC_CHECK_DST_TYPE_MIN + DST_TYPE_NELEM - 1, nDstType + IDC_CHECK_DST_TYPE_MIN ) ;
               return TRUE ;
            }
            return TRUE ;
         }

         // ϊ̎s
         if ( LOWORD ( wParam ) == IDOK ) {

            // 
            IsAborted = 0 ;

            static CONVERSIONTHREADINFO ConversionThreadInfo ;
            ConversionThreadInfo.hWnd = hWnd ;
            ConversionThreadInfo.pIsAborted = & IsAborted ;
#ifdef _MT
            // s֐Ăяoi}`Xbhj
            if ( BeginThread ( ConversionThread, (void*) & ConversionThreadInfo, NULL ) ) {
               if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameW, MB_OK ) ;
               else           MessageBoxA ( hWnd, GetStringA ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameA, MB_OK ) ;
            }
#else
            // s֐ĂяoiVOXbhj
            ConversionThread ( (void*) & ConversionThreadInfo ) ;
#endif
            return TRUE ;
         }

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {

            if ( IsConfirmOnExit ) {
               int nResult ;
               if ( IsNT () ) nResult = MessageBoxW ( hWnd, GetStringW ( IDS_CONFIRM_ON_EXIT ), szCommandNameW, MB_YESNO ) ;
               else           nResult = MessageBoxA ( hWnd, GetStringA ( IDS_CONFIRM_ON_EXIT ), szCommandNameA, MB_YESNO ) ;
               if ( nResult != IDYES ) return TRUE ;
            }

            if ( IsSaveWindow ) {
               RECT Rect ;
               if ( ! GetWindowNormalPositionRect ( hWnd, & Rect ) ) IsSaveWindow = 0 ;
               nSavedWindowX = Rect.left ;
               nSavedWindowY = Rect.top ;
            }

            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         // ̓t@C̎Q
         if ( (unsigned) ( LOWORD ( wParam ) - IDC_BUTTON_SRC_MIN ) < SRC_DST_FILE_NELEM ) {
            BrowseSrcFileName ( hWnd, LOWORD ( wParam ) - IDC_BUTTON_SRC_MIN ) ;
            return TRUE ;
         }

         // o͂̃t@C̎Q
         if ( (unsigned) ( LOWORD ( wParam ) - IDC_BUTTON_DST_MIN ) < SRC_DST_FILE_NELEM ) {
            BrowseDstFileName ( hWnd, LOWORD ( wParam ) - IDC_BUTTON_DST_MIN ) ;
            return TRUE ;
         }


         // ̓WI{^̐ݒ
         if ( (unsigned) ( LOWORD ( wParam ) - IDC_CHECK_SRC_TYPE_MIN ) < SRC_TYPE_NELEM ) {

            nSrcType = LOWORD ( wParam ) - IDC_CHECK_SRC_TYPE_MIN ;

            // bZ[W̏
            for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) SetStatusIcon ( hWnd, nCount, STATUS_ICON_READY ) ;

            if ( nSrcType == TYPE_TXT ) {
               Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_SRC_CODE, nSrcCode ) ;
               Dlg_EnableWindow ( hWnd, IDC_COMBO_SRC_CODE, TRUE ) ;
            }
            else {
               Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_SRC_CODE, -1 ) ;
               Dlg_EnableWindow ( hWnd, IDC_COMBO_SRC_CODE, FALSE ) ;
            }

            return TRUE ;
         }

         // o̓WI{^̐ݒ
         if ( (unsigned) ( LOWORD ( wParam ) - IDC_CHECK_DST_TYPE_MIN ) < DST_TYPE_NELEM ) {

            nDstType = LOWORD ( wParam ) - IDC_CHECK_DST_TYPE_MIN ;

            // bZ[W̏
            for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) SetStatusIcon ( hWnd, nCount, STATUS_ICON_READY ) ;

            if ( nDstType == TYPE_TXT ) {
               Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_DST_CODE, nDstCode ) ;
               Dlg_EnableWindow ( hWnd, IDC_COMBO_DST_CODE, TRUE ) ;
            }
            else {
               Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_DST_CODE, -1 ) ;
               Dlg_EnableWindow ( hWnd, IDC_COMBO_DST_CODE, FALSE ) ;
            }

            return TRUE ;
         }


         // IvVݒ
         if ( LOWORD ( wParam ) == IDC_BUTTON_OPTION ) {
            SetOption ( hWnd ) ;
            return TRUE ;
         }

         // wv\
         if ( LOWORD ( wParam ) == IDC_BUTTON_HELP ) {
            DisplayHelp ( hWnd ) ;
            return TRUE ;
         }

         // R{{bNX̑IύXƂ
         if ( HIWORD ( wParam ) == CBN_SELCHANGE ) {
            if ( LOWORD ( wParam ) == IDC_COMBO_SRC_CODE ) {
               nSrcCode = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_SRC_CODE ) ;
               return TRUE ;
            }
            if ( LOWORD ( wParam ) == IDC_COMBO_DST_CODE ) {
               nDstCode = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_DST_CODE ) ;
               return TRUE ;
            }
         }

         break ;


      case WM_CTLCOLOREDIT :
      case WM_CTLCOLORSTATIC :
         nId = GetDlgCtrlID ( (HWND) lParam ) ;
         if ( (unsigned) ( nId - IDC_EDIT_SRC_MIN ) < SRC_DST_FILE_NELEM ) return (INT_PTR) GetStockObject ( WHITE_BRUSH ) ;
         if ( (unsigned) ( nId - IDC_EDIT_DST_MIN ) < SRC_DST_FILE_NELEM ) return (INT_PTR) GetStockObject ( WHITE_BRUSH ) ;
         break ;


      case WM_DRAWITEM :
         if ( (unsigned) ( wParam - IDC_STATIC_STATUS_MIN ) < SRC_DST_FILE_NELEM ) {
            DrawStatusIcon ( (DRAWITEMSTRUCT*) lParam, (int) ( wParam - IDC_STATIC_STATUS_MIN ) ) ;
            SetDlgMsgResult ( hWnd, TRUE ) ;
            return TRUE ;
         }
         break ;


      case WM_DESTROY :
         if ( hFont ) {
            DeleteObject ( hFont ) ;
            hFont = NULL ;
         }
         if ( hIcon ) {
            DestroyIcon ( hIcon ) ;
            hIcon = NULL ;
         }
         break ;


      default :
         break ;
   }

   return FALSE ;
}



int SetDraggedFileNameA ( HWND hWnd, HDROP hDrop ) ;
int SetDraggedFileNameW ( HWND hWnd, HDROP hDrop ) ;



// GfBbgRg[tbNAhbOAhhbviANSIŁj
LRESULT CALLBACK NewEditCtrlProcA ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {
   if ( dwMessage == WM_DROPFILES ) return SetDraggedFileNameA ( hWnd, (HDROP) wParam ) ;
   else                             return CallWindowProcA ( OldEditCtrlProc, hWnd, dwMessage, wParam, lParam ) ;
}



// GfBbgRg[tbNAhbOAhhbviUNICODEŁj
LRESULT CALLBACK NewEditCtrlProcW ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {
   if ( dwMessage == WM_DROPFILES ) return SetDraggedFileNameW ( hWnd, (HDROP) wParam ) ;
   else                             return CallWindowProcW ( OldEditCtrlProc, hWnd, dwMessage, wParam, lParam ) ;
}



// hbOꂽt@CEBhEɕ\iANSIŁj
int SetDraggedFileNameA ( HWND hWnd, HDROP hDrop ) {

   char szFileName [ MAX_PATH ] ;

   if ( DragQueryFileA ( hDrop, (unsigned int) -1, NULL, 0 ) == 1 ) {
      DragQueryFileA ( hDrop, 0, szFileName, MAX_PATH ) ;
      char *szExtension = GetExtensionA ( szFileName ) ;
      if ( szExtension && ( ! stricmp ( szExtension, "lnk" ) || ! stricmp ( szExtension, "url" ) ) ) GetShortcutA ( szFileName, szFileName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL ) ;
      Wm_SetTextA ( hWnd, szFileName ) ;
      MoveCaretToEnd ( hWnd ) ;
   }
   else {
      SetForegroundWindow ( GetParent ( hWnd ) ) ;
      MessageBoxA ( GetParent ( hWnd ), GetStringA ( IDS_DRAGFILE_ERROR ), szCommandNameA, MB_OK ) ;
   }

   DragFinish ( hDrop ) ;

   return 0 ;
}



// hbOꂽt@CEBhEɕ\iUNICODEŁj
int SetDraggedFileNameW ( HWND hWnd, HDROP hDrop ) {

   wchar_t szFileName [ MAX_PATH ] ;

   if ( DragQueryFileW ( hDrop, (unsigned int) -1, NULL, 0 ) == 1 ) {
      DragQueryFileW ( hDrop, 0, szFileName, MAX_PATH ) ;
      wchar_t *szExtension = GetExtensionW ( szFileName ) ;
      if ( szExtension && ( ! wcsicmp ( szExtension, L"lnk" ) || ! wcsicmp ( szExtension, L"url" ) ) ) GetShortcutW ( szFileName, szFileName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL ) ;
      Wm_SetTextW ( hWnd, szFileName ) ;
      MoveCaretToEnd ( hWnd ) ;
   }
   else {
      SetForegroundWindow ( GetParent ( hWnd ) ) ;
      MessageBoxW ( GetParent ( hWnd ), GetStringW ( IDS_DRAGFILE_ERROR ), szCommandNameW, MB_OK ) ;
   }

   DragFinish ( hDrop ) ;

   return 0 ;
}



// C_CAÕ^Cgݒ
// nMode  0 łȂ΁Auϊv
int SetMainDialogTitle ( HWND hWnd, int nMode ) {

   if ( nMode ) {
      if ( IsNT () ) Wm_SetTextFormatW ( hWnd, L"%s  -  %s", szCommandNameW, GetStringW ( IDS_NOWCONVERTING ) ) ;
      else           Wm_SetTextFormatA ( hWnd,  "%s  -  %s", szCommandNameA, GetStringA ( IDS_NOWCONVERTING ) ) ;
   }
   else {
      if ( IsNT () ) Wm_SetTextFormatW ( hWnd, L"%s  -  version %d.%02d", szCommandNameW, nVersion / 100, nVersion % 100 ) ;
      else           Wm_SetTextFormatA ( hWnd,  "%s  -  version %d.%02d", szCommandNameA, nVersion / 100, nVersion % 100 ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//            t@C̎Q            //
////////////////////////////////////////////



int CallBrowseForFileA ( HWND hWnd, int nFileNumber, int IsDestination ) ;
int CallBrowseForFileW ( HWND hWnd, int nFileNumber, int IsDestination ) ;



// Qƃ{^̓
int CallBrowseForFile ( HWND hWnd, int nFileNumber, int IsDestination ) {

   if ( IsNT () ) return CallBrowseForFileW ( hWnd, nFileNumber, IsDestination ) ;
   else           return CallBrowseForFileA ( hWnd, nFileNumber, IsDestination ) ;
}



char *GetFileNameFilterA ( int IsRtf ) ;
wchar_t *GetFileNameFilterW ( int IsRtf ) ;



// Qƃ{^̓iANSIŁj
int CallBrowseForFileA ( HWND hWnd, int nFileNumber, int IsDestination ) {

   char szFileName [ MAX_PATH ] ;
   char szResult [ MAX_PATH ] ;

   // bZ[W̏
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) SetStatusIcon ( hWnd, nCount, STATUS_ICON_READY ) ;

   // o̓t@C擾
   if ( IsDestination ) {
      Dlg_Wm_GetTextA ( hWnd, IDC_EDIT_DST_MIN + nFileNumber, szFileName, MAX_PATH ) ;
      UnquoteFileNameA ( szFileName ) ;
      if ( ! IsFolderExistA ( szFileName ) ) {
         ChangeFileNameA ( szFileName, NULL ) ;
         if ( ! IsFolderExistA ( szFileName ) ) *szFileName = 0 ;
      }
   }

   // ̓t@C擾
   if ( ! IsDestination || ! *szFileName ) {
      Dlg_Wm_GetTextA ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber, szFileName, MAX_PATH ) ;
      UnquoteFileNameA ( szFileName ) ;
      if ( ! IsFolderExistA ( szFileName ) ) {
         ChangeFileNameA ( szFileName, NULL ) ;
         if ( ! IsFolderExistA ( szFileName ) ) strcpy ( szFileName, szBrowsePathA ) ;
      }
   }

   char *szFilter = GetFileNameFilterA ( ( ! IsDestination && nSrcType == TYPE_RTF ) || ( IsDestination && nDstType == TYPE_RTF ) ) ;

   if ( ! IsDestination ) {
      if ( ! BrowseForFileA ( hWnd, szResult, MAX_PATH, szFileName, GetStringA ( IDS_SELECTSRCFILE ), szFilter, 0 ) ) {
         Dlg_SetFocus ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber ) ;
         Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber, szResult ) ;
         Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber ) ;
         strcpy ( szBrowsePathA, szResult ) ;
         ChangeFileNameA ( szBrowsePathA, NULL ) ;
      }
   }
   else {
      if ( ! BrowseForFileA ( hWnd, szResult, MAX_PATH, szFileName, GetStringA ( IDS_SELECTDSTFILE ), szFilter, 0 ) ) {
         Dlg_SetFocus ( hWnd, IDC_EDIT_DST_MIN + nFileNumber ) ;
         Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_DST_MIN + nFileNumber, szResult ) ;
         Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_DST_MIN + nFileNumber ) ;
      }
   }

   return 0 ;
}



// Qƃ{^̓iUNICODEŁj
int CallBrowseForFileW ( HWND hWnd, int nFileNumber, int IsDestination ) {

   wchar_t szFileName [ MAX_PATH ] ;
   wchar_t szResult [ MAX_PATH ] ;

   // bZ[W̏
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) SetStatusIcon ( hWnd, nCount, STATUS_ICON_READY ) ;

   // o̓t@C擾
   if ( IsDestination ) {
      Dlg_Wm_GetTextW ( hWnd, IDC_EDIT_DST_MIN + nFileNumber, szFileName, MAX_PATH ) ;
      UnquoteFileNameW ( szFileName ) ;
      if ( ! IsFolderExistW ( szFileName ) ) {
         ChangeFileNameW ( szFileName, NULL ) ;
         if ( ! IsFolderExistW ( szFileName ) ) *szFileName = 0 ;
      }
   }

   // ̓t@C擾
   if ( ! IsDestination || ! *szFileName ) {
      Dlg_Wm_GetTextW ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber, szFileName, MAX_PATH ) ;
      UnquoteFileNameW ( szFileName ) ;
      if ( ! IsFolderExistW ( szFileName ) ) {
         ChangeFileNameW ( szFileName, NULL ) ;
         if ( ! IsFolderExistW ( szFileName ) ) wcscpy ( szFileName, szBrowsePathW ) ;
      }
   }

   wchar_t *szFilter = GetFileNameFilterW ( ( ! IsDestination && nSrcType == TYPE_RTF ) || ( IsDestination && nDstType == TYPE_RTF ) ) ;

   if ( ! IsDestination ) {
      if ( ! BrowseForFileW ( hWnd, szResult, MAX_PATH, szFileName, GetStringW ( IDS_SELECTSRCFILE ), szFilter, 0 ) ) {
         Dlg_SetFocus ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber ) ;
         Dlg_Wm_SetTextW ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber, szResult ) ;
         Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber ) ;
         wcscpy ( szBrowsePathW, szResult ) ;
         ChangeFileNameW ( szBrowsePathW, NULL ) ;
      }
   }
   else {
      if ( ! BrowseForFileW ( hWnd, szResult, MAX_PATH, szFileName, GetStringW ( IDS_SELECTDSTFILE ), szFilter, 0 ) ) {
         Dlg_SetFocus ( hWnd, IDC_EDIT_DST_MIN + nFileNumber ) ;
         Dlg_Wm_SetTextW ( hWnd, IDC_EDIT_DST_MIN + nFileNumber, szResult ) ;
         Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_DST_MIN + nFileNumber ) ;
      }
   }

   return 0 ;
}



// t@CQƂŎgtB^𓾂iANSIŁj
// IsRtf  0 ȂeLXgt@C
//          1 Ȃ烊b`eCXgt@C
char *GetFileNameFilterA ( int IsRtf ) {

   static char szFilter [ MAX_PATH ] ;
   char *szNext = szFilter ;

   if ( IsRtf ) {
      strcpy ( szNext, GetStringA ( IDS_RICHTEXTFILES ) ) ;
      szNext = strend ( szNext ) + 1 ;
      strcpy ( szNext, "*.RTF" ) ;
   }
   else {
      strcpy ( szNext, GetStringA ( IDS_TEXTFILES ) ) ;
      szNext = strend ( szNext ) + 1 ;
      strcpy ( szNext, "*.TXT" ) ;
   }
   szNext = strend ( szNext ) + 1 ;

   strcpy ( szNext, GetStringA ( IDS_ALLFILES ) ) ;
   szNext = strend ( szNext ) + 1 ;
   strcpy ( szNext, "*.*" ) ;
   szNext = strend ( szNext ) + 1 ;

   strcpy ( szNext, "" ) ;

   return szFilter ;
}



// t@CQƂŎgtB^𓾂iUNICODEŁj
// IsRtf  0 ȂeLXgt@C
//          1 Ȃ烊b`eCXgt@C
wchar_t *GetFileNameFilterW ( int IsRtf ) {

   static wchar_t szFilter [ MAX_PATH ] ;
   wchar_t *szNext = szFilter ;

   if ( IsRtf ) {
      wcscpy ( szNext, GetStringW ( IDS_RICHTEXTFILES ) ) ;
      szNext = wcsend ( szNext ) + 1 ;
      wcscpy ( szNext, L"*.RTF" ) ;
   }
   else {
      wcscpy ( szNext, GetStringW ( IDS_TEXTFILES ) ) ;
      szNext = wcsend ( szNext ) + 1 ;
      wcscpy ( szNext, L"*.TXT" ) ;
   }
   szNext = wcsend ( szNext ) + 1 ;

   wcscpy ( szNext, GetStringW ( IDS_ALLFILES ) ) ;
   szNext = wcsend ( szNext ) + 1 ;
   wcscpy ( szNext, L"*.*" ) ;
   szNext = wcsend ( szNext ) + 1 ;

   wcscpy ( szNext, L"" ) ;

   return szFilter ;
}



////////////////////////////////////////////
//               r                 //
////////////////////////////////////////////



// nValue  0 ܂͐ȂAli[
// nValue ̑O̒lԂ
int SetSemaphore ( int nValue ) {

   static volatile long nSemaphore ;

   if ( nValue < 0 ) return nSemaphore ;

   return InterlockedExchange ( & nSemaphore, nValue ) ;
}



////////////////////////////////////////////
//            Ro[^N              //
////////////////////////////////////////////



int CallRtfconv ( HWND hWnd, volatile int *pIsAborted ) ;



int ConversionThread ( void *pConversionThreadInfo ) {

   // Z}tH擾
   // ȍ~AUnlockSemaphore() ܂ return Ȃ
   if ( LockSemaphore () ) return 1 ;

   HWND hWnd = ( (CONVERSIONTHREADINFO*) pConversionThreadInfo )->hWnd ;
   volatile int *pIsAborted = ( (CONVERSIONTHREADINFO*) pConversionThreadInfo )->pIsAborted ;

   // hbOAhhbv֎~
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
      DragAcceptFiles ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN + nCount ), FALSE ) ;
      DragAcceptFiles ( GetDlgItem ( hWnd, IDC_EDIT_DST_MIN + nCount ), FALSE ) ;
   }

   // GfBbgRg[̕ҏW֎~
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
      Dlg_Edit_SetReadOnly ( hWnd, IDC_EDIT_SRC_MIN + nCount, TRUE ) ;
      Dlg_Edit_SetReadOnly ( hWnd, IDC_EDIT_DST_MIN + nCount, TRUE ) ;
   }

   // s֐Ăяo
   if ( DEBUG_SEMAPHORE ) {
      for ( int nCount = 10 ; nCount && ! *pIsAborted ; nCount -- ) {
         Dlg_Wm_SetTextFormatA ( hWnd, IDC_EDIT_DST_MIN + SRC_DST_FILE_NELEM - 1, "%d", nCount ) ;
         Sleep ( 1000 ) ;
      }
      Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_DST_MIN + SRC_DST_FILE_NELEM - 1, "" ) ;
      MessageBeep ( MB_OK ) ;
   }
   else {
      CallRtfconv ( hWnd, pIsAborted ) ;
   }

   // GfBbgRg[̕ҏWĊJ
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
      Dlg_Edit_SetReadOnly ( hWnd, IDC_EDIT_SRC_MIN + nCount, FALSE ) ;
      Dlg_Edit_SetReadOnly ( hWnd, IDC_EDIT_DST_MIN + nCount, FALSE ) ;
   }

   // hbOAhhbvĊJ
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
      DragAcceptFiles ( GetDlgItem ( hWnd, IDC_EDIT_SRC_MIN + nCount ), TRUE ) ;
      DragAcceptFiles ( GetDlgItem ( hWnd, IDC_EDIT_DST_MIN + nCount ), TRUE ) ;
   }

   // Z}tH
   UnlockSemaphore () ;

   return 0 ;
}



int CallRtfconvA ( HWND hWnd, volatile int *pIsAborted, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
int CallRtfconvW ( HWND hWnd, volatile int *pIsAborted, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
int TestFileNameA ( HWND hWnd, int nCount ) ;
int TestFileNameW ( HWND hWnd, int nCount ) ;



// t@C擾Ro[^N
int CallRtfconv ( HWND hWnd, volatile int *pIsAborted ) {


   // bZ[W̏
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) SetStatusIcon ( hWnd, nCount, STATUS_ICON_READY ) ;


   // ̓R[h
   if ( nSrcType == TYPE_RTF && nDstType == TYPE_RTF ) {
      if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_CANNOTSAMECODE ), szCommandNameW, MB_OK ) ;
      else           MessageBoxA ( hWnd, GetStringA ( IDS_CANNOTSAMECODE ), szCommandNameA, MB_OK ) ;
      return 1 ;
   }


   // n Ԗڂ̃t@C
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {
      int nResult ;
      if ( IsNT () ) nResult = TestFileNameW ( hWnd, nCount ) ;
      else           nResult = TestFileNameA ( hWnd, nCount ) ;
      if ( nResult ) return 1 ;
   }


   // DLLW[擾
   if ( LoadRtfconvDll () ) {
      if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_DLLERROR ), szCommandNameW, MB_OK ) ;
      else           MessageBoxA ( hWnd, GetStringA ( IDS_DLLERROR ), szCommandNameA, MB_OK ) ;
      return -1 ;
   }


   // o̓R[hy[Wݒ
   int nSrcCodePage = ( AvailableCodeList [ nSrcCode ] ).nCodePage ;
   int nDstCodePage = ( AvailableCodeList [ nDstCode ] ).nCodePage ;
   if ( nSrcType == TYPE_RTF ) nSrcCodePage = 0 ;
   if ( nDstType == TYPE_RTF ) nDstCodePage = 0 ;


   RTFCONVEXINFO Info = { sizeof(RTFCONVEXINFO) } ;
   RTFFONTINFO FontInfo = { sizeof(RTFFONTINFO) } ;
   RTFFONTEXINFO FontExInfo [ UNICODE_BLOCK_NELEM ] ;
   RTFLANGIDINFO LangIdInfo [ UNICODE_BLOCK_NELEM ] ;

   Info.pFontInfo = & FontInfo ;
   Info.pFontExInfo = FontExInfo ;
   Info.pLangIdInfo = LangIdInfo ;


   // IvVݒ
   Info.dwConversionMode = TranslateConversionMode ( nSrcCodePage, nDstCodePage ) ;
   Info.dwAdditionalMode = TranslateAdditionalMode ( nSrcCodePage, nDstCodePage ) ;
#ifndef _MT
   Info.dwConversionMode |= CONVMODE_PEEKMESSAGE ;
#endif


   // TXTo͎̎w
   if ( nDstCodePage ) {

      // o̓tH[}bgݒ
      if ( ( UnmappableCharFormatList [ nUnmappableCharFormat ] ).szFormat ) {
         Info.dwConversionMode |= CONVMODE_OUTPUT_CODE_NUMBER ;
         Info.pUnmappableCharFormat = ( UnmappableCharFormatList [ nUnmappableCharFormat ] ).szFormat ;
      }

   }
   // RTFo͎̎w
   else {

      // Unicode̎w
      Info.nDefaultCjkCodePage = nDefaultCjkCodePage ;

      // tHgTCY
      Info.nFontSize = nFontSize ;
      if ( ! Info.nFontSize ) Info.nFontSize = -1 ;

      // o̓tHgݒ
      if ( *szEuropeanFont )           FontInfo.pEuropean = szEuropeanFont ;
      if ( *szJapaneseFont )           FontInfo.pJapanese = szJapaneseFont ;
      if ( *szChineseSimplifiedFont )  FontInfo.pChineseSimplified = szChineseSimplifiedFont ;
      if ( *szChineseTraditionalFont ) FontInfo.pChineseTraditional = szChineseTraditionalFont ;
      if ( *szKoreanFont )             FontInfo.pKorean = szKoreanFont ;
      if ( *szThaiFont )               FontInfo.pThai = szThaiFont ;

      // tHgڍׂݒ
      if ( FontNameArray ) {
         for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
            char *szFontName = FontNameArray + nIndex * LF_FACESIZE ;
            if ( *szFontName ) {
               int C ;
               if ( GetUnicodeBlock ( 0, 0, nIndex, & C, NULL, NULL ) < 0 ) continue ;
               ( FontExInfo [ Info.nNumberOfFontExInfo ] ).C = C ;
               ( FontExInfo [ Info.nNumberOfFontExInfo ] ).pFontName = szFontName ;
               Info.nNumberOfFontExInfo ++ ;
            }
         }
      }

      // ꎯʎqݒ
      if ( IsLangIdUserDefined ) {
         for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
            if ( LangIdArray [ nIndex ] ) {
               int C ;
               if ( GetUnicodeBlock ( 0, 0, nIndex, & C, NULL, NULL ) < 0 ) continue ;
               ( LangIdInfo [ Info.nNumberOfLangIdInfo ] ).C = C ;
               ( LangIdInfo [ Info.nNumberOfLangIdInfo ] ).nLangId = LangIdArray [ nIndex ] ;
               Info.nNumberOfLangIdInfo ++ ;
            }
         }
      }
   }

   if ( IsNT () ) return CallRtfconvW ( hWnd, pIsAborted, nSrcCodePage, nDstCodePage, & Info ) ;
   else           return CallRtfconvA ( hWnd, pIsAborted, nSrcCodePage, nDstCodePage, & Info ) ;
}



// mԖڂ̃t@CiANSIŁj
// G[Ȃ烁bZ[W\ 0 ȊOԂ
int TestFileNameA ( HWND hWnd, int nFileNumber ) {

   char szSrcFileName [ MAX_PATH ] ;
   char szDstFileName [ MAX_PATH ] ;

   Dlg_Wm_GetTextA ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber, szSrcFileName, MAX_PATH ) ;
   Dlg_Wm_GetTextA ( hWnd, IDC_EDIT_DST_MIN + nFileNumber, szDstFileName, MAX_PATH ) ;
   UnquoteFileNameA ( szSrcFileName ) ;
   UnquoteFileNameA ( szDstFileName ) ;

   int nResult = 0 ;
   if ( ! *szSrcFileName && *szDstFileName ) nResult = 1 ;
   if ( *szSrcFileName && ! *szDstFileName ) nResult = 2 ;
   if ( nResult ) {
      if ( nResult == 1 ) MessageBoxFormatA ( hWnd, szCommandNameA, MB_OK, GetStringA ( IDS_SPECIFYSRCFILE ), nFileNumber + 1 ) ;
      else                MessageBoxFormatA ( hWnd, szCommandNameA, MB_OK, GetStringA ( IDS_SPECIFYDSTFILE ), nFileNumber + 1 ) ;
      return 1 ;
   }

   return 0 ;
}



// mԖڂ̃t@CiUNICODEŁj
// G[Ȃ烁bZ[W\ 0 ȊOԂ
int TestFileNameW ( HWND hWnd, int nFileNumber ) {

   wchar_t szSrcFileName [ MAX_PATH ] ;
   wchar_t szDstFileName [ MAX_PATH ] ;

   Dlg_Wm_GetTextW ( hWnd, IDC_EDIT_SRC_MIN + nFileNumber, szSrcFileName, MAX_PATH ) ;
   Dlg_Wm_GetTextW ( hWnd, IDC_EDIT_DST_MIN + nFileNumber, szDstFileName, MAX_PATH ) ;
   UnquoteFileNameW ( szSrcFileName ) ;
   UnquoteFileNameW ( szDstFileName ) ;

   int nResult = 0 ;
   if ( ! *szSrcFileName && *szDstFileName ) nResult = 1 ;
   if ( *szSrcFileName && ! *szDstFileName ) nResult = 2 ;
   if ( nResult ) {
      if ( nResult == 1 ) MessageBoxFormatW ( hWnd, szCommandNameW, MB_OK, GetStringW ( IDS_SPECIFYSRCFILE ), nFileNumber + 1 ) ;
      else                MessageBoxFormatW ( hWnd, szCommandNameW, MB_OK, GetStringW ( IDS_SPECIFYDSTFILE ), nFileNumber + 1 ) ;
      return 1 ;
   }

   return 0 ;
}



int ExecRtfconvA ( HWND hWnd, char *szSrcFileName, char *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
int ExecRtfconvW ( HWND hWnd, wchar_t *szSrcFileName, wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
char *GetRtfconvErrorMessageA ( int nError ) ;
wchar_t *GetRtfconvErrorMessageW ( int nError ) ;



// t@C擾Ro[^NiANSIŁj
int CallRtfconvA ( HWND hWnd, volatile int *pIsAborted, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   char szSrcFileName [ MAX_PATH + 0x10 ] ;
   char szDstFileName [ MAX_PATH + 0x10 ] ;
   char szMessageBuffer [ MESSAGE_BUFFER_SIZE ] ;

   int nCountSuccess = 0 ;
   int nCountFailure = 0 ;

   // ^CgύX
   // ȍ~A^Cg߂܂ return Ȃ
   SetMainDialogTitle ( hWnd, 1 ) ;

   // bZ[Wobt@̏
   *szMessageBuffer = 0 ;

   // t@C擾ϊs
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {

      // ߂Ɏ擾A牺֐ŃG[Ԃ
      Dlg_Wm_GetTextA ( hWnd, IDC_EDIT_SRC_MIN + nCount, szSrcFileName, MAX_PATH + 4 ) ;
      Dlg_Wm_GetTextA ( hWnd, IDC_EDIT_DST_MIN + nCount, szDstFileName, MAX_PATH + 4 ) ;
      UnquoteFileNameA ( szSrcFileName ) ;
      UnquoteFileNameA ( szDstFileName ) ;
      if ( ! *szSrcFileName ) continue ;

      SetStatusIcon ( hWnd, nCount, STATUS_ICON_NOW ) ;

      // KvȂΓ̓t@CɊgqt
      if ( ! IsNoAddExtension && ! IsFileExistA ( szSrcFileName ) ) {
         char *szSrcExtension = GetExtensionA ( szSrcFileName ) ;
         if ( nSrcType == TYPE_RTF ) {
            if ( ! szSrcExtension || stricmp ( szSrcExtension, "RTF" ) ) {
               strcat ( szSrcFileName, ".rtf" ) ;
               Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_SRC_MIN + nCount, szSrcFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_SRC_MIN + nCount ) ;
            }
         }
         if ( nSrcType == TYPE_TXT ) {
            if ( ! szSrcExtension || stricmp ( szSrcExtension, "TXT" ) ) {
               strcat ( szSrcFileName, ".txt" ) ;
               Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_SRC_MIN + nCount, szSrcFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_SRC_MIN + nCount ) ;
            }
         }
      }

      // KvȂΏo̓t@CɊgqt
      if ( ! IsNoAddExtension && ! IsFileExistA ( szDstFileName ) ) {
         char *szDstExtension = GetExtensionA ( szDstFileName ) ;
         if ( nDstType == TYPE_RTF ) {
            if ( ! szDstExtension || stricmp ( szDstExtension, "RTF" ) ) {
               strcat ( szDstFileName, ".rtf" ) ;
               Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_DST_MIN + nCount, szDstFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_DST_MIN + nCount ) ;
            }
         }
         if ( nDstType == TYPE_TXT ) {
            if ( ! szDstExtension || stricmp ( szDstExtension, "TXT" ) ) {
               strcat ( szDstFileName, ".txt" ) ;
               Dlg_Wm_SetTextA ( hWnd, IDC_EDIT_DST_MIN + nCount, szDstFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_DST_MIN + nCount ) ;
            }
         }
      }

      // ϊs
      int nResult ;
      if ( *pIsAborted ) nResult = CONV_ERROR_ABORTED ;
      else               nResult = ExecRtfconvA ( hWnd, szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, pInfo ) ;
      if ( nResult ) {
         SetStatusIcon ( hWnd, nCount, STATUS_ICON_FAILURE ) ;
         nCountFailure ++ ;
         if ( nResult < 0 ) break ;
         if ( ! IsNoMessage ) {
            char *szTail = strend ( szMessageBuffer ) ;
            char *Auguments [] = { (char*)(LONG_PTR) ( nCount + 1 ), GetRtfconvErrorMessageA ( nResult ), 0 } ;
            FormatMessageA ( FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY, GetStringA ( IDS_RTFCONVERROR ), 0, 0, szTail, (int) ( MESSAGE_BUFFER_SIZE - ( szTail - szMessageBuffer ) ), (va_list*) Auguments ) ;
         }
      }
      else {
         SetStatusIcon ( hWnd, nCount, STATUS_ICON_SUCCESS ) ;
         nCountSuccess ++ ;
      }
   }

   // ^Cg߂
   SetMainDialogTitle ( hWnd, 0 ) ;

   if ( ! IsNoMessage ) {
      if ( nCountSuccess || nCountFailure ) sprintf ( strend ( szMessageBuffer ), GetStringA ( IDS_CONVERTEDFILE ), nCountSuccess ) ;
      else                                  strcpy ( szMessageBuffer, GetStringA ( IDS_DIDNOTCONVERTFILE ) ) ;
      MessageBoxA ( hWnd, szMessageBuffer, szCommandNameA, MB_OK ) ;
   }

   return 0 ;
}



// t@C擾Ro[^NiUNICODEŁj
int CallRtfconvW ( HWND hWnd, volatile int *pIsAborted, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   wchar_t szSrcFileName [ MAX_PATH + 0x10 ] ;
   wchar_t szDstFileName [ MAX_PATH + 0x10 ] ;
   wchar_t szMessageBuffer [ MESSAGE_BUFFER_SIZE ] ;

   int nCountSuccess = 0 ;
   int nCountFailure = 0 ;

   // ^CgύX
   // ȍ~A^Cg߂܂ return Ȃ
   SetMainDialogTitle ( hWnd, 1 ) ;

   // bZ[Wobt@̏
   *szMessageBuffer = 0 ;

   // t@C擾ϊs
   for ( int nCount = 0 ; nCount < SRC_DST_FILE_NELEM ; nCount ++ ) {

      // ߂Ɏ擾A牺֐ŃG[Ԃ
      Dlg_Wm_GetTextW ( hWnd, IDC_EDIT_SRC_MIN + nCount, szSrcFileName, MAX_PATH + 4 ) ;
      Dlg_Wm_GetTextW ( hWnd, IDC_EDIT_DST_MIN + nCount, szDstFileName, MAX_PATH + 4 ) ;
      UnquoteFileNameW ( szSrcFileName ) ;
      UnquoteFileNameW ( szDstFileName ) ;
      if ( ! *szSrcFileName ) continue ;

      SetStatusIcon ( hWnd, nCount, STATUS_ICON_NOW ) ;

      // KvȂΓ̓t@CɊgqt
      if ( ! IsNoAddExtension && ! IsFileExistW ( szSrcFileName ) ) {
         wchar_t *szSrcExtension = GetExtensionW ( szSrcFileName ) ;
         if ( nSrcType == TYPE_RTF ) {
            if ( ! szSrcExtension || wcsicmp ( szSrcExtension, L"RTF" ) ) {
               wcscat ( szSrcFileName, L".rtf" ) ;
               Dlg_Wm_SetTextW ( hWnd, IDC_EDIT_SRC_MIN + nCount, szSrcFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_SRC_MIN + nCount ) ;
            }
         }
         if ( nSrcType == TYPE_TXT ) {
            if ( ! szSrcExtension || wcsicmp ( szSrcExtension, L"TXT" ) ) {
               wcscat ( szSrcFileName, L".txt" ) ;
               Dlg_Wm_SetTextW ( hWnd, IDC_EDIT_SRC_MIN + nCount, szSrcFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_SRC_MIN + nCount ) ;
            }
         }
      }

      // KvȂΏo̓t@CɊgqt
      if ( ! IsNoAddExtension && ! IsFileExistW ( szDstFileName ) ) {
         wchar_t *szDstExtension = GetExtensionW ( szDstFileName ) ;
         if ( nDstType == TYPE_RTF ) {
            if ( ! szDstExtension || wcsicmp ( szDstExtension, L"RTF" ) ) {
               wcscat ( szDstFileName, L".rtf" ) ;
               Dlg_Wm_SetTextW ( hWnd, IDC_EDIT_DST_MIN + nCount, szDstFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_DST_MIN + nCount ) ;
            }
         }
         if ( nDstType == TYPE_TXT ) {
            if ( ! szDstExtension || wcsicmp ( szDstExtension, L"TXT" ) ) {
               wcscat ( szDstFileName, L".txt" ) ;
               Dlg_Wm_SetTextW ( hWnd, IDC_EDIT_DST_MIN + nCount, szDstFileName ) ;
               Dlg_MoveCaretToEnd ( hWnd, IDC_EDIT_DST_MIN + nCount ) ;
            }
         }
      }

      // ϊs
      int nResult ;
      if ( *pIsAborted ) nResult = CONV_ERROR_ABORTED ;
      else               nResult = ExecRtfconvW ( hWnd, szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, pInfo ) ;
      if ( nResult ) {
         SetStatusIcon ( hWnd, nCount, STATUS_ICON_FAILURE ) ;
         nCountFailure ++ ;
         if ( nResult < 0 ) break ;
         if ( ! IsNoMessage ) {
            wchar_t *szTail = wcsend ( szMessageBuffer ) ;
            wchar_t *Auguments [] = { (wchar_t*)(LONG_PTR) ( nCount + 1 ), GetRtfconvErrorMessageW ( nResult ), 0 } ;
            FormatMessageW ( FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY, GetStringW ( IDS_RTFCONVERROR ), 0, 0, szTail, (int) ( MESSAGE_BUFFER_SIZE - ( szTail - szMessageBuffer ) ), (va_list*) Auguments ) ;
         }
      }
      else {
         SetStatusIcon ( hWnd, nCount, STATUS_ICON_SUCCESS ) ;
         nCountSuccess ++ ;
      }
   }

   // ^Cg߂
   SetMainDialogTitle ( hWnd, 0 ) ;

   if ( ! IsNoMessage ) {
      if ( nCountSuccess || nCountFailure ) swprintf ( wcsend ( szMessageBuffer ), GetStringW ( IDS_CONVERTEDFILE ), nCountSuccess ) ;
      else                                  wcscpy ( szMessageBuffer, GetStringW ( IDS_DIDNOTCONVERTFILE ) ) ;
      MessageBoxW ( hWnd, szMessageBuffer, szCommandNameW, MB_OK ) ;
   }

   return 0 ;
}



// RTFCONV ĂяoiANSIŁj
// ߂ĺARTFCONV ̕Ԃl̂܂ܕԂ
int ExecRtfconvA ( HWND hWnd, char *szSrcFileName, char *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   char szRealSrcFileName [ MAX_PATH ] ;
   char szRealDstFileName [ MAX_PATH ] ;
   char szTempFileName [ MAX_PATH ] ;
   char szCurrentFolder [ MAX_PATH ] ;

   int nExitCode ;


   // {̃t@C擾
   GetSpecialFolderPathA ( hWnd, szCurrentFolder, CSIDL_PERSONAL, 0 ) ;
   SetCurrentDirectoryA ( szCurrentFolder ) ;
   if ( ! GetFullPathNameExA ( szSrcFileName, szRealSrcFileName, MAX_PATH ) ) return CONV_ERROR_READ ;
   strcpy ( szCurrentFolder, szRealSrcFileName ) ;
   ChangeFileNameA ( szCurrentFolder, NULL ) ;
   SetPathSeparatorA ( szCurrentFolder, ADD_SEPARATOR_ONLY_ROOT ) ;
   SetCurrentDirectoryA ( szCurrentFolder ) ;
   if ( ! GetFullPathNameExA ( szDstFileName, szRealDstFileName, MAX_PATH ) ) return CONV_ERROR_WRITE ;

   // JgfBNg߂
   GetModuleFileNameA ( NULL, szCurrentFolder, MAX_PATH ) ;
   ChangeFileNameA ( szCurrentFolder, NULL ) ;
   SetPathSeparatorA ( szCurrentFolder, ADD_SEPARATOR_ONLY_ROOT ) ;
   SetCurrentDirectoryA ( szCurrentFolder ) ;


   // t@CANZX̌
   if ( ! IsFileReadableA ( szRealSrcFileName ) ) return CONV_ERROR_READ ;
   if ( ! IsFileWritableA ( szRealDstFileName ) ) return CONV_ERROR_WRITE ;


   // ϊs
   if ( IsFileExistA ( szRealDstFileName ) ) {
      if ( GetTempFileNameExA ( TEMPFILE_HEADER_A, szTempFileName ) ) return CONV_ERROR_UNKNOWN ;
      nExitCode = RtfconvExA ( szRealSrcFileName, szTempFileName, nSrcCodePage, nDstCodePage, pInfo ) ;
      if ( ! nExitCode && IsFileExistA ( szRealDstFileName ) ) {
         nExitCode = CONV_ERROR_ABORTED ;
         if ( IsNoConfirm || MessageBoxFormatA ( hWnd, szCommandNameA, MB_OKCANCEL, GetStringA ( IDS_ASKOVERWRITE ), szDstFileName ) == IDOK ) {
            nExitCode = CONV_ERROR_WRITE ;
            if ( IsFileWritableA ( szRealDstFileName ) ) {
               if ( IsNoRecycle ) DeleteFileA ( szRealDstFileName ) ;
               else               ShellDeleteFileA ( NULL, szRealDstFileName, 0 ) ;
               if ( MoveFileA ( szTempFileName, szRealDstFileName ) ) nExitCode = 0 ;
            }
         }
      }
      if ( IsFileExistA ( szTempFileName ) ) DeleteFileA ( szTempFileName ) ;
   }
   else {
      nExitCode = RtfconvExA ( szRealSrcFileName, szRealDstFileName, nSrcCodePage, nDstCodePage, pInfo ) ;
      if ( nExitCode && IsFileExistA ( szRealDstFileName ) ) DeleteFileA ( szRealDstFileName ) ;
   }

   if ( ! nExitCode ) ShellChangeNotifyA ( 0, szRealDstFileName, NULL, 0 ) ;

#ifndef _MT
   if ( pInfo->dwConversionMode & CONVMODE_PEEKMESSAGE ) {
      MSG Msg ;
      while ( PeekMessageA ( & Msg, hWnd, 0, 0, PM_REMOVE ) ) {
         TranslateMessage ( & Msg ) ;
         DispatchMessageA ( & Msg ) ;
      }
   }
#endif

   return nExitCode ;
}



// RTFCONV ĂяoiUNICODEŁj
// ߂ĺARTFCONV ̕Ԃl̂܂ܕԂ
int ExecRtfconvW ( HWND hWnd, wchar_t *szSrcFileName, wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   wchar_t szRealSrcFileName [ MAX_PATH ] ;
   wchar_t szRealDstFileName [ MAX_PATH ] ;
   wchar_t szTempFileName [ MAX_PATH ] ;
   wchar_t szCurrentFolder [ MAX_PATH ] ;

   int nExitCode ;


   // {̃t@C擾
   GetSpecialFolderPathW ( hWnd, szCurrentFolder, CSIDL_PERSONAL, 0 ) ;
   SetCurrentDirectoryW ( szCurrentFolder ) ;
   if ( ! GetFullPathNameExW ( szSrcFileName, szRealSrcFileName, MAX_PATH ) ) return CONV_ERROR_READ ;
   wcscpy ( szCurrentFolder, szRealSrcFileName ) ;
   ChangeFileNameW ( szCurrentFolder, NULL ) ;
   SetPathSeparatorW ( szCurrentFolder, ADD_SEPARATOR_ONLY_ROOT ) ;
   SetCurrentDirectoryW ( szCurrentFolder ) ;
   if ( ! GetFullPathNameExW ( szDstFileName, szRealDstFileName, MAX_PATH ) ) return CONV_ERROR_WRITE ;

   // JgfBNg߂
   GetModuleFileNameW ( NULL, szCurrentFolder, MAX_PATH ) ;
   ChangeFileNameW ( szCurrentFolder, NULL ) ;
   SetPathSeparatorW ( szCurrentFolder, ADD_SEPARATOR_ONLY_ROOT ) ;
   SetCurrentDirectoryW ( szCurrentFolder ) ;


   // t@CANZX̌
   if ( ! IsFileReadableW ( szRealSrcFileName ) ) return CONV_ERROR_READ ;
   if ( ! IsFileWritableW ( szRealDstFileName ) ) return CONV_ERROR_WRITE ;


   // ϊs
   if ( IsFileExistW ( szRealDstFileName ) ) {
      if ( GetTempFileNameExW ( TEMPFILE_HEADER_W, szTempFileName ) ) return CONV_ERROR_UNKNOWN ;
      nExitCode = RtfconvExW ( szRealSrcFileName, szTempFileName, nSrcCodePage, nDstCodePage, pInfo ) ;
      if ( ! nExitCode && IsFileExistW ( szRealDstFileName ) ) {
         nExitCode = CONV_ERROR_ABORTED ;
         if ( IsNoConfirm || MessageBoxFormatW ( hWnd, szCommandNameW, MB_OKCANCEL, GetStringW ( IDS_ASKOVERWRITE ), szDstFileName ) == IDOK ) {
            nExitCode = CONV_ERROR_WRITE ;
            if ( IsFileWritableW ( szRealDstFileName ) ) {
               if ( IsNoRecycle ) DeleteFileW ( szRealDstFileName ) ;
               else               ShellDeleteFileW ( NULL, szRealDstFileName, 0 ) ;
               if ( MoveFileW ( szTempFileName, szRealDstFileName ) ) nExitCode = 0 ;
            }
         }
      }
      if ( IsFileExistW ( szTempFileName ) ) DeleteFileW ( szTempFileName ) ;
   }
   else {
      nExitCode = RtfconvExW ( szRealSrcFileName, szRealDstFileName, nSrcCodePage, nDstCodePage, pInfo ) ;
      if ( nExitCode && IsFileExistW ( szRealDstFileName ) ) DeleteFileW ( szRealDstFileName ) ;
   }

   if ( ! nExitCode ) ShellChangeNotifyW ( 0, szRealDstFileName, NULL, 0 ) ;

#ifndef _MT
   if ( pInfo->dwConversionMode & CONVMODE_PEEKMESSAGE ) {
      MSG Msg ;
      while ( PeekMessageW ( & Msg, hWnd, 0, 0, PM_REMOVE ) ) {
         TranslateMessage ( & Msg ) ;
         DispatchMessageW ( & Msg ) ;
      }
   }
#endif

   return nExitCode ;
}



// GetRtfconvErrorMessage ̉֐
int GetStringIdFromRtfconvError ( int nError ) {
   switch ( nError ) {
      case CONV_ERROR_READ :     return IDS_CONV_ERROR_READ ;
      case CONV_ERROR_WRITE :    return IDS_CONV_ERROR_WRITE ;
      case CONV_ERROR_FORMAT :
      case CONV_ERROR_BRACE :    return IDS_CONV_ERROR_FORMAT ;
      case CONV_ERROR_MEMORY :   return IDS_CONV_ERROR_MEMORY ;
      case CONV_ERROR_CODEPAGE : return IDS_CONV_ERROR_CODEPAGE ;
      case CONV_ERROR_ABORTED :  return IDS_CONV_ERROR_ABORTED ;
      default :                  return IDS_CONV_ERROR_UNKNOWN ;
   }
}



// RTFCONV ̃G[lɂӂ킵ԂiANSIŁj
char *GetRtfconvErrorMessageA ( int nError ) {
   static char szBuffer [ SHORTSTRING_MAX ] ;
   LoadStringExA ( GetStringIdFromRtfconvError ( nError ), szBuffer, SHORTSTRING_MAX ) ;
   return szBuffer ;
}



// RTFCONV ̃G[lɂӂ킵ԂiUNICODEŁj
wchar_t *GetRtfconvErrorMessageW ( int nError ) {
   static wchar_t szBuffer [ SHORTSTRING_MAX ] ;
   LoadStringExW ( GetStringIdFromRtfconvError ( nError ), szBuffer, SHORTSTRING_MAX ) ;
   return szBuffer ;
}



////////////////////////////////////////////
//               wv\               //
////////////////////////////////////////////



INT_PTR CALLBACK HelpDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
int DisplayVersion ( HWND hWnd ) ;



int DisplayHelp ( HWND hWnd ) {
   // EBhE\
   if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_HELP, hWnd, HelpDialogProc ) ;
   else           DialogBoxExA ( IDD_DIALOG_HELP, hWnd, HelpDialogProc ) ;
   return 0 ;
}



// wv\_CAO
INT_PTR CALLBACK HelpDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         MoveWindowToCenter ( hWnd ) ;

         // ^Cgݒ
         if ( IsNT () ) Wm_SetTextFormatW ( hWnd, L"%s  %s  %s", szCommandNameW, GetVersionCopyrightW (), WIN3264BIT_SUFFIX_W ) ;
         else           Wm_SetTextFormatA ( hWnd,  "%s  %s  %s", szCommandNameA, GetVersionCopyrightA (), WIN3264BIT_SUFFIX_A ) ;

         if ( IsNT () ) {
            wchar_t szBuffer [ MESSAGE_BUFFER_SIZE ] ;
            swprintf ( szBuffer, GetStringW ( IDS_DISPLAYEXEVERSION ), nVersion / 100, nVersion % 100 ) ;
            wcscat ( szBuffer, GetStringW ( IDS_FORDLLVERSION ) ) ;
            Dlg_Wm_SetTextW ( hWnd, IDC_STATIC_CALL_VERSION, szBuffer ) ;
         }
         else {
            char szBuffer [ MESSAGE_BUFFER_SIZE ] ;
            sprintf ( szBuffer, GetStringA ( IDS_DISPLAYEXEVERSION ), nVersion / 100, nVersion % 100 ) ;
            strcat ( szBuffer, GetStringA ( IDS_FORDLLVERSION ) ) ;
            Dlg_Wm_SetTextA ( hWnd, IDC_STATIC_CALL_VERSION, szBuffer ) ;
         }

         return TRUE ;


      case WM_COMMAND :

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         // t@CJ
         if ( LOWORD ( wParam ) == IDC_BUTTON_OPENFILE ) {

            if ( IsNT () ) {
               wchar_t szFileName [ MAX_PATH ] ;
               GetModuleFileNameW ( NULL, szFileName, MAX_PATH ) ;
               ChangeFileNameW ( szFileName, GetStringW ( IDS_HELPFILE_FILENAME ) ) ;
               int nResult = ( ShellExecuteW ( hWnd, NULL, szFileName, NULL, szFileName, SW_SHOWDEFAULT ) <= (HINSTANCE) 32 ) ;
               if ( nResult ) MessageBoxFormatW ( hWnd, szCommandNameW, MB_OK, GetStringW ( IDS_CANNOTOPENHELPFILE ), GetFileNameW ( szFileName ) ) ;
               else           EndDialog ( hWnd, IDCANCEL ) ;
            }
            else {
               char szFileName [ MAX_PATH ] ;
               GetModuleFileNameA ( NULL, szFileName, MAX_PATH ) ;
               ChangeFileNameA ( szFileName, GetStringA ( IDS_HELPFILE_FILENAME ) ) ;
               int nResult = ( ShellExecuteA ( hWnd, NULL, szFileName, NULL, szFileName, SW_SHOWDEFAULT ) <= (HINSTANCE) 32 ) ;
               if ( nResult ) MessageBoxFormatA ( hWnd, szCommandNameA, MB_OK, GetStringA ( IDS_CANNOTOPENHELPFILE ), GetFileNameA ( szFileName ) ) ;
               else           EndDialog ( hWnd, IDCANCEL ) ;
            }

            return TRUE ;
         }

         // o[W\
         if ( LOWORD ( wParam ) == IDC_BUTTON_CALL_VERSION ) {
            DisplayVersion ( hWnd ) ;
            return TRUE ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// DLLo[W\_CAO
int DisplayVersion ( HWND hWnd ) {

   if ( IsNT () ) {

      int nDllError = 0 ;

      wchar_t szBuffer [ MESSAGE_BUFFER_SIZE ] ;
      *szBuffer = 0 ;   // bZ[Wobt@̏

      for ( int nCount = 0 ; nCount < 4 ; nCount ++ ) {

         const wchar_t *szDllName ;
         switch ( nCount ) {
            case 0 : szDllName = L"RTFCONV.DLL" ; break ;
            case 1 : szDllName = L"SBCTBL.DLL" ; break ;
            case 2 : szDllName = L"MBCTBL.DLL" ; break ;
            case 3 : szDllName = L"CNSTBL.DLL" ; break ;
            default : szDllName = NULL ; break ;
         }
         if ( ! szDllName ) break ;

         HINSTANCE hDllInstance = LoadDllW ( NULL, szDllName ) ;
         GETDLLVERSION GetDllVersion = NULL ;

         if ( GETPROCADDRESS ( hDllInstance, GETDLLVERSION, GetDllVersion ) ) {
            swprintf ( wcsend ( szBuffer ), GetStringW ( IDS_DISPLAYDLLVERSIONERROR ), szDllName ) ;
            nDllError ++ ;
         }
         else {
            int nDllVersion = GetDllVersion () ;
            swprintf ( wcsend ( szBuffer ), GetStringW ( IDS_DISPLAYDLLVERSION ), szDllName, nDllVersion / 100, nDllVersion % 100 ) ;
         }
         if ( hDllInstance ) FreeLibrary ( hDllInstance ) ;
      }

      if ( ! nDllError ) {
         wcscpy ( wcsend ( szBuffer ), L"\n" ) ;
         wcscpy ( wcsend ( szBuffer ), GetStringW ( IDS_DISPLAYDLLVERSIONSUCCESS ) ) ;
      }

      wcscpy ( wcsend ( szBuffer ), L"\n" ) ;
      MessageBoxW ( hWnd, szBuffer, szCommandNameW, MB_OK ) ;

   }
   else {

      int nDllError = 0 ;

      char szBuffer [ MESSAGE_BUFFER_SIZE ] ;
      *szBuffer = 0 ;   // bZ[Wobt@̏

      for ( int nCount = 0 ; nCount < 4 ; nCount ++ ) {

         const char *szDllName ;
         switch ( nCount ) {
            case 0 : szDllName = "RTFCONV.DLL" ; break ;
            case 1 : szDllName = "SBCTBL.DLL" ; break ;
            case 2 : szDllName = "MBCTBL.DLL" ; break ;
            case 3 : szDllName = "CNSTBL.DLL" ; break ;
            default : szDllName = NULL ; break ;
         }
         if ( ! szDllName ) break ;

         HINSTANCE hDllInstance = LoadDllA ( NULL, szDllName ) ;
         GETDLLVERSION GetDllVersion = NULL ;

         if ( GETPROCADDRESS ( hDllInstance, GETDLLVERSION, GetDllVersion ) ) {
            sprintf ( strend ( szBuffer ), GetStringA ( IDS_DISPLAYDLLVERSIONERROR ), szDllName ) ;
            nDllError ++ ;
         }
         else {
            int nDllVersion = GetDllVersion () ;
            sprintf ( strend ( szBuffer ), GetStringA ( IDS_DISPLAYDLLVERSION ), szDllName, nDllVersion / 100, nDllVersion % 100 ) ;
         }
         if ( hDllInstance ) FreeLibrary ( hDllInstance ) ;
      }

      if ( ! nDllError ) {
         strcpy ( strend ( szBuffer ), "\n" ) ;
         strcpy ( strend ( szBuffer ), GetStringA ( IDS_DISPLAYDLLVERSIONSUCCESS ) ) ;
      }

      strcpy ( strend ( szBuffer ), "\n" ) ;
      MessageBoxA ( hWnd, szBuffer, szCommandNameA, MB_OK ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//            IvVݒ              //
////////////////////////////////////////////



INT_PTR CALLBACK OptionDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog1Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog2Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog3Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog4Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog5Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog6Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK OptionDialog3InvalidProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;

int TabInsertItem ( HWND hTabControl, int nIndex, int nCaptionId ) ;

int GetOptionDialogFontName ( HWND hWnd, int nSingleByteComboBox, int nMultiByteComboBox, int nCodePage ) ;
int SetOptionDialogFontName ( HWND hWnd, int nSingleByteComboBox, int nMultiByteComboBox, int nIdDlgItem, int nCodePage ) ;
int SetComboBoxFontName ( HWND hWnd, int nComboBox, int nCodePage ) ;

int SetComboBoxFontSize ( HWND hWnd, int nComboBox, int nDefaultFontSize ) ;
int SetComboBoxFontSizeValue ( HWND hWnd, int nComboBox, int nFontSize ) ;
int GetComboBoxFontSizeValue ( HWND hWnd, int nComboBox ) ;

int SetComboBoxMuiLangId ( HWND hWnd, int nComboBox ) ;
int GetComboBoxMuiLangId ( HWND hWnd, int nComboBox ) ;
int AllocMuiLangIdArray ( void ) ;
int FreeMuiLangIdArray ( void ) ;

INT_PTR CALLBACK FontExDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK ExConvModeJpDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK ExConvModeCnsDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK ExConvModeCntDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;
INT_PTR CALLBACK ExConvModeKsDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) ;

// tHgwŎgp
static char szAsciiFontTmp [ FONTNAME_MAX ] ;
static char szJapaneseFontTmp [ FONTNAME_MAX ] ;
static char szKoreanFontTmp [ FONTNAME_MAX ] ;
static char szGbFontTmp [ FONTNAME_MAX ] ;
static char szBig5FontTmp [ FONTNAME_MAX ] ;
static char szThaiFontTmp [ FONTNAME_MAX ] ;
int nDefaultMultiByteCodePageSub ;
#define FONTSIZE_BUFFER_MAX   8

static int IsNoVariantTmp ;


typedef struct {
   int nTemplate ;
   int nCaption ;
   DLGPROC pDlgProc ;
   HWND hWnd ;
} PROPSHEET_DATA ;


typedef struct {
   int nPages ;
   PROPSHEET_DATA *pData ;
} PROPSHEET ;



static PROPSHEET_DATA OptionPropSheetData [] = {
   { IDD_PROPPAGE_1, IDS_PROPPAGE_1_CAPTION, OptionDialog1Proc, NULL },
   { IDD_PROPPAGE_2, IDS_PROPPAGE_2_CAPTION, OptionDialog2Proc, NULL },
   { IDD_PROPPAGE_3, IDS_PROPPAGE_3_CAPTION, OptionDialog3Proc, NULL },
   { IDD_PROPPAGE_4, IDS_PROPPAGE_4_CAPTION, OptionDialog4Proc, NULL },
   { IDD_PROPPAGE_5, IDS_PROPPAGE_5_CAPTION, OptionDialog5Proc, NULL },
   { IDD_PROPPAGE_6, IDS_PROPPAGE_6_CAPTION, OptionDialog6Proc, NULL },
} ;


static PROPSHEET_DATA OptionPropSheetDataInvalid [] = {
   { IDD_PROPPAGE_1, IDS_PROPPAGE_1_CAPTION, OptionDialog1Proc, NULL },
   { IDD_PROPPAGE_2, IDS_PROPPAGE_2_CAPTION, OptionDialog2Proc, NULL },
   { IDD_PROPPAGE_3_INVALID, IDS_PROPPAGE_3_CAPTION, OptionDialog3InvalidProc, NULL },
   { IDD_PROPPAGE_4, IDS_PROPPAGE_4_CAPTION, OptionDialog4Proc, NULL },
   { IDD_PROPPAGE_5, IDS_PROPPAGE_5_CAPTION, OptionDialog5Proc, NULL },
   { IDD_PROPPAGE_6, IDS_PROPPAGE_6_CAPTION, OptionDialog6Proc, NULL },
} ;



int SetOption ( HWND hWnd ) {

   PROPSHEET PropSheet ;

   if ( nSrcType != TYPE_TXT || nDstType != TYPE_RTF ) {
      PropSheet.nPages = NELEMOF ( OptionPropSheetDataInvalid ) ;
      PropSheet.pData = OptionPropSheetDataInvalid ;
   }
   else {
      PropSheet.nPages = NELEMOF ( OptionPropSheetData ) ;
      PropSheet.pData = OptionPropSheetData ;
   }

   if ( IsNT () ) DialogBoxParamExW ( IDD_DIALOG_OPTION, hWnd, OptionDialogProc, (LPARAM) & PropSheet ) ;
   else           DialogBoxParamExA ( IDD_DIALOG_OPTION, hWnd, OptionDialogProc, (LPARAM) & PropSheet ) ;

   if ( IsOptionChanged ) {
      int nResult ;
      if ( IsNT () ) nResult = WriteOptionW ( szConfigFileNameW ) ;
      else           nResult = WriteOptionA ( szConfigFileNameA ) ;
      if ( nResult ) {
         if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_CANNOT_SAVE_OPTION ), szCommandNameW, MB_OK ) ;
         else           MessageBoxA ( hWnd, GetStringA ( IDS_CANNOT_SAVE_OPTION ), szCommandNameA, MB_OK ) ;
      }
      else IsOptionChanged = 0 ;
   }

   return 0 ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   static PROPSHEET_DATA *pPropSheetData ;
   static int nPropSheetPages ;
   static HWND hTabControl ;
   static RECT Rect ;
   static int nCurrentPage ;
   static HWND *pTabOrderList ;

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         pPropSheetData = ( (PROPSHEET*) lParam )->pData ;
         nPropSheetPages = ( (PROPSHEET*) lParam )->nPages ;

         for ( int nCount = 0 ; nCount < nPropSheetPages ; nCount ++ ) ( pPropSheetData [ nCount ] ).hWnd = NULL ;

         if ( IsNT () ) Wm_SetTextFormatW ( hWnd, L"%s  -  %s", szCommandNameW, GetStringW ( IDS_OPTIONTITLE ) ) ;
         else           Wm_SetTextFormatA ( hWnd,  "%s  -  %s", szCommandNameA, GetStringA ( IDS_OPTIONTITLE ) ) ;

         hTabControl = GetDlgItem ( hWnd, IDC_TAB_OPTION_PROPPAGE ) ;

         // ^uRg[ɃACe}
         for ( int nCount = 0 ; nCount < nPropSheetPages ; nCount ++ ) TabInsertItem ( hTabControl, nCount, ( pPropSheetData [ nCount ] ).nCaption ) ;

         // ^uRg[̈ʒu𒲐
         GetClientRect ( hTabControl, & Rect ) ;
         TabCtrl_AdjustRect ( hTabControl, FALSE, & Rect ) ;
         MapWindowPoints ( hTabControl, hWnd, (LPPOINT) & Rect, sizeof(RECT)/sizeof(POINT) ) ;
         if ( DEBUG_TAB_CONTROL ) MessageBoxFormatA ( hWnd, "", MB_OK, "%d, %d", Rect.right - Rect.left, Rect.bottom - Rect.top ) ;

         // ^uI[_[u郊Xg쐬
         if ( 1 ) {
            int nCount = 0 ;
            for ( HWND hWndNext = hTabControl ; ( hWndNext = GetWindow ( hWndNext, GW_HWNDNEXT ) ) != NULL && hWndNext != hTabControl ; ) nCount ++ ;
            pTabOrderList = (HWND*) malloc ( sizeof(HWND*) * ( nCount + 1 ) ) ;
         }
         if ( pTabOrderList ) {
            int nCount = 0 ;
            for ( HWND hWndNext = hTabControl ; ( hWndNext = GetWindow ( hWndNext, GW_HWNDNEXT ) ) != NULL && hWndNext != hTabControl ; nCount ++ ) pTabOrderList [ nCount ] = hWndNext ;
            pTabOrderList [ nCount ] = NULL ;
         }

         // Py[Wڂ쐬
         nCurrentPage = 0 ;
         if ( IsNT () ) ( pPropSheetData [ nCurrentPage ] ).hWnd = CreateDialogExW ( ( pPropSheetData [ nCurrentPage ] ).nTemplate, hWnd, ( pPropSheetData [ nCurrentPage ] ).pDlgProc ) ;
         else           ( pPropSheetData [ nCurrentPage ] ).hWnd = CreateDialogExA ( ( pPropSheetData [ nCurrentPage ] ).nTemplate, hWnd, ( pPropSheetData [ nCurrentPage ] ).pDlgProc ) ;
         if ( EnableThemeDialogTexture ) EnableThemeDialogTexture ( ( pPropSheetData [ nCurrentPage ] ).hWnd, ETDT_ENABLETAB ) ;
         MoveWindow ( ( pPropSheetData [ nCurrentPage ] ).hWnd, Rect.left, Rect.top, Rect.right - Rect.left, Rect.bottom - Rect.top, FALSE ) ;

         // ^uI[_[u
         if ( pTabOrderList ) {
            for ( int nCount = 0 ; pTabOrderList [ nCount ] ; nCount ++ ) SetWindowPos ( pTabOrderList [ nCount ], HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOREDRAW | SWP_NOACTIVATE ) ;
         }

         ShowWindow ( ( pPropSheetData [ nCurrentPage ] ).hWnd, SW_SHOW ) ;

         return TRUE ;


      case WM_NOTIFY :

         // ^uRg[̃V[g؂ւʒmȂ
         if ( ( (NMHDR*) lParam )->code == TCN_SELCHANGE ) {
            if ( ( (NMHDR*) lParam )->hwndFrom == hTabControl ) {

               int nPrevPage = nCurrentPage ;
               nCurrentPage = TabCtrl_GetCurSel ( hTabControl ) ;

               // Vy[W쐬Ȃ쐬
               if ( ! ( pPropSheetData [ nCurrentPage ] ).hWnd ) {
                  if ( IsNT () ) ( pPropSheetData [ nCurrentPage ] ).hWnd = CreateDialogExW ( ( pPropSheetData [ nCurrentPage ] ).nTemplate, hWnd, ( pPropSheetData [ nCurrentPage ] ).pDlgProc ) ;
                  else           ( pPropSheetData [ nCurrentPage ] ).hWnd = CreateDialogExA ( ( pPropSheetData [ nCurrentPage ] ).nTemplate, hWnd, ( pPropSheetData [ nCurrentPage ] ).pDlgProc ) ;
                  if ( EnableThemeDialogTexture ) EnableThemeDialogTexture ( ( pPropSheetData [ nCurrentPage ] ).hWnd, ETDT_ENABLETAB ) ;
                  MoveWindow ( ( pPropSheetData [ nCurrentPage ] ).hWnd, Rect.left, Rect.top, Rect.right - Rect.left, Rect.bottom - Rect.top, FALSE ) ;

                  // ^uI[_[u
                  if ( pTabOrderList ) {
                     for ( int nCount = 0 ; pTabOrderList [ nCount ] ; nCount ++ ) SetWindowPos ( pTabOrderList [ nCount ], HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOREDRAW | SWP_NOACTIVATE ) ;
                  }
               }

               NMHDR NotifyMessage ;

               NotifyMessage.hwndFrom = hWnd ;
               NotifyMessage.idFrom = IDC_TAB_OPTION_PROPPAGE ;
               NotifyMessage.code = PSN_KILLACTIVE ;
               Wm_Notify ( ( pPropSheetData [ nPrevPage ] ).hWnd, & NotifyMessage ) ;

               NotifyMessage.hwndFrom = hWnd ;
               NotifyMessage.idFrom = IDC_TAB_OPTION_PROPPAGE ;
               NotifyMessage.code = PSN_SETACTIVE ;
               Wm_Notify ( ( pPropSheetData [ nCurrentPage ] ).hWnd, & NotifyMessage ) ;

               ShowWindow ( ( pPropSheetData [ nPrevPage ] ).hWnd, SW_HIDE ) ;
               ShowWindow ( ( pPropSheetData [ nCurrentPage ] ).hWnd, SW_SHOW ) ;

               return TRUE ;
            }
         }
         break ;


      case WM_COMMAND :

         // ݒs
         if ( LOWORD ( wParam ) == IDOK ) {
            for ( int nCount = 0 ; nCount < nPropSheetPages ; nCount ++ ) {
               if ( ( pPropSheetData [ nCount ] ).hWnd ) {
                  NMHDR NotifyMessage ;
                  NotifyMessage.hwndFrom = hWnd ;
                  NotifyMessage.idFrom = IDC_TAB_OPTION_PROPPAGE ;
                  NotifyMessage.code = PSN_APPLY ;
                  Wm_Notify ( ( pPropSheetData [ nCount ] ).hWnd, & NotifyMessage ) ;
               }
            }
            EndDialog ( hWnd, IDOK ) ;
            return TRUE ;
         }

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         if ( IsNT () ) return SendMessageW ( ( pPropSheetData [ nCurrentPage ] ).hWnd, dwMessage, wParam, lParam ) ;
         else           return SendMessageA ( ( pPropSheetData [ nCurrentPage ] ).hWnd, dwMessage, wParam, lParam ) ;


      case WM_DESTROY :
         free ( pTabOrderList ) ;
         break ;
   }

   return FALSE ;
}



// ^uRg[ɃACe}
int TabInsertItem ( HWND hTabControl, int nIndex, int nCaptionId ) {

   if ( IsNT () ) {
      wchar_t szBuffer [ SHORTSTRING_MAX ] ;
      LoadStringExW ( nCaptionId, szBuffer, SHORTSTRING_MAX ) ;
      TC_ITEMW TcItem ;
      TcItem.mask = TCIF_TEXT ;
      TcItem.pszText = szBuffer ;
      TabCtrl_InsertItemW ( hTabControl, nIndex, & TcItem ) ;
   }
   else {
      char szBuffer [ SHORTSTRING_MAX ] ;
      LoadStringExA ( nCaptionId, szBuffer, SHORTSTRING_MAX ) ;
      TC_ITEMA TcItem ;
      TcItem.mask = TCIF_TEXT ;
      TcItem.pszText = szBuffer ;
      TabCtrl_InsertItemA ( hTabControl, nIndex, & TcItem ) ;
   }

   return 0 ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog1Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         if ( DEBUG_TAB_CONTROL ) {
            RECT Rect ;
            GetWindowRect ( hWnd, & Rect ) ;
            MessageBoxFormatA ( hWnd, "", MB_OK, "%d, %d", Rect.right - Rect.left, Rect.bottom - Rect.top ) ;
         }

         // l
         if ( IsNoVariant ) CheckDlgButton ( hWnd, IDC_CHECK_NO_VARIANT, BST_CHECKED ) ;
         if ( IsNoByteOrderMark ) CheckDlgButton ( hWnd, IDC_CHECK_NO_BYTEORDERMARK, BST_CHECKED ) ;
         if ( IsUtf8ByteOrderMark ) CheckDlgButton ( hWnd, IDC_CHECK_UTF8_BYTEORDERMARK, BST_CHECKED ) ;
         if ( IsOutputRuby ) CheckDlgButton ( hWnd, IDC_CHECK_OUTPUT_RUBY, BST_CHECKED ) ;
         if ( IsOutputLink ) CheckDlgButton ( hWnd, IDC_CHECK_OUTPUT_LINK, BST_CHECKED ) ;
         if ( IsOutputHidden ) CheckDlgButton ( hWnd, IDC_CHECK_OUTPUT_HIDDEN, BST_CHECKED ) ;
         if ( IsInputNumberHtml ) CheckDlgButton ( hWnd, IDC_CHECK_INPUT_NUMBER_HTML, BST_CHECKED ) ;
         if ( IsInputNumberC ) CheckDlgButton ( hWnd, IDC_CHECK_INPUT_NUMBER_C, BST_CHECKED ) ;
         if ( IsEndianAuto ) CheckDlgButton ( hWnd, IDC_CHECK_ENDIAN_AUTO, BST_CHECKED ) ;
         if ( IsOutputNumberSurrogate ) CheckDlgButton ( hWnd, IDC_CHECK_NUMBER_SURROGATE, BST_CHECKED ) ;

         // R{{bNX̐ݒ
         Dlg_ComboBox_ResetContent ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR ) ;
         for ( int nCount = 0 ; nCount < UNMAPPABLE_CHAR_FORMAT_LIST_NELEM ; nCount ++ ) {
            if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR, -1, GetStringW ( ( UnmappableCharFormatList [ nCount ] ).nNameId ) ) ;
            else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR, -1, GetStringA ( ( UnmappableCharFormatList [ nCount ] ).nNameId ) ) ;
         }
         Dlg_ComboBox_SetMaxHeight ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR, 12 ) ;
         Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR, nUnmappableCharFormat ) ;

         Dlg_EnableWindow ( hWnd, IDC_CHECK_NUMBER_SURROGATE, ( ( UnmappableCharFormatList [ nUnmappableCharFormat ] ).szFormat != NULL && CountPrintfArgumentsA ( ( UnmappableCharFormatList [ nUnmappableCharFormat ] ).szFormat ) > 0 ) ) ;

         return TRUE ;


      case WM_COMMAND :

         if ( LOWORD ( wParam ) == IDC_COMBO_UNMAPPABLE_CHAR ) {
            // R{{bNX̑IύXƂ
            if ( HIWORD ( wParam ) == CBN_SELCHANGE ) {
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR ) ;
               Dlg_EnableWindow ( hWnd, IDC_CHECK_NUMBER_SURROGATE, ( ( UnmappableCharFormatList [ nCount ] ).szFormat != NULL && CountPrintfArgumentsA ( ( UnmappableCharFormatList [ nCount ] ).szFormat ) > 0 ) ) ;
               return TRUE ;
            }
         }

         break ;


      case WM_NOTIFY :

         // ݒ
         if ( ((NMHDR*) lParam )->code == PSN_APPLY ) {
            IsNoVariant = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_VARIANT ) ;
            IsNoByteOrderMark = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_BYTEORDERMARK ) ;
            IsUtf8ByteOrderMark = IsDlgButtonChecked ( hWnd, IDC_CHECK_UTF8_BYTEORDERMARK ) ;
            IsOutputRuby = IsDlgButtonChecked ( hWnd, IDC_CHECK_OUTPUT_RUBY ) ;
            IsOutputLink = IsDlgButtonChecked ( hWnd, IDC_CHECK_OUTPUT_LINK ) ;
            IsOutputHidden = IsDlgButtonChecked ( hWnd, IDC_CHECK_OUTPUT_HIDDEN ) ;
            IsInputNumberHtml = IsDlgButtonChecked ( hWnd, IDC_CHECK_INPUT_NUMBER_HTML ) ;
            IsInputNumberC = IsDlgButtonChecked ( hWnd, IDC_CHECK_INPUT_NUMBER_C ) ;
            IsEndianAuto = IsDlgButtonChecked ( hWnd, IDC_CHECK_ENDIAN_AUTO ) ;
            nUnmappableCharFormat = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_UNMAPPABLE_CHAR ) ;
            IsOutputNumberSurrogate = IsDlgButtonChecked ( hWnd, IDC_CHECK_NUMBER_SURROGATE ) ;
            IsOptionChanged = 1 ;
            return TRUE ;
         }

         // ؂ւ
         if ( ((NMHDR*) lParam )->code == PSN_KILLACTIVE ) {
            IsNoVariantTmp = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_VARIANT ) ;
            break ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog2Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsNoHanKana ) CheckDlgButton ( hWnd, IDC_CHECK_NO_HANKANA, BST_CHECKED ) ;
         if ( IsNoZenAlpha ) CheckDlgButton ( hWnd, IDC_CHECK_NO_ZENALPHA, BST_CHECKED ) ;
         if ( IsNoZenSpace ) CheckDlgButton ( hWnd, IDC_CHECK_NO_ZENSPACE, BST_CHECKED ) ;
         if ( IsNoZenSymbol ) CheckDlgButton ( hWnd, IDC_CHECK_NO_ZENSYMBOL, BST_CHECKED ) ;
         if ( IsYenNoBackslash ) CheckDlgButton ( hWnd, IDC_CHECK_YEN_NO_BACKSLASH, BST_CHECKED ) ;
         if ( IsOverlineNoTilde ) CheckDlgButton ( hWnd, IDC_CHECK_OVERLINE_NO_TILDE, BST_CHECKED ) ;

         return TRUE ;


      case WM_NOTIFY :

         // ݒ
         if ( ((NMHDR*) lParam )->code == PSN_APPLY ) {
            IsNoHanKana = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_HANKANA ) ;
            IsNoZenAlpha = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_ZENALPHA ) ;
            IsNoZenSpace = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_ZENSPACE ) ;
            IsNoZenSymbol = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_ZENSYMBOL ) ;
            IsYenNoBackslash = IsDlgButtonChecked ( hWnd, IDC_CHECK_YEN_NO_BACKSLASH ) ;
            IsOverlineNoTilde = IsDlgButtonChecked ( hWnd, IDC_CHECK_OVERLINE_NO_TILDE ) ;
            IsOptionChanged = 1 ;
            return TRUE ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog3Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   static HFONT hAsciiFont ;
   static HFONT hMultiByteFont ;
   static int nUniDefaultCodeTmp ;
   int nCodePage ;

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         nCodePage = GetWinCodePage ( GetModelessCodePage ( ( AvailableCodeList [ nSrcCode ] ).nCodePage ) ) ;

         SetComboBoxFontSize ( hWnd, IDC_COMBO_FONTSIZE, nFontSize ) ;

         strcpy ( szAsciiFontTmp, szEuropeanFont ) ;
         strcpy ( szJapaneseFontTmp, szJapaneseFont ) ;
         strcpy ( szKoreanFontTmp, szKoreanFont ) ;
         strcpy ( szGbFontTmp, szChineseSimplifiedFont ) ;
         strcpy ( szBig5FontTmp, szChineseTraditionalFont ) ;
         strcpy ( szThaiFontTmp, szThaiFont ) ;

         for ( int nCount = 0 ; nCount < MULTIBYTE_CODE_LIST_NELEM ; nCount ++ ) {
            if ( nDefaultCjkCodePage == ( MultiByteCodeList [ nCount ] ).nCodePage ) nUniDefaultCodeTmp = nCount ;
         }

         if ( nCodePage != CP_UTF16 && ! IsMultiByteCodePage ( nCodePage ) && nCodePage != CP_THAI ) {
            if ( IsNT () ) Dlg_Wm_SetTextW ( hWnd, IDC_CJKLANG_GUIDE, GetStringW ( IDS_GUIDE_CJKLANG_NOTSPECIFIED ) ) ;
            else           Dlg_Wm_SetTextA ( hWnd, IDC_CJKLANG_GUIDE, GetStringA ( IDS_GUIDE_CJKLANG_NOTSPECIFIED ) ) ;
         }

         if ( nCodePage == CP_THAI ) {
            if ( IsNT () ) Dlg_Wm_SetTextW ( hWnd, IDC_CHECK_SAMEFONT, GetStringW ( IDS_GUIDE_USESAMEFONT_THAI ) ) ;
            else           Dlg_Wm_SetTextA ( hWnd, IDC_CHECK_SAMEFONT, GetStringA ( IDS_GUIDE_USESAMEFONT_THAI ) ) ;
         }

         // Unicode ̊R{{bNX
         Dlg_ComboBox_ResetContent ( hWnd, IDC_COMBO_CJKLANG ) ;
         for ( int nCount = 0 ; nCount < MULTIBYTE_CODE_LIST_NELEM ; nCount ++ ) {
            if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_CJKLANG, -1, GetStringW ( ( MultiByteCodeList [ nCount ] ).nNameId ) ) ;
            else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_CJKLANG, -1, GetStringA ( ( MultiByteCodeList [ nCount ] ).nNameId ) ) ;
         }

         // GfBbgRg[͂̒̐
         Dlg_ComboBox_LimitText ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, FONTNAME_MAX ) ;
         Dlg_ComboBox_LimitText ( hWnd, IDC_COMBO_MULTIBYTE_FONT, FONTNAME_MAX ) ;
         Dlg_ComboBox_LimitText ( hWnd, IDC_COMBO_FONTSIZE, FONTSIZE_BUFFER_MAX - 1 ) ;

         // tHg̏l
         // tHgEBhE͉̓s̐ݒ
         if ( nCodePage == CP_UTF16 || ! IsMultiByteCodePage ( nCodePage ) && nCodePage != CP_THAI && IsUseSameFont ) {

            if ( IsUseSameFont ) CheckDlgButton ( hWnd, IDC_CHECK_SAMEFONT, BST_CHECKED ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, ! IsDlgButtonChecked ( hWnd, IDC_CHECK_SAMEFONT ) ) ;

            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_CJKLANG, nUniDefaultCodeTmp ) ;
            nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;
         }
         else {
            if ( ! IsMultiByteCodePage ( nCodePage ) && nCodePage != CP_THAI && ! IsUseSameFont ) Dlg_EnableWindow ( hWnd, IDC_COMBO_MULTIBYTE_FONT, FALSE ) ;

            if ( IsUseSameFont ) CheckDlgButton ( hWnd, IDC_CHECK_SAMEFONT, BST_CHECKED ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, ! IsDlgButtonChecked ( hWnd, IDC_CHECK_SAMEFONT ) ) ;
            Dlg_EnableWindow ( hWnd, IDC_COMBO_CJKLANG, FALSE ) ;
         }

         // tHg쐬
         if ( IsNT () ) {
            hAsciiFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_3 ), 0 ) ;
            if ( hAsciiFont ) Dlg_Wm_SetFont ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, hAsciiFont, TRUE ) ;
            if ( hAsciiFont ) Dlg_Wm_SetFont ( hWnd, IDC_COMBO_MULTIBYTE_FONT, hAsciiFont, TRUE ) ;
            SetComboBoxFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, CP_ANSI_EUROPEAN ) ;
            SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
            SetOptionDialogFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
         }
         else {
            hAsciiFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_3 ), 0 ) ;
            hMultiByteFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_3 ), nCodePage ) ;
            if ( hAsciiFont ) Dlg_Wm_SetFont ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, hAsciiFont, TRUE ) ;
            if ( hMultiByteFont ) Dlg_Wm_SetFont ( hWnd, IDC_COMBO_MULTIBYTE_FONT, hMultiByteFont, TRUE ) ;
            SetComboBoxFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, CP_ANSI_EUROPEAN ) ;
            SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
            SetOptionDialogFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
         }

         return TRUE ;


      case WM_COMMAND :

         // W֖߂iPoCgj
         if ( LOWORD ( wParam ) == IDC_SINGLEBYTE_FONT_DEFAULT ) {
            if ( ! Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_SINGLEBYTE_FONT ) ) return TRUE ;
            strcpy ( szAsciiFontTmp, szDefaultAsciiFont ) ;
            SetOptionDialogFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, 0, 0, 0 ) ;
            return TRUE ;
         }

         // W֖߂iQoCgj
         if ( LOWORD ( wParam ) == IDC_MULTIBYTE_FONT_DEFAULT ) {
            if ( ! Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_MULTIBYTE_FONT ) ) return TRUE ;

            nCodePage = GetWinCodePage ( GetModelessCodePage ( ( AvailableCodeList [ nSrcCode ] ).nCodePage ) ) ;
            if ( Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_CJKLANG ) ) nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;

            if ( nCodePage == CP_JAPANESE )            strcpy ( szJapaneseFontTmp, szDefaultJapaneseFont ) ;
            if ( nCodePage == CP_KOREAN )              strcpy ( szKoreanFontTmp, szDefaultKoreanFont ) ;
            if ( nCodePage == CP_CHINESE_SIMPLIFIED )  strcpy ( szGbFontTmp, szDefaultChineseSimplifiedFont ) ;
            if ( nCodePage == CP_CHINESE_TRADITIONAL ) strcpy ( szBig5FontTmp, szDefaultChineseTraditionalFont ) ;
            if ( nCodePage == CP_THAI )                strcpy ( szThaiFontTmp, szDefaultThaiFont ) ;

            SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, 0, nCodePage ) ;
            return TRUE ;
         }

         // W֖߂iCJKLANGj
         if ( LOWORD ( wParam ) == IDC_CJKLANG_DEFAULT ) {
            if ( ! Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_CJKLANG ) ) return TRUE ;

            nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;
            GetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;

            for ( int nCount = 0 ; nCount < MULTIBYTE_CODE_LIST_NELEM ; nCount ++ ) {
               if ( CP_JAPANESE == ( MultiByteCodeList [ nCount ] ).nCodePage ) nUniDefaultCodeTmp = nCount ;
               if ( nOsCodePage == ( MultiByteCodeList [ nCount ] ).nCodePage ) { nUniDefaultCodeTmp = nCount ; break ; }
            }
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_CJKLANG, nUniDefaultCodeTmp ) ;
            nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;

            if ( IsNT () ) {
               SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
               SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
            }
            else {
               Dlg_Wm_SetTextA ( hWnd, IDC_COMBO_MULTIBYTE_FONT, "" ) ;
               HFONT hNewFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_3 ), nCodePage ) ;
               if ( hNewFont ) {
                  Dlg_Wm_SetFont ( hWnd, IDC_COMBO_MULTIBYTE_FONT, hNewFont, TRUE ) ;
                  if ( hMultiByteFont ) DeleteObject ( hMultiByteFont ) ;
                  hMultiByteFont = hNewFont ;
               }
               SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
               SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
            }

            return TRUE ;
         }

         // W֖߂itHgTCYj
         if ( LOWORD ( wParam ) == IDC_FONTSIZE_DEFAULT ) {
            if ( ! Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_FONTSIZE ) ) return TRUE ;
            SetComboBoxFontSizeValue ( hWnd, IDC_COMBO_FONTSIZE, DEFAULT_FONT_SIZE ) ;
            return TRUE ;
         }

         // xȐݒ
         if ( LOWORD ( wParam ) == IDC_BUTTON_FONT_EX ) {
            nDefaultMultiByteCodePageSub = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;
            if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_FONT_EX, hWnd, FontExDialogProc ) ;
            else           DialogBoxExA ( IDD_DIALOG_FONT_EX, hWnd, FontExDialogProc ) ;
            return TRUE ;
         }


         if ( LOWORD ( wParam ) == IDC_COMBO_CJKLANG ) {
            // R{{bNX̑IύXƂ
            if ( HIWORD ( wParam ) == CBN_SELCHANGE ) {
               nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;
               GetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;

               nUniDefaultCodeTmp = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_CJKLANG ) ;
               nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;

               if ( IsNT () ) {
                  SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
                  SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
               }
               else {
                  Dlg_Wm_SetTextA ( hWnd, IDC_COMBO_MULTIBYTE_FONT, "" ) ;
                  HFONT hNewFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_3 ), nCodePage ) ;
                  if ( hNewFont ) {
                     Dlg_Wm_SetFont ( hWnd, IDC_COMBO_MULTIBYTE_FONT, hNewFont, TRUE ) ;
                     if ( hMultiByteFont ) DeleteObject ( hMultiByteFont ) ;
                     hMultiByteFont = hNewFont ;
                  }
                  SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
                  SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
               }
               return TRUE ;
            }
         }

         // tHg̎gpw肵Ƃ
         if ( LOWORD ( wParam ) == IDC_CHECK_SAMEFONT ) {

            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_SAMEFONT ) ) {

               nCodePage = GetWinCodePage ( GetModelessCodePage ( ( AvailableCodeList [ nSrcCode ] ).nCodePage ) ) ;

               GetOptionDialogFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, 0, nCodePage ) ;
               Dlg_EnableWindow ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, FALSE ) ;
               if ( IsNT () ) Dlg_Wm_SetTextW ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, L"" ) ;
               else           Dlg_Wm_SetTextA ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, "" ) ;

               if ( nCodePage != CP_UTF16 && ! IsMultiByteCodePage ( nCodePage ) && nCodePage != CP_THAI ) {

                  Dlg_EnableWindow ( hWnd, IDC_COMBO_CJKLANG, TRUE ) ;
                  Dlg_EnableWindow ( hWnd, IDC_COMBO_MULTIBYTE_FONT, TRUE ) ;

                  Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_CJKLANG, nUniDefaultCodeTmp ) ;
                  nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;

                  if ( IsNT () ) {
                     SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
                     SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
                  }
                  else {
                     Dlg_Wm_SetTextA ( hWnd, IDC_COMBO_MULTIBYTE_FONT, "" ) ;
                     HFONT hNewFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_3 ), nCodePage ) ;
                     if ( hNewFont ) {
                        Dlg_Wm_SetFont ( hWnd, IDC_COMBO_MULTIBYTE_FONT, hNewFont, TRUE ) ;
                        if ( hMultiByteFont ) DeleteObject ( hMultiByteFont ) ;
                        hMultiByteFont = hNewFont ;
                     }
                     SetComboBoxFontName ( hWnd, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;
                     SetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, IDC_MULTIBYTE_GUIDE, nCodePage ) ;
                  }
               }
            }
            else {

               nCodePage = GetWinCodePage ( GetModelessCodePage ( ( AvailableCodeList [ nSrcCode ] ).nCodePage ) ) ;

               Dlg_EnableWindow ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, TRUE ) ;
               SetOptionDialogFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, 0, 0, nCodePage ) ;

               if ( nCodePage != CP_UTF16 && ! IsMultiByteCodePage ( nCodePage ) && nCodePage != CP_THAI ) {

                  nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;
                  GetOptionDialogFontName ( hWnd, 0, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;

                  Dlg_EnableWindow ( hWnd, IDC_COMBO_MULTIBYTE_FONT, FALSE ) ;
                  if ( IsNT () ) Dlg_Wm_SetTextW ( hWnd, IDC_COMBO_MULTIBYTE_FONT, L"" ) ;
                  else           Dlg_Wm_SetTextA ( hWnd, IDC_COMBO_MULTIBYTE_FONT, "" ) ;
                  Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_CJKLANG, -1 ) ;
                  Dlg_EnableWindow ( hWnd, IDC_COMBO_CJKLANG, FALSE ) ;
               }
            }
            return TRUE ;
         }

         break ;


      case WM_NOTIFY :

         // ݒ
         if ( ((NMHDR*) lParam )->code == PSN_APPLY ) {

            if ( Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_FONTSIZE ) ) nFontSize = GetComboBoxFontSizeValue ( hWnd, IDC_COMBO_FONTSIZE ) ;
            if ( Dlg_IsWindowEnabled ( hWnd, IDC_CHECK_SAMEFONT ) ) IsUseSameFont = IsDlgButtonChecked ( hWnd, IDC_CHECK_SAMEFONT ) ;
            if ( Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_CJKLANG ) ) nUniDefaultCodeTmp = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_CJKLANG ) ;

            // tHg̎擾
            nCodePage = GetWinCodePage ( GetModelessCodePage ( ( AvailableCodeList [ nSrcCode ] ).nCodePage ) ) ;
            if ( Dlg_IsWindowEnabled ( hWnd, IDC_COMBO_CJKLANG ) ) nCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;
            GetOptionDialogFontName ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, IDC_COMBO_MULTIBYTE_FONT, nCodePage ) ;

            strcpy ( szEuropeanFont, szAsciiFontTmp ) ;
            strcpy ( szJapaneseFont, szJapaneseFontTmp ) ;
            strcpy ( szKoreanFont, szKoreanFontTmp ) ;
            strcpy ( szChineseSimplifiedFont, szGbFontTmp ) ;
            strcpy ( szChineseTraditionalFont, szBig5FontTmp ) ;
            strcpy ( szThaiFont, szThaiFontTmp ) ;
            if ( ! *szEuropeanFont )            strcpy ( szEuropeanFont, szDefaultAsciiFont ) ;
            if ( ! *szJapaneseFont )            strcpy ( szJapaneseFont, szDefaultJapaneseFont ) ;
            if ( ! *szKoreanFont )              strcpy ( szKoreanFont, szDefaultKoreanFont ) ;
            if ( ! *szChineseSimplifiedFont )   strcpy ( szChineseSimplifiedFont, szDefaultChineseSimplifiedFont ) ;
            if ( ! *szChineseTraditionalFont )  strcpy ( szChineseTraditionalFont, szDefaultChineseTraditionalFont ) ;
            if ( ! *szThaiFont )                strcpy ( szThaiFont, szDefaultThaiFont ) ;
            nDefaultCjkCodePage = ( MultiByteCodeList [ nUniDefaultCodeTmp ] ).nCodePage ;

            IsOptionChanged = 1 ;
            return TRUE ;
         }

         break ;


      case WM_DESTROY :
         if ( hAsciiFont ) {
            DeleteObject ( hAsciiFont ) ;
            hAsciiFont = NULL ;
         }
         if ( hMultiByteFont ) {
            DeleteObject ( hMultiByteFont ) ;
            hMultiByteFont = NULL ;
         }
         break ;


      default :
         break ;
   }

   return FALSE ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog3InvalidProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :
         return TRUE ;

      default :
         break ;
   }

   return FALSE ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog4Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsOutputLF && IsOutputCR ) CheckDlgButton ( hWnd, IDC_CHECK_CRLF, BST_CHECKED ) ;
         else if ( IsOutputLF )          CheckDlgButton ( hWnd, IDC_CHECK_LF, BST_CHECKED ) ;
         else if ( IsOutputCR )          CheckDlgButton ( hWnd, IDC_CHECK_CR, BST_CHECKED ) ;

         if ( IsDeleteEof ) CheckDlgButton ( hWnd, IDC_CHECK_DELETE_EOF, BST_CHECKED ) ;

         return TRUE ;


      case WM_NOTIFY :

         // ݒ
         if ( ((NMHDR*) lParam )->code == PSN_APPLY ) {
            IsOutputLF = IsOutputCR = 0 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_CRLF ) ) IsOutputLF = IsOutputCR = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_LF ) ) IsOutputLF = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_CR ) ) IsOutputCR = 1 ;
            IsDeleteEof = IsDlgButtonChecked ( hWnd, IDC_CHECK_DELETE_EOF ) ;
            IsOptionChanged = 1 ;
            return TRUE ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog5Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( ! IsUseArray && ! IsNotUseArray ) CheckDlgButton ( hWnd, IDC_CHECK_TABLE_AUTO, BST_CHECKED ) ;
         else if ( IsNotUseArray )              CheckDlgButton ( hWnd, IDC_CHECK_TABLE_SYSTEM, BST_CHECKED ) ;
         else if ( IsUseArray )                 CheckDlgButton ( hWnd, IDC_CHECK_TABLE_ARRAY, BST_CHECKED ) ;

         if ( IsConvertPua ) CheckDlgButton ( hWnd, IDC_CHECK_CONVERT_PUA, BST_CHECKED ) ;

         return TRUE ;


      case WM_COMMAND :

         // xȐݒ
         if ( LOWORD ( wParam ) == IDC_BUTTON_EXMODE_JP ) {
            if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_EXMODE_JP, hWnd, ExConvModeJpDialogProc ) ;
            else           DialogBoxExA ( IDD_DIALOG_EXMODE_JP, hWnd, ExConvModeJpDialogProc ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDC_BUTTON_EXMODE_CN_S ) {
            if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_EXMODE_CN_S, hWnd, ExConvModeCnsDialogProc ) ;
            else           DialogBoxExA ( IDD_DIALOG_EXMODE_CN_S, hWnd, ExConvModeCnsDialogProc ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDC_BUTTON_EXMODE_CN_T ) {
            if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_EXMODE_CN_T, hWnd, ExConvModeCntDialogProc ) ;
            else           DialogBoxExA ( IDD_DIALOG_EXMODE_CN_T, hWnd, ExConvModeCntDialogProc ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDC_BUTTON_EXMODE_KS ) {
            if ( IsNT () ) DialogBoxExW ( IDD_DIALOG_EXMODE_KS, hWnd, ExConvModeKsDialogProc ) ;
            else           DialogBoxExA ( IDD_DIALOG_EXMODE_KS, hWnd, ExConvModeKsDialogProc ) ;
            return TRUE ;
         }
         break ;


      case WM_NOTIFY :

         // ݒ
         if ( ((NMHDR*) lParam )->code == PSN_APPLY ) {
            IsUseArray = IsNotUseArray = 0 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_TABLE_AUTO ) ) IsUseArray = IsNotUseArray = 0 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_TABLE_SYSTEM ) ) IsNotUseArray = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_TABLE_ARRAY ) ) IsUseArray = 1 ;
            IsConvertPua = IsDlgButtonChecked ( hWnd, IDC_CHECK_CONVERT_PUA ) ;
            IsOptionChanged = 1 ;
            return TRUE ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// IvVݒ_CAO
INT_PTR CALLBACK OptionDialog6Proc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   static HFONT hFont ;

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsSaveCode ) CheckDlgButton ( hWnd, IDC_CHECK_SAVETYPE, BST_CHECKED ) ;
         if ( IsSavePath ) CheckDlgButton ( hWnd, IDC_CHECK_SAVEPATH, BST_CHECKED ) ;
         if ( IsSaveWindow ) CheckDlgButton ( hWnd, IDC_CHECK_SAVEWINDOW, BST_CHECKED ) ;
         if ( IsNoConfirm ) CheckDlgButton ( hWnd, IDC_CHECK_NO_CONFIRM, BST_CHECKED ) ;
         if ( IsNoRecycle ) CheckDlgButton ( hWnd, IDC_CHECK_NO_RECYCLE, BST_CHECKED ) ;
         if ( IsNoMessage) CheckDlgButton ( hWnd, IDC_CHECK_NO_MESSAGE, BST_CHECKED ) ;
         if ( IsNoAddExtension ) CheckDlgButton ( hWnd, IDC_CHECK_NO_ADD_EXT, BST_CHECKED ) ;
         if ( IsConfirmOnExit ) CheckDlgButton ( hWnd, IDC_CHECK_CONFIRM_ON_EXIT, BST_CHECKED ) ;

         // tHg쐬
         hFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_PROPPAGE_6 ), 0 ) ;
         if ( hFont ) Dlg_Wm_SetFont ( hWnd, IDC_COMBO_MUI_LANG, hFont, TRUE ) ;

         SetComboBoxMuiLangId ( hWnd, IDC_COMBO_MUI_LANG ) ;

         return TRUE ;


      case WM_NOTIFY :

         // ݒ
         if ( ((NMHDR*) lParam )->code == PSN_APPLY ) {
            IsSaveCode = IsDlgButtonChecked ( hWnd, IDC_CHECK_SAVETYPE ) ;
            IsSavePath = IsDlgButtonChecked ( hWnd, IDC_CHECK_SAVEPATH ) ;
            IsSaveWindow = IsDlgButtonChecked ( hWnd, IDC_CHECK_SAVEWINDOW ) ;
            IsNoConfirm = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_CONFIRM ) ;
            IsNoRecycle = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_RECYCLE ) ;
            IsNoMessage = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_MESSAGE ) ;
            IsNoAddExtension = IsDlgButtonChecked ( hWnd, IDC_CHECK_NO_ADD_EXT ) ;
            IsConfirmOnExit = IsDlgButtonChecked ( hWnd, IDC_CHECK_CONFIRM_ON_EXIT ) ;
            nSavedMuiLangId = GetComboBoxMuiLangId ( hWnd, IDC_COMBO_MUI_LANG ) ;
            IsOptionChanged = 1 ;
            return TRUE ;
         }

         break ;


      case WM_DESTROY :
         if ( hFont ) {
            DeleteObject ( hFont ) ;
            hFont = NULL ;
         }
         FreeMuiLangIdArray () ;
         break ;


      default :
         break ;
   }

   return FALSE ;
}



////////////////////////////////////////////
//       IvVݒitHgj       //
////////////////////////////////////////////



// _CAÕtHg擾
int GetOptionDialogFontName ( HWND hWnd, int nSingleByteComboBox, int nMultiByteComboBox, int nCodePage ) {

   // PoCgtHg̎擾
   if ( nSingleByteComboBox && Dlg_IsWindowEnabled ( hWnd, nSingleByteComboBox ) ) {
      if ( IsNT () ) {
         wchar_t szBuffer [ FONTNAME_MAX ] ;
         int nCodePageTmp = CP_ANSI_EUROPEAN ;
         Dlg_Wm_GetTextW ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, szBuffer, FONTNAME_MAX ) ;
         w2alcpy_cp ( szAsciiFontTmp, szBuffer, FONTNAME_MAX, nCodePageTmp ) ;
      }
      else {
         Dlg_Wm_GetTextA ( hWnd, IDC_COMBO_SINGLEBYTE_FONT, szAsciiFontTmp, FONTNAME_MAX ) ;
      }
   }

   // QoCgtHg̎擾
   if ( nMultiByteComboBox && Dlg_IsWindowEnabled ( hWnd, nMultiByteComboBox ) ) {

      char *szFontTmp = NULL ;
      if ( nCodePage == CP_JAPANESE )                 szFontTmp = szJapaneseFontTmp ;
      else if ( nCodePage == CP_KOREAN )              szFontTmp = szKoreanFontTmp ;
      else if ( nCodePage == CP_CHINESE_SIMPLIFIED )  szFontTmp = szGbFontTmp ;
      else if ( nCodePage == CP_CHINESE_TRADITIONAL ) szFontTmp = szBig5FontTmp ;
      else if ( nCodePage == CP_THAI )                szFontTmp = szThaiFontTmp ;
      if ( ! szFontTmp ) return 1 ;

      if ( IsNT () ) {
         wchar_t szBuffer [ FONTNAME_MAX ] ;
         Dlg_Wm_GetTextW ( hWnd, IDC_COMBO_MULTIBYTE_FONT, szBuffer, FONTNAME_MAX ) ;
         int nCodePageTmp = CP_ANSI_EUROPEAN ;
         if ( IsMultiByteCodePage ( nCodePage ) && IsValidCodePage ( nCodePage ) ) nCodePageTmp = nCodePage ;
         w2alcpy_cp ( szFontTmp, szBuffer, FONTNAME_MAX, nCodePageTmp ) ;
      }
      else {
         Dlg_Wm_GetTextA ( hWnd, IDC_COMBO_MULTIBYTE_FONT, szFontTmp, FONTNAME_MAX ) ;
      }
   }

   return 0 ;
}



// _CAÕtHgуKChύX
int SetOptionDialogFontName ( HWND hWnd, int nSingleByteComboBox, int nMultiByteComboBox, int nIdDlgItem, int nCodePage ) {

   // QoCgtHg̃KCh̎w
   if ( nIdDlgItem ) {
      int nStringId = IDS_GUIDE_NOTSPECIFIED ;
      if ( nCodePage == CP_JAPANESE )                 nStringId = IDS_GUIDE_JAPANESE ;
      else if ( nCodePage == CP_KOREAN )              nStringId = IDS_GUIDE_KOREAN ;
      else if ( nCodePage == CP_CHINESE_SIMPLIFIED )  nStringId = IDS_GUIDE_CHINESE_SIMPLIFIED ;
      else if ( nCodePage == CP_CHINESE_TRADITIONAL ) nStringId = IDS_GUIDE_CHINESE_TRADITIONAL ;
      else if ( nCodePage == CP_THAI )                nStringId = IDS_GUIDE_THAI ;

      if ( IsNT () ) Dlg_Wm_SetTextW ( hWnd, nIdDlgItem, GetStringW ( nStringId ) ) ;
      else           Dlg_Wm_SetTextA ( hWnd, nIdDlgItem, GetStringA ( nStringId ) ) ;
   }

   // PoCgtHg̎w
   if ( nSingleByteComboBox && Dlg_IsWindowEnabled ( hWnd, nSingleByteComboBox ) ) {
      if ( IsNT () ) {
         wchar_t szBuffer [ FONTNAME_MAX ] ;
         a2wlcpy_cp ( szBuffer, szAsciiFontTmp, FONTNAME_MAX, CP_ANSI_EUROPEAN ) ;
         Dlg_Wm_SetTextW ( hWnd, nSingleByteComboBox, szBuffer ) ;
      }
      else {
         Dlg_Wm_SetTextA ( hWnd, nSingleByteComboBox, szAsciiFontTmp ) ;
      }
   }

   // QoCgtHg̎w
   if ( nMultiByteComboBox && Dlg_IsWindowEnabled ( hWnd, nMultiByteComboBox ) ) {

      char *szFontTmp = NULL ;
      if ( nCodePage == CP_JAPANESE )                 szFontTmp = szJapaneseFontTmp ;
      else if ( nCodePage == CP_KOREAN )              szFontTmp = szKoreanFontTmp ;
      else if ( nCodePage == CP_CHINESE_SIMPLIFIED )  szFontTmp = szGbFontTmp ;
      else if ( nCodePage == CP_CHINESE_TRADITIONAL ) szFontTmp = szBig5FontTmp ;
      else if ( nCodePage == CP_THAI )                szFontTmp = szThaiFontTmp ;
      if ( ! szFontTmp ) return 1 ;

      if ( IsNT () ) {
         wchar_t szBuffer [ FONTNAME_MAX ] ;
         int nCodePageTmp = CP_ANSI_EUROPEAN ;
         if ( IsMultiByteCodePage ( nCodePage ) && IsValidCodePage ( nCodePage ) ) nCodePageTmp = nCodePage ;
         a2wlcpy_cp ( szBuffer, szFontTmp, FONTNAME_MAX, nCodePageTmp ) ;
         Dlg_Wm_SetTextW ( hWnd, nMultiByteComboBox, szBuffer ) ;
      }
      else {
         Dlg_Wm_SetTextA ( hWnd, nMultiByteComboBox, szFontTmp ) ;
      }
   }

   return 0 ;
}



int CALLBACK SetComboBoxFontFamilyProcA ( const LOGFONTA *lplfe, const TEXTMETRICA *lptme, unsigned long nFontType, LPARAM lParam ) ;
int CALLBACK SetComboBoxFontFamilyProcW ( const LOGFONTW *lplfe, const TEXTMETRICW *lptme, unsigned long nFontType, LPARAM lParam ) ;
int CALLBACK SetComboBoxFontNameProcA ( const LOGFONTA *lplfe, const TEXTMETRICA *lptme, unsigned long nFontType, LPARAM lParam ) ;
int CALLBACK SetComboBoxFontNameProcW ( const LOGFONTW *lplfe, const TEXTMETRICW *lptme, unsigned long nFontType, LPARAM lParam ) ;

typedef struct {
   HWND hComboBox ;
   HDC hDC ;
   int nCharset ;
   int IsFontSjis ;
   int IsFontGb ;
   int IsFontBig5 ;
   int IsFontKs ;
} SEARCHFONTNAMEINFO ;



// tHgR{{bNXɃtHgW߂
// nCodePage  -1 ȂSẴtHgW߂
int SetComboBoxFontName ( HWND hWnd, int nComboBox, int nCodePage ) {

   Dlg_ComboBox_ResetContent ( hWnd, nComboBox ) ;

   SEARCHFONTNAMEINFO SearchFontNameInfo = { 0 } ;
   SearchFontNameInfo.hComboBox = GetDlgItem ( hWnd, nComboBox ) ;
   if ( nCodePage < 0 ) SearchFontNameInfo.nCharset = DEFAULT_CHARSET ;
   else                 SearchFontNameInfo.nCharset = CodePageToCharset ( nCodePage ) ;
   SearchFontNameInfo.hDC = CreateCompatibleDC ( NULL ) ;

   if ( IsNT () ) {
      LOGFONTW LogFont = { 0 } ;
      LogFont.lfCharSet = SearchFontNameInfo.nCharset ;
      EnumFontFamiliesExW ( SearchFontNameInfo.hDC, & LogFont, SetComboBoxFontFamilyProcW, (LPARAM) & SearchFontNameInfo, 0 ) ;
   }
   else {
      LOGFONTA LogFont = { 0 } ;
      LogFont.lfCharSet = SearchFontNameInfo.nCharset ;
      EnumFontFamiliesExA ( SearchFontNameInfo.hDC, & LogFont, SetComboBoxFontFamilyProcA, (LPARAM) & SearchFontNameInfo, 0 ) ;
   }

   DeleteDC ( SearchFontNameInfo.hDC ) ;

   Dlg_ComboBox_SetMaxHeight ( hWnd, nComboBox, 16 ) ;

   return 0 ;
}



// R{{bNXɃtHgW߂vV[W (1)iANSIŁj
int CALLBACK SetComboBoxFontFamilyProcA ( const LOGFONTA *lplfe, const TEXTMETRICA *lptme, unsigned long nFontType, LPARAM lParam ) {

   SEARCHFONTNAMEINFO *pSearchFontNameInfo = (SEARCHFONTNAMEINFO*)lParam ;
   int IsSuitableFont = 0 ;

   if ( ! ( nFontType & TRUETYPE_FONTTYPE ) ) return TRUE ;
   if ( *( lplfe->lfFaceName ) == '@' ) return TRUE ;

   // SẴtHgW߂Ȃ
   if ( pSearchFontNameInfo->nCharset == DEFAULT_CHARSET ) IsSuitableFont = 1 ;
   // ̕Zbg̃tHgW߂Ȃ
   else {

      LOGFONTA LogFont = { 0 } ;
      strcpy ( LogFont.lfFaceName, lplfe->lfFaceName ) ;
      LogFont.lfCharSet = DEFAULT_CHARSET ;

      pSearchFontNameInfo->IsFontSjis = 0 ;
      pSearchFontNameInfo->IsFontGb = 0 ;
      pSearchFontNameInfo->IsFontBig5 = 0 ;
      pSearchFontNameInfo->IsFontKs = 0 ;

      // tHg̏W
      EnumFontFamiliesExA ( pSearchFontNameInfo->hDC, & LogFont, SetComboBoxFontNameProcA, (LPARAM) pSearchFontNameInfo, 0 ) ;

      // QoCgptHgW߂Ȃ
      if ( IsMultiByteCodePage ( CharsetToCodePage ( pSearchFontNameInfo->nCharset ) ) ) {
         if ( pSearchFontNameInfo->nCharset == SHIFTJIS_CHARSET    && pSearchFontNameInfo->IsFontSjis ) IsSuitableFont = 1 ;
         if ( pSearchFontNameInfo->nCharset == GB2312_CHARSET      && pSearchFontNameInfo->IsFontGb )   IsSuitableFont = 1 ;
         if ( pSearchFontNameInfo->nCharset == CHINESEBIG5_CHARSET && pSearchFontNameInfo->IsFontBig5 ) IsSuitableFont = 1 ;
         if ( pSearchFontNameInfo->nCharset == HANGUL_CHARSET      && pSearchFontNameInfo->IsFontKs )   IsSuitableFont = 1 ;
      }
      // PoCgptHgW߂Ȃ
      else {
         int nAddResult = pSearchFontNameInfo->IsFontSjis + pSearchFontNameInfo->IsFontGb + pSearchFontNameInfo->IsFontBig5 + pSearchFontNameInfo->IsFontKs ;
         if ( nAddResult == 0 || nAddResult == 4 ) IsSuitableFont = 1 ;
      }
   }

   if ( IsSuitableFont && ComboBox_FindStringExactA ( pSearchFontNameInfo->hComboBox, 0, lplfe->lfFaceName ) == CB_ERR ) {
      ComboBox_AddStringA ( pSearchFontNameInfo->hComboBox, lplfe->lfFaceName ) ;
   }

   return TRUE ;
}



// R{{bNXɃtHgW߂vV[W (1)iUNICODEŁj
int CALLBACK SetComboBoxFontFamilyProcW ( const LOGFONTW *lplfe, const TEXTMETRICW *lptme, unsigned long nFontType, LPARAM lParam ) {

   SEARCHFONTNAMEINFO *pSearchFontNameInfo = (SEARCHFONTNAMEINFO*)lParam ;
   int IsSuitableFont = 0 ;

   if ( ! ( nFontType & TRUETYPE_FONTTYPE ) ) return TRUE ;
   if ( *( lplfe->lfFaceName ) == '@' ) return TRUE ;

   // SẴtHgW߂Ȃ
   if ( pSearchFontNameInfo->nCharset == DEFAULT_CHARSET ) IsSuitableFont = 1 ;
   // ̕Zbg̃tHgW߂Ȃ
   else {

      LOGFONTW LogFont = { 0 } ;
      wcscpy ( LogFont.lfFaceName, lplfe->lfFaceName ) ;
      LogFont.lfCharSet = DEFAULT_CHARSET ;

      pSearchFontNameInfo->IsFontSjis = 0 ;
      pSearchFontNameInfo->IsFontGb = 0 ;
      pSearchFontNameInfo->IsFontBig5 = 0 ;
      pSearchFontNameInfo->IsFontKs = 0 ;

      // tHg̏W
      EnumFontFamiliesExW ( pSearchFontNameInfo->hDC, & LogFont, SetComboBoxFontNameProcW, (LPARAM) pSearchFontNameInfo, 0 ) ;

      // QoCgptHgW߂Ȃ
      if ( IsMultiByteCodePage ( CharsetToCodePage ( pSearchFontNameInfo->nCharset ) ) ) {
         if ( pSearchFontNameInfo->nCharset == SHIFTJIS_CHARSET    && pSearchFontNameInfo->IsFontSjis ) IsSuitableFont = 1 ;
         if ( pSearchFontNameInfo->nCharset == GB2312_CHARSET      && pSearchFontNameInfo->IsFontGb )   IsSuitableFont = 1 ;
         if ( pSearchFontNameInfo->nCharset == CHINESEBIG5_CHARSET && pSearchFontNameInfo->IsFontBig5 ) IsSuitableFont = 1 ;
         if ( pSearchFontNameInfo->nCharset == HANGUL_CHARSET      && pSearchFontNameInfo->IsFontKs )   IsSuitableFont = 1 ;
      }
      // PoCgptHgW߂Ȃ
      else {
         int nAddResult = pSearchFontNameInfo->IsFontSjis + pSearchFontNameInfo->IsFontGb + pSearchFontNameInfo->IsFontBig5 + pSearchFontNameInfo->IsFontKs ;
         if ( nAddResult == 0 || nAddResult == 4 ) IsSuitableFont = 1 ;
      }
   }

   if ( IsSuitableFont && ComboBox_FindStringExactW ( pSearchFontNameInfo->hComboBox, 0, lplfe->lfFaceName ) == CB_ERR ) {
      ComboBox_AddStringW ( pSearchFontNameInfo->hComboBox, lplfe->lfFaceName ) ;
   }

   return TRUE ;
}



// R{{bNXɃtHgW߂vV[W (2)iANSIŁj
int CALLBACK SetComboBoxFontNameProcA ( const LOGFONTA *lplfe, const TEXTMETRICA *lptme, unsigned long nFontType, LPARAM lParam ) {

   SEARCHFONTNAMEINFO *pSearchFontNameInfo = (SEARCHFONTNAMEINFO*)lParam ;

   if ( lptme->tmCharSet == SHIFTJIS_CHARSET )    pSearchFontNameInfo->IsFontSjis = 1 ;
   if ( lptme->tmCharSet == GB2312_CHARSET )      pSearchFontNameInfo->IsFontGb = 1 ;
   if ( lptme->tmCharSet == CHINESEBIG5_CHARSET ) pSearchFontNameInfo->IsFontBig5 = 1 ;
   if ( lptme->tmCharSet == HANGUL_CHARSET )      pSearchFontNameInfo->IsFontKs = 1 ;

   return TRUE ;
}



// R{{bNXɃtHgW߂vV[W (2)iUNICODEŁj
int CALLBACK SetComboBoxFontNameProcW ( const LOGFONTW *lplfe, const TEXTMETRICW *lptme, unsigned long nFontType, LPARAM lParam ) {

   SEARCHFONTNAMEINFO *pSearchFontNameInfo = (SEARCHFONTNAMEINFO*)lParam ;

   if ( lptme->tmCharSet == SHIFTJIS_CHARSET )    pSearchFontNameInfo->IsFontSjis = 1 ;
   if ( lptme->tmCharSet == GB2312_CHARSET )      pSearchFontNameInfo->IsFontGb = 1 ;
   if ( lptme->tmCharSet == CHINESEBIG5_CHARSET ) pSearchFontNameInfo->IsFontBig5 = 1 ;
   if ( lptme->tmCharSet == HANGUL_CHARSET )      pSearchFontNameInfo->IsFontKs = 1 ;

   return TRUE ;
}



// tHgTCYR{{bNXZbg
int SetComboBoxFontSize ( HWND hWnd, int nComboBox, int nDefaultFontSize ) {

   if ( IsNT () ) {

      wchar_t szBuffer [ FONTSIZE_BUFFER_MAX ] ;

      for ( int nCount = 50 ; nCount >= 10 ; nCount -- ) {
         if ( ( nCount > 26 || nCount < 18 ) && nCount % 2 ) continue ;
         swprintf ( szBuffer, L"%d%s", nCount / 2, ( nCount % 2 ) ? L".5" : L"" ) ;
         Dlg_ComboBox_InsertStringW ( hWnd, nComboBox, 0, szBuffer ) ;
      }

      swprintf ( szBuffer, L"%d%s", nDefaultFontSize / 2, ( nDefaultFontSize % 2 ) ? L".5" : L"" ) ;
      Dlg_Wm_SetTextW ( hWnd, nComboBox, szBuffer ) ;
   }
   else {

      char szBuffer [ FONTSIZE_BUFFER_MAX ] ;

      for ( int nCount = 50 ; nCount >= 10 ; nCount -- ) {
         if ( ( nCount > 26 || nCount < 18 ) && nCount % 2 ) continue ;
         sprintf ( szBuffer, "%d%s", nCount / 2, ( nCount % 2 ) ? ".5" : "" ) ;
         Dlg_ComboBox_InsertStringA ( hWnd, nComboBox, 0, szBuffer ) ;
      }

      sprintf ( szBuffer, "%d%s", nDefaultFontSize / 2, ( nDefaultFontSize % 2 ) ? ".5" : "" ) ;
      Dlg_Wm_SetTextA ( hWnd, nComboBox, szBuffer ) ;
   }

   Dlg_ComboBox_SetMaxHeight ( hWnd, nComboBox, 8 ) ;

   return 0 ;
}



// tHgTCYR{{bNXl擾
int GetComboBoxFontSizeValue ( HWND hWnd, int nComboBox ) {

   int nSize = DEFAULT_FONT_SIZE ;

   if ( IsNT () ) {
      wchar_t szBuffer [ FONTSIZE_BUFFER_MAX ] ;
      Dlg_Wm_GetTextW ( hWnd, nComboBox, szBuffer, FONTSIZE_BUFFER_MAX ) ;
      if ( isascii ( *szBuffer ) && isdigit ( *szBuffer ) ) {
         wchar_t *szString ;
         nSize = wcstoul ( szBuffer, & szString, 10 ) * 2 ;
         if ( szString && ! wcscmp ( szString, L".5" ) ) nSize ++ ;
      }
   }
   else {
      char szBuffer [ FONTSIZE_BUFFER_MAX ] ;
      Dlg_Wm_GetTextA ( hWnd, nComboBox, szBuffer, FONTSIZE_BUFFER_MAX ) ;
      if ( isascii ( *szBuffer ) && isdigit ( *szBuffer ) ) {
         char *szString ;
         nSize = strtoul ( szBuffer, & szString, 10 ) * 2 ;
         if ( szString && ! strcmp ( szString, ".5" ) ) nSize ++ ;
      }
   }

   return nSize ;
}



// tHgTCYR{{bNXɒlZbg
int SetComboBoxFontSizeValue ( HWND hWnd, int nComboBox, int nFontSize ) {

   if ( IsNT () ) {
      wchar_t szBuffer [ FONTSIZE_BUFFER_MAX ] ;
      swprintf ( szBuffer, L"%d%s", nFontSize / 2, ( nFontSize % 2 ) ? L".5" : L"" ) ;
      Dlg_Wm_SetTextW ( hWnd, nComboBox, szBuffer ) ;
   }
   else {
      char szBuffer [ FONTSIZE_BUFFER_MAX ] ;
      sprintf ( szBuffer, "%d%s", nFontSize / 2, ( nFontSize % 2 ) ? ".5" : "" ) ;
      Dlg_Wm_SetTextA ( hWnd, nComboBox, szBuffer ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//       IvVݒi\j       //
////////////////////////////////////////////



int SetComboBoxMuiLangIdData ( HWND hWnd, int nComboBox, int nLangId ) ;
int SetMuiLangIdArrayData ( int nIndex, int nLangId ) ;
int GetMuiLangIdArray ( int nIndex ) ;
BOOL CALLBACK SetComboBoxMuiLangIdProc ( wchar_t *szName ) ;



// \ĨR{{bNXZbg
int SetComboBoxMuiLangId ( HWND hWnd, int nComboBox ) {

   if ( ! IsNT () || AllocMuiLangIdArray () ) {
      if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, nComboBox, 0, GetStringW ( IDS_DEFAULT_MUI_LANG ) ) ;
      else           Dlg_ComboBox_InsertStringA ( hWnd, nComboBox, 0, GetStringA ( IDS_DEFAULT_MUI_LANG ) ) ;
      Dlg_ComboBox_SetCurSel ( hWnd, nComboBox, 0 ) ;
      Dlg_EnableWindow ( hWnd, IDC_COMBO_MUI_LANG, FALSE ) ;
   }
   else {

      int nCurrentLangId = GetResourceLanguage () ;

      SetComboBoxMuiLangIdData ( hWnd, nComboBox, 0 ) ;
      EnumSystemLocalesW ( SetComboBoxMuiLangIdProc, LCID_INSTALLED ) ;

      LoadMuiLibrary ( nCurrentLangId ) ;

      Dlg_ComboBox_InsertStringW ( hWnd, nComboBox, 0, GetStringW ( IDS_DEFAULT_MUI_LANG ) ) ;
      SetMuiLangIdArrayData ( 0, LANGID_NEUTRAL ) ;

      int nIndexMax = Dlg_ComboBox_GetCount ( hWnd, nComboBox ) ;
      int nIndex = 0 ;

      for ( int nCount = 0 ; nCount < nIndexMax ; nCount ++ ) {
         if ( nSavedMuiLangId == GetMuiLangIdArray ( nCount ) ) {
            nIndex = nCount ;
            break ;
         }
      }

      Dlg_ComboBox_SetCurSel ( hWnd, nComboBox, nIndex ) ;

      Dlg_ComboBox_SetMaxHeight ( hWnd, nComboBox, 10 ) ;
   }

   return 0 ;
}



// \I񋓂vV[W
BOOL CALLBACK SetComboBoxMuiLangIdProc ( wchar_t *szName ) {
   LCID nLcid = wcstol ( szName, NULL, 0x10 ) ;
   LANGID nLangId = LANGIDFROMLCID ( nLcid ) ;
   if ( nLangId ) SetComboBoxMuiLangIdData ( NULL, 0, nLangId ) ;
   return TRUE ;
}



// \I񋓂vV[W̉֐
// ŏ nLangId  0 w肵āAhWnd, nComboBox 
int SetComboBoxMuiLangIdData ( HWND hWnd, int nComboBox, int nLangId ) {

   static HWND hSavedWnd ;
   static int nSavedComboBox ;

   if ( ! nLangId ) {
      hSavedWnd = hWnd ;
      nSavedComboBox = nComboBox ;
   }
   else {
      if ( ! LoadMuiLibrary ( nLangId ) ) {

         wchar_t szBuffer [ SHORTSTRING_MAX * 2 ] ;

         if ( GetLocaleInfoW ( MAKELCID ( nLangId, SORT_DEFAULT ), LOCALE_SLANGUAGE, szBuffer, SHORTSTRING_MAX ) ) {

            wchar_t szBufferSub [ SHORTSTRING_MAX ] ;
            if ( GetLocaleInfoW ( MAKELCID ( nLangId, SORT_DEFAULT ), LOCALE_SNATIVELANGNAME, szBufferSub, SHORTSTRING_MAX ) && wcscmp ( szBuffer, szBufferSub ) ) {
               wcscat ( szBuffer, L" - " ) ;
               wcscat ( szBuffer, szBufferSub ) ;
            }

            int nIndex = Dlg_ComboBox_AddStringW ( hSavedWnd, nSavedComboBox, szBuffer ) ;
            SetMuiLangIdArrayData ( nIndex, nLangId ) ;
         }
      }
   }

   return 0 ;
}



int GetComboBoxMuiLangId ( HWND hWnd, int nComboBox ) {
   int nIndex = Dlg_ComboBox_GetCurSel ( hWnd, nComboBox ) ;
   return GetMuiLangIdArray ( nIndex ) ;
}



BOOL CALLBACK CountMuiLangIdProc ( wchar_t *szName ) ;
static LANGID *pMuiLangIdArray ;
static int nMuiLangIdMax ;



// \I̔z񃁃m
// s 0 ȊOԂ
int AllocMuiLangIdArray ( void ) {

   nMuiLangIdMax = 0 ;
   EnumSystemLocalesW ( CountMuiLangIdProc, LCID_INSTALLED ) ;
   nMuiLangIdMax += 0x10 ;    // Ƃ LANGID_NEUTRAL Ȃǂ邽

   pMuiLangIdArray = (LANGID*) malloc ( ( nMuiLangIdMax ) * sizeof(LANGID) ) ;
   if ( ! pMuiLangIdArray ) return 1 ;

   return 0 ;
}



// \I̔z񃁃
int FreeMuiLangIdArray ( void ) {
   free ( pMuiLangIdArray ) ;
   pMuiLangIdArray = NULL ;
   return 0 ;
}



// \I̍ő吔𒲂ׂvV[W
BOOL CALLBACK CountMuiLangIdProc ( wchar_t *szName ) {
   nMuiLangIdMax ++ ;
   return TRUE ;
}



// \I̔z񃁃Ƀf[^i[
int SetMuiLangIdArrayData ( int nIndex, int nLangId ) {

   if ( nIndex >= 0 && nIndex < nMuiLangIdMax ) {
      memmove ( pMuiLangIdArray + nIndex + 1, pMuiLangIdArray + nIndex, sizeof(LANGID) * ( nMuiLangIdMax - nIndex - 1 ) ) ;
      pMuiLangIdArray [ nIndex ] = nLangId ;
   }

   return 0 ;
}



// \I̔z񃁃f[^擾
int GetMuiLangIdArray ( int nIndex ) {
   if ( nIndex >= 0 && nIndex < nMuiLangIdMax ) return pMuiLangIdArray [ nIndex ] ;
   return 0 ;
}



////////////////////////////////////////////
//         xȃIvVݒ           //
////////////////////////////////////////////



int GetFontExComboBox ( HWND hWnd, int nDefaultCheckBox, int nFontComboBox, int nUnicodeBlock, char *FontNameBuffer ) ;
int SetFontExComboBox ( HWND hWnd, int nDefaultCheckBox, int nFontComboBox, int nUnicodeBlock, char *FontNameBuffer ) ;
int SetLangIdComboBox ( HWND hWnd, int nDefaultCheckBox, int nLangIdComboBox, int nUnicodeBlock, unsigned short *LangIdArray ) ;



// xȐݒ_CAO
INT_PTR CALLBACK FontExDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   static unsigned short *LangIdArrayTmp ;
   static char *FontNameArrayTmp ;

   static HFONT hFont ;
   int nLangId ;


   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // GfBbgRg[͂̒̐
         Dlg_ComboBox_LimitText ( hWnd, IDC_COMBO_FONT, FONTNAME_MAX ) ;

         // tHg
         if ( ! FontNameArray ) {
            FontNameArray = (char*) malloc ( FONTNAME_ARRAY_SIZE ) ;
            if ( ! FontNameArray ) {
               if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameW, MB_OK ) ;
               else           MessageBoxA ( hWnd, GetStringA ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameA, MB_OK ) ;
               EndDialog ( hWnd, IDCANCEL ) ;
               return FALSE ;
            }
            atexit ( FreeFontNameArray ) ;
            memzero ( FontNameArray, FONTNAME_ARRAY_SIZE ) ;
         }
         if ( ! FontNameArrayTmp ) {
            FontNameArrayTmp = (char*) malloc ( FONTNAME_ARRAY_SIZE ) ;
            if ( ! FontNameArrayTmp ) {
               if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameW, MB_OK ) ;
               else           MessageBoxA ( hWnd, GetStringA ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameA, MB_OK ) ;
               EndDialog ( hWnd, IDCANCEL ) ;
               return FALSE ;
            }
            memmove ( FontNameArrayTmp, FontNameArray, FONTNAME_ARRAY_SIZE ) ;
         }

         // tHg쐬
         hFont = CreateFontByCodePage ( GetDialogFontHeight ( IDD_DIALOG_FONT_EX ), 0 ) ;
         if ( hFont ) Dlg_Wm_SetFont ( hWnd, IDC_COMBO_FONT, hFont, TRUE ) ;


         // ubN
         for ( int nCount = 0 ; nCount < UNICODE_BLOCK_NELEM ; nCount ++ ) {
            int nStart, nEnd ;
            if ( GetUnicodeBlock ( 0, nCount, 0, & nStart, & nEnd, NULL ) < 0 ) break ;
            const wchar_t *szName = GetUnicodeBlockNameW ( nCount ) ;
            if ( ! szName ) break ;
            if ( IsNT () ) {
               Dlg_ComboBox_InsertStringFormatW ( hWnd, IDC_COMBO_FONT_BLOCKNAME, -1, L"%04X-%04X: %s", nStart, nEnd, szName ) ;
            }
            else {
               char szNameA [ MAX_PATH ] ;
               w2alcpy ( szNameA, szName, MAX_PATH ) ;
               Dlg_ComboBox_InsertStringFormatA ( hWnd, IDC_COMBO_FONT_BLOCKNAME, -1, "%04X-%04X: %s", nStart, nEnd, szNameA ) ;
            }
         }

         // ubN
         if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_FONT_BLOCKNAME, -1, GetStringW ( IDS_ALL_UNICODE_BLOCKS ) ) ;
         else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_FONT_BLOCKNAME, -1, GetStringA ( IDS_ALL_UNICODE_BLOCKS ) ) ;
         Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_FONT_BLOCKNAME, 0 ) ;

         Dlg_ComboBox_SetMaxHeight ( hWnd, IDC_COMBO_FONT_BLOCKNAME, 16 ) ;

         SetFontExComboBox ( hWnd, IDC_CHECK_FONT_DEFAULT, IDC_COMBO_FONT, 0, FontNameArrayTmp ) ;
         Dlg_EnableWindow ( hWnd, IDC_BUTTON_FONT_ALL_DEFAULT, ! IsZeroMemory ( FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ) ;


         // ꎯʎq
         if ( ! LangIdArrayTmp ) {
            LangIdArrayTmp = (unsigned short*) malloc ( LANGID_ARRAY_SIZE ) ;
            if ( ! LangIdArrayTmp ) {
               if ( IsNT () ) MessageBoxW ( hWnd, GetStringW ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameW, MB_OK ) ;
               else           MessageBoxA ( hWnd, GetStringA ( IDS_NOT_ENOUGH_MEMORY ), szCommandNameA, MB_OK ) ;
               EndDialog ( hWnd, IDCANCEL ) ;
               return FALSE ;
            }
            memmove ( LangIdArrayTmp, LangIdArray, LANGID_ARRAY_SIZE ) ;
         }

         // ubN
         for ( int nCount = 0 ; nCount < UNICODE_BLOCK_NELEM ; nCount ++ ) {
            int nStart, nEnd ;
            if ( GetUnicodeBlock ( 0, nCount, 0, & nStart, & nEnd, NULL ) < 0 ) break ;
            const wchar_t *szName = GetUnicodeBlockNameW ( nCount ) ;
            if ( ! szName ) break ;
            if ( IsNT () ) {
               Dlg_ComboBox_InsertStringFormatW ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, -1, L"%04X-%04X: %s", nStart, nEnd, szName ) ;
            }
            else {
               char szNameA [ MAX_PATH ] ;
               w2alcpy ( szNameA, szName, MAX_PATH ) ;
               Dlg_ComboBox_InsertStringFormatA ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, -1, "%04X-%04X: %s", nStart, nEnd, szNameA ) ;
            }
         }

         // ubN
         if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, -1, GetStringW ( IDS_ALL_UNICODE_BLOCKS ) ) ;
         else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, -1, GetStringA ( IDS_ALL_UNICODE_BLOCKS ) ) ;
         Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, 0 ) ;

         Dlg_ComboBox_SetMaxHeight ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, 16 ) ;

         SetLangIdComboBox ( hWnd, IDC_CHECK_LANGID_DEFAULT, IDC_COMBO_LANGID, 0, LangIdArrayTmp ) ;
         Dlg_EnableWindow ( hWnd, IDC_BUTTON_LANGID_ALL_DEFAULT, ! IsZeroMemory ( LangIdArrayTmp, LANGID_ARRAY_SIZE ) ) ;

         return TRUE ;


      case WM_COMMAND :

         // tHg̕ubN
         if ( LOWORD ( wParam ) == IDC_COMBO_FONT_BLOCKNAME ) {
            // R{{bNX̑IύXƂ
            if ( HIWORD ( wParam ) == CBN_SELCHANGE ) {
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_FONT_BLOCKNAME ) ;
               SetFontExComboBox ( hWnd, IDC_CHECK_FONT_DEFAULT, IDC_COMBO_FONT, nCount, FontNameArrayTmp ) ;
               Dlg_EnableWindow ( hWnd, IDC_BUTTON_FONT_ALL_DEFAULT, ! IsZeroMemory ( FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ) ;
               return TRUE ;
            }
         }

         // ID̕ubN
         if ( LOWORD ( wParam ) == IDC_COMBO_LANGID_BLOCKNAME ) {
            // R{{bNX̑IύXƂ
            if ( HIWORD ( wParam ) == CBN_SELCHANGE ) {
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_LANGID_BLOCKNAME ) ;
               SetLangIdComboBox ( hWnd, IDC_CHECK_LANGID_DEFAULT, IDC_COMBO_LANGID, nCount, LangIdArrayTmp ) ;
               Dlg_EnableWindow ( hWnd, IDC_BUTTON_LANGID_ALL_DEFAULT, ! IsZeroMemory ( LangIdArrayTmp, LANGID_ARRAY_SIZE ) ) ;
               return TRUE ;
            }
         }

         // tHg
         if ( LOWORD ( wParam ) == IDC_COMBO_FONT ) {
            // R{{bNXtH[JXƂ
            if ( HIWORD ( wParam ) == CBN_KILLFOCUS ) {
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_FONT_BLOCKNAME ) ;
               GetFontExComboBox ( hWnd, IDC_CHECK_FONT_DEFAULT, IDC_COMBO_FONT, nCount, FontNameArrayTmp ) ;
               Dlg_EnableWindow ( hWnd, IDC_BUTTON_FONT_ALL_DEFAULT, ! IsZeroMemory ( FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ) ;
               return TRUE ;
            }
         }

         // ID
         if ( LOWORD ( wParam ) == IDC_COMBO_LANGID ) {
            // R{{bNX̑IύXƂ
            if ( HIWORD ( wParam ) == CBN_SELCHANGE ) {
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_LANGID ) ;
               nLangId = ( LangIdNameList [ nCount ] ).nLangId ;
               nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_LANGID_BLOCKNAME ) ;
               if ( (unsigned) nCount < (unsigned) UNICODE_BLOCK_NELEM ) {
                  int nIndex = GetUnicodeBlock ( 0, nCount, 0, NULL, NULL, NULL ) ;
                  if ( (unsigned) nIndex < (unsigned) UNICODE_BLOCK_NELEM ) LangIdArrayTmp [ nIndex ] = nLangId ;
               }
               else {
                  for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) LangIdArrayTmp [ nIndex ] = nLangId ;
               }
               Dlg_EnableWindow ( hWnd, IDC_BUTTON_LANGID_ALL_DEFAULT, ! IsZeroMemory ( LangIdArrayTmp, LANGID_ARRAY_SIZE ) ) ;
               return TRUE ;
            }
         }

         // tHg̊l
         if ( LOWORD ( wParam ) == IDC_CHECK_FONT_DEFAULT ) {
            // `FbN{bNXύXƂ
            if ( HIWORD ( wParam ) == BN_CLICKED ) {
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_FONT_BLOCKNAME ) ;
               int nIndex = GetUnicodeBlock ( 0, nCount, 0, NULL, NULL, NULL ) ;
               const char *szFontName ;
               if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_FONT_DEFAULT ) ) szFontName = "" ;
               else                                                       szFontName = GetDefaultFontName ( nIndex ) ;
               if ( (unsigned) nIndex < (unsigned) UNICODE_BLOCK_NELEM ) {
                  memzero ( FontNameArrayTmp + nIndex * LF_FACESIZE, LF_FACESIZE * sizeof(char) ) ;
                  strlcpy ( FontNameArrayTmp + nIndex * LF_FACESIZE, szFontName, LF_FACESIZE ) ;
               }
               SetFontExComboBox ( hWnd, IDC_CHECK_FONT_DEFAULT, IDC_COMBO_FONT, nCount, FontNameArrayTmp ) ;
               Dlg_EnableWindow ( hWnd, IDC_BUTTON_FONT_ALL_DEFAULT, ! IsZeroMemory ( FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ) ;
               return TRUE ;
            }
         }

         // ID̊l
         if ( LOWORD ( wParam ) == IDC_CHECK_LANGID_DEFAULT ) {
            // `FbN{bNXύXƂ
            if ( HIWORD ( wParam ) == BN_CLICKED ) {
               if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_LANGID_DEFAULT ) ) nLangId = 0 ;
               else                                                         nLangId = ( LangIdNameList [ 0 ] ).nLangId ;
               int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_LANGID_BLOCKNAME ) ;
               int nIndex = GetUnicodeBlock ( 0, nCount, 0, NULL, NULL, NULL ) ;
               if ( (unsigned) nIndex < (unsigned) UNICODE_BLOCK_NELEM ) LangIdArrayTmp [ nIndex ] = nLangId ;
               SetLangIdComboBox ( hWnd, IDC_CHECK_LANGID_DEFAULT, IDC_COMBO_LANGID, nCount, LangIdArrayTmp ) ;
               Dlg_EnableWindow ( hWnd, IDC_BUTTON_LANGID_ALL_DEFAULT, ! IsZeroMemory ( LangIdArrayTmp, LANGID_ARRAY_SIZE ) ) ;
               return TRUE ;
            }
         }

         // Wɖ߂
         if ( LOWORD ( wParam ) == IDC_BUTTON_FONT_ALL_DEFAULT ) {
            memzero ( FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ;
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_FONT_BLOCKNAME, 0 ) ;
            SetFontExComboBox ( hWnd, IDC_CHECK_FONT_DEFAULT, IDC_COMBO_FONT, 0, FontNameArrayTmp ) ;
            Dlg_EnableWindow ( hWnd, IDC_BUTTON_FONT_ALL_DEFAULT, ! IsZeroMemory ( FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ) ;
            return TRUE ;
         }

         // Wɖ߂
         if ( LOWORD ( wParam ) == IDC_BUTTON_LANGID_ALL_DEFAULT ) {
            memzero ( LangIdArrayTmp, LANGID_ARRAY_SIZE ) ;
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_LANGID_BLOCKNAME, 0 ) ;
            SetLangIdComboBox ( hWnd, IDC_CHECK_LANGID_DEFAULT, IDC_COMBO_LANGID, 0, LangIdArrayTmp ) ;
            Dlg_EnableWindow ( hWnd, IDC_BUTTON_LANGID_ALL_DEFAULT, ! IsZeroMemory ( LangIdArrayTmp, LANGID_ARRAY_SIZE ) ) ;
            return TRUE ;
         }

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDOK ) {
            memmove ( FontNameArray, FontNameArrayTmp, FONTNAME_ARRAY_SIZE ) ;
            memmove ( LangIdArray, LangIdArrayTmp, LANGID_ARRAY_SIZE ) ;
            IsLangIdUserDefined = ! IsZeroMemory ( LangIdArray, LANGID_ARRAY_SIZE ) ;
            IsOptionChanged = 1 ;
            EndDialog ( hWnd, IDOK ) ;
            return TRUE ;
         }

         break ;


      case WM_DESTROY :
         if ( hFont ) {
            DeleteObject ( hFont ) ;
            hFont = NULL ;
         }
         if ( FontNameArrayTmp ) {
            free ( FontNameArrayTmp ) ;
            FontNameArrayTmp = NULL ;
         }
         if ( LangIdArrayTmp ) {
            free ( LangIdArrayTmp ) ;
            LangIdArrayTmp = NULL ;
         }
         break ;


      default :
         break ;
   }

   return FALSE ;
}



// xȐݒ_CAỎ֐
int GetFontExComboBox ( HWND hWnd, int nDefaultCheckBox, int nFontComboBox, int nUnicodeBlock, char *FontNameBuffer ) {

   char szFontName [ FONTNAME_MAX ] ;

   if ( IsDlgButtonChecked ( hWnd, nDefaultCheckBox ) ) *szFontName = 0 ;
   else {
      if ( IsNT () ) {
         wchar_t szBuffer [ FONTNAME_MAX ] ;
         Dlg_Wm_GetTextW ( hWnd, nFontComboBox, szBuffer, FONTNAME_MAX ) ;
         w2alcpy ( szFontName, szBuffer, FONTNAME_MAX ) ;
      }
      else {
         Dlg_Wm_GetTextA ( hWnd, nFontComboBox, szFontName, FONTNAME_MAX ) ;
      }
   }

   if ( (unsigned) nUnicodeBlock < (unsigned) UNICODE_BLOCK_NELEM ) {
      int nIndex = GetUnicodeBlock ( 0, nUnicodeBlock, 0, NULL, NULL, NULL ) ;
      if ( (unsigned) nIndex < (unsigned) UNICODE_BLOCK_NELEM ) {
         memzero ( FontNameBuffer + nIndex * LF_FACESIZE, LF_FACESIZE * sizeof(char) ) ;
         strlcpy ( FontNameBuffer + nIndex * LF_FACESIZE, szFontName, LF_FACESIZE ) ;
      }
   }
   else if ( *szFontName ) {
      for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
         memzero ( FontNameBuffer + nIndex * LF_FACESIZE, LF_FACESIZE * sizeof(char) ) ;
         strlcpy ( FontNameBuffer + nIndex * LF_FACESIZE, szFontName, LF_FACESIZE ) ;
      }
   }

   return 0 ;
}



// xȐݒ_CAỎ֐
int SetFontExComboBox ( HWND hWnd, int nDefaultCheckBox, int nFontComboBox, int nUnicodeBlock, char *FontNameBuffer ) {

   LRESULT nCountContent = Dlg_ComboBox_GetCount ( hWnd, nFontComboBox ) ;
   if ( nCountContent ) Dlg_ComboBox_ResetContent ( hWnd, nFontComboBox ) ;

   char *szFontName = (char*) -1 ;
   if ( (unsigned) nUnicodeBlock < (unsigned) UNICODE_BLOCK_NELEM ) {
      int nIndex = GetUnicodeBlock ( 0, nUnicodeBlock, 0, NULL, NULL, NULL ) ;
      if ( (unsigned) nIndex < (unsigned) UNICODE_BLOCK_NELEM ) szFontName = FontNameBuffer + nIndex * LF_FACESIZE ;
      else                                                      szFontName = NULL ;
   }

   if ( ! szFontName || szFontName != (char*) -1 && ! *szFontName ) {
      CheckDlgButton ( hWnd, nDefaultCheckBox, BST_CHECKED ) ;
      Dlg_EnableWindow ( hWnd, nDefaultCheckBox, TRUE ) ;
      Dlg_EnableWindow ( hWnd, nFontComboBox, FALSE ) ;
   }
   else {
      CheckDlgButton ( hWnd, nDefaultCheckBox, BST_UNCHECKED ) ;
      Dlg_EnableWindow ( hWnd, nDefaultCheckBox, ( szFontName != (char*) -1 ) ) ;
      Dlg_EnableWindow ( hWnd, nFontComboBox, TRUE ) ;
      SetComboBoxFontName ( hWnd, nFontComboBox, -1 ) ;
      if ( szFontName != (char*) -1 ) {
         if ( IsNT () ) {
            wchar_t szBuffer [ FONTNAME_MAX ] ;
            a2wlcpy ( szBuffer, szFontName, FONTNAME_MAX ) ;
            Dlg_Wm_SetTextW ( hWnd, nFontComboBox, szBuffer ) ;
         }
         else {
            Dlg_Wm_SetTextA ( hWnd, nFontComboBox, szFontName ) ;
         }
      }
      Dlg_ComboBox_SetMaxHeight ( hWnd, nFontComboBox, 16 ) ;
   }

   return 0 ;
}



// xȐݒ_CAỎ֐
int SetLangIdComboBox ( HWND hWnd, int nDefaultCheckBox, int nLangIdComboBox, int nUnicodeBlock, unsigned short *LangIdArray ) {

   LRESULT nCountContent = Dlg_ComboBox_GetCount ( hWnd, nLangIdComboBox ) ;
   if ( nCountContent ) Dlg_ComboBox_ResetContent ( hWnd, nLangIdComboBox ) ;

   int nLangId = -1 ;
   if ( (unsigned) nUnicodeBlock < (unsigned) UNICODE_BLOCK_NELEM ) {
      int nIndex = GetUnicodeBlock ( 0, nUnicodeBlock, 0, NULL, NULL, NULL ) ;
      if ( (unsigned) nIndex < (unsigned) UNICODE_BLOCK_NELEM ) nLangId = LangIdArray [ nIndex ] ;
      else                                                      nLangId = 0 ;
   }

   if ( ! nLangId ) {
      CheckDlgButton ( hWnd, nDefaultCheckBox, BST_CHECKED ) ;
      Dlg_EnableWindow ( hWnd, nDefaultCheckBox, TRUE ) ;
      Dlg_EnableWindow ( hWnd, nLangIdComboBox, FALSE ) ;
   }
   else {
      CheckDlgButton ( hWnd, nDefaultCheckBox, BST_UNCHECKED ) ;
      Dlg_EnableWindow ( hWnd, nDefaultCheckBox, ( nLangId != -1 ) ) ;
      Dlg_EnableWindow ( hWnd, nLangIdComboBox, TRUE ) ;
      for ( int nCount = 0 ; nCount < LANGID_NAME_LIST_NELEM ; nCount ++ ) {
         if ( IsNT () ) {
            Dlg_ComboBox_InsertStringW ( hWnd, nLangIdComboBox, -1, ( LangIdNameList [ nCount ] ).szName ) ;
         }
         else {
            char szName [ MAX_PATH ] ;
            w2alcpy ( szName, ( LangIdNameList [ nCount ] ).szName, MAX_PATH ) ;
            Dlg_ComboBox_InsertStringA ( hWnd, nLangIdComboBox, -1, szName ) ;
         }
      }
      for ( int nCount = 0 ; nCount < LANGID_NAME_LIST_NELEM ; nCount ++ ) {
         if ( nLangId == ( LangIdNameList [ nCount ] ).nLangId ) {
            Dlg_ComboBox_SetCurSel ( hWnd, nLangIdComboBox, nCount ) ;
            break ;
         }
      }
      Dlg_ComboBox_SetMaxHeight ( hWnd, nLangIdComboBox, 16 ) ;
   }

   return 0 ;
}



// xȐݒ_CAOi{j
INT_PTR CALLBACK ExConvModeJpDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsNoJis83New ) CheckDlgButton ( hWnd, IDC_CHECK_JIS_NO_JIS83_NEW, BST_CHECKED ) ;
         if ( IsNoJis83Old ) CheckDlgButton ( hWnd, IDC_CHECK_JIS_NO_JIS83_OLD, BST_CHECKED ) ;
         if ( IsEucOutputHojo ) CheckDlgButton ( hWnd, IDC_CHECK_EUC_HOJO, BST_CHECKED ) ;

         if ( IsInputJis2004 )          CheckDlgButton ( hWnd, IDC_CHECK_JIS_INPUT_2004, BST_CHECKED ) ;
         else if ( IsInputJisNoVendor ) CheckDlgButton ( hWnd, IDC_CHECK_JIS_INPUT_NO_VENDOR, BST_CHECKED ) ;
         else if ( IsInputJisMac )      CheckDlgButton ( hWnd, IDC_CHECK_JIS_INPUT_MAC, BST_CHECKED ) ;
         else                           CheckDlgButton ( hWnd, IDC_CHECK_JIS_INPUT_WIN, BST_CHECKED ) ;

         if ( IsOutputJis2004 )          CheckDlgButton ( hWnd, IDC_CHECK_JIS_OUTPUT_2004, BST_CHECKED ) ;
         else if ( IsOutputJisNoVendor ) CheckDlgButton ( hWnd, IDC_CHECK_JIS_OUTPUT_NO_VENDOR, BST_CHECKED ) ;
         else if ( IsOutputJisMac )      CheckDlgButton ( hWnd, IDC_CHECK_JIS_OUTPUT_MAC, BST_CHECKED ) ;
         else                            CheckDlgButton ( hWnd, IDC_CHECK_JIS_OUTPUT_WIN, BST_CHECKED ) ;

         if ( IsNoVariantTmp ) {
            Dlg_EnableWindow ( hWnd, IDC_CHECK_JIS_NO_JIS83_NEW, FALSE ) ;
            Dlg_EnableWindow ( hWnd, IDC_CHECK_JIS_NO_JIS83_OLD, FALSE ) ;
         }

         if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_OUTPUT_2004 ) ) {
            Dlg_EnableWindow ( hWnd, IDC_CHECK_EUC_HOJO, FALSE ) ;
         }

         return TRUE ;


      case WM_COMMAND :

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDOK ) {
            IsInputJisMac = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_INPUT_MAC ) ;
            IsInputJisNoVendor = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_INPUT_NO_VENDOR ) ;
            IsInputJis2004 = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_INPUT_2004 ) ;
            IsOutputJisMac = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_OUTPUT_MAC ) ;
            IsOutputJisNoVendor = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_OUTPUT_NO_VENDOR ) ;
            IsOutputJis2004 = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_OUTPUT_2004 ) ;
            IsNoJis83New = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_NO_JIS83_NEW ) ;
            IsNoJis83Old = IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_NO_JIS83_OLD ) ;
            IsEucOutputHojo = IsDlgButtonChecked ( hWnd, IDC_CHECK_EUC_HOJO ) ;

            IsOptionChanged = 1 ;
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         if ( LOWORD ( wParam ) == IDC_CHECK_JIS_NO_JIS83_NEW ) {
            // `FbN{bNXύXƂ
            if ( HIWORD ( wParam ) == BN_CLICKED ) {
               if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_NO_JIS83_NEW ) ) CheckDlgButton ( hWnd, IDC_CHECK_JIS_NO_JIS83_OLD, BST_UNCHECKED ) ;
            }
         }
         if ( LOWORD ( wParam ) == IDC_CHECK_JIS_NO_JIS83_OLD ) {
            // `FbN{bNXύXƂ
            if ( HIWORD ( wParam ) == BN_CLICKED ) {
               if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_JIS_NO_JIS83_OLD ) ) CheckDlgButton ( hWnd, IDC_CHECK_JIS_NO_JIS83_NEW, BST_UNCHECKED ) ;
            }
         }
         if ( LOWORD ( wParam ) >= IDC_CHECK_JIS_OUTPUT_WIN && LOWORD ( wParam ) <= IDC_CHECK_JIS_OUTPUT_2004 ) {
            // `FbN{bNXύXƂ
            if ( HIWORD ( wParam ) == BN_CLICKED ) {
               if ( LOWORD ( wParam ) != IDC_CHECK_JIS_OUTPUT_2004 ) Dlg_EnableWindow ( hWnd, IDC_CHECK_EUC_HOJO, TRUE ) ;
               else                                                  Dlg_EnableWindow ( hWnd, IDC_CHECK_EUC_HOJO, FALSE ) ;
            }
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// xȐݒ_CAOiȑ̎j
INT_PTR CALLBACK ExConvModeCnsDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsGbk2Pua ) CheckDlgButton ( hWnd, IDC_CHECK_GBK_TO_PUA, BST_CHECKED ) ;
         if ( IsPua2Gbk ) CheckDlgButton ( hWnd, IDC_CHECK_PUA_TO_GBK, BST_CHECKED ) ;

         if ( IsGbOutput == 0 )      CheckDlgButton ( hWnd, IDC_CHECK_GB2312_ONLY, BST_CHECKED ) ;
         else if ( IsGbOutput == 1 ) CheckDlgButton ( hWnd, IDC_CHECK_GBK, BST_CHECKED ) ;
         else                        CheckDlgButton ( hWnd, IDC_CHECK_GB18030, BST_CHECKED ) ;

         if ( ! IsGbkOutputFantizi ) CheckDlgButton ( hWnd, IDC_CHECK_GBK_JIANTIZI, BST_CHECKED ) ;
         else                        CheckDlgButton ( hWnd, IDC_CHECK_GBK_FANTIZI, BST_CHECKED ) ;

         if ( IsNoVariantTmp || IsDlgButtonChecked ( hWnd, IDC_CHECK_GB2312_ONLY ) ) {
            Dlg_EnableWindow ( hWnd, IDC_CHECK_GBK_JIANTIZI, FALSE ) ;
            Dlg_EnableWindow ( hWnd, IDC_CHECK_GBK_FANTIZI, FALSE ) ;
         }

         return TRUE ;


      case WM_COMMAND :

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDOK ) {
            IsGbk2Pua = IsDlgButtonChecked ( hWnd, IDC_CHECK_GBK_TO_PUA ) ;
            IsPua2Gbk = IsDlgButtonChecked ( hWnd, IDC_CHECK_PUA_TO_GBK ) ;

            IsGbOutput = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GB2312_ONLY ) ) IsGbOutput = 0 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GBK ) ) IsGbOutput = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GB18030 ) ) IsGbOutput = 2 ;

            IsGbkOutputFantizi = 0 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GBK_FANTIZI ) ) IsGbkOutputFantizi = 1 ;

            IsOptionChanged = 1 ;
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         if ( LOWORD ( wParam ) >= IDC_CHECK_GB2312_ONLY && LOWORD ( wParam ) <= IDC_CHECK_GB18030 ) {
            // `FbN{bNXύXƂ
            if ( ! IsNoVariantTmp && HIWORD ( wParam ) == BN_CLICKED ) {
               if ( LOWORD ( wParam ) == IDC_CHECK_GB2312_ONLY ) {
                  Dlg_EnableWindow ( hWnd, IDC_CHECK_GBK_JIANTIZI, FALSE ) ;
                  Dlg_EnableWindow ( hWnd, IDC_CHECK_GBK_FANTIZI, FALSE ) ;
               }
               else {
                  Dlg_EnableWindow ( hWnd, IDC_CHECK_GBK_JIANTIZI, TRUE ) ;
                  Dlg_EnableWindow ( hWnd, IDC_CHECK_GBK_FANTIZI, TRUE ) ;
               }
            }
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// xȐݒ_CAOiɑ̎j
INT_PTR CALLBACK ExConvModeCntDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   static const int nHkscsCountMax = 4 ;

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsPua2Hkscs ) CheckDlgButton ( hWnd, IDC_CHECK_UNI2HKS_PUA, BST_CHECKED ) ;
         if ( IsHkscsCjka2Pua ) CheckDlgButton ( hWnd, IDC_CHECK_HKS2UNI_CJKA_PUA, BST_CHECKED ) ;
         if ( IsHkscsCjkb2Pua ) CheckDlgButton ( hWnd, IDC_CHECK_HKS2UNI_CJKB_PUA, BST_CHECKED ) ;
         if ( IsHkscsCjk412Pua ) CheckDlgButton ( hWnd, IDC_CHECK_HKS2UNI_CJK41_PUA, BST_CHECKED ) ;

         for ( int nCount = 0 ; nCount < nHkscsCountMax ; nCount ++ ) {
            if ( IsNT () ) Dlg_ComboBox_InsertStringW ( hWnd, IDC_COMBO_HKSCS_VER, -1, GetStringW ( IDS_HKSCS_1999 + nCount ) ) ;
            else           Dlg_ComboBox_InsertStringA ( hWnd, IDC_COMBO_HKSCS_VER, -1, GetStringA ( IDS_HKSCS_1999 + nCount ) ) ;
         }

         int nTmp ;
         if ( IsOutputHkscs1999 )      nTmp = 1 ;
         else if ( IsOutputHkscs2001 ) nTmp = 2 ;
         else if ( IsOutputHkscs2004 ) nTmp = 3 ;
         else                          nTmp = nHkscsCountMax ;
         Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_HKSCS_VER, nTmp - 1 ) ;

         return TRUE ;


      case WM_COMMAND :

         // W
         if ( LOWORD ( wParam ) == IDC_BUTTON_HKSCS_VER_DEFAULT ) {
            Dlg_ComboBox_SetCurSel ( hWnd, IDC_COMBO_HKSCS_VER, nHkscsCountMax - 1 ) ;
            return TRUE ;
         }
         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDOK ) {
            IsPua2Hkscs = IsDlgButtonChecked ( hWnd, IDC_CHECK_UNI2HKS_PUA ) ;
            IsHkscsCjka2Pua = IsDlgButtonChecked ( hWnd, IDC_CHECK_HKS2UNI_CJKA_PUA ) ;
            IsHkscsCjkb2Pua = IsDlgButtonChecked ( hWnd, IDC_CHECK_HKS2UNI_CJKB_PUA ) ;
            IsHkscsCjk412Pua = IsDlgButtonChecked ( hWnd, IDC_CHECK_HKS2UNI_CJK41_PUA ) ;

            IsGbOutput = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GB2312_ONLY ) ) IsGbOutput = 0 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GBK ) ) IsGbOutput = 1 ;
            if ( IsDlgButtonChecked ( hWnd, IDC_CHECK_GB18030 ) ) IsGbOutput = 2 ;

            IsOutputHkscs1999 = 0 ;
            IsOutputHkscs2001 = 0 ;
            IsOutputHkscs2004 = 0 ;
            int nCount = Dlg_ComboBox_GetCurSel ( hWnd, IDC_COMBO_HKSCS_VER ) ;
            switch ( nCount ) {
               case 0 : IsOutputHkscs1999 = 1 ; break ;
               case 1 : IsOutputHkscs2001 = 1 ; break ;
               case 2 : IsOutputHkscs2004 = 1 ; break ;
            }

            IsOptionChanged = 1 ;
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



// xȐݒ_CAOi؍j
INT_PTR CALLBACK ExConvModeKsDialogProc ( HWND hWnd, unsigned int dwMessage, WPARAM wParam, LPARAM lParam ) {

   switch ( dwMessage ) {

      case WM_INITDIALOG :

         // l
         if ( IsKsNoUhc ) CheckDlgButton ( hWnd, IDC_CHECK_KS_NO_VENDOR, BST_CHECKED ) ;
         else             CheckDlgButton ( hWnd, IDC_CHECK_KS_OUTPUT_WIN, BST_CHECKED ) ;

         if ( IsConvKsPunc ) CheckDlgButton ( hWnd, IDC_CHECK_KS_PUNC, BST_CHECKED ) ;

         return TRUE ;


      case WM_COMMAND :

         // I
         if ( LOWORD ( wParam ) == IDCANCEL ) {
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }
         if ( LOWORD ( wParam ) == IDOK ) {
            IsKsNoUhc = IsDlgButtonChecked ( hWnd, IDC_CHECK_KS_NO_VENDOR ) ;
            IsConvKsPunc = IsDlgButtonChecked ( hWnd, IDC_CHECK_KS_PUNC ) ;
            IsOptionChanged = 1 ;
            EndDialog ( hWnd, IDCANCEL ) ;
            return TRUE ;
         }

         break ;


      default :
         break ;
   }

   return FALSE ;
}



////////////////////////////////////////////
//               ꎯʎq               //
////////////////////////////////////////////



int UnicodeToCharset ( int nIndex ) ;



const char *GetDefaultFontName ( int nIndex ) {

   int nCharset = UnicodeToCharset ( nIndex ) ;

   switch ( nCharset ) {
      case ANSI_CHARSET :        return szEuropeanFont ;
      case SHIFTJIS_CHARSET :    return szJapaneseFont ;
      case HANGUL_CHARSET :      return szKoreanFont ;
      case GB2312_CHARSET :      return szChineseSimplifiedFont ;
      case CHINESEBIG5_CHARSET : return szChineseTraditionalFont ;
      case GREEK_CHARSET :       return szEuropeanFont ;
      case HEBREW_CHARSET :      return szEuropeanFont ;
      case ARABIC_CHARSET :      return szEuropeanFont ;
      case RUSSIAN_CHARSET :     return szEuropeanFont ;
      case THAI_CHARSET :        return szThaiFont ;
   }

   if ( nIndex == UB_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A || nIndex == UB_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B ) {
      if ( IsWinVISTAorLater () ) {
         if ( nDefaultMultiByteCodePageSub == CP_CHINESE_TRADITIONAL ) return szChineseTraditionalFont ;
         else                                                          return szChineseSimplifiedFont ;
      }
      return szDefaultGb18030Font ;
   }

   return szEuropeanFont ;
}



int UnicodeToCharset ( int nIndex ) {

   int nCharset = UnicodeBlockToCharset ( nIndex ) ;

   if ( nCharset == UNICS_CJK_CHARSET ) {
      return CodePageToCharset ( nDefaultCjkCodePage ) ;
   }
   if ( nCharset == UNICS_CHINESE_CHARSET ) {
      if ( nOsCodePage == CP_CHINESE_TRADITIONAL ) return CHINESEBIG5_CHARSET ;
      else                                         return GB2312_CHARSET ;
   }
   if ( nCharset != UNICS_CHARSET_UNDEFINED ) return nCharset ;

   return -1 ;
}



////////////////////////////////////////////
//            PRINTF ARGUMENTS            //
////////////////////////////////////////////



// printf tH[}bg̈̐ԂiANSIŁj
int CountPrintfArgumentsA ( const char *szFormat ) {

   int nCount = 0 ;

   while ( *szFormat ) {

      if ( *szFormat == '%' ) {

         szFormat ++ ;

         if ( *szFormat && strchr ( "+- #", *szFormat ) ) szFormat ++ ;

         if ( *szFormat == '*' ) { szFormat ++ ; nCount ++ ; }
         else while ( isascii ( *szFormat ) && isdigit ( *szFormat ) ) szFormat ++ ;

         if ( *szFormat == '.' ) {
            szFormat ++ ;
            if ( *szFormat == '*' ) { szFormat ++ ; nCount ++ ; }
            else while ( isascii ( *szFormat ) && isdigit ( *szFormat ) ) szFormat ++ ;
         }

         if      ( ! strheadcmp ( szFormat, "I32" ) ) szFormat += strlen ( "I32" ) ;
         else if ( ! strheadcmp ( szFormat, "I64" ) ) szFormat += strlen ( "I64" ) ;
         else if ( *szFormat && strchr ( "hlI", *szFormat ) ) szFormat ++ ;

         if ( *szFormat && strchr ( "cCdiouxXeEfgGnpsS", *szFormat ) ) nCount ++ ;
      }

      szFormat ++ ;
   }

   return nCount ;
}



////////////////////////////////////////////
//                 ̑                 //
////////////////////////////////////////////



// [Să[ǂ
// nSize ɂ̓oCgw
int IsZeroMemory ( const void *pMemory, size_t nSize ) {

   for ( const char *p = (const char*) pMemory, *end = p + nSize ; p < end ; p ++ ) {
      if ( *p ) return 0 ;
   }

   return 1 ;
}



