// rtfconv_dll.cpp

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <limits.h>
#include "msc.h"
#include "winmain.h"

#include "cjkctype.h"
#include "convchar.h"
#include "codepage.h"
#include "langid.h"
#include "nls.h"
#include "res.h"
#include "storechr.h"
#include "stream.h"
#include "stringex.h"

#include "conv_err.h"
#include "rtfconv.h"
#include "rtfconv_dll.h"
#include "conv_abort.h"
#include "conv_pkmsg.h"
#include "htmlcr.h"
#include "fontname.h"

#include "unicode.h"
#include "uninorm.h"
#include "unichain.h"
#include "uniprior.h"
#include "uniblock.h"
#include "unilang.h"
#include "unicharset.h"
#include "gbx.h"
#include "unijtz.h"
#include "uniftz.h"
#include "hkscsver.h"
#include "tamil.h"
#include "viet.h"

#include "conv_tbl.h"



// for debug
#define DEBUG_FIELD     0
#define DEBUG_RTFTAG    0



static HINSTANCE hInstance ;     // RTFCONV.DLL̃CX^X
static int nOsCodePage ;         // OS̃R[hy[W
static unsigned long nTlsIndex ;

static CRITICAL_SECTION LoadSbctblSection ;
static CRITICAL_SECTION LoadMbctblSection ;
static CRITICAL_SECTION LoadCnstblSection ;
static CRITICAL_SECTION AvailableCodePageSection ;

#define PEEK_MESSAGE_CONUT_MASK  0xFF

// ̕R[h
#define CP_UNICODE   CP_UTF32


static int InitRtfconvData ( RTFCONVDATA *pData, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) ;
static int InitConversionMode ( RTFCONVDATA *pData ) ;
static int AllocWritingData ( RTFCONVDATA *pData, const RTFCONVEXINFO *pInfo ) ;
static int FreeWritingData ( RTFCONVDATA *pData ) ;
static int GetCharSize ( int nCodePage ) ;
static int ConvertTxt ( RTFCONVDATA *pData ) ;
static int ConvertRtf ( RTFCONVDATA *pData ) ;
static int WriteFileStart ( RTFCONVDATA *pData ) ;
static int WriteFileEnd ( RTFCONVDATA *pData ) ;



////////////////////////////////////////////
//    RTFCONV.DLL  LOAD  FREE     //
////////////////////////////////////////////



int WINAPI DllMain ( HINSTANCE hInst, unsigned long dwReason, void *pReserved ) {

   if ( dwReason == DLL_PROCESS_ATTACH ) {

      // LastError p̃CfbNXm
      if ( ( nTlsIndex = TlsAlloc () ) == TLS_OUT_OF_INDEXES ) return FALSE ;

      if ( InitializeCriticalSectionEx ( & LoadSbctblSection ) ) return FALSE ;
      if ( InitializeCriticalSectionEx ( & LoadMbctblSection ) ) return FALSE ;
      if ( InitializeCriticalSectionEx ( & LoadCnstblSection ) ) return FALSE ;
      if ( InitializeCriticalSectionEx ( & AvailableCodePageSection ) ) return FALSE ;

      // ϐ
      hInstance = hInst ;
      nOsCodePage = GetACP () ;

      // \[X̃CX^X
      SetResourceInstance ( hInst ) ;

      // ~tOe[u
      if ( AllocAbortTable () ) return FALSE ;
   }

   if ( dwReason == DLL_PROCESS_DETACH ) {

      FreeAbortTable () ;

      DeleteCriticalSection ( & LoadSbctblSection ) ;
      DeleteCriticalSection ( & LoadMbctblSection ) ;
      DeleteCriticalSection ( & LoadCnstblSection ) ;
      DeleteCriticalSection ( & AvailableCodePageSection ) ;

      TlsFree ( nTlsIndex ) ;
   }

   return TRUE ;
}



////////////////////////////////////////////
//            RTFCONViANSIŁj           //
////////////////////////////////////////////



//  CONV_SUCCESS Ԃ
// s CONV_ERROR_UNKNOWN ȏ̒萔Ԃ
int WINAPI RtfconvExA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   int nError = CONV_ERROR_PARAM ;

   if ( ! pInfo || pInfo->cbSize == sizeof(RTFCONVEXINFO) ) {

      RTFCONVDATA Data = { 0 } ;

      __try {

         // f[^̏E[̊m
         if ( InitRtfconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( AllocWritingData ( & Data, pInfo ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
         if ( RegisterAbortFlag ( & Data.pAbortFlag ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

         // ̓Xg[J
         if ( ! ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) ) {
            Data.StreamIn = OpenFileStreamForReadingA ( szSrcFileName ) ;
            if ( Data.StreamIn == NULL ) { nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         }

         // o̓Xg[J
         Data.StreamOut = OpenFileStreamForWritingA ( szDstFileName ) ;
         if ( Data.StreamOut == NULL ) { nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }

         // s
         if ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) {
            if      ( Data.dwReserved & RESERVEDMODE_ONLY_START ) nError = WriteFileStart ( & Data ) ;
            else if ( Data.dwReserved & RESERVEDMODE_ONLY_END )   nError = WriteFileEnd ( & Data ) ;
         }
         else {
            if ( Data.nSrcCodePage ) nError = ConvertTxt ( & Data ) ;
            else                     nError = ConvertRtf ( & Data ) ;
         }

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      __try {

         // Xg[
         if ( Data.StreamIn && CloseStream ( Data.StreamIn, NULL ) && ! nError ) nError = CONV_ERROR_READ ;
         if ( Data.StreamOut && CloseStream ( Data.StreamOut, NULL ) && ! nError ) nError = CONV_ERROR_WRITE ;

         // [̉
         FreeWritingData ( & Data ) ;
         UnregisterAbortFlag ( Data.pAbortFlag ) ;

      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

   }

   return SetRtfconvLastError ( nError ) ;
}



////////////////////////////////////////////
//          RTFCONViUNICODEŁj          //
////////////////////////////////////////////



//  CONV_SUCCESS Ԃ
// s CONV_ERROR_UNKNOWN ȏ̒萔Ԃ
int WINAPI RtfconvExW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   int nError = CONV_ERROR_PARAM ;

   if ( ! pInfo || pInfo->cbSize == sizeof(RTFCONVEXINFO) ) {

      RTFCONVDATA Data = { 0 } ;

      __try {

         // f[^̏E[̊m
         if ( InitRtfconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( AllocWritingData ( & Data, pInfo ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
         if ( RegisterAbortFlag ( & Data.pAbortFlag ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

         // ̓Xg[J
         if ( ! ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) ) {
            Data.StreamIn = OpenFileStreamForReadingW ( szSrcFileName ) ;
            if ( Data.StreamIn == NULL ) { nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         }

         // o̓Xg[J
         Data.StreamOut = OpenFileStreamForWritingW ( szDstFileName ) ;
         if ( Data.StreamOut == NULL ) { nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }

         // s
         if ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) {
            if      ( Data.dwReserved & RESERVEDMODE_ONLY_START ) nError = WriteFileStart ( & Data ) ;
            else if ( Data.dwReserved & RESERVEDMODE_ONLY_END )   nError = WriteFileEnd ( & Data ) ;
         }
         else {
            if ( Data.nSrcCodePage ) nError = ConvertTxt ( & Data ) ;
            else                     nError = ConvertRtf ( & Data ) ;
         }

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      __try {

         // Xg[
         if ( Data.StreamIn && CloseStream ( Data.StreamIn, NULL ) && ! nError ) nError = CONV_ERROR_READ ;
         if ( Data.StreamOut && CloseStream ( Data.StreamOut, NULL ) && ! nError ) nError = CONV_ERROR_WRITE ;

         // [̉
         FreeWritingData ( & Data ) ;
         UnregisterAbortFlag ( Data.pAbortFlag ) ;

      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

   }

   return SetRtfconvLastError ( nError ) ;
}



////////////////////////////////////////////
//           RTFCONViSTRINGŁj          //
////////////////////////////////////////////



// ϊ
// ϊ̒̕Ԃ
// s -1 Ԃ
intptr_t WINAPI RtfconvStringEx ( const void *szSrcBuffer, void *szDstBuffer, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo, size_t nMaxLen ) {

   int nError = CONV_ERROR_PARAM ;
   size_t nResultLen ;

   if ( ! pInfo || pInfo->cbSize == sizeof(RTFCONVEXINFO) ) {

      RTFCONVDATA Data = { 0 } ;

      __try {

         // f[^̏E[̊m
         if ( InitRtfconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( AllocWritingData ( & Data, pInfo ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
         if ( RegisterAbortFlag ( & Data.pAbortFlag ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

         // ̓Xg[J
         if ( ! ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) ) {
            Data.StreamIn = OpenStringStreamForReading ( szSrcBuffer, (size_t) -1, GetCharSize ( nSrcCodePage ) ) ;
            if ( Data.StreamIn == NULL ) { nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         }

         // o̓Xg[J
         Data.StreamOut = OpenStringStreamForWriting ( szDstBuffer, nMaxLen, GetCharSize ( nDstCodePage ) ) ;
         if ( Data.StreamOut == NULL ) { nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }

         // s
         if ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) {
            if      ( Data.dwReserved & RESERVEDMODE_ONLY_START ) nError = WriteFileStart ( & Data ) ;
            else if ( Data.dwReserved & RESERVEDMODE_ONLY_END )   nError = WriteFileEnd ( & Data ) ;
         }
         else {
            if ( Data.nSrcCodePage ) nError = ConvertTxt ( & Data ) ;
            else                     nError = ConvertRtf ( & Data ) ;
         }

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      __try {

         // Xg[
         if ( Data.StreamIn && CloseStream ( Data.StreamIn, NULL ) && ! nError ) nError = CONV_ERROR_READ ;
         if ( Data.StreamOut && CloseStream ( Data.StreamOut, & nResultLen ) && ! nError ) nError = CONV_ERROR_WRITE ;

         // [̉
         FreeWritingData ( & Data ) ;
         UnregisterAbortFlag ( Data.pAbortFlag ) ;

      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

   }

   if ( SetRtfconvLastError ( nError ) ) return -1 ;
   return nResultLen ;
}



////////////////////////////////////////////
//              ϊ̑O              //
////////////////////////////////////////////



// ϊf[^
//  0 As炻ȊOԂ
static int InitRtfconvData ( RTFCONVDATA *pData, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   pData->nSrcCodePage = nSrcCodePage ;
   pData->nDstCodePage = nDstCodePage ;

   if ( pInfo ) {

      pData->dwConversionMode = pInfo->dwConversionMode ;
      pData->dwAdditionalMode = pInfo->dwAdditionalMode ;

      if ( pInfo->pReserved ) {
         const RESERVEDINFO *pReserved = (const RESERVEDINFO*) pInfo->pReserved ;
         if ( pReserved->cbSize != sizeof(RESERVEDINFO) ) return 1 ;
         pData->dwReserved = pReserved->dwMode ;
      }

      if ( pData->dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) pData->nSrcCodePage = CP_UNICODE ; // dummy
   }

   return 0 ;
}



static int AllocFontArray ( RTFCONVDATA *pData, const RTFCONVEXINFO *pInfo ) ;
static int AllocLangIdArray ( RTFCONVDATA *pData, const RTFCONVEXINFO *pInfo ) ;
static int FreeFontArray ( RTFCONVDATA *pData ) ;
static int FreeLangIdArray ( RTFCONVDATA *pData ) ;



// ݗpf[^̃[mۂl
//  0 As炻ȊOԂ
static int AllocWritingData ( RTFCONVDATA *pData, const RTFCONVEXINFO *pInfo ) {

   if ( pInfo && pInfo->pUnmappableCharFormat ) {
      wchar_t *szFormat = a2wdup ( pInfo->pUnmappableCharFormat ) ;
      if ( ! szFormat ) return 1 ;
      wcstrunc ( szFormat, UNMAPPABLE_CHAR_FORMAT_MAX - 1 ) ;
      pData->szUnmappableCharFormat = szFormat ;
   }

   if ( pData->nDstCodePage ) {

      WRITINGTEXTDATA *wt = (WRITINGTEXTDATA*) malloc ( sizeof(WRITINGTEXTDATA) ) ;
      if ( ! wt ) return 1 ;

      memzero ( wt, sizeof(WRITINGTEXTDATA) ) ;
      pData->wt = wt ;

      pData->wt->C1  = EOF ;
      pData->wt->C2  = EOF ;
      pData->wt->C3  = EOF ;
      pData->wt->C4  = EOF ;
      pData->wt->U1  = EOF ;
      pData->wt->U2  = EOF ;
      pData->wt->U3  = EOF ;
      pData->wt->U4  = EOF ;
   }
   else {

      WRITINGRTFDATA *wr = (WRITINGRTFDATA*) malloc ( sizeof(WRITINGRTFDATA) ) ;
      if ( ! wr ) return 1 ;

      memzero ( wr, sizeof(WRITINGRTFDATA) ) ;
      pData->wr = wr ;

      if ( pInfo ) {
         pData->wr->nDefaultCodePage = pInfo->nDefaultCjkCodePage ;
         pData->wr->nFontSize = pInfo->nFontSize ;
         pData->wr->pFontInfo = pInfo->pFontInfo ;
      }

      if ( ! IsMultiByteCodePage ( pData->wr->nDefaultCodePage ) ) {
         if ( IsMultiByteCodePage ( nOsCodePage ) ) pData->wr->nDefaultCodePage = nOsCodePage ;
         else                                       pData->wr->nDefaultCodePage = CP_JAPANESE ;
      }

      pData->wr->nFontPrev          = -1 ;
      pData->wr->nLowByteFontPrev   = -1 ;

      if ( ! pData->wr->nFontSize ) pData->wr->nFontSize = DEFAULT_FONT_SIZE ;
      else if ( pData->wr->nFontSize < 0 ) pData->wr->nFontSize = 0 ;

      if ( pInfo ) {
         if ( AllocFontArray ( pData, pInfo ) ) return 1 ;
         if ( AllocLangIdArray ( pData, pInfo ) ) return 1 ;
      }
   }

   return 0 ;
}



// ݗpf[^̃[
static int FreeWritingData ( RTFCONVDATA *pData ) {

   if ( ! pData->nDstCodePage && pData->wr ) {
      FreeFontArray ( pData ) ;
      FreeLangIdArray ( pData ) ;
   }

   free ( pData->szUnmappableCharFormat ) ;

   free ( pData->wt ) ;
   if ( offsetof ( RTFCONVDATA, wt ) != offsetof ( RTFCONVDATA, wr ) ) free ( pData->wr ) ;

   return 0 ;
}



static int InitConversionModeSub ( RTFCONVDATA *pData, int IsSrc ) ;

static int GetPseudoCodePage ( int nCodePage ) ;
static int IsAvailableCodePageWithConversionMode ( int nCodePage, unsigned long dwConversionMode ) ;
static int CountPrintfArgumentsW ( const wchar_t *szFormat ) ;



// ϊ[h
//  AllocWritingData() ŏ邱
//  0 As炻ȊOԂ
static int InitConversionMode ( RTFCONVDATA *pData ) {

   if ( ! pData->nSrcCodePage && ! pData->nDstCodePage ) return 1 ;

   if ( InitConversionModeSub ( pData, 1 ) ) return 1 ;
   if ( InitConversionModeSub ( pData, 0 ) ) return 1 ;

   if ( ! pData->nDstCodePage && pData->wr ) {
      if ( IsMultiByteCodePage ( pData->nSrcWinCodePage ) ) pData->wr->nDefaultCodePage = pData->nSrcWinCodePage ;
   }

   return 0 ;
}



// InitConversionMode ̉֐
//  0 As炻ȊOԂ
static int InitConversionModeSub ( RTFCONVDATA *pData, int IsSrc ) {

   // l擾
   int nCodePage ;
   if ( IsSrc ) nCodePage = pData->nSrcCodePage ;
   else         nCodePage = pData->nDstCodePage ;
   unsigned long dwConversionMode = pData->dwConversionMode ;
   unsigned long dwAdditionalMode = pData->dwAdditionalMode ;


   // {֌W

   if ( nCodePage == CP_ISO_JAPANESE_AND_KANA ) nCodePage = CP_ISO_JAPANESE ;
   if ( nCodePage == CP_ISO_JAPANESE_AND_KANA_SIO ) nCodePage = CP_ISO_JAPANESE ;

   if ( IsSrc ) {
      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR ) dwAdditionalMode &= ~ ADDITIONAL_MODE_JIS_INPUT_MAC ;
      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_2004 ) dwAdditionalMode &= ~ ( ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_INPUT_MAC ) ;
   }
   else {
      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR ) dwAdditionalMode &= ~ ADDITIONAL_MODE_JIS_OUTPUT_MAC ;
      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 ) dwAdditionalMode &= ~ ( ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_OUTPUT_MAC ) ;
      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 ) dwConversionMode &= ~ CONVMODE_EUC_HOJO ;
   }

   if ( ! nCodePage ) {
      if ( IsSrc ) dwAdditionalMode &= ~ ( ADDITIONAL_MODE_JIS_INPUT_MAC | ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_INPUT_2004 ) ;
      else         dwAdditionalMode &= ~ ( ADDITIONAL_MODE_JIS_OUTPUT_MAC | ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_OUTPUT_2004 ) ;
   }

   if ( nCodePage == CP_MAC_JAPANESE ) {
      nCodePage = CP_JAPANESE ;
      if ( IsSrc ) {
         dwAdditionalMode |= ADDITIONAL_MODE_JIS_INPUT_MAC ;
         dwAdditionalMode &= ~ ( ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_INPUT_2004 ) ;
      }
      else {
         dwAdditionalMode |= ADDITIONAL_MODE_JIS_OUTPUT_MAC ;
         dwAdditionalMode &= ~ ( ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_OUTPUT_2004 ) ;
         dwConversionMode &= ~ CONVMODE_EUC_HOJO ;
      }
   }

   if ( nCodePage != CP_EUC_JAPANESE ) {
      if ( ! IsSrc ) dwConversionMode &= ~ CONVMODE_EUC_HOJO ;
   }


   // ȑ̎֌W

   if ( nCodePage == CP_CHINESE_SIMPLIFIED_EX ) {
      nCodePage = CP_CHINESE_SIMPLIFIED ;
      if ( ! IsSrc ) dwConversionMode |= CONVMODE_OUTPUT_GB18030 ;
   }

   if ( nCodePage == CP_HZ_CHINESE_SIMPLIFIED ) {
      if ( ! IsSrc ) dwConversionMode |= CONVMODE_NO_GBK ;
   }

   if ( nCodePage == CP_EUC_CHINESE_SIMPLIFIED || nCodePage == CP_EUC_CHINESE_SIMPLIFIED_2 ) {
      nCodePage = CP_CHINESE_SIMPLIFIED ;
      if ( ! IsSrc ) {
         dwConversionMode |= CONVMODE_NO_GBK ;
         dwConversionMode &= ~ CONVMODE_OUTPUT_GB18030 ;
      }
   }

   if ( ! IsSrc ) {
      if ( dwConversionMode & CONVMODE_OUTPUT_GB18030 ) dwConversionMode &= ~ CONVMODE_NO_GBK ;
      if ( dwConversionMode & CONVMODE_NO_GBK ) dwAdditionalMode &= ~ ADDITIONAL_MODE_GBK_OUTPUT_FANTIZI ;
   }


   // ɑ̎֌W

   if ( nCodePage == CP_CHINESE_TRADITIONAL_2 ) nCodePage = CP_CHINESE_TRADITIONAL ;


   // ؍֌W

   if ( nCodePage == CP_EUC_KOREAN || nCodePage == CP_EUC_KOREAN_2 ) {
      nCodePage = CP_KOREAN ;
      if ( ! IsSrc ) dwAdditionalMode |= ADDITIONAL_MODE_KS_NO_UHC ;
   }

   if ( nCodePage == CP_ISO_KOREAN ) {
      if ( ! IsSrc ) dwAdditionalMode |= ADDITIONAL_MODE_KS_NO_UHC ;
   }


   // UNICODE֌W

   if ( nCodePage == CP_UTF32_2 ) nCodePage = CP_UTF32 ;
   if ( nCodePage == CP_UTF32_2_BE ) nCodePage = CP_UTF32_BE ;

   if ( ! IsSrc && nCodePage == CP_UTF8 ) {
      if ( dwConversionMode & CONVMODE_UTF8_WITH_BOM ) dwConversionMode &= ~ CONVMODE_NO_OUTPUT_BOM ;
      else                                             dwConversionMode |= CONVMODE_NO_OUTPUT_BOM ;
   }


   // DLL֘A

   if ( ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) && ( dwConversionMode & CONVMODE_USE_DLL_TABLE ) ) dwConversionMode &= ~ ( CONVMODE_USE_SYSTEM_TABLE | CONVMODE_USE_DLL_TABLE ) ;


   // ϊpR[hy[W擾

   int nPseudoCodePage = GetPseudoCodePage ( nCodePage ) ;
   if ( nCodePage && nPseudoCodePage != CP_UNICODE && ! IsAvailableCodePageWithConversionMode ( nPseudoCodePage, dwConversionMode ) ) return 1 ;


   // ̑

   if ( IsSrc ) {

      if ( ! ( dwConversionMode & ( CONVMODE_OUTPUT_CR | CONVMODE_OUTPUT_LF ) ) ) dwConversionMode |= ( CONVMODE_OUTPUT_CR | CONVMODE_OUTPUT_LF ) ;

      if ( dwConversionMode & CONVMODE_VARIANT_ONLY_ROUNDTRIP && nPseudoCodePage == CP_UNICODE ) {
         dwConversionMode &= ~ CONVMODE_VARIANT_ONLY_ROUNDTRIP ;
         dwConversionMode |= CONVMODE_NO_VARIANT_EUROPEAN | CONVMODE_NO_VARIANT_KANJI ;
      }

      // PeekMessage
      if ( dwConversionMode & CONVMODE_PEEKMESSAGE ) {
         if ( InitPeekMessage ( & pData->hDialog, & pData->IsMessageUnicode ) ) dwConversionMode &= ~ CONVMODE_PEEKMESSAGE ;
      }
   }
   else {

      if ( pData->szUnmappableCharFormat && ! CountPrintfArgumentsW ( pData->szUnmappableCharFormat ) ) dwConversionMode &= ~ CONVMODE_OUTPUT_NUMBER_SURROGATE ;

      if ( dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) dwConversionMode &= ~ CONVMODE_OUTPUT_CODE_NUMBER_HTML ;

      if ( ( dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_VISTA ) && ( dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_XP ) ) {
         dwAdditionalMode &= ~ ( ADDITIONAL_MODE_RTFFONT_VISTA | ADDITIONAL_MODE_RTFFONT_XP ) ;
      }

      if ( ! ( dwAdditionalMode & ( ADDITIONAL_MODE_RTFFONT_VISTA | ADDITIONAL_MODE_RTFFONT_XP ) ) ) {
         if ( IsWinVISTAorLater () ) dwAdditionalMode |= ADDITIONAL_MODE_RTFFONT_VISTA ;
      }

      // dwReserved
      if ( ( dwConversionMode & CONVMODE_NO_OUTPUT_BOM ) && nPseudoCodePage == CP_UNICODE ) pData->dwReserved |= RESERVEDMODE_NO_START_END ;
   }


   // li[
   if ( IsSrc ) {
      pData->nSrcCodePage = nCodePage ;
      pData->nSrcPseudoCodePage = nPseudoCodePage ;
      pData->nSrcWinCodePage = GetWinCodePage ( nPseudoCodePage ) ;
   }
   else {
      pData->nDstCodePage = nCodePage ;
      pData->nDstPseudoCodePage = nPseudoCodePage ;
      pData->nDstWinCodePage = GetWinCodePage ( nPseudoCodePage ) ;
   }
   pData->dwConversionMode = dwConversionMode ;
   pData->dwAdditionalMode = dwAdditionalMode ;


   return 0 ;
}



////////////////////////////////////////////
//             ̓[eB             //
////////////////////////////////////////////



static int Putc ( int C, int nSrcCodePage, RTFCONVDATA *pData ) ;
static int Puts ( const char *szString, int nSrcCodePage, RTFCONVDATA *pData ) ;
static int Putws ( const wchar_t *szString, RTFCONVDATA *pData ) ;
static int PutUnmappableChar ( int cUnicode, const char *szDefaultString, RTFCONVDATA *pData ) ;

static int SjisNec2ibm ( int cSjis ) ;
static int SjisIbm2nec ( int cSjis ) ;

static int CharToUnicode ( int C, int nCodePage, unsigned long dwConversionMode ) ;
static int UnicodeToChar ( int C, int nCodePage, unsigned long dwConversionMode ) ;

static int CharToUnicode_JisHojo ( int C, unsigned long dwConversionMode ) ;
static int UnicodeToChar_JisHojo ( int C, unsigned long dwConversionMode ) ;

static int IsAsciiString ( const char *szString ) ;

static inline int IsAborted ( RTFCONVDATA *pData ) { return *pData->pAbortFlag ; }



////////////////////////////////////////////
//           TXT ̓[eB           //
////////////////////////////////////////////



static int Getwc ( int C1, RTFCONVDATA *pData ) ;
static int DecodeByteOrderMark ( int C, RTFCONVDATA *pData ) ;
static int Getc ( int C1, RTFCONVDATA *pData ) ;
static int GetcModal ( int C1, RTFCONVDATA *pData, int IsFirstByte ) ;

static const int IsIsoKrStrict = 0 ;   // ISO-2022-KR ̔邩H

#define GETC_ONLY_ESCAPE   (EOF-1)

// nEncodeMode
#define  EM_ASCII       0
#define  EM_LOCAL       1
#define  EM_HANKANA     2
#define  EM_LOCAL_EX    3



static int DecodeCharRefHtml ( int C, RTFCONVDATA *pData ) ;
static int DecodeCharRefC ( int C, RTFCONVDATA *pData ) ;
static int UnreadDecodeCharRefBuffer ( int C, RTFCONVDATA *pData ) ;
static int FlushReverseBuffer ( int C, RTFCONVDATA *pData ) ;



// eLXgt@Cϊ
//  0 Ԃ
// s 1 ȏ̎w肳ꂽ萔Ԃ
static int ConvertTxt ( RTFCONVDATA *pData ) {

   int nCountChar = 0 ;
   int IsFirstByte = 1 ;         // (UNICODE) t@C擪H (ISO-2022-KR) s̐擪H

   int nSrcPseudoCodePage = pData->nSrcPseudoCodePage ;
   int IsModalEncode = ( nSrcPseudoCodePage != pData->nSrcCodePage || pData->nSrcCodePage == CP_EUC_CHINESE_TRADITIONAL ) ;
   int IsIsciiOrTamil = IsCodePageIscii ( pData->nSrcCodePage ) || IsCodePageTamil ( pData->nSrcCodePage ) ;

   int IsReverse = 0 ;


   // ǂݍݗpf[^̏l
   READINGTEXTDATA rt = { 0 } ;
   pData->rt = & rt ;
   pData->rt->C1 = EOF ;
   pData->rt->C2 = EOF ;


   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_START ) ) WriteFileStart ( pData ) ;


   while ( 1 ) {

      if ( pData->nError ) goto CAUGHT_ERROR ;

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }


      int C ;

      // 擾
      if ( ( C = PopStoredCharsW32 ( pData->rt->CharRefUnreadBuffer ) ) == EOF ) {

         int C1 = StreamRead ( pData->StreamIn ) ;
         if ( C1 == EOF ) break ;

         // UNICODEȂ
         if ( nSrcPseudoCodePage == CP_UNICODE ) {
            C = Getwc ( C1, pData ) ;
            if ( C == GETC_ONLY_ESCAPE ) continue ;
            if ( IsFirstByte ) {
               IsFirstByte = 0 ;
               if ( DecodeByteOrderMark ( C, pData ) ) continue ;
            }
            if ( C == EOF ) {
               PutUnmappableChar ( REPLACEMENT_CHARACTER, "?", pData ) ;
               continue ;
            }
         }
         // [ĥGR[hȂ
         else if ( IsModalEncode ) {
            C = GetcModal ( C1, pData, IsFirstByte ) ;
            IsFirstByte = 0 ;
            if ( C == GETC_ONLY_ESCAPE ) continue ;
            if ( C == EOF ) {
               PutUnmappableChar ( REPLACEMENT_CHARACTER, "??", pData ) ;
               continue ;
            }
            if ( pData->nSrcCodePage == CP_ISO_KOREAN && C == '\n' ) IsFirstByte = 1 ;
         }
         // [ĥȂGR[hȂ
         else {
            C = Getc ( C1, pData ) ;
            IsFirstByte = 0 ;
            if ( C == EOF ) {
               PutUnmappableChar ( REPLACEMENT_CHARACTER, "??", pData ) ;
               continue ;
            }
         }
      }


      // QƂ͂
      if ( pData->dwConversionMode & ( CONVMODE_INPUT_CODE_NUMBER_HTML | CONVMODE_INPUT_CODE_NUMBER_C ) ) {
         // HTML̕QƂ͂
         if ( pData->dwConversionMode & CONVMODE_INPUT_CODE_NUMBER_HTML ) {
            C = DecodeCharRefHtml ( C, pData ) ;
            if ( C == EOF ) continue ;
         }
         // C^Cv̕QƂ͂
         if ( pData->dwConversionMode & CONVMODE_INPUT_CODE_NUMBER_C ) {
            C = DecodeCharRefC ( C, pData ) ;
            if ( C == EOF ) continue ;
         }
         // G[Ȃobt@߂
         if ( *GetStoredCharsA ( pData->rt->CharRefBuffer ) ) {
            C = UnreadDecodeCharRefBuffer ( C, pData ) ;
            if ( pData->rt->CharRefUnreadBuffer == STOREDCHARS_ERROR ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
            if ( C == EOF ) continue ;
         }
      }


      // UTF16TQ[g̏
      if ( pData->rt->cUnicodeLead ) {
         if ( nSrcPseudoCodePage == CP_UNICODE && IsUnicodeLowSurrogate ( C ) ) C = MAKE_UNICODE_SCALAR ( pData->rt->cUnicodeLead, C ) ;
         else                                                                   Putc ( pData->rt->cUnicodeLead, CP_UNICODE, pData ) ;
         pData->rt->cUnicodeLead = 0 ;
      }
      if ( nSrcPseudoCodePage == CP_UNICODE && IsUnicodeHighSurrogate ( C ) ) {
         pData->rt->cUnicodeLead = C ;
         continue ;
      }


      // ISCII, TSCII, VIQR ̏
      if ( IsIsciiOrTamil || pData->nSrcCodePage == CP_VIETNAMESE_VIQR ) {
         FlushReverseBuffer ( C, pData ) ;
         continue ;
      }
      // wuCꎋȍ
      if ( nSrcPseudoCodePage == CP_ISO_HEBREW_VISUAL ) {
         if ( C >= 0xE0 ) IsReverse = 1 ;
         else if ( isascii ( C ) && ( isalpha ( C ) || iscntrl ( C ) ) ) IsReverse = 0 ;
         if ( IsReverse ) {
            pData->rt->ReverseBuffer = StoreCharsA ( C, pData->rt->ReverseBuffer ) ;
            if ( pData->rt->ReverseBuffer == STOREDCHARS_ERROR ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
            continue ;
         }
         FlushReverseBuffer ( C, pData ) ;
      }


      // ̏o
      Putc ( C, nSrcPseudoCodePage, pData ) ;
   }


   // ܂Ăobt@fo
   FlushReverseBuffer ( EOF, pData ) ;


   CAUGHT_ERROR :

   // t@C̖ɌĂяo
   Putc ( EOF, nSrcPseudoCodePage, pData ) ;
   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_END ) ) WriteFileEnd ( pData ) ;

   FreeStoredCharsA ( pData->rt->CharRefBuffer ) ;
   FreeStoredCharsW32 ( pData->rt->CharRefUnreadBuffer ) ;
   FreeStoredCharsA ( pData->rt->ReverseBuffer ) ;

   return pData->nError ;
}



// HTML`̕QƂ
// Ԃ
// ŏAEOF Ԃ
static int DecodeCharRefHtml ( int C, RTFCONVDATA *pData ) {

   char *szBuffer = GetStoredCharsA ( pData->rt->CharRefBuffer ) ;

   // [
   // obt@̐擪 '&' Ɣr̂ DecodeCharRefC Ƌʂ邽
   if ( C == '&' && ! *szBuffer ||
        *szBuffer == '&' && ! *( szBuffer + 1 ) && C == '#' ||
        *szBuffer == '&' && isascii ( C ) && ( isdigit ( C ) || isalpha ( C ) )
   ) {

      // 
      if ( strlen ( szBuffer ) >= 0x20 ) return C ;

      pData->rt->CharRefBuffer = StoreCharsA ( C, pData->rt->CharRefBuffer ) ;
      if ( pData->rt->CharRefBuffer == STOREDCHARS_ERROR ) pData->nError = CONV_ERROR_MEMORY ;

      return EOF ;
   }

   // l
   if ( C == ';' && *szBuffer == '&' ) {

      int cUnicode = HtmlCharRefToUnicode ( szBuffer ) ;
      if ( cUnicode > 0 ) {

         ClearStoredCharsA ( pData->rt->CharRefBuffer ) ;

         // Ô
         if ( pData->rt->cUnicodeLead ) {
            if ( IsUnicodeLowSurrogate ( cUnicode ) ) cUnicode = MAKE_UNICODE_SCALAR ( pData->rt->cUnicodeLead, C ) ;
            else                                      Putc ( pData->rt->cUnicodeLead, CP_UNICODE, pData ) ;
            pData->rt->cUnicodeLead = 0 ;
         }
         if ( IsUnicodeHighSurrogate ( cUnicode ) ) {
            pData->rt->cUnicodeLead = C ;
            return EOF ;
         }

         if ( pData->nSrcCodePage != CP_UNICODE ) {
            int cResult = UnicodeToChar ( cUnicode, pData->nSrcCodePage, pData->dwConversionMode ) ;
            if ( cResult && cResult != EOF ) return cResult ;
         }

         if ( pData->nSrcCodePage == CP_ISO_HEBREW_VISUAL ) FlushReverseBuffer ( 0, pData ) ;
         else                                               FlushReverseBuffer ( EOF, pData ) ;

         Putc ( cUnicode, CP_UNICODE, pData ) ;
         return EOF ;
      }
   }

   return C ;
}



// C`̕QƂ
// Ԃ
// ŏAEOF Ԃ
static int DecodeCharRefC ( int C, RTFCONVDATA *pData ) {

   char *szBuffer = GetStoredCharsA ( pData->rt->CharRefBuffer ) ;

   // [
   // obt@̐擪 '\\' Ɣr̂ DecodeCharRefHtml Ƌʂ邽
   if ( C == '\\' && ! *szBuffer ||
       *szBuffer == '\\' && ! *( szBuffer + 1 ) && ( C == 'x' || C == 'u' || C == 'U' ) ||
       *szBuffer == '\\' &&   *( szBuffer + 1 ) && isascii ( C ) && isxdigit ( C )
   ) {

      pData->rt->CharRefBuffer = StoreCharsA ( C, pData->rt->CharRefBuffer ) ;
      if ( pData->rt->CharRefBuffer == STOREDCHARS_ERROR ) {
         pData->nError = CONV_ERROR_MEMORY ;
         return EOF ;
      }

      // l
      szBuffer = GetStoredCharsA ( pData->rt->CharRefBuffer ) ;
      if ( ! ( ( *( szBuffer + 1 ) == 'x' || *( szBuffer + 1 ) == 'u' ) && strlen ( szBuffer ) == ( 4 + 2 ) ||
                 *( szBuffer + 1 ) == 'U' && strlen ( szBuffer ) == ( 8 + 2 ) )
      ) {
         return EOF ;
      }

      int cUnicode = strtoul ( szBuffer + 2, NULL, 0x10 ) ;
      if ( cUnicode > 0 ) {

         ClearStoredCharsA ( pData->rt->CharRefBuffer ) ;

         // Ô
         if ( pData->rt->cUnicodeLead ) {
            if ( IsUnicodeLowSurrogate ( cUnicode ) ) cUnicode = MAKE_UNICODE_SCALAR ( pData->rt->cUnicodeLead, C ) ;
            else                                      Putc ( pData->rt->cUnicodeLead, CP_UNICODE, pData ) ;
            pData->rt->cUnicodeLead = 0 ;
         }
         if ( IsUnicodeHighSurrogate ( cUnicode ) ) {
            pData->rt->cUnicodeLead = C ;
            return EOF ;
         }

         if ( pData->nSrcCodePage != CP_UNICODE ) {
            int cResult = UnicodeToChar ( cUnicode, pData->nSrcCodePage, pData->dwConversionMode ) ;
            if ( cResult && cResult != EOF ) return cResult ;
         }

         if ( pData->nSrcCodePage == CP_ISO_HEBREW_VISUAL ) FlushReverseBuffer ( 0, pData ) ;
         else                                               FlushReverseBuffer ( EOF, pData ) ;

         Putc ( cUnicode, CP_UNICODE, pData ) ;
         return EOF ;
      }
   }

   return C ;
}



// HTML`/C`̕QƂ̃obt@ǂݖ߂AɏׂԂ
static int UnreadDecodeCharRefBuffer ( int C, RTFCONVDATA *pData ) {

   int cStart = PopStoredCharsA ( pData->rt->CharRefBuffer ) ;

   for ( int cTmp ; ( cTmp = PopStoredCharsA ( pData->rt->CharRefBuffer ) ) != EOF ; ) {
      pData->rt->CharRefUnreadBuffer = StoreCharsW32 ( cTmp, pData->rt->CharRefUnreadBuffer ) ;
   }

   pData->rt->CharRefUnreadBuffer = StoreCharsW32 ( C, pData->rt->CharRefUnreadBuffer ) ;

   return cStart ;
}



static int HebrewReverseString ( char *szSrcBuffer, int IsLineEndOfVtl ) ;
static int DecodeIscii ( int C1, int C2, RTFCONVDATA *pData ) ;
static int DecodeTamil ( int C1, int C2, int C3, RTFCONVDATA *pData ) ;
static int DecodeVietnameseViqr ( int C1, int C2, int C3, RTFCONVDATA *pData ) ;



// wuC̏ꍇ́AC ͎QƂ邾ŏȂ
// ISCII, TSCII, VIQR ̏ꍇ́AC 
// t@C C  EOF w肵ČĂяo
static int FlushReverseBuffer ( int C, RTFCONVDATA *pData ) {

   if ( pData->nSrcCodePage == CP_ISO_HEBREW_VISUAL ) {

      if ( ! *GetStoredCharsA ( pData->rt->ReverseBuffer ) ) return 0 ;

      pData->rt->ReverseBuffer = StoreCharsA ( 0, pData->rt->ReverseBuffer ) ;   // dummy, to increase buffer size
      if ( pData->rt->ReverseBuffer == STOREDCHARS_ERROR ) {
         pData->nError = CONV_ERROR_MEMORY ;
         return 1 ;
      }

      char *szBuffer = GetStoredCharsA ( pData->rt->ReverseBuffer ) ;

      int IsLineEndOfVtl = ( C == '\r' || C == '\n' || C == 0x1A || C == EOF ) ? 1 : 0 ;
      if ( HebrewReverseString ( szBuffer, IsLineEndOfVtl ) ) {
         pData->nError = CONV_ERROR_MEMORY ;
         return 1 ;
      }

      Puts ( szBuffer, pData->nSrcCodePage, pData ) ;
      ClearStoredCharsA ( pData->rt->ReverseBuffer ) ;
   }

   if ( IsCodePageIscii ( pData->nSrcCodePage ) ) {
      int nResult = 1 ;
      if ( pData->rt->C1 != EOF ) nResult = DecodeIscii ( pData->rt->C1, C, pData ) ;
      if ( nResult == 1 ) pData->rt->C1 = C ;
      else                pData->rt->C1 = EOF ;    // if ( nResult == 2 )
   }

   if ( IsCodePageTamil ( pData->nSrcCodePage ) ) {
      int nResult = 1 ;
      if ( pData->rt->C1 != EOF ) nResult = DecodeTamil ( pData->rt->C1, pData->rt->C2, C, pData ) ;
      if ( nResult == 1 )      { pData->rt->C1 = pData->rt->C2 ;  pData->rt->C2 = C ; }
      else if ( nResult == 2 ) { pData->rt->C1 = EOF ;            pData->rt->C2 = C ; }
      else                     { pData->rt->C1 = EOF ;            pData->rt->C2 = EOF ; }    // if ( nResult == 3 )
      if ( C == EOF ) FlushReverseBuffer ( 0, pData ) ;
   }

   if ( pData->nSrcCodePage == CP_VIETNAMESE_VIQR ) {
      int nResult = 1 ;
      if ( pData->rt->C1 != EOF ) nResult = DecodeVietnameseViqr ( pData->rt->C1, pData->rt->C2, C, pData ) ;
      if ( nResult == 1 )      { pData->rt->C1 = pData->rt->C2 ;  pData->rt->C2 = C ; }
      else if ( nResult == 2 ) { pData->rt->C1 = EOF ;            pData->rt->C2 = C ; }
      else                     { pData->rt->C1 = EOF ;            pData->rt->C2 = EOF ; }    // if ( nResult == 3 )
      if ( C == EOF ) FlushReverseBuffer ( 0, pData ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//     TXT ̓[eB̉֐     //
////////////////////////////////////////////



static int DecodeUtf8 ( int C, RTFCONVDATA *pData ) ;
static int DecodeUtf7 ( int C, RTFCONVDATA *pData ) ;
static int eucjp2sjis ( int C, int Is2004 ) ;
static int jistojms2004 ( int c ) ;



// sȃoCgȂ EOF Ԃ
// CP_UTF7 Ń[hύX݂̂Ȃ GETC_ONLY_ESCAPE Ԃ
static int Getwc ( int C1, RTFCONVDATA *pData ) {

   STREAM *StreamIn = pData->StreamIn ;

   switch ( pData->nSrcCodePage ) {

      // UTF-16 ̏
      case CP_UTF16 :
      {
         int C2 ;
         if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         return ( C2 << 8 ) | C1 ;
      }
      case CP_UTF16_BE :
      {
         int C2 ;
         if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         return ( C1 << 8 ) | C2 ;
      }

      // UTF-32 ̏
      case CP_UTF32 :
      {
         int C2, C3, C4 ;
         if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( ( C4 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         return ( C4 << 24 ) | ( C3 << 16 ) | ( C2 << 8 ) | C1 ;
      }
      case CP_UTF32_BE :
      {
         int C2, C3, C4 ;
         if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( ( C4 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         return ( C1 << 24 ) | ( C2 << 16 ) | ( C3 << 8 ) | C4 ;
      }

      // UTF-8 ̏
      case CP_UTF8 :
      {
         int cResult = DecodeUtf8 ( C1, pData ) ;
         if ( cResult == EOF ) return EOF ;
         return cResult ;
      }

      // UTF-7 ̏
      case CP_UTF7 :
      {
         int cResult = DecodeUtf7 ( C1, pData ) ;
         if ( cResult == EOF ) return GETC_ONLY_ESCAPE ;
         return cResult ;
      }
   }

   return EOF ;
}



// 폜Ȃ 0 ȊOԂ
static int DecodeByteOrderMark ( int C, RTFCONVDATA *pData ) {

   int IsEndianAuto = pData->dwConversionMode & CONVMODE_ENDIAN_AUTO ;

   switch ( pData->nSrcCodePage ) {

      case CP_UTF16 :
         if ( C == BYTE_ORDER_MARK ) return 1 ;
         if ( C == BYTE_ORDER_MARK_BE ) {
            if ( IsEndianAuto ) pData->nSrcCodePage = CP_UTF16_BE ;
            return 1 ;
         }
         break ;

      case CP_UTF16_BE :
         if ( C == BYTE_ORDER_MARK ) return 1 ;
         if ( C == BYTE_ORDER_MARK_BE ) {
            if ( IsEndianAuto ) pData->nSrcCodePage = CP_UTF16 ;
            return 1 ;
         }
         break ;

      case CP_UTF32 :
         if ( C == BYTE_ORDER_MARK ) return 1 ;
         if ( C == BYTE_ORDER_MARK_BE << 16 ) {
            if ( IsEndianAuto ) pData->nSrcCodePage = CP_UTF32_BE ;
            return 1 ;
         }
         break ;

      case CP_UTF32_BE :
         if ( C == BYTE_ORDER_MARK ) return 1 ;
         if ( C == BYTE_ORDER_MARK_BE << 16 ) {
            if ( IsEndianAuto ) pData->nSrcCodePage = CP_UTF32 ;
            return 1 ;
         }

      case CP_UTF8 :
         if ( C == BYTE_ORDER_MARK || C == BYTE_ORDER_MARK_BE ) return 1 ;
         break ;

      case CP_UTF7 :
         if ( C == BYTE_ORDER_MARK || C == BYTE_ORDER_MARK_BE ) return 1 ;
         break ;

   }

   return 0 ;
}



// sȃoCgȂ EOF Ԃ
static int Getc ( int C1, RTFCONVDATA *pData ) {

   STREAM *StreamIn = pData->StreamIn ;
   int nSrcCodePage = pData->nSrcCodePage ;

   if ( ismbblead_cp ( C1, nSrcCodePage ) ) {

      int C2 ;
      if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;

      if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED && ismbbsurrogate_cp ( C2, CP_CHINESE_SIMPLIFIED_EX ) ) {
         int C3, C4 ;
         if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( ( C4 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( ! ismbblead_cp ( C3, nSrcCodePage ) || ! ismbbsurrogate_cp ( C4, CP_CHINESE_SIMPLIFIED_EX ) ) return EOF ;
         return ( C1 << 24 ) | ( C2 << 16 ) | ( C3 << 8 ) | C4 ;
      }

      if ( ! ismbbtrail_cp ( C2, nSrcCodePage ) ) return EOF ;
      return  ( C1 << 8 ) | C2 ;
   }

   return C1 ;
}



// sȃoCgȂ EOF Ԃ
// GXP[vɂ郂[hύX݂̂Ȃ GETC_ONLY_ESCAPE Ԃ
static int GetcModal ( int C1, RTFCONVDATA *pData, int IsFirstByte ) {

   STREAM *StreamIn = pData->StreamIn ;

   switch ( pData->nSrcCodePage ) {

      // EUC-JP ̏
      case CP_EUC_JAPANESE :
      {
         if ( C1 == 0x8F ) {
            int C2, C3 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            int cResult = eucjp2sjis ( ( C1 << 16 ) + ( C2 << 8 ) + C3, pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_2004 ) ;
            if ( ! cResult ) return EOF ;
            return cResult ;
         }
         if ( C1 >= 0x80 ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            int cResult = eucjp2sjis ( ( C1 << 8 ) + C2, pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_2004 ) ;
            if ( ! cResult ) return EOF ;
            return cResult ;
         }
         return C1 ;
      }

      // EUC-TW ̏
      case CP_EUC_CHINESE_TRADITIONAL :
      {
         if ( C1 == 0x8E ) {
            int C2, C3, C4 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( ( C4 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            return ( C1 << 24 ) + ( C2 << 16 ) + ( C3 << 8 ) + C4 ;
         }
         if ( C1 >= 0x80 ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            return ( C1 << 8 ) + C2 ;
         }
         return C1 ;
      }

      // ISO-2022-JP ̏
      case CP_ISO_JAPANESE :
      {
         if ( C1 == 0x0E ) {     // SO
            pData->rt->nEncodeMode = EM_HANKANA ;
            return GETC_ONLY_ESCAPE ;
         }
         if ( C1 == 0x0F ) {     // SI
            pData->rt->nEncodeMode = EM_ASCII ;
            return GETC_ONLY_ESCAPE ;
         }
         if ( C1 == 0x1B ) {
            int C2, C3, C4 = EOF ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( C2 == '(' && ( C3 == 'B' || C3 == 'J' ) ) {
               pData->rt->nEncodeMode = EM_ASCII ;
               return GETC_ONLY_ESCAPE ;
            }
            if ( C2 == '$' && ( C3 == 'B' || C3 == '@' ) ) {
               pData->rt->nEncodeMode = EM_LOCAL ;
               return GETC_ONLY_ESCAPE ;
            }
            if ( C2 == '(' && C3 == 'I' ) {
               pData->rt->nEncodeMode = EM_HANKANA ;
               return GETC_ONLY_ESCAPE ;
            }
            if ( C2 == '$' && C3 == '(' && ( pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_2004 ) ) {
               if ( ( C4 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
               if ( C4 == 'Q' ) {
                  pData->rt->nEncodeMode = EM_LOCAL ;
                  return GETC_ONLY_ESCAPE ;
               }
               if ( C4 == 'P' ) {
                  pData->rt->nEncodeMode = EM_LOCAL_EX ;
                  return GETC_ONLY_ESCAPE ;
               }
               if ( C4 == 'O' ) {      // JIS2000
                  pData->rt->nEncodeMode = EM_LOCAL ;
                  return GETC_ONLY_ESCAPE ;
               }
            }
            // caught illegal
            StreamUnread ( C2, StreamIn ) ;
            StreamUnread ( C3, StreamIn ) ;
            if ( C4 != EOF ) StreamUnread ( C4, StreamIn ) ;
            return C1 ;
         }
         if ( pData->rt->nEncodeMode == EM_LOCAL_EX ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            int cResult = jistojms2004 ( ( ( C1 << 8 ) | C2 ) & 0x7F7F ) ;
            if ( ! cResult ) return EOF ;
            return cResult ;
         }
         if ( pData->rt->nEncodeMode == EM_LOCAL ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            int cResult = jistojms ( ( ( C1 << 8 ) | C2 ) & 0x7F7F ) ;
            if ( ! cResult ) return EOF ;
            return cResult ;
         }

         if ( ismbblead_sjis ( C1 ) ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            return ( C1 << 8 ) | C2 ;
         }
         if ( pData->rt->nEncodeMode == EM_HANKANA ) return C1 | 0x80 ;
         return C1 ;
      }

      // HZ ̏
      case CP_HZ_CHINESE_SIMPLIFIED :
      {
         if ( pData->rt->nEncodeMode == EM_ASCII ) {
            if ( C1 == '~' ) {
               int C2 ;
               if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
               if ( C2 == 0x7B ) {
                  pData->rt->nEncodeMode = EM_LOCAL ;
                  return GETC_ONLY_ESCAPE ;
               }
               if ( C2 == '\n' ) {
                  return GETC_ONLY_ESCAPE ;
               }
               if ( C2 != '~' ) StreamUnread ( C2, StreamIn ) ;
            }
         }
         else {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( C1 == '~' && C2 == 0x7D ) {
               pData->rt->nEncodeMode = EM_ASCII ;
               return GETC_ONLY_ESCAPE ;
            }
            if ( C1 >= 0x21 && C1 <= 0x7D && C2 >= 0x21 && C2 <= 0x7E ) {
               return ( ( C1 << 8 ) | C2 ) | 0x8080 ;
            }
            // caught illegal
            if ( C1 == 0x20 || C2 == 0x20 ) C1 = 0x20 ;
            else                            StreamUnread ( C2, StreamIn ) ;
         }

         if ( ismbblead_gb ( C1 ) ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            return ( C1 << 8 ) | C2 ;
         }
         return C1 ;
      }

      // ISO-2022-KR ̏
      case CP_ISO_KOREAN :
      {
         if ( C1 == 0x1B && ( ! IsIsoKrStrict || IsFirstByte ) ) {
            int C2, C3, C4 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( ( C3 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( ( C4 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            if ( C2 == '$' && C3 == ')' && C4 == 'C' ) {
               pData->rt->IsIsoKrEscFound = 1 ;
               return GETC_ONLY_ESCAPE ;
            }
            // caught illegal
            StreamUnread ( C2, StreamIn ) ;
            StreamUnread ( C3, StreamIn ) ;
            StreamUnread ( C4, StreamIn ) ;
            return C1 ;
         }

         if ( C1 == 0x0F ) {
            pData->rt->nEncodeMode = EM_ASCII ;
            return GETC_ONLY_ESCAPE ;
         }
         if ( C1 == 0x0E && ( ! IsIsoKrStrict || pData->rt->IsIsoKrEscFound ) ) {
            pData->rt->nEncodeMode = EM_LOCAL ;
            return GETC_ONLY_ESCAPE ;
         }
         if ( pData->rt->nEncodeMode == EM_LOCAL || ismbblead_ks ( C1 ) ) {
            int C2 ;
            if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
            return ( ( C1 << 8 ) | C2 ) | 0x8080 ;
         }
         return C1 ;
      }
   }

   return C1 ;
}



////////////////////////////////////////////
//           RTF ̓[eB           //
////////////////////////////////////////////



static STOREDCHARSW *StoreCharsExW ( int C, STOREDCHARSW *StoredChars ) ;

typedef struct {
   STOREDCHARSW   *StoredChars ;
   unsigned long  *AttribBuffer ;
} STOREDCHARSATTRW ;

static STOREDCHARSATTRW *StoreCharsAttrW ( int C, STOREDCHARSATTRW *StoredChars, unsigned long dwAttributes ) ;
static wchar_t *GetStoredCharsAttrW ( STOREDCHARSATTRW *StoredChars ) ;
static void ClearStoredCharsAttrW ( STOREDCHARSATTRW *StoredChars ) ;
static void FreeStoredCharsAttrW ( STOREDCHARSATTRW *StoredChars ) ;


static int PutRtfField ( STOREDCHARSATTRW *FieldInst, STOREDCHARSW *FieldResult, RTFCONVDATA *pData ) ;
static int FontNameToPseudoCodePageA ( int nCodePage, const char *szFontName ) ;
static int FontNameToPseudoCodePageW ( int nCodePage, const wchar_t *szFontName ) ;
static int FontNameToCharset ( const char *szFontName, int nDefaultCharset ) ;
static int PseudoCharToChar ( int C, int cUnicode, int *pSrcCodePage, int *pC2, RTFCONVDATA *pData ) ;

static int wcswordicmp ( const wchar_t *szString, const wchar_t *szKeyword ) ;



// b`eLXg̊e[x̏
typedef struct {
   int nFont ;                   // tHgԍ
   int nCharset ;                // Zbg
   int nCharToSkip ;             // UnicodêƃXLbv镶
   unsigned long dwAttributes ;  // 
   int nDefaultCodePage ;
} DEPTH_INFO ;


// for dwAttributes
#define RTF_FONT_TABLE         1
#define RTF_HIDDEN             2
#define RTF_FIELD           0x10
#define RTF_FIELD_INST      0x20
#define RTF_FIELD_RESULT    0x40
#define RTF_SHAPE          0x100


// for nFont
#define CP_DEFINE_CODEPAGE    0x0000FFFF     // R[hy[W̒`
#define CP_DEFINE_PSEUDOCODE  0x00FF0000     // [R[h̒`
#define CP_DEFINE_OPTIONAL    0x0F000000     // ɑ̎̒`Ȃ
#define CP_DEFINE_FONT        0xF0000000     // ̒` (for CHNCONV.DLL)

// for nFont (Chinese Writer & Korean Writer)
#define CP_JAPANESE_CW        ( CP_JAPANESE     +   0x10000 )
#define CP_JAPANESE_KW        ( CP_JAPANESE     +   0x20000 )
#define CP_JAPANESE_KW_OLD    ( CP_JAPANESE     +   0x50000 )
#define CP_JAPANESE_CW_FANTI  ( CP_JAPANESE_CW  + 0x1000000 )

// for nFont (cWnn & kWnn)
#define CP_JAPANESE_CWN       ( CP_JAPANESE     +   0x30000 )
#define CP_JAPANESE_KWN       ( CP_JAPANESE     +   0x40000 )
#define CP_JAPANESE_CWN_FANTI ( CP_JAPANESE_CWN + 0x1000000 )


// for nFont (Symbol)
#define CP_SYMBOL_DEFAULT     ( CP_SYMBOL + 0x10000 )    // Symbol
#define CP_SYMBOL_WINGDINGS   ( CP_SYMBOL + 0x20000 )    // Wingdings
#define CP_SYMBOL_IPA         ( CP_SYMBOL + 0x30000 )    // SIL IPA Encore
#define CP_SYMBOL_IPA93       ( CP_SYMBOL + 0x40000 )    // SIL IPA 93
#define CP_SYMBOL_IPAPHON     ( CP_SYMBOL + 0x50000 )    // IPAPhon
#define CP_SYMBOL_IPASAM      ( CP_SYMBOL + 0x60000 )    // Ipa-samd Uclphon1 SILDoulosL
#define CP_SYMBOL_GALATIA     ( CP_SYMBOL + 0x70000 )    // SIL Galatia
#define CP_SYMBOL_GALATIAEX   ( CP_SYMBOL + 0x80000 )    // SIL Galatia Extras
#define CP_SYMBOL_GREEKTRNS   ( CP_SYMBOL + 0x90000 )    // SIL Greek Trans
#define CP_SYMBOL_HEBTRANS    ( CP_SYMBOL + 0xA0000 )    // SIL Heb Trans
#define CP_SYMBOL_HEBTRANSCAPS ( CP_SYMBOL + 0xB0000 )   // SIL Heb Trans Caps
#define CP_SYMBOL_DAIBANNA    ( CP_SYMBOL + 0xC0000 )    // SIL Dai Banna
#define CP_SYMBOL_APPARATUS   ( CP_SYMBOL + 0xD0000 )    // SIL Apparatus
#define CP_SYMBOL_WINGREEK    ( CP_SYMBOL + 0xE0000 )    // WinGreek
#define CP_SYMBOL_ATHENIAN    ( CP_SYMBOL + 0xF0000 )    // Athenian
#define CP_SYMBOL_MIDEAST     ( CP_SYMBOL + 0x100000 )   // MidEastTimes


#define CP_MAC_EUROPEAN_PHONETIC    ( CP_MAC_EUROPEAN  + 0x10000 )   // Times New Roman Phonetic (MAC)
#define CP_ANSI_EUROPEAN_PHONETIC   ( CP_ANSI_EUROPEAN + 0x10000 )   // Times New Roman Phonetic
#define CP_ANSI_EUROPEAN_KULDIPA2   ( CP_ANSI_EUROPEAN + 0x20000 )   // KULDIPA2
#define CP_ANSI_EUROPEAN_SGREEK     ( CP_ANSI_EUROPEAN + 0x30000 )   // Sgreek
#define CP_ANSI_EUROPEAN_SPIONIC    ( CP_ANSI_EUROPEAN + 0x40000 )   // SPIonic
#define CP_ANSI_EUROPEAN_WINGREEK   ( CP_ANSI_EUROPEAN + 0x50000 )   // WinGreek
#define CP_ANSI_EUROPEAN_MOUNCE     ( CP_ANSI_EUROPEAN + 0x60000 )   // Mounce
#define CP_ANSI_EUROPEAN_TEKNIAGREEK ( CP_ANSI_EUROPEAN + 0x70000 )  // TekniaGreek
#define CP_ANSI_EUROPEAN_VNTIMES    ( CP_ANSI_EUROPEAN + 0x80000 )   // VnTimes
#define CP_ANSI_EUROPEAN_CIRILICA   ( CP_ANSI_EUROPEAN + 0x90000 )   // Serbian Orthodox Church


typedef struct {
   DEPTH_INFO *pTable ;
   int nTableSize ;     // number of items
   int nDepth ;
} DEPTH_INFO_TABLE ;

static DEPTH_INFO *GetDepthInfo ( DEPTH_INFO_TABLE *pDepthInfoTable, int nIncrease ) ;



typedef struct {
   int *pTable ;        // RTF ̃tHgԍɑΉii[鉿͊etHg̃R[hy[Wj
   int nTableSize ;     // number of items
} FONT_TABLE_INFO ;

static int GetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nDefaultCodePage ) ;
static int SetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nCodePage ) ;


static int GetRtfControlValue ( const char *szTag, const char *szKeyword, int *pValue ) ;

#define RTF_TAG_MAXLEN     0x80     // 32{



// b`eLXgeLXgt@Cϊ
//  0 Ԃ
// s 1 ȏ̎w肳ꂽ萔Ԃ
static int ConvertRtf ( RTFCONVDATA *pData ) {

   char szTag [ RTF_TAG_MAXLEN ] ;
   int IsRtfFormat = 0 ;            // 0: , 1: ς

   DEPTH_INFO_TABLE DepthInfoTable = { 0 } ;
   FONT_TABLE_INFO FontTableInfo = { 0 } ;

   STOREDCHARSA *FontName = NULL ;

   STOREDCHARSATTRW *FieldInst = NULL ;
   STOREDCHARSW *FieldResult = NULL ;

   int C ;
   int cLeadChar = 0 ;           // w肳ĂA C ͂QoCg
   int IsCodeDirect = 0 ;        // 𐔎łȂÂ܂ܒ
   int cUnicode = 0 ;
   int cUnicodeLead = 0 ;        // Unicode SoCg̐擪QoCg
   int nCharToSkip = 0 ;         // UnicodêƂ̃XLbv镶

   int C2 = 0 ;                  // [R[hp
   int cUnicode2 = 0 ;           // [R[hp

   unsigned long nCountChar = 0 ;


   // [e[u
   DEPTH_INFO *pDepthInfo = GetDepthInfo ( & DepthInfoTable, 0 ) ;
   if ( ! pDepthInfo ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
   pDepthInfo->nCharToSkip = 1 ;
   pDepthInfo->nFont = -1 ;
   pDepthInfo->nCharset = DEFAULT_CHARSET ;
   pDepthInfo->nDefaultCodePage = CP_ANSI_EUROPEAN ;



   while ( ( C = StreamRead ( pData->StreamIn ) ) != EOF ) {

      if ( pData->nError ) goto CAUGHT_ERROR ;

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }


      // oCi̔r
      if ( isascii ( C ) && iscntrl ( C ) && C != '\t' ) continue ;

      // [̐ݒ
      if ( ! IsCodeDirect || ! cLeadChar ) {
         if ( C == BRACE_END ) {

            pDepthInfo = GetDepthInfo ( & DepthInfoTable, -1 ) ;
            if ( ! pDepthInfo ) break ;

            if ( ! ( pDepthInfo->dwAttributes & RTF_FIELD ) && ( pDepthInfo + 1 )->dwAttributes & RTF_FIELD ) {
               if ( *GetStoredCharsAttrW ( FieldInst ) && ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) {
                  PutRtfField ( FieldInst, FieldResult, pData ) ;
               }
               ClearStoredCharsAttrW ( FieldInst ) ;
               ClearStoredCharsW ( FieldResult ) ;
            }
            if ( ! ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) && ( pDepthInfo + 1 )->dwAttributes & RTF_FONT_TABLE ) {
               // ʏ͎gȂ
               ClearStoredCharsA ( FontName ) ;
            }

            continue ;
         }
         if ( C == BRACE_START ) {

            pDepthInfo = GetDepthInfo ( & DepthInfoTable, 1 ) ;
            if ( ! pDepthInfo ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

            continue ;
         }
      }


      // GXP[v̏
      if ( C == ESCAPE ) {

         // QoCg𒼐ړ͂ꍇ̂QoCgڂ̉
         // ʏ͎gȂ
         if ( IsCodeDirect && cLeadChar && ismbbtrail_cp ( C, ( GetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, pDepthInfo->nDefaultCodePage ) ) & CP_DEFINE_CODEPAGE ) ) {
            goto PROCEDURE_OUTPUT_CHAR ;
         }


         if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;


         // ̎擾i16iR[hj
         if ( C == '\'' ) {
            int C0, C1 ;
            if ( ( C0 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            if ( ( C1 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            szTag [ 0 ] = C0 ;
            szTag [ 1 ] = C1 ;
            szTag [ 2 ] = 0 ;
            C = strtoul ( szTag, NULL, 0x10 ) ;
            IsCodeDirect = 0 ;
            goto PROCEDURE_OUTPUT_CHAR ;
         }

         // ^Oobt@Ɏ擾
         if ( isascii ( C ) && ( ispunct ( C ) || iscntrl ( C ) ) ) {
            szTag [ 0 ] = C ;
            szTag [ 1 ] = 0 ;
         }
         else {
            StreamUnread ( C, pData->StreamIn ) ;
            int nCount = 0 ;
            for ( ; nCount < RTF_TAG_MAXLEN - 1 ; nCount ++ ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( ! isascii ( C ) || ( isspace ( C ) || iscntrl ( C ) || ispunct ( C ) ) && C != '-' ) break ;
               szTag [ nCount ] = C ;
            }
            if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
            szTag [ nCount ] = 0 ;
         }

         if ( DEBUG_RTFTAG ) {
            IsRtfFormat = 1 ;
            wchar_t szTagW [ RTF_TAG_MAXLEN ] ;
            MultiByteToWideChar ( CP_ANSI_EUROPEAN, 0, szTag, -1, szTagW, RTF_TAG_MAXLEN ) ;
            Putws ( L"[", pData ) ;
            Putws ( szTagW, pData ) ;
            Putws ( L"]", pData ) ;
            continue ;
         }

         // ̎擾iLj
         for ( int nCount = 0 ; nCount < RTF_ESCAPE_LIST_NELEM ; nCount ++ ) {
            if ( ! strcmp ( szTag, ( RtfEscapeList [ nCount ] ).szName ) ) {
               int nCode = ( RtfEscapeList [ nCount ] ).nCode ;
               if ( isascii ( nCode ) || cLeadChar && nCode == 0xA0 ) C = nCode ;      // 0xA0 ̃P[X͒ʏ͎gȂ
               else                                                   cUnicode = nCode ;
               goto PROCEDURE_OUTPUT_CHAR ;
            }
         }

         // ̎擾iǂݎ݂̂̋Lj
         for ( int nCount = 0 ; nCount < RTF_ESCAPE_COMP_LIST_NELEM ; nCount ++ ) {
            if ( ! strcmp ( szTag, ( RtfEscapeCompatibleList [ nCount ] ).szName ) ) {
               int nCode = ( RtfEscapeCompatibleList [ nCount ] ).nCode ;
               if ( isascii ( nCode ) ) C = nCode ;
               else                     cUnicode = nCode ;
               goto PROCEDURE_OUTPUT_CHAR ;
            }
         }

         // tHgԍ̎擾
         if ( ! GetRtfControlValue ( szTag, "f", & pDepthInfo->nFont ) ) {
            // ʏ͎gȂ
            if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) ClearStoredCharsA ( FontName ) ;
            continue ;
         }

         // tHg̕Zbg擾
         if ( ! GetRtfControlValue ( szTag, "fcharset", & pDepthInfo->nCharset ) ) {
            // Ô
            if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) {
               if ( SetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, CharsetToCodePage ( pDepthInfo->nCharset ) ) ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
            }
            continue ;
         }

         // tHge[uubN̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) ) {
            if ( ! strcmp ( szTag, "fonttbl" ) ) {
               pDepthInfo->dwAttributes |= RTF_FONT_TABLE ;
               continue ;
            }
         }

         // ̎擾iUnicodej
         if ( ! GetRtfControlValue ( szTag, "u", & cUnicode ) ) {
            cUnicode = (wchar_t) cUnicode ;
            nCharToSkip = pDepthInfo->nCharToSkip ;
            goto PROCEDURE_OUTPUT_CHAR ;
         }

         // UnicodeŃXLbv镶̎擾
         if ( ! GetRtfControlValue ( szTag, "uc", & pDepthInfo->nCharToSkip ) ) continue ;

         // tB[h̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_FIELD ) ) {
            if ( ! strcmp ( szTag, "field" ) ) {
               pDepthInfo->dwAttributes |= RTF_FIELD ;
               continue ;
            }
         }
         else {
            if ( ! strcmp ( szTag, "fldinst" ) ) {
               pDepthInfo->dwAttributes |= RTF_FIELD_INST ;
               continue ;
            }
            if ( ! strcmp ( szTag, "*" ) ) {
               pDepthInfo->dwAttributes &= ~ RTF_FIELD_INST ;
               pDepthInfo->dwAttributes |= RTF_HIDDEN ;
               continue ;
            }
            if ( ! strcmp ( szTag, "fldrslt" ) ) {
               pDepthInfo->dwAttributes &= ~ RTF_FIELD_INST ;
               // FieldInst  EQ, SYMBOL ̂Ƃ FiledResult ɂȂ
               wchar_t *szString = GetStoredCharsAttrW ( FieldInst ) ;
               while ( isascii ( *szString ) && isspace ( *szString ) ) szString ++ ;
               if ( ! wcswordicmp ( szString, L"EQ" ) || ! wcswordicmp ( szString, L"SYMBOL" ) ) pDepthInfo->dwAttributes |= RTF_HIDDEN ;
               else                                                                              pDepthInfo->dwAttributes |= RTF_FIELD_RESULT ;
               continue ;
            }
         }

         // eLXg{bNX̃eLXg
         if ( ! ( pDepthInfo->dwAttributes & RTF_SHAPE ) ) {
            if ( ! strcmp ( szTag, "shp" ) ) {
               pDepthInfo->dwAttributes |= RTF_SHAPE ;
               continue ;
            }
         }
         else {
            if ( ! strcmp ( szTag, "shptxt" ) ) {
               pDepthInfo->dwAttributes &= ~ RTF_HIDDEN ;
               continue ;
            }
            if ( ! strcmp ( szTag, "shprslt" ) ) {
               pDepthInfo->dwAttributes |= RTF_HIDDEN ;
               continue ;
            }
         }

         // B̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) {
            for ( int nCount = 0 ; nCount < HIDDEN_TAG_LIST_NELEM ; nCount ++ ) {
               if ( ! strcmp ( szTag, HiddenTagList [ nCount ] ) ) {
                  if ( pData->dwConversionMode & CONVMODE_OUTPUT_HIDDEN && ! strcmp ( szTag, "v" ) ) goto PROCEDURE_ESCAPE_END ;
                  pDepthInfo->dwAttributes |= RTF_HIDDEN ;
                  goto PROCEDURE_ESCAPE_END ;
               }
            }
         }

         // 
         if ( ! IsRtfFormat ) {
            if ( GetRtfControlValue ( szTag, "rtf", NULL ) ) { pData->nError = CONV_ERROR_FORMAT ; goto CAUGHT_ERROR ; }
            IsRtfFormat = 1 ;
            if ( ! ( pData->dwReserved & RESERVEDMODE_NO_START ) ) WriteFileStart ( pData ) ;
            continue ;
         }

         // ̃R[hy[W̎擾
         if ( ! GetRtfControlValue ( szTag, "ansicpg", & pDepthInfo->nDefaultCodePage ) ) continue ;

         // ̃R[hy[W̎擾iʏ͎gȂj
         if ( ! strcmp ( szTag, "mac" ) ) {
            pDepthInfo->nDefaultCodePage = CP_MAC_EUROPEAN ;
            continue ;
         }
         if ( ! strcmp ( szTag, "pc" ) ) {
            pDepthInfo->nDefaultCodePage = CP_US ;
            continue ;
         }
         if ( ! strcmp ( szTag, "pca" ) ) {
            pDepthInfo->nDefaultCodePage = CP_EUROPEAN ;
            continue ;
         }

         // tHg̃R[hy[W擾iʏ͎gȂj
         if ( ! GetRtfControlValue ( szTag, "cpg", & pDepthInfo->nCharset ) ) continue ;

         if ( ! strcmp ( szTag, "j" ) ) {
            pDepthInfo->nCharset = SHIFTJIS_CHARSET ;
            continue ;
         }

         PROCEDURE_ESCAPE_END :
         continue ;
      }


      // ڂǂ
      IsCodeDirect = 1 ;


      PROCEDURE_OUTPUT_CHAR :

      if ( ! IsRtfFormat ) { pData->nError = CONV_ERROR_FORMAT ; goto CAUGHT_ERROR ; }

      if ( nCharToSkip && ! cUnicode ) {
         nCharToSkip -- ;
         continue ;
      }

      // tHge[ȕ
      if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) {
         if ( C == ';' ) {
            char *szFontName = GetStoredCharsA ( FontName ) ;
            if ( *szFontName == '@' ) szFontName ++ ;
            if ( pDepthInfo->nCharset == DEFAULT_CHARSET ) pDepthInfo->nCharset = FontNameToCharset ( szFontName, CodePageToCharset ( pDepthInfo->nDefaultCodePage ) ) ;
            int nCodePage = FontNameToPseudoCodePageA ( CharsetToCodePage ( pDepthInfo->nCharset ), szFontName ) ;
            ClearStoredCharsA ( FontName ) ;
            if ( pData->dwConversionMode & CONVMODE_NO_CW && nCodePage != CP_SYMBOL_DEFAULT && nCodePage != CP_SYMBOL_WINGDINGS ) nCodePage &= CP_DEFINE_CODEPAGE ;
            if ( SetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, nCodePage ) ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
            continue ;
         }
         // tHg̎擾
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) {
            FontName = StoreCharsA ( C, FontName ) ;
            if ( FontName == STOREDCHARS_ERROR ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         }
         continue ;
      }


      // Surrogate Pair
      if ( cUnicodeLead ) {
         if ( cUnicode && IsUnicodeLowSurrogate ( cUnicode ) ) {
            cUnicode = MAKE_UNICODE_SCALAR ( cUnicodeLead, cUnicode ) ;
            cUnicodeLead = 0 ;
         }
         else {
            if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) Putc ( cUnicodeLead, CP_UNICODE, pData ) ;
            cUnicodeLead = 0 ;
         }
      }
      if ( IsUnicodeHighSurrogate ( cUnicode ) ) {
         cUnicodeLead = cUnicode ;
         cUnicode = 0 ;
         continue ;
      }


      // R[hy[W擾
      int nSrcCodePage = GetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, pDepthInfo->nDefaultCodePage ) ;
      // tB[hpɁA[R[h̏O nSrcCodePage ۑĂ
      int nSrcCodePageSaved = nSrcCodePage ;

      // 擾
      if ( ! cUnicode ) {
         // QoCg̃[h
         if ( ! cLeadChar && ismbblead_cp ( C, nSrcCodePage & CP_DEFINE_CODEPAGE ) ) {
            cLeadChar = C ;
            continue ;
         }
         if ( cLeadChar ) {
            if ( ! ismbbtrail_cp ( C, nSrcCodePage & CP_DEFINE_CODEPAGE ) ) {
               if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) PutUnmappableChar ( REPLACEMENT_CHARACTER, "??", pData ) ;
               cLeadChar = 0 ;
            }
            else {
               C = ( cLeadChar << 8 ) | C ;
               cLeadChar = 0 ;
            }
         }
      }


      // [R[h̏
      if ( nSrcCodePage & CP_DEFINE_PSEUDOCODE ) {
         if ( ! ( pDepthInfo->dwAttributes & RTF_FIELD_INST ) ) {
            int cTmp2 ;
            int cTmp = PseudoCharToChar ( C, cUnicode, & nSrcCodePage, & cTmp2, pData ) ;
            if ( cTmp == EOF ) { pData->nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
            if ( cTmp ) {
               cUnicode = 0 ;
               if ( nSrcCodePage == CP_UNICODE ) cUnicode = cTmp ;
               else                              C = cTmp ;
            }
            if ( cTmp2 ) {
               cUnicode2 = 0 ;
               if ( nSrcCodePage == CP_UNICODE ) cUnicode2 = cTmp2 ;
               else                              C2 = cTmp2 ;
            }
         }
         //  nSrcCodePage ͋[R[h\Ȃ
         nSrcCodePage = nSrcCodePage & CP_DEFINE_CODEPAGE ;
      }


      // tB[h̏
      if ( pDepthInfo->dwAttributes & RTF_FIELD ) {
         int cUnicodeTmp ;
         if ( cUnicode ) cUnicodeTmp = cUnicode ;
         else {
            if ( nSrcCodePage == CP_SYMBOL ) cUnicodeTmp = C ;
            else {
               if ( nSrcCodePage == CP_JAPANESE ) C = SjisNec2ibm ( C ) ;
               cUnicodeTmp = CharToUnicode ( C, ( nSrcCodePage == CP_CHINESE_HKSCS ) ? CP_CHINESE_TRADITIONAL : nSrcCodePage, pData->dwConversionMode ) ;
               if ( cUnicodeTmp == EOF ) { pData->nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
            }
         }
         if ( pDepthInfo->dwAttributes & RTF_FIELD_INST ) {
            FieldInst = StoreCharsAttrW ( cUnicodeTmp, FieldInst, nSrcCodePageSaved ) ;
            if ( FieldInst == (STOREDCHARSATTRW*) -1 ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         }
         else if ( pDepthInfo->dwAttributes & RTF_FIELD_RESULT ) {
            FieldResult = StoreCharsExW ( cUnicodeTmp, FieldResult ) ;
            if ( FieldResult == (STOREDCHARSW*) -1 ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         }
      }


      // V{R[hy[W̓VXeϊłȂ̂
      if ( nSrcCodePage == CP_SYMBOL && ! cUnicode ) {
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) PutUnmappableChar ( ( C & 0xFF ) | 0xF000, "?", pData ) ;
         continue ;
      }


      // o
      if ( cUnicode ) {
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) Putc ( cUnicode, CP_UNICODE, pData ) ;
         cUnicode = 0 ;
         continue ;
      }
      if ( C ) {
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) Putc ( C, nSrcCodePage, pData ) ;
         continue ;
      }

      // o (2)
      if ( cUnicode2 ) {
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) Putc ( cUnicode2, CP_UNICODE, pData ) ;
         cUnicode2 = 0 ;
         continue ;
      }
      if ( C2 ) {
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) Putc ( C2, nSrcCodePage, pData ) ;
         continue ;
      }
   }

   CAUGHT_ERROR :

   // t@C̖ɌĂяo
   Putc ( EOF, CP_ANSI_EUROPEAN, pData ) ;
   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_END ) ) WriteFileEnd ( pData ) ;

   free ( FontTableInfo.pTable ) ;
   free ( DepthInfoTable.pTable ) ;
   FreeStoredCharsAttrW ( FieldInst ) ;
   FreeStoredCharsW ( FieldResult ) ;
   FreeStoredCharsA ( FontName ) ;

   if ( ! pData->nError && DepthInfoTable.nDepth ) pData->nError = CONV_ERROR_BRACE ;
   return pData->nError ;
}



#define INIT_DEPTHINFOTABLE_SIZE  0x20



static DEPTH_INFO *GetDepthInfo ( DEPTH_INFO_TABLE *pDepthInfoTable, int nIncrease ) {

   pDepthInfoTable->nDepth += nIncrease ;

   if ( nIncrease < 0 ) {
      if ( pDepthInfoTable->nDepth <= 0 ) return NULL ;
   }
   else {
      if ( pDepthInfoTable->nDepth >= pDepthInfoTable->nTableSize ) {

         if ( pDepthInfoTable->nDepth > USHRT_MAX ) return NULL ; // too large

         int nNewTableSize = pDepthInfoTable->nTableSize + INIT_DEPTHINFOTABLE_SIZE ;
         DEPTH_INFO *pNewTable = (DEPTH_INFO *) realloc ( pDepthInfoTable->pTable, nNewTableSize * sizeof(DEPTH_INFO) ) ;
         if ( ! pNewTable ) return NULL ;

         if ( ! pDepthInfoTable->nTableSize ) memzero ( pNewTable, sizeof(DEPTH_INFO) ) ;

         pDepthInfoTable->pTable = pNewTable ;
         pDepthInfoTable->nTableSize = nNewTableSize ;
      }
   }

   DEPTH_INFO *pCurrent = pDepthInfoTable->pTable + pDepthInfoTable->nDepth ;
   if ( nIncrease > 0 ) *pCurrent = *( pCurrent - 1 ) ;
   return pCurrent ;
}



// L[[hl擾
// pValue  NULL ł
// 擾 0 Ԃ
static int GetRtfControlValue ( const char *szTag, const char *szKeyword, int *pValue ) {

   int c1, c2 ;

   while ( 1 ) {
      c1 = *szTag ;
      c2 = *szKeyword ;
      if ( ! c2 && ( isdigit ( c1 ) || c1 == '-' || ! c1 ) ) {
         if ( pValue ) *pValue = (unsigned short) strtol ( szTag, NULL, 10 ) ;
         return 0 ;
      }
      if ( c1 != c2 ) break ;
      szTag ++ ;
      szKeyword ++ ;
   }

   return 1 ;
}



////////////////////////////////////////////
//          RTF tHge[u          //
////////////////////////////////////////////



#define INIT_FONTTABLE_SIZE      0x400



// tHg̃R[hy[WԂ
static int GetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nDefaultCodePage ) {

   int nCodePage ;
   if ( (unsigned) nFont < (unsigned) pFontTableInfo->nTableSize ) nCodePage = pFontTableInfo->pTable [ nFont ] ;
   else                                                            nCodePage = 0 ;

   if ( nCodePage ) return nCodePage ;
   if ( nDefaultCodePage ) return nDefaultCodePage ;
   return CP_ANSI_EUROPEAN ;
}



// tHg̃R[hy[Wݒ
//  0 As炻ȊOԂ
static int SetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nCodePage ) {

   if ( (unsigned) nFont > SHRT_MAX ) return 0 ;

   if ( nFont >= pFontTableInfo->nTableSize ) {

      int nNewTableSize = ( (unsigned) nFont / INIT_FONTTABLE_SIZE + 1 ) * INIT_FONTTABLE_SIZE ;
      int *pNewTable = (int *) realloc ( pFontTableInfo->pTable, nNewTableSize * sizeof(int) ) ;
      if ( ! pNewTable ) return 1 ;

      for ( int nCount = pFontTableInfo->nTableSize ; nCount < nNewTableSize ; nCount ++ ) pNewTable [ nCount ] = 0 ;

      pFontTableInfo->pTable = pNewTable ;
      pFontTableInfo->nTableSize = nNewTableSize ;
   }

   pFontTableInfo->pTable [ nFont ] = nCodePage ;

   return 0 ;
}



////////////////////////////////////////////
//     RTF ̓[eB̉֐     //
////////////////////////////////////////////



static int SymbolToUnicode ( int C, int nSrcCodePage, int *pC2 ) ;
static wchar_t *SearchPairParenthesis ( wchar_t **pString ) ;
static int PutwsAttr ( const wchar_t *szString, int nSrcCodePage, const unsigned long *pSrcCodePage, RTFCONVDATA *pData ) ;


typedef struct {
   unsigned long  dwAttributes ;
   wchar_t        *szString ;
} FIELD_STRING ;

#define FILED_STRING_ARRAY_SIZE  0x40


// for dwAttributes
#define FIELD_ATTRIB_S        1
#define FIELD_ATTRIB_UP       2
#define FIELD_ATTRIB_DO       4



// b`eLXg Field wo
// o͂ 1 ȏAo͂Ȃ 0 Ԃ
// FieldInst ͕K{BFieldResult  NULL łB
// FieldResult ͎QƂ邾ŁAo͂͂Ȃ
static int PutRtfField ( STOREDCHARSATTRW *FieldInst, STOREDCHARSW *FieldResult, RTFCONVDATA *pData ) {

   wchar_t *szInst = GetStoredCharsAttrW ( FieldInst ) ;
   unsigned long *InstCodePage = FieldInst->AttribBuffer ;
   wchar_t *szResult = GetStoredCharsW ( FieldResult ) ;


   if ( DEBUG_FIELD ) {
      Putws ( L"[FLDINST:", pData ) ;
      Putws ( szInst, pData ) ;
      Putws ( L"]", pData ) ;
      Putws ( L"[FLDRSLT:", pData ) ;
      Putws ( szResult, pData ) ;
      Putws ( L"]", pData ) ;
   }


   wchar_t *szString ;

   wchar_t *szInstStart = szInst ;

   // tB[h̃^O𕪗
   wchar_t *szFieldInstTag = wcstokex ( szInst, & szInst ) ;
   if ( ! szFieldInstTag ) return 0 ;


   // tB[h
   if ( ! wcsicmp ( szFieldInstTag, L"EQ" ) ) {

      int nCountOutput = 0 ;
      int nPseudoCodePage = 0 ;

      for ( szString = szInst ; *szString ; ) {
         while ( isascii ( *szString ) && isspace ( *szString ) ) szString ++ ;
         if ( *szString == '\\' ) {
            if ( *( szString + 1 ) != '*' ) break ;
            szString += 2 ;
            wchar_t *szToken = wcstokex ( szString, & szString ) ;
            if ( szToken && ! wcsheadicmp ( szToken, L"Font:" ) ) {
               wchar_t *szFontName = wcschr ( szToken, ':' ) + 1 ;
               nPseudoCodePage = FontNameToPseudoCodePageW ( 0, szFontName ) ;
            }
            continue ;
         }
         break ;
      }


      FIELD_STRING FieldString [ FILED_STRING_ARRAY_SIZE ] ;
      int nFieldStringMax = 0 ;
      int nCountMain = 0 ;
      int IsStartWithControl = 0 ;
      int IsOverwriteField = ! wcsheadicmp ( szString, L"\\o" ) || ! wcsheadicmp ( szString, L"\\x" ) ;

      // e̎擾
      unsigned long dwAttributes = 0 ;
      while ( nFieldStringMax < FILED_STRING_ARRAY_SIZE ) {

         wchar_t *szStart = szString ;

         while ( isascii ( *szString ) && isspace ( *szString ) ) szString ++ ;
         if ( ! wcsheadicmp ( szString, L"\\a" )   // z
           || ! wcsheadicmp ( szString, L"\\b" )   // 
           || ! wcsheadicmp ( szString, L"\\o" )   // dˑł
           || ! wcsheadicmp ( szString, L"\\x" )   // ͂
           || ! wcsheadicmp ( szString, L"\\s" )  && ( dwAttributes |= FIELD_ATTRIB_S )
           || ! wcsheadicmp ( szString, L"\\up" ) && ( dwAttributes |= FIELD_ATTRIB_UP )
           || ! wcsheadicmp ( szString, L"\\do" ) && ( dwAttributes |= FIELD_ATTRIB_DO )
         ) {
            if ( ! nFieldStringMax ) IsStartWithControl = 1 ;
            *szString ++ = 0 ;
            wchar_t *szEnd = SearchPairParenthesis ( & szString ) ;
            if ( ! szEnd ) break ;
            *szEnd = WEOF ;
            continue ;
         }

         ( FieldString [ nFieldStringMax ] ).szString = szStart ;
         ( FieldString [ nFieldStringMax ] ).dwAttributes = dwAttributes ;

         nFieldStringMax ++ ;
         if ( ! dwAttributes ) nCountMain ++ ;
         dwAttributes = 0 ;

         szString = wcspbrk ( szString, L",\\" ) ;
         if ( ! szString ) break ;
         if ( *szString != '\\' ) *szString ++ = 0 ;
      }

      if ( ! IsStartWithControl ) return nCountOutput ;


      // ̘͂g̍폜
      if ( IsOverwriteField && nCountMain > 1 ) {
         for ( int nCount = 0 ; nCount < nFieldStringMax ; nCount ++ ) {
            if ( ! ( FieldString [ nCount ] ).dwAttributes && ( FieldString [ nCount ] ).szString ) {
               if ( ( ( *( FieldString [ nCount ] ).szString ) & 0xFF00 ) == 0x2500 ) ( FieldString [ nCount ] ).szString = NULL ;
            }
         }
      }

      // ȑo
      if ( IsOverwriteField && nCountMain ) {

         for ( int nCount = 0 ; nCount < nFieldStringMax ; nCount ++ ) {
            if ( ! ( FieldString [ nCount ] ).dwAttributes && ( FieldString [ nCount ] ).szString ) {
               PutwsAttr ( ( FieldString [ nCount ] ).szString, 0, InstCodePage + ( ( FieldString [ nCount ] ).szString - szInstStart ), pData ) ;
               ( FieldString [ nCount ] ).szString = NULL ;
               nCountOutput ++ ;
            }
         }

         if ( pData->dwConversionMode & CONVMODE_OUTPUT_RUBY ) {
            for ( int nCount = 0 ; nCount < nFieldStringMax ; nCount ++ ) {
               if ( ( FieldString [ nCount ] ).szString ) {
                  Putws ( L"(", pData ) ;
                  PutwsAttr ( ( FieldString [ nCount ] ).szString, nPseudoCodePage, NULL, pData ) ;
                  Putws ( L")", pData ) ;
                  ( FieldString [ nCount ] ).szString = NULL ;
                  nCountOutput ++ ;
               }
            }
         }

      }
      // gݕ̏o  iIsOverwriteField 0 ȊȌꍇj
      // zAʂ̏óiIsOverwriteField 0 ̏ꍇj
      else {
         for ( int nCount = 0 ; nCount < nFieldStringMax ; nCount ++ ) {
            if ( ( FieldString [ nCount ] ).szString ) {
               PutwsAttr ( ( FieldString [ nCount ] ).szString, 0, InstCodePage + ( ( FieldString [ nCount ] ).szString - szInstStart ), pData ) ;
               if ( ! IsOverwriteField ) Putc ( L'\n', CP_UNICODE, pData ) ;
               ( FieldString [ nCount ] ).szString = NULL ;
               nCountOutput ++ ;
            }
         }
      }

      return nCountOutput ;
   }


   // V{tB[h
   if ( ! wcsicmp ( szFieldInstTag, L"SYMBOL" ) ) {

      szString = szInst ;
      int nSrcCodePage = 0 ;

      wchar_t *szToken = wcstokex ( szString, & szString ) ;
      if ( ! szToken ) return 0 ;
      if ( ! isascii ( *szToken ) || ! isdigit ( *szToken ) ) return 0 ;

      int C = (int) wcstoul ( szToken, & szToken, 10 ) ;

      while ( 1 ) {
         wchar_t *szToken = wcstokex ( szString, & szString ) ;
         if ( ! szToken ) break ;
         if ( ! wcsicmp ( szToken, L"\\f" ) ) {
            wchar_t *szFontName = wcstokex ( szString, & szString ) ;
            if ( ! szFontName ) break ;
            nSrcCodePage = FontNameToPseudoCodePageW ( CP_SYMBOL, szFontName ) ;
            continue ;
         }
      }

      if ( pData->dwConversionMode & CONVMODE_NO_CW && nSrcCodePage != CP_SYMBOL_DEFAULT && nSrcCodePage != CP_SYMBOL_WINGDINGS ) nSrcCodePage &= CP_DEFINE_CODEPAGE ;

      int cUnicode2 ;
      int cUnicode = SymbolToUnicode ( C, nSrcCodePage, & cUnicode2 ) ;

      if ( cUnicode ) Putc ( cUnicode, CP_UNICODE, pData ) ;
      else            PutUnmappableChar ( ( C & 0xFF ) | 0xF000, "?", pData ) ;

      if ( cUnicode2 ) Putc ( cUnicode2, CP_UNICODE, pData ) ;

      return 1 ;
   }


   // nCp[NtB[h
   if ( ! wcsicmp ( szFieldInstTag, L"HYPERLINK" ) ) {
      if ( pData->dwConversionMode & CONVMODE_OUTPUT_LINK ) {

         szString = szInst ;

         wchar_t *szLinkName = wcstokex ( szString, & szString ) ;
         if ( ! szLinkName ) return 0 ;
         wchar_t *szLinkNameTail = wcsend ( szLinkName ) ;

         while ( 1 ) {
            wchar_t *szToken = wcstokex ( szString, & szString ) ;
            if ( ! szToken ) break ;
            if ( ! wcsicmp ( szToken, L"\\l" ) ) {
               *szLinkNameTail = L'#' ;
               wcsmove ( szLinkNameTail + 1, szToken ) ;
               break ;
            }
         }

         static const wchar_t szFieldMailto [] = L"mailto:" ;
         if ( ! wcsheadicmp ( szLinkName, szFieldMailto ) ) wcsmove ( szLinkName, szLinkName + wcslen ( szFieldMailto ) ) ;

         for ( szString = szLinkName ; ( szString = wcsstr ( szString, L"\\\\" ) ) ; ) wcsmove ( szString, szString + 1 ) ;

         if ( szResult ) {
            const wchar_t *szLinkNameInResult = wcsstr ( szResult, szLinkName ) ;
            if ( szLinkNameInResult ) {
               int C = *( szLinkNameInResult + wcslen ( szLinkName ) ) ;
               if ( isascii ( C ) && ( ! isgraph ( C ) || C == '\'' || C == '\"' || C == '|' || C == '<' || C == '>' || C == '(' || C == ')' || C == '[' || C == ']' ) ) return 0 ;
            }
         }

         Putws ( L"(", pData ) ;
         Putws ( szLinkName, pData ) ;
         Putws ( L")", pData ) ;
         return 1 ;
      }
   }

   return 0 ;
}



// 񂩂玟̍ʂAyAɂȂEʂԂ
// ʂ̎̃̕AhX pString Ɋi[
// yÅʂȂ NULL Ԃ
static wchar_t *SearchPairParenthesis ( wchar_t **pString ) {

   wchar_t *string = wcschr ( *pString, L'(' ) ;
   if ( ! string ) return NULL ;

   *pString = ++ string ;

   for ( int depth = 0 ; *string ; string ++ ) {
      if ( *string == '(' ) depth ++ ;
      if ( *string == ')' ) {
         if ( ! depth ) return string ;
         depth -- ;
      }
   }

   return NULL ;
}



// [R[hwɏ]ďo
//  WEOF ͏o͂
static int PutwsAttr ( const wchar_t *szString, int nSrcCodePage, const unsigned long *pSrcCodePage, RTFCONVDATA *pData ) {

   for ( ; *szString ; szString ++ ) {

      int C ;
      int C2 = 0 ;
      int nCodePage = ( pSrcCodePage ) ? *pSrcCodePage ++ : nSrcCodePage ;
      if ( ! nCodePage ) nCodePage = CP_UNICODE ;

      if ( *szString == WEOF ) continue ;

      if ( nCodePage & CP_DEFINE_PSEUDOCODE ) {
         C = PseudoCharToChar ( 0, *szString, & nCodePage, & C2, pData ) ;
         if ( C == EOF ) {
            pData->nError = CONV_ERROR_CODEPAGE ;
            return 0 ;
         }
      }
      else {
         if ( IsUnicodeHighSurrogate ( *szString ) && IsUnicodeLowSurrogate ( *( szString + 1 ) ) ) {
            C = MAKE_UNICODE_SCALAR ( *szString, *( szString + 1 ) ) ;
            szString ++ ;
            if ( pSrcCodePage ) pSrcCodePage ++ ;
            nCodePage = CP_UNICODE ;
         }
         else {
            C = *szString ;
            if ( nCodePage != CP_UNICODE ) {
               int cTmp = UnicodeToChar ( C, ( nCodePage == CP_CHINESE_HKSCS ) ? CP_CHINESE_TRADITIONAL : nCodePage, pData->dwConversionMode ) ;
               if ( cTmp == EOF ) {
                  pData->nError  = CONV_ERROR_CODEPAGE ;
                  return 0 ;
               }
               if ( cTmp ) C = cTmp ;
               else        nCodePage = CP_UNICODE ;
            }
         }
      }

      if ( nCodePage == CP_SYMBOL ) {
         if ( ! C ) C = *szString ;
         PutUnmappableChar ( ( C & 0xFF ) | 0xF000, "?", pData ) ;
         continue ;
      }

      Putc ( C, nCodePage, pData ) ;
      if ( C2 ) Putc ( C2, nCodePage, pData ) ;
   }

   return 0 ;
}



// tHg[R[h擾iANSIŁj
// R[hy[WȂ nCodePage  0 w肷邱
// YtHgȂ΁AnCodePage ̂܂ܕԂ
static int FontNameToPseudoCodePageA ( int nCodePage, const char *szFontName ) {

   wchar_t szTmp [ FONTNAME_MAX ] ;

   MultiByteToWideChar ( ( nCodePage && nCodePage != CP_SYMBOL && IsValidCodePage ( nCodePage ) ) ? nCodePage : GetACP (), 0, szFontName, -1, szTmp, FONTNAME_MAX ) ;
   wcstrunc ( szTmp, FONTNAME_MAX - 1 ) ; // Ô

   return FontNameToPseudoCodePageW ( nCodePage, szTmp ) ;
}



// tHg[R[h擾iUNICODEŁj
// R[hy[WȂ nCodePage  0 w肷邱
// YtHgȂ΁AnCodePage ̂܂ܕԂ
static int FontNameToPseudoCodePageW ( int nCodePage, const wchar_t *szFontName ) {

   if ( wcsstr ( szFontName, szUnicode ) ) return nCodePage ;

   // Chinese/Korean Writer
   if ( ! nCodePage || nCodePage == CP_JAPANESE ) {
      if ( ! wcsheadcmp ( szFontName, szCwHead ) ) return CP_JAPANESE_CW ;
      if ( ! wcsheadcmp ( szFontName, szCwFantiHead ) ) return CP_JAPANESE_CW_FANTI ;
      if ( ! wcsheadcmp ( szFontName, szCw7Head ) ) return CP_JAPANESE_CW ;
      if ( ! wcsheadcmp ( szFontName, szCw7FantiHead ) ) return CP_JAPANESE_CW_FANTI ;
      if ( ! wcsheadcmp ( szFontName, szKwHead ) && wcsstr ( szFontName, szKwName ) ) return CP_JAPANESE_KW ;
      if ( ! wcsheadcmp ( szFontName, szKwHeadOld ) && wcsstr ( szFontName, szKwName ) ) return CP_JAPANESE_KW_OLD ;
      if ( wcsstr ( szFontName, szKwCompat ) ) return CP_JAPANESE_KW ;
      if ( ! wcsheadcmp ( szFontName, szCwnHead ) ) return CP_JAPANESE_CWN ;
      if ( ! wcsheadcmp ( szFontName, szCwnFantiHead ) ) return CP_JAPANESE_CWN_FANTI ;
      if ( ! wcsheadcmp ( szFontName, szKwnHead ) ) return CP_JAPANESE_KWN ;
      if ( nCodePage ) return nCodePage ;
   }

   // Symbol, Wingdings, SIL, etc
   if ( ! nCodePage || nCodePage == CP_SYMBOL ) {
      if ( ! wcscmp ( szFontName, szSymbolFont ) ) return CP_SYMBOL_DEFAULT ;
      if ( ! wcscmp ( szFontName, szWingdingsFont ) ) return CP_SYMBOL_WINGDINGS ;
      if ( ! wcsheadcmp ( szFontName, szSil ) ) {
         if ( wcsstr ( szFontName, szIpa ) ) {
            if ( wcsstr ( szFontName, sz93 ) ) return CP_SYMBOL_IPA93 ;
            else                               return CP_SYMBOL_IPA ;
         }
         if ( wcsstr ( szFontName, szGalatiaExtras ) ) return CP_SYMBOL_GALATIAEX ;
         if ( wcsstr ( szFontName, szGalatia ) ) return CP_SYMBOL_GALATIA ;
         if ( wcsstr ( szFontName, szGreekTrans ) ) return CP_SYMBOL_GREEKTRNS ;
         if ( wcsstr ( szFontName, szHebTransCaps ) ) return CP_SYMBOL_HEBTRANSCAPS ;
         if ( wcsstr ( szFontName, szHebTrans ) ) return CP_SYMBOL_HEBTRANS ;
         if ( wcsstr ( szFontName, szDaiBanna ) ) return CP_SYMBOL_DAIBANNA ;
         if ( wcsstr ( szFontName, szApparatus ) ) return CP_SYMBOL_APPARATUS ;
      }
      if ( ! wcsheadcmp ( szFontName, szIpaphon ) ) return CP_SYMBOL_IPAPHON ;
      if ( ! wcsheadcmp ( szFontName, szIpasam ) ) return CP_SYMBOL_IPASAM ;
      if ( ! wcsheadicmp ( szFontName, szGreek ) ) return CP_SYMBOL_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szAthenian ) ) return CP_SYMBOL_ATHENIAN ;
      if ( ! wcsheadicmp ( szFontName, szMidEastTimes ) ) return CP_SYMBOL_MIDEAST ;

      if ( nCodePage ) return nCodePage ;
   }

   // Times New Roman Phonetic, Sgreek, etc
   if ( ! nCodePage || nCodePage == CP_ANSI_EUROPEAN ) {
      if ( ! wcsheadicmp ( szFontName, szTimesNewRomanPhonetic ) ) return CP_ANSI_EUROPEAN_PHONETIC ;
      if ( ! wcsheadicmp ( szFontName, szSgreek ) ) return CP_ANSI_EUROPEAN_SGREEK ;
      if ( ! wcsheadicmp ( szFontName, szSGkClassic ) ) return CP_ANSI_EUROPEAN_SGREEK ;
      if ( ! wcsheadicmp ( szFontName, szSPIonic ) ) return CP_ANSI_EUROPEAN_SPIONIC ;
      if ( ! wcsheadicmp ( szFontName, szGreek ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szStandardGreek ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szMilanGreek ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szAngaros ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szKorinthus ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szGrecsDuRoiWg ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsheadicmp ( szFontName, szGrammata ) ) return CP_ANSI_EUROPEAN_WINGREEK ;
      if ( ! wcsicmp ( szFontName, szMounce ) ) return CP_ANSI_EUROPEAN_MOUNCE ;
      if ( ! wcsheadicmp ( szFontName, szTekniaGreek ) ) return CP_ANSI_EUROPEAN_TEKNIAGREEK ;
      if ( ! wcsheadcmp ( szFontName, szTcvn ) ) return CP_VIETNAMESE_TCVN ;
      if ( ! wcsheadcmp ( szFontName, szVps ) ) return CP_VIETNAMESE_VPS ;
      if ( ! wcsheadcmp ( szFontName, szVni ) ) return CP_VIETNAMESE_VNI ;
      if ( ! wcsheadcmp ( szFontName, szViscii1 ) ) return CP_VIETNAMESE_VISCII ;
      if ( ! wcsheadcmp ( szFontName, szViscii2 ) ) return CP_VIETNAMESE_VISCII ;
      for ( int nCount = 0 ; nCount < VISCII_TRICHLO_LIST_NELEM ; nCount ++ ) { if ( ! wcsheadcmp ( szFontName, VisciiTrichloList [ nCount ] ) ) return CP_VIETNAMESE_VISCII ; }
      if ( ! wcscmp ( szFontName, szVnTimes ) ) return CP_ANSI_EUROPEAN_VNTIMES ;
      if ( wcsstr ( szFontName, szCirilica ) ) return CP_ANSI_EUROPEAN_CIRILICA ;
      if ( nCodePage ) return nCodePage ;
   }

   // Times New Roman Phonetic (MAC)
   if ( ! nCodePage || nCodePage == CP_MAC_EUROPEAN ) {
      if ( ! wcsheadicmp ( szFontName, szTimesNewRomanPhonetic ) ) return CP_MAC_EUROPEAN_PHONETIC ;
      if ( nCodePage ) return nCodePage ;
   }

   // HKSCS
   if ( ! nCodePage || nCodePage == CP_CHINESE_TRADITIONAL ) {
      if ( wcsstri ( szFontName, szHkscsName ) ) return CP_CHINESE_HKSCS ;
      if ( nCodePage ) return nCodePage ;
   }

   return nCodePage ;
}



static int Cw2gb ( int cCw ) ;
static int Kg2gb ( int cCw ) ;
static int Kw2ks ( int cKw ) ;
static int Cwn2cw ( int cCwn ) ;
static int Kwn2ks ( int cKwn ) ;



// R[hy[W̃G[Ȃ EOF Ԃ
// ϊłȂ 0 Ԃ
static int PseudoCharToChar ( int C, int cUnicode, int *pSrcCodePage, int *pC2, RTFCONVDATA *pData ) {

   int nSrcCodePage = *pSrcCodePage ;
   *pSrcCodePage = *pSrcCodePage & CP_DEFINE_CODEPAGE ;

   *pC2 = 0 ;

   // Unicode]R[h
   if ( cUnicode ) {
      if ( ( nSrcCodePage & CP_DEFINE_CODEPAGE ) == CP_SYMBOL ) {
         C = cUnicode & 0xFF ;
         cUnicode = 0 ;
      }
      else {
         C = UnicodeToChar ( cUnicode, nSrcCodePage & CP_DEFINE_CODEPAGE, pData->dwConversionMode ) ;
         if ( C == EOF ) return EOF ;
         if ( C ) cUnicode = 0 ;
      }
   }


   // {n̋[R[h
   if ( ( nSrcCodePage & CP_DEFINE_CODEPAGE ) == CP_JAPANESE ) {

      if ( C != 0xEEF9 ) C = SjisNec2ibm ( C ) ;

      int IsConvertViaCw = 0 ;

      // cWnn  GB ܂ Chinese Writer ɕϊ
      if ( ( nSrcCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == CP_JAPANESE_CWN ) {
         if ( nSrcCodePage == CP_JAPANESE_CWN_FANTI || ! ( C >= 0xF090 && C <= 0xF4FF ) ) {
            int cCw = 0 ;
            if ( nSrcCodePage != CP_JAPANESE_CWN_FANTI && ( C >= 0xF540 && C <= 0xF65F ) ) {
               if ( ( C >> 8 ) == 0xF5 ) cCw = Cwn2cwTable_NoFanti_F5 [ ( C & 0xFF ) - 0x40 ] ;
               if ( ( C >> 8 ) == 0xF6 ) cCw = Cwn2cwTable_NoFanti_F6 [ ( C & 0xFF ) - 0x40 ] ;
            }
            if ( ! cCw && nSrcCodePage == CP_JAPANESE_CWN_FANTI ) {
               int cGb = Cwn2gbFanti_NoCwOnly ( C ) ;
               if ( cGb ) {
                  *pSrcCodePage = CP_CHINESE_SIMPLIFIED ;
                  return cGb ;
               }
            }
            if ( ! cCw ) {
               int cUnicode = Cwn2uc_NoCwOnly ( C ) ;
               if ( cUnicode ) {
                  *pSrcCodePage = CP_UNICODE ;
                  return cUnicode ;
               }
               int cGb = Cwn2gb_NoCwOnly ( C ) ;
               if ( cGb ) {
                  *pSrcCodePage = CP_CHINESE_SIMPLIFIED ;
                  return cGb ;
               }
               cCw = Cwn2cw ( C ) ;
            }
            if ( cCw ) {
               C = cCw ;
               if ( nSrcCodePage == CP_JAPANESE_CWN_FANTI ) nSrcCodePage = CP_JAPANESE_CW_FANTI ;
               else                                         nSrcCodePage = CP_JAPANESE_CW ;
               IsConvertViaCw = 1 ;
               // fall through
            }
         }
      }

      // Chinese Writer ̏
      if ( ( nSrcCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == CP_JAPANESE_CW ) {
         if ( nSrcCodePage == CP_JAPANESE_CW_FANTI ) {
            int cUnicode = Kg2uc_NoGbOnly ( C ) ;
            if ( cUnicode ) {
               *pSrcCodePage = CP_UNICODE ;
               return cUnicode ;
            }
         }
         if ( C >= 0x8740 && ( nSrcCodePage == CP_JAPANESE_CW_FANTI || ! ( C >= 0xF040 && C <= 0xF4FF ) || IsConvertViaCw ) ) {
            int cGb ;
            if ( nSrcCodePage == CP_JAPANESE_CW_FANTI ) cGb = Kg2gb ( C ) ;
            else                                        cGb = Cw2gb ( C ) ;
            if ( cGb ) {
               *pSrcCodePage = CP_CHINESE_SIMPLIFIED ;
               return cGb ;
            }
         }
         if ( C >= 0x80 && C <= 0xFF ) {
            int cUnicode = CwOneByteCharToUnicode_0080_00FF [ C - 0x80 ] ;
            // GB13000Վ̏
            if ( ! ( pData->dwAdditionalMode & ADDITIONAL_MODE_GBK_TO_LINSHIDAIMA ) ) {
               int cNonLinshiDaima = GetNonLinshiDaima ( cUnicode ) ;
               if ( cNonLinshiDaima ) cUnicode = cNonLinshiDaima ;
            }
            *pSrcCodePage = CP_UNICODE ;
            return cUnicode ;
         }
         if ( isascii ( C ) ) {
            *pSrcCodePage = CP_CHINESE_SIMPLIFIED ;
            return C ;
         }
         return C ;
      }

      // Korean Writer ̏
      if ( nSrcCodePage == CP_JAPANESE_KW ) {
         if ( C >= 0xF5F9 && C <= 0xF5FC ) {
            int cUnicode = 0 ;
            if      ( C == 0xF5F9 ) cUnicode = 0x217A ;
            else if ( C == 0xF5FA ) cUnicode = 0x217B ;
            else if ( C == 0xF5FB ) cUnicode = 0x216A ;
            else if ( C == 0xF5FC ) cUnicode = 0x216B ;
            *pSrcCodePage = CP_UNICODE ;
            return cUnicode ;
         }
         if ( C >= 0x8800 ) {
            int cKs = Kw2ks ( C ) ;
            if ( cKs ) {
               *pSrcCodePage = CP_KOREAN ;
               return cKs ;
            }
         }
         if ( isascii ( C ) ) {
            *pSrcCodePage = CP_KOREAN ;
            return C ;
         }
         return C ;
      }
      if ( nSrcCodePage == CP_JAPANESE_KW_OLD ) {
         if ( C >= 0x8000 ) {
            int cKs = jmstojis ( SjisIbm2nec ( C ) ) ;
            if ( cKs ) {
               cKs |= 0x8080 ;
               *pSrcCodePage = CP_KOREAN ;
               return cKs ;
            }
         }
         if ( isascii ( C ) ) {
            *pSrcCodePage = CP_KOREAN ;
            return C ;
         }
         return C ;
      }

      // kWnn ̏
      if ( nSrcCodePage == CP_JAPANESE_KWN ) {
         if ( C >= 0x80 ) {
            int cKs = Kwn2ks ( C ) ;
            if ( cKs ) {
               *pSrcCodePage = CP_KOREAN ;
               return cKs ;
            }
         }
         if ( isascii ( C ) ) {
            *pSrcCodePage = CP_KOREAN ;
            return C ;
         }
         return C ;
      }

      return 0 ;
   }


   // Symbol ̏
   if ( ( nSrcCodePage & CP_DEFINE_CODEPAGE ) == CP_SYMBOL ) {

      int cUnicode = SymbolToUnicode ( C, nSrcCodePage, pC2 ) ;

      if ( cUnicode ) {
         *pSrcCodePage = CP_UNICODE ;
         return cUnicode ;
      }

      return C ;
   }


   // PoCgn̋[R[h
   if ( ( nSrcCodePage & CP_DEFINE_CODEPAGE ) == CP_ANSI_EUROPEAN || ( nSrcCodePage & CP_DEFINE_CODEPAGE ) == CP_MAC_EUROPEAN ) {

      *pSrcCodePage = CP_UNICODE ;

      switch ( nSrcCodePage ) {
         case CP_ANSI_EUROPEAN_PHONETIC :
         case CP_MAC_EUROPEAN_PHONETIC :
            if ( cUnicode ) return TimesNewRomanPhoneticToUnicode ( CP_UNICODE, cUnicode, pData->dwConversionMode ) ;
            else            return TimesNewRomanPhoneticToUnicode ( nSrcCodePage & CP_DEFINE_CODEPAGE, C, pData->dwConversionMode ) ;
      }

      if ( (unsigned) C >= 0x100 ) return 0 ;

      switch ( nSrcCodePage ) {
         case CP_ANSI_EUROPEAN_SGREEK :      *pC2 = SgreekToUnicodeTableSub [ C ] ; return SgreekToUnicodeTableMain [ C ] ;
         case CP_ANSI_EUROPEAN_SPIONIC :     *pC2 = SpionicToUnicodeTableSub [ C ] ; return SpionicToUnicodeTableMain [ C ] ;
         case CP_ANSI_EUROPEAN_WINGREEK :    return WinGreekToUnicodeTable [ C ] ;
         case CP_ANSI_EUROPEAN_MOUNCE :      return MounceToUnicodeTable [ C ] ;
         case CP_ANSI_EUROPEAN_TEKNIAGREEK : *pC2 = TekniaGreekToUnicodeTableSub [ C ] ; return TekniaGreekToUnicodeTableMain [ C ] ;
         case CP_ANSI_EUROPEAN_VNTIMES :     return VnTimesToUnicodeTable [ C ] ;
         case CP_ANSI_EUROPEAN_CIRILICA :    return CirilicaToUnicodeTable [ C ] ;
      }
   }


   *pSrcCodePage = CP_SYMBOL ;
   return 0 ;
}



// V{tHg Unicode Ԃ
// ϊłȂ 0 Ԃ
static int SymbolToUnicode ( int C, int nSrcCodePage, int *pC2 ) {

   *pC2 = 0 ;

   if ( (unsigned) C >= 0x100 ) return 0 ;

   switch ( nSrcCodePage ) {
      case CP_SYMBOL_DEFAULT :      return SymbolToUnicodeTable [ C ] ;
      case CP_SYMBOL_WINGDINGS :    return WingdingsToUnicodeTable [ C ] ;
      case CP_SYMBOL_IPA :          return SilIpaToUnicodeTable [ C ] ;
      case CP_SYMBOL_IPA93 :        return SilIpa93ToUnicodeTable [ C ] ;
      case CP_SYMBOL_GALATIA :      return GalatiaToUnicodeTable [ C ] ;
      case CP_SYMBOL_GALATIAEX :    return GalatiaExToUnicodeTable [ C ] ;
      case CP_SYMBOL_GREEKTRNS :    *pC2 = GreekTransToUnicodeTableSub [ C ] ; return GreekTransToUnicodeTableMain [ C ] ;
      case CP_SYMBOL_HEBTRANS :     *pC2 = HebTransToUnicodeTableSub [ C ] ; return HebTransToUnicodeTableMain [ C ] ;
      case CP_SYMBOL_HEBTRANSCAPS : *pC2 = HebTransCapsToUnicodeTableSub [ C ] ; return HebTransCapsToUnicodeTableMain [ C ] ;
      case CP_SYMBOL_DAIBANNA :     return DaiBannaToUnicodeTable [ C ] ;
      case CP_SYMBOL_APPARATUS :    return ApparatusToUnicodeTable [ C ] ;
      case CP_SYMBOL_IPAPHON :      return IpaphonToUnicodeTable [ C ] ;
      case CP_SYMBOL_IPASAM :       return IpasamToUnicodeTable [ C ] ;
      case CP_SYMBOL_WINGREEK :     return WinGreekToUnicodeTable [ C ] ;
      case CP_SYMBOL_ATHENIAN :     return AthenianToUnicodeTable [ C ] ;
      case CP_SYMBOL_MIDEAST :      return MidEastTimesToUnicodeTable [ C ] ;
   }

   return 0 ;
}



////////////////////////////////////////////
//             o̓[eB             //
////////////////////////////////////////////



static int PutcTxt ( int C, int cUnicode, int nSrcCodePage, const char *szUnknown, RTFCONVDATA *pData ) ;
static int PutcRtf ( int C, int cUnicode, int nSrcCodePage, const char *szUnknown, RTFCONVDATA *pData ) ;

static int ZenSymbolToHanSymbol ( int cUnicode ) ;
static int ZenAlphaToHanAlpha ( int cUnicode ) ;
static int HanKanaToZenKana ( int cUnicode ) ;
static int ZenKanaToDakuon ( int cUnicode ) ;
static int ZenKanaToHanDakuon ( int cUnicode ) ;
static int SjisToUnicode_Mac ( int cSjis, int cUnicode, const wchar_t **pszUnicode ) ;
static int UnicodeToSjis_Mac ( int cUnicode, int cSjis ) ;
static int SjisToUnicode_Org ( int cSjis, int cUnicode ) ;
static int UnicodeToSjis_Org ( int cUnicode, int cSjis ) ;
static int SjisToUnicode_2004 ( int cSjis, int cUnicode, const wchar_t **pszUnicode, unsigned long dwConversionMode ) ;
static int UnicodeToSjis_2004 ( int cUnicode, int cSjis, const wchar_t *pszUnicode, unsigned long dwConversionMode ) ;
static int DecodeVietnameseVni ( int C, RTFCONVDATA *pData ) ;
static int EncodeVietnamese ( int cUnicode, RTFCONVDATA *pData ) ;



// o
// ċAĂяo
// nSrcCodePage ̓[ĥȂR[ĥ
// Unicode  Scalar 
// GBX SoCg C  SurrogatePair 
// JIS⏕ + 0x10000 
// StreamOut  NULL ȂΕԂďIiUnicodeȂXJ[lj
// ȊO 0 Ԃ
// G[ StreamOut ɃG[lۑ
// t@C̖ŕK C  EOF w肵ČĂяo
static int Putc ( int C, int nSrcCodePage, RTFCONVDATA *pData ) {

   int nDstCodePage = pData->nDstCodePage ;
   STREAM *StreamOut =  pData->StreamOut ;
   unsigned long dwConversionMode = pData->dwConversionMode ;
   unsigned long dwAdditionalMode = pData->dwAdditionalMode ;

   int cUnicode ;
   int cSaved = C ;
   const char *szUnknown = "?" ;


   // t@C̍Ō
   if ( C == EOF ) {
      // c cHanKanaPrev ͂
      if ( pData->cHanKanaPrev ) {
         if ( StreamOut ) {         // for others than RtfconvChar()
            int cTmp = pData->cHanKanaPrev ;
            pData->cHanKanaPrev = 0 ;
            Putc ( cTmp, CP_UNICODE, pData ) ;
         }
      }
      if ( IsCodePageVietnamese ( nDstCodePage ) ) EncodeVietnamese ( EOF, pData ) ;
      if ( nDstCodePage ) return PutcTxt ( EOF, 0, nSrcCodePage, szUnknown, pData ) ;
      else                return PutcRtf ( EOF, 0, nSrcCodePage, szUnknown, pData ) ;
   }


   // Unicode ͂̂܂
   if ( nSrcCodePage == CP_UNICODE ) {
      cUnicode = C ;
      if ( cUnicode >= CJK_RADICALS_START ) {
         if ( cUnicode >= 0x2E80 && cUnicode <= 0x9FFF ||      // CJK, Kana etc
              cUnicode >= 0xAC00 && cUnicode <= 0xD7AF ||      // Hangul
              cUnicode >= 0xE000 && cUnicode <= 0xFAFF ||      // Private Use Area, CJK Compatibility
              cUnicode >= 0xFE30 && cUnicode <= 0xFE6F ||      // CJK Compatibility Forms, Small Form
              cUnicode >= 0xFF00 && cUnicode <= 0xFF60 ||      // Fullwidth Forms
              cUnicode >= 0xFFE0 && cUnicode <= 0xFFE7 ||      // Fullwidth Forms
              cUnicode >= 0x20000 ||
              cUnicode == REPLACEMENT_CHARACTER && IsMultiByteCodePage ( GetWinCodePage ( pData->nDstPseudoCodePage ) )
         ) {
            szUnknown = "??" ;
         }
      }
   }
   // SoCg Unicode ɕϊiJIS⏕܂ށj
   else if ( IsFourByteLetter ( C ) ) {
      szUnknown = "??" ;
      if ( nSrcCodePage == CP_JAPANESE ) {
         cUnicode = CharToUnicode_JisHojo ( C & 0xFFFF, dwConversionMode ) ;
         if ( cUnicode == EOF ) cUnicode = 0 ;  // ⏕̕ϊs̓G[ԂȂ
      }
      else if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED )      cUnicode = GbxToUnicode ( C ) ;
      else if ( nSrcCodePage == CP_EUC_CHINESE_TRADITIONAL ) cUnicode = CharToUnicode ( C, nSrcCodePage, dwConversionMode ) ;
      else                                                   cUnicode = 0 ;
   }
   // PoCgEQoCg Unicode ɕϊ
   else {
      if ( C >= 0x100 ) {
         szUnknown = "??" ;
         if ( nSrcCodePage == CP_JAPANESE ) C = SjisNec2ibm ( C ) ;
         if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED && C >= 0xA8A1 && C <= 0xA8BF ) szUnknown = "?" ;
      }
      cUnicode = CharToUnicode ( C, nSrcCodePage, dwConversionMode ) ;
      if ( cUnicode == EOF ) goto CAUGHT_ERROR ;
   }


   // wuC
   if ( ( nSrcCodePage == CP_ISO_HEBREW_VISUAL || nSrcCodePage == CP_ISO_HEBREW_LOGICAL ) ) {
      if ( C == 0xFD ) cUnicode = 0x200E ;
      if ( C == 0xFE ) cUnicode = 0x200F ;
   }
   // HKSCS
   if ( nSrcCodePage == CP_CHINESE_HKSCS && (
         dwAdditionalMode & ADDITIONAL_MODE_HKSCS_CJKB_TO_PUA && IsCjkExB ( cUnicode ) ||
         dwAdditionalMode & ADDITIONAL_MODE_HKSCS_CJKA_TO_PUA && IsCjkExA ( cUnicode ) ||
         dwAdditionalMode & ADDITIONAL_MODE_HKSCS_CJK41_TO_PUA && IsHkscsUnicode410orLater ( cUnicode ) )
   ) {
      cUnicode = CharToUnicode ( C, CP_CHINESE_TRADITIONAL, dwConversionMode ) ;
      if ( cUnicode == EOF ) goto CAUGHT_ERROR ;
   }
   // HKSCS
   if ( nDstCodePage == CP_CHINESE_HKSCS ) {
      if ( dwAdditionalMode & ADDITIONAL_MODE_PUA_TO_HKSCS && IsUnicodePua ( cUnicode ) ) {
         int cBig5 = UnicodeToChar ( cUnicode, CP_CHINESE_TRADITIONAL, dwConversionMode ) ;
         if ( cBig5 == EOF ) goto CAUGHT_ERROR ;
         cUnicode = CharToUnicode ( cBig5, CP_CHINESE_HKSCS, dwConversionMode ) ;
         if ( cUnicode == EOF ) goto CAUGHT_ERROR ;
      }
   }
   // MacintoshpVtgJIS
   if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_MAC && nSrcCodePage == CP_JAPANESE ) {
      const wchar_t *szUnicode ;
      C = cSaved ;
      cUnicode = SjisToUnicode_Mac ( C, cUnicode, & szUnicode ) ;
      if ( szUnicode ) {
         if ( ! StreamOut ) return 0 ;          // for RtfconvChar()
         return Putws ( szUnicode, pData ) ;
      }
   }
   // @ˑȂ̃VtgJIS
   if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR && nSrcCodePage == CP_JAPANESE ) {
      C = cSaved ;
      cUnicode = SjisToUnicode_Org ( C, cUnicode ) ;
   }
   // VtgJIS2004
   if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_2004 && nSrcCodePage == CP_JAPANESE ) {
      const wchar_t *szUnicode ;
      C = cSaved ;
      cUnicode = SjisToUnicode_2004 ( C, cUnicode, & szUnicode, dwConversionMode ) ;
      if ( cUnicode == EOF ) goto CAUGHT_ERROR ;
      if ( szUnicode ) {
         if ( ! StreamOut ) return 0 ;          // for RtfconvChar()
         return Putws ( szUnicode, pData ) ;
      }
   }
   // KS̋Ǔ_̕ϊ
   if ( dwAdditionalMode & ADDITIONAL_MODE_KS_PUNC && ( nDstCodePage == CP_KOREAN || nDstCodePage == CP_KOREAN_JOHAB || nDstCodePage == CP_ISO_KOREAN || ! nDstCodePage && ( nSrcCodePage == CP_KOREAN || nSrcCodePage == CP_UNICODE && pData->wr->nDefaultCodePage == CP_KOREAN ) ) ) {
      if ( cUnicode == L'B' ) {
         if ( ! StreamOut ) return '.' ;           // for RtfconvChar()
         return Putws ( L". ", pData ) ;
      }
      if ( cUnicode == L'A' ) {
         if ( ! StreamOut ) return ',' ;           // for RtfconvChar()
         return Putws ( L", ", pData ) ;
      }
   }
   // xgi̓
   if ( nSrcCodePage == CP_VIETNAMESE_VNI && ! cUnicode ) {
      if ( StreamOut ) {            // for others than RtfconvChar()
         if ( DecodeVietnameseVni ( C, pData ) ) return 0 ;
      }
   }
   // xgȉo
   if ( IsCodePageVietnamese ( nDstCodePage ) ) {
      if ( StreamOut ) {            // for others than RtfconvChar()
         if ( EncodeVietnamese ( cUnicode, pData ) ) return 0 ;
      }
   }
   // ŌɊO
   if ( nSrcCodePage != CP_UNICODE && IsUnicodePua ( cUnicode ) ) {
      // GB13000Վ̏
      int cNonLinshiDaima ;
      if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED && ( cNonLinshiDaima = GetNonLinshiDaima ( cUnicode ) ) && ! IsFourByteLetter ( C ) ) {
         if ( ! ( dwAdditionalMode & ADDITIONAL_MODE_GBK_TO_LINSHIDAIMA ) ) cUnicode = cNonLinshiDaima ;
      }
      // Ȍ
      else if ( ! ( dwConversionMode & CONVMODE_CONVERT_PUA ) ) { // Don't remove "else"!
         if ( ! StreamOut ) return 0 ;          // for RtfconvChar()
         return PutUnmappableChar ( cUnicode, szUnknown, pData ) ;
      }
   }


   // ~L̏
   if ( dwConversionMode & CONVMODE_YEN_NO_BACKSLASH && ! pData->IsInPutUnmmapleChar ) {
      if ( nSrcCodePage == CP_JAPANESE ) {
         if ( C == '\\' ) cUnicode = 0xA5 ;
         if ( dwAdditionalMode & ( ADDITIONAL_MODE_JIS_INPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_INPUT_2004 ) ) {
            if ( C == 0x815F ) cUnicode = 0x5C ;
         }
      }
      if ( nSrcCodePage == CP_KOREAN ) {
         if ( C == '\\' ) cUnicode = 0x20A9 ;
         if ( C == 0xA1AC ) cUnicode = 0x5C ;
      }
      if ( nSrcCodePage == CP_KOREAN_JOHAB ) {
         if ( C == '\\' ) cUnicode = 0x20A9 ;
         if ( C == 0xD93C ) cUnicode = 0x5C ;
      }
   }
   // `_̏
   if ( dwAdditionalMode & ADDITIONAL_MODE_OVERLINE_NO_TILDE ) {
      if ( nSrcCodePage == CP_JAPANESE ) {
         if ( C == '~' ) cUnicode = 0x203E ;
      }
   }


   // C  0 łȂ̂ cUnicode  0 A͐K̕ł͂Ȃ
   if ( cUnicode == 0 && ( ( nSrcCodePage == CP_US_ASCII ) ? ( C & 0x7F ) : C ) != 0 ) {
      if ( ! StreamOut ) return 0 ;       // for RtfconvChar()
      return PutUnmappableChar ( REPLACEMENT_CHARACTER, szUnknown, pData ) ;
   }


   // SpXy[X̉
   if ( dwConversionMode & CONVMODE_NO_ZENSPACE && cUnicode == 0x3000 ) {
      if ( ! StreamOut ) return 0x20 ;    // for RtfconvChar()
      Putc ( 0x20, CP_UNICODE, pData ) ;
      Putc ( 0x20, CP_UNICODE, pData ) ;
      return 0 ;
   }

   // SpL̉
   if ( dwConversionMode & CONVMODE_NO_ZENSYMBOL ) {
      int cResult = ZenSymbolToHanSymbol ( cUnicode ) ;
      if ( cResult ) cUnicode = cResult ;
   }

   // Sp[}Ẻ
   if ( dwConversionMode & CONVMODE_NO_ZENALPHA ) {
      int cResult = ZenAlphaToHanAlpha ( cUnicode ) ;
      if ( cResult ) cUnicode = cResult ;
   }

   // pJỉ
   if ( dwConversionMode & CONVMODE_NO_HANKANA ) {
      if ( pData->cHanKanaPrev ) {
         int cResult ;
         if ( cUnicode == L'' && ( cResult = ZenKanaToDakuon ( pData->cHanKanaPrev ) ) ||
              cUnicode == L'' && ( cResult = ZenKanaToHanDakuon ( pData->cHanKanaPrev ) ) ) {
            cUnicode = cResult ;
            pData->cHanKanaPrev = 0 ;
         }
         else {
            int cTmp = pData->cHanKanaPrev ;
            pData->cHanKanaPrev = 0 ;
            Putc ( cTmp, CP_UNICODE, pData ) ;
         }
      }
      int cResult = HanKanaToZenKana ( cUnicode ) ;
      if ( cResult ) {
         if ( ! StreamOut ) return cResult ;    // for RtfconvChar()
         else {
            pData->cHanKanaPrev = cResult ;
            return 0 ;
         }
      }
   }


   // EOF, WEOF
   if ( dwConversionMode & CONVMODE_DELETE_EOF || ! nDstCodePage ) {
      if ( cUnicode == 0x1A ) return 0 ;                             // CTRL-Z
      if ( ( cUnicode & 0xFFFE ) == 0xFFFE ) return 0 ;              // U+FFFE, U+FFFF
//    if ( cUnicode >= 0xFDD0 && cUnicode <= 0xFDEF ) return 0 ;     // U+FDD0 - U+FDEF
   }


   // s̏
   if ( StreamOut ) {            // for others than RtfconvChar()
      if ( pData->IsPrevCr ) {
         pData->IsPrevCr = 0 ;
         if ( cUnicode == '\n' ) return 0 ;
      }
      if ( cUnicode == '\r' ) {
         pData->IsPrevCr = 1 ;
         C = '\n' ;
         cUnicode = '\n' ;
      }
   }


   if ( nDstCodePage ) return PutcTxt ( C, cUnicode, nSrcCodePage, szUnknown, pData ) ;
   else                return PutcRtf ( C, cUnicode, nSrcCodePage, szUnknown, pData ) ;


   CAUGHT_ERROR :
   pData->nError = CONV_ERROR_CODEPAGE ;
   return 0 ;

}



////////////////////////////////////////////
//         TXT o̓Tu[eB         //
////////////////////////////////////////////



static const wchar_t *GetDecomposedForm ( int cUnicode ) ;
static int GetPriorChar ( int cUnicodeSrc, int *pUnicodeDst, int nSrcCodePage, int nDstCodePage, int nSrcWinCodePage, int nDstWinCodePage, unsigned long dwConversionMode, unsigned long dwAdditionalMode ) ;
static int EncodeSjis2004Composition ( int C, int cUnicode, int nSrcCodePage, RTFCONVDATA *pData ) ;
static int EncodeIscii ( int C, int cUnicode, RTFCONVDATA *pData ) ;
static int EncodeTamil ( int C, int cUnicode, RTFCONVDATA *pData ) ;
static int EncodeUtf8 ( int cUnicode, RTFCONVDATA *pData ) ;
static int EncodeUtf7 ( int cUnicode, RTFCONVDATA *pData ) ;
static int jmstojis2004 ( int c ) ;
static int sjis2eucjp ( int C, int Is2004 ) ;

static int Writewc ( int C, RTFCONVDATA *pData ) ;
static int Writec ( int C, RTFCONVDATA *pData ) ;
static int WritecModal ( int C, RTFCONVDATA *pData ) ;



// Putc ̉֐ (TXTt@C̏o͂)
// StreamOut  NULL ȂΕԂďIiUnicodeȂXJ[lj
// ȊO 0 Ԃ
// G[ StreamOut ɃG[lۑ
// t@C̖ŕK C  EOF w肵ČĂяo
static int PutcTxt ( int C, int cUnicode, int nSrcCodePage, const char *szUnknown, RTFCONVDATA *pData ) {

   int nDstCodePage = pData->nDstCodePage ;
   STREAM *StreamOut =  pData->StreamOut ;
   unsigned long dwConversionMode = pData->dwConversionMode ;
   unsigned long dwAdditionalMode = pData->dwAdditionalMode ;

   int nDstPseudoCodePage = pData->nDstPseudoCodePage ;
   int IsModalEncode = ( nDstPseudoCodePage != nDstCodePage ) ;

   int cMbchar ;


   // t@C̍Ō
   if ( C == EOF ) {
      // c cUnicodePrev ͂
      if ( StreamOut ) {            // for others than RtfconvChar()
         if ( nDstPseudoCodePage == CP_JAPANESE ) EncodeSjis2004Composition ( EOF, EOF, nSrcCodePage, pData ) ;
         if ( IsCodePageTamil ( nDstCodePage ) ) EncodeTamil ( EOF, EOF, pData ) ;
         if ( nDstCodePage == CP_UTF7 ) EncodeUtf7 ( EOF, pData ) ;
      }
      if ( pData->wt->nEncodeMode != EM_ASCII ) {
         if ( StreamOut ) {            // for others than RtfconvChar()
            if ( nDstCodePage == CP_ISO_JAPANESE ) StreamWriteString ( "\x1B(B", StreamOut ) ;
            if ( nDstCodePage == CP_ISO_JAPANESE_AND_KANA_SIO ) StreamWriteString ( "\x1B(B", StreamOut ) ;
            if ( nDstCodePage == CP_HZ_CHINESE_SIMPLIFIED ) StreamWriteString ( "~\x7D", StreamOut ) ;
            if ( nDstCodePage == CP_ISO_KOREAN ) StreamWrite ( 0x0F, StreamOut ) ;
         }
      }
      if ( nDstCodePage == CP_ISO_HEBREW_VISUAL ) {
         if ( StreamOut ) {            // for others than RtfconvChar()
            char *szBuffer = GetStoredCharsA ( pData->wt->ReverseBuffer ) ;
            if ( *szBuffer ) {
               if ( HebrewReverseString ( szBuffer, 0 ) ) {
                  if ( StreamOut ) pData->nError = CONV_ERROR_MEMORY ;
               }
               else {
                  StreamWriteString ( szBuffer, StreamOut ) ;
                  ClearStoredCharsA ( pData->wt->ReverseBuffer ) ;
               }
            }
         }
         FreeStoredCharsA ( pData->wt->ReverseBuffer ) ;
      }

      return 0 ;
   }


   // VtgJIS2004
   if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 && nDstPseudoCodePage == CP_JAPANESE ) {
      if ( StreamOut ) {            // for others than RtfconvChar()
         int cResult = EncodeSjis2004Composition ( C, cUnicode, nSrcCodePage, pData ) ;
         if ( cResult == 0 ) return 0 ;
         if ( cResult > 0 ) {
            cMbchar = cResult ;
            goto OUTPUT_CHAR_PROC ;
         }
      }
   }


   // BOM
   if ( cUnicode == BYTE_ORDER_MARK && nDstPseudoCodePage != CP_UNICODE && StreamOut ) return 0 ;


   // s̏o
   if ( cUnicode == '\n' ) {
      if ( ! StreamOut ) return cUnicode ;      // for RtfconvChar()
      if ( dwConversionMode & CONVMODE_OUTPUT_CR ) PutcTxt ( '\r', L'\r', nSrcCodePage, "?", pData ) ;
      if ( ! ( dwConversionMode & CONVMODE_OUTPUT_LF ) ) return 0 ;
   }


   // Unichain gpĈّ̎
   if ( nDstPseudoCodePage != CP_UNICODE ) {
      if ( pData->IsInPutUnmmapleChar ) {
         if ( isascii ( cUnicode ) ) cMbchar = cUnicode ; // [v邽
         else                        return 0 ;
      }
      else {
         int nSrcWinCodePage = ( nSrcCodePage == pData->nSrcPseudoCodePage ) ? pData->nSrcWinCodePage : GetWinCodePage ( nSrcCodePage ) ;
         cMbchar = GetPriorChar ( cUnicode, NULL, nSrcCodePage, nDstPseudoCodePage, nSrcWinCodePage, pData->nDstWinCodePage, dwConversionMode, dwAdditionalMode ) ;
         if ( cMbchar == EOF ) {
            if ( StreamOut ) pData->nError = CONV_ERROR_CODEPAGE ;
            return 0 ;
         }
      }
   }


   // for RtfconvChar()
   if ( ! StreamOut ) {
      if ( nDstPseudoCodePage == CP_UNICODE ) return cUnicode ;
      else                                    return cMbchar ;
   }


   // ISCII ̏o
   if ( IsCodePageIscii ( nDstCodePage ) ) {
      if ( EncodeIscii ( cMbchar, cUnicode, pData ) ) return 0 ;
   }
   // TSCII/TAB/TAM ̏o
   if ( IsCodePageTamil ( nDstCodePage ) ) {
      if ( EncodeTamil ( cMbchar, cUnicode, pData ) ) return 0 ;
   }


   // s̏̕o
   if ( nDstPseudoCodePage != CP_UNICODE && cMbchar == 0 && cUnicode != 0 ||
        (unsigned) cUnicode > UTF16_MAX && ( nDstCodePage == CP_UTF16 || nDstCodePage == CP_UTF16_BE || nDstCodePage == CP_UTF7 ) ||
        (unsigned) cUnicode > UTF32_MAX && ( nDstCodePage == CP_UTF8 || nDstCodePage == CP_UTF32 || nDstCodePage == CP_UTF32_BE ) ) {

      if ( ! ( dwConversionMode & ( ( cUnicode < CJK_RADICALS_START ) ? CONVMODE_NO_VARIANT_EUROPEAN : CONVMODE_NO_VARIANT_KANJI ) ) ) {
         const wchar_t *szString = GetDecomposedForm ( cUnicode ) ;
         if ( szString ) {
            int IsValid = 1 ;
            for ( const wchar_t *szTmp = szString ; *szTmp && IsValid ; szTmp ++ ) {
               int nSrcWinCodePage = ( nSrcCodePage == pData->nSrcPseudoCodePage ) ? pData->nSrcWinCodePage : GetWinCodePage ( nSrcCodePage ) ;
               if ( ! GetPriorChar ( *szTmp, NULL, nSrcCodePage, nDstPseudoCodePage, nSrcWinCodePage, pData->nDstWinCodePage, dwConversionMode, dwAdditionalMode ) ) {
                  IsValid = 0 ;
                  break ;
               }
            }
            if ( IsValid ) return Putws ( szString, pData ) ;
         }
      }

      return PutUnmappableChar ( cUnicode, szUnknown, pData ) ;
   }


   // wuCꎋȍ
   if ( nDstCodePage == CP_ISO_HEBREW_VISUAL ) {
      if ( cMbchar == 0xFD ) pData->wt->IsReverse = 0 ;
      else if ( cMbchar >= 0xE0 ) pData->wt->IsReverse = 1 ;
      else if ( isascii ( cMbchar ) && ( isalpha ( cMbchar ) || iscntrl ( cMbchar ) ) ) pData->wt->IsReverse = 0 ;
      if ( pData->wt->IsReverse ) {
         if ( cMbchar == 0xFE ) return 0 ;
         pData->wt->ReverseBuffer = StoreCharsA ( cMbchar, pData->wt->ReverseBuffer ) ;
         if ( pData->wt->ReverseBuffer == STOREDCHARS_ERROR ) {
            if ( StreamOut ) pData->nError = CONV_ERROR_MEMORY ;
         }
         return 0 ;
      }
      else {
         char *szBuffer = GetStoredCharsA ( pData->wt->ReverseBuffer ) ;
         if ( *szBuffer ) {
            if ( HebrewReverseString ( szBuffer, 0 ) ) {
               if ( StreamOut ) pData->nError = CONV_ERROR_MEMORY ;
            }
            else {
               StreamWriteString ( szBuffer, StreamOut ) ;
               ClearStoredCharsA ( pData->wt->ReverseBuffer ) ;
            }
         }
         if ( cMbchar == 0xFD ) return 0 ;
      }
   }


   OUTPUT_CHAR_PROC :


   // o
   // UNICODEȂ
   if ( nDstPseudoCodePage == CP_UNICODE ) {
      Writewc ( cUnicode, pData ) ;
   }
   // [ĥGR[hȂ
   else if ( IsModalEncode ) {
      if ( WritecModal ( cMbchar, pData ) == EOF ) {
         PutUnmappableChar ( cUnicode, szUnknown, pData ) ;
      }
   }
   // [ĥȂGR[hȂ
   else {
      Writec ( cMbchar, pData ) ;
   }

   return 0 ;
}



static int Writewc ( int C, RTFCONVDATA *pData ) {

   STREAM *StreamOut = pData->StreamOut ;

   switch ( pData->nDstCodePage ) {

      // UTF-16 ̏
      case CP_UTF16 :
      {
         if ( IsFourByteLetter ( C ) ) {
            C = MAKE_UNICODE_SURROGATE ( C ) ;
            StreamWrite ( ( C >> 16 ) & 0xFF, StreamOut ) ;
            StreamWrite ( ( ( C >> 16 ) >> 8 ) & 0xFF, StreamOut ) ;
         }
         StreamWrite ( C & 0xFF, StreamOut ) ;
         StreamWrite ( ( C >> 8 ) & 0xFF, StreamOut ) ;
         return 0 ;
      }
      case CP_UTF16_BE :
      {
         if ( IsFourByteLetter ( C ) ) {
            C = MAKE_UNICODE_SURROGATE ( C ) ;
            StreamWrite ( ( ( C >> 16 ) >> 8 ) & 0xFF, StreamOut ) ;
            StreamWrite ( ( C >> 16 ) & 0xFF, StreamOut ) ;
         }
         StreamWrite ( ( C >> 8 ) & 0xFF, StreamOut ) ;
         StreamWrite ( C & 0xFF, StreamOut ) ;
         return 0 ;
      }

      // UTF-32 ̏
      case CP_UTF32 :
      {
         StreamWrite ( C & 0xFF, StreamOut ) ;
         StreamWrite ( ( C >> 8 ) & 0xFF, StreamOut ) ;
         StreamWrite ( ( C >> 16 ) & 0xFF, StreamOut ) ;
         StreamWrite ( ( C >> 24 ) & 0xFF, StreamOut ) ;
         return 0 ;
      }
      case CP_UTF32_BE :
      {
         StreamWrite ( ( C >> 24 ) & 0xFF, StreamOut ) ;
         StreamWrite ( ( C >> 16 ) & 0xFF, StreamOut ) ;
         StreamWrite ( ( C >> 8 ) & 0xFF, StreamOut ) ;
         StreamWrite ( C & 0xFF, StreamOut ) ;
         return 0 ;
      }

      // UTF-8 ̏
      case CP_UTF8 :
      {
         EncodeUtf8 ( C, pData ) ;
         return 0 ;
      }

      // UTF-7 ̏
      case CP_UTF7 :
      {
         if ( IsFourByteLetter ( C ) ) {
            C = MAKE_UNICODE_SURROGATE ( C ) ;
            EncodeUtf7 ( ( C >> 16 ) & 0xFFFF, pData ) ;
         }
         EncodeUtf7 ( C & 0xFFFF, pData ) ;
         return 0 ;
      }
   }

   return 0 ;
}



static int Writec ( int C, RTFCONVDATA *pData ) {

   STREAM *StreamOut = pData->StreamOut ;

   if ( C & 0xFF000000 ) StreamWrite ( ( C >> 24 ) & 0xFF, StreamOut ) ;
   if ( C & 0xFFFF0000 ) StreamWrite ( ( C >> 16 ) & 0xFF, StreamOut ) ;
   if ( C & 0xFFFFFF00 ) StreamWrite ( ( C >> 8 ) & 0xFF, StreamOut ) ;
   StreamWrite ( C & 0xFF, StreamOut ) ;

   return 0 ;
}



// sȕȂ EOF Ԃ
static int WritecModal ( int C, RTFCONVDATA *pData ) {

   STREAM *StreamOut = pData->StreamOut ;

   switch ( pData->nDstCodePage ) {

      // EUC-JP ̏
      case CP_EUC_JAPANESE :
      {
         int cResult = sjis2eucjp ( C, pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 ) ;
         if ( C != 0 && cResult == 0 ) return EOF ;
         Writec ( cResult, pData ) ;
         return 0 ;
      }

      // ISO-2022-JP ̏
      case CP_ISO_JAPANESE :
      case CP_ISO_JAPANESE_AND_KANA_SIO :
      {
         int nEncodeMode = EM_ASCII ;

         if ( C >= 0x100 ) {
            int cResult = 0 ;
            if ( pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 ) {
               cResult = jmstojis2004 ( C ) ;
               if ( cResult ) {
                  C = cResult ;
                  nEncodeMode = EM_LOCAL_EX ;
               }
            }
            if ( ! cResult ) {
               C = jmstojis ( SjisIbm2nec ( C ) ) ;
               if ( ! C ) return EOF ;
               nEncodeMode = EM_LOCAL ;
            }
         }
         else if ( C >= 0x80 ) {
            C = C & 0x7F ;
            nEncodeMode = EM_HANKANA ;
         }

         if ( pData->wt->nEncodeMode != nEncodeMode ) {
            if ( nEncodeMode == EM_HANKANA ) {
               if ( pData->nDstCodePage == CP_ISO_JAPANESE_AND_KANA_SIO ) StreamWriteString ( "\x0E", StreamOut ) ;
               else                                                       StreamWriteString ( "\x1B(I", StreamOut ) ;
            }
            else {
               if ( pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 ) {
                  if ( nEncodeMode == EM_LOCAL )         StreamWriteString ( "\x1B$(Q", StreamOut ) ;
                  else if ( nEncodeMode == EM_LOCAL_EX ) StreamWriteString ( "\x1B$(P", StreamOut ) ;
                  else                                   StreamWriteString ( "\x1B(B", StreamOut ) ;
               }
               else {
                  if ( nEncodeMode == EM_LOCAL ) StreamWriteString ( "\x1B$B", StreamOut ) ;
                  else                           StreamWriteString ( "\x1B(B", StreamOut ) ;
               }
            }
            pData->wt->nEncodeMode = nEncodeMode ;
         }

         Writec ( C, pData ) ;
         return 0 ;
      }

      // HZ ̏
      case CP_HZ_CHINESE_SIMPLIFIED :
      {
         int nEncodeMode ;

         if ( C >= 0x100 ) nEncodeMode = EM_LOCAL ;
         else              nEncodeMode = EM_ASCII ;

         if ( pData->wt->nEncodeMode != nEncodeMode ) {
            if ( nEncodeMode == EM_LOCAL ) StreamWriteString ( "~\x7B", StreamOut ) ;
            else                           StreamWriteString ( "~\x7D", StreamOut ) ;
            pData->wt->nEncodeMode = nEncodeMode ;
         }

         if ( nEncodeMode == EM_LOCAL ) C &= 0x7F7F ;
         if ( C == '~' ) StreamWrite ( '~', StreamOut ) ;

         Writec ( C, pData ) ;
         return 0 ;
      }

      // ISO-2022-KR ̏
      case CP_ISO_KOREAN :
      {
         int nEncodeMode ;

         if ( C >= 0x100 ) nEncodeMode = EM_LOCAL ;
         else              nEncodeMode = EM_ASCII ;

         if ( pData->wt->nEncodeMode != nEncodeMode ) {
            if ( nEncodeMode == EM_LOCAL ) StreamWrite ( 0x0E, StreamOut ) ;
            else                           StreamWrite ( 0x0F, StreamOut ) ;
            pData->wt->nEncodeMode = nEncodeMode ;
         }

         Writec ( C, pData ) ;
         return 0 ;
      }
   }

   return EOF ;
}



//  Putc 𗘗pďo (͂PoCĝ)
static int Puts ( const char *szString, int nSrcCodePage, RTFCONVDATA *pData ) {

   while ( *szString ) {
      Putc ( (unsigned char) *szString, nSrcCodePage, pData ) ;
      szString ++ ;
   }

   return 0 ;
}



//  Putc 𗘗pďo
static int Putws ( const wchar_t *szString, RTFCONVDATA *pData ) {

   while ( *szString ) {
      if ( IsUnicodeHighSurrogate ( *szString ) && IsUnicodeLowSurrogate ( *( szString + 1 ) ) ) {
         int cScalar = MAKE_UNICODE_SCALAR ( *szString, *( szString + 1 ) ) ;
         Putc ( cScalar, CP_UNICODE, pData ) ;
         szString ++ ;
         szString ++ ;
         continue ;
      }
      Putc ( *szString, CP_UNICODE, pData ) ;
      szString ++ ;
   }

   return 0 ;
}



// ϊłȂ UNICODE ̔ԍŏo͂
static int PutCodeNumber ( int cUnicode, RTFCONVDATA *pData ) {

   const wchar_t *szFormat = pData->szUnmappableCharFormat ;
   if ( ! szFormat ) szFormat = szUnmappableCharFormatDefault ;

   wchar_t szBuffer [ UNMAPPABLE_CHAR_FORMAT_MAX + 0x20 ] ;
   snwprintf ( szBuffer, sizeof(szBuffer)/sizeof(wchar_t), szFormat, cUnicode ) ;
   szBuffer [ sizeof(szBuffer)/sizeof(wchar_t) - 1 ] = 0 ;

   return Putws ( szBuffer, pData ) ;
}



// ϊłȂ UNICODE ̔ԍŏo͂(HTML`)
static int PutCodeNumberHtml ( int cUnicode, RTFCONVDATA *pData ) {

   wchar_t szBuffer [ 0x20 ] ;
   swprintf ( szBuffer, L"&#x%04X;", cUnicode ) ;

   return Putws ( szBuffer, pData ) ;
}



// Unicode ɕϊłȂo
// szDefaultString ɂ͔ԍŏo͂Ȃꍇɏo͂镶w
static int PutUnmappableChar ( int cUnicode, const char *szDefaultString, RTFCONVDATA *pData ) {

   if ( cUnicode == REPLACEMENT_CHARACTER ) {
      if ( ! pData->nDstCodePage || GetModelessCodePage ( pData->nDstCodePage ) == CP_UNICODE ) {
         return Putc ( REPLACEMENT_CHARACTER, CP_UNICODE, pData ) ;
      }
   }

   if ( pData->IsInPutUnmmapleChar ) return 0 ;
   pData->IsInPutUnmmapleChar = 1 ;

   if ( pData->dwConversionMode & ( CONVMODE_OUTPUT_CODE_NUMBER | CONVMODE_OUTPUT_CODE_NUMBER_HTML ) ) {
      if ( pData->dwConversionMode & CONVMODE_OUTPUT_NUMBER_SURROGATE && IsFourByteLetter ( cUnicode ) && (unsigned) cUnicode <= UTF16_MAX ) {
         cUnicode = MAKE_UNICODE_SURROGATE ( cUnicode ) ;
         if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER_HTML ) {
            PutCodeNumberHtml ( ( cUnicode >> 16 ) & 0xFFFF, pData ) ;
            PutCodeNumberHtml ( cUnicode & 0xFFFF, pData ) ;
         }
         else if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) {
            PutCodeNumber ( ( cUnicode >> 16 ) & 0xFFFF, pData ) ;
            PutCodeNumber ( cUnicode & 0xFFFF, pData ) ;
         }
      }
      else {
         if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER_HTML ) PutCodeNumberHtml ( cUnicode, pData ) ;
         else if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) PutCodeNumber ( cUnicode, pData ) ;
      }
   }
   else {
      Puts ( szDefaultString, CP_ANSI_EUROPEAN, pData ) ;
   }

   pData->IsInPutUnmmapleChar = 0 ;
   return 0 ;
}



////////////////////////////////////////////
//               ّ̎               //
////////////////////////////////////////////



static int AdjustJisOldNew ( int cUnicode, unsigned long dwConversionMode, unsigned long dwAdditionalMode ) ;
static int IsGb2312 ( int C ) ;
static int IsKsX1001 ( int cKs ) ;

#define IsBasicCyrillic(c)    ( (c)>=0x0410 && (c)<=0x044F )
#define IsBasicGreek(c)       ( (c)>=0x0391 && (c)<=0x03A9 || (c)>=0x03B1 && (c)<=0x03C9 )



// Unichain ỸR[hy[W̍łœKȕ
// nSrcCodePage, nDstCodePage ̓[ĥȂR[ĥ
// JIS⏕ + 0x10000 ŏo
// œKȕȂꍇ 0 Ԃ
// R[hy[WsK؂Ȃ EOF Ԃ
static int GetPriorChar ( int cUnicodeSrc, int *pUnicodeDst, int nSrcCodePage, int nDstCodePage, int nSrcWinCodePage, int nDstWinCodePage, unsigned long dwConversionMode, unsigned long dwAdditionalMode ) {

   int IsBetweenSameCodePage = ( nSrcWinCodePage == nDstWinCodePage ) ;
   int IsNoVariant = dwConversionMode & ( ( cUnicodeSrc < CJK_RADICALS_START ) ? CONVMODE_NO_VARIANT_EUROPEAN : CONVMODE_NO_VARIANT_KANJI ) ;

   int cMbcharResult = 0 ;
   int cUnicodeResult = 0 ;
   int cMbcharFirstHit = 0 ;

   int IsRoundTrip = ( dwConversionMode & CONVMODE_VARIANT_ONLY_ROUNDTRIP ) ;
   dwConversionMode &= ~ CONVMODE_VARIANT_ONLY_ROUNDTRIP ;


   // CJKgّ̈̎邩H
   if ( ! IsNoVariant && IsCjkEx ( cUnicodeSrc ) ) {
      switch ( nDstCodePage ) {
         case CP_EUC_CHINESE_TRADITIONAL :
            break ;
         case CP_CHINESE_SIMPLIFIED :
            if ( dwConversionMode & CONVMODE_OUTPUT_GB18030 ) break ;
            if ( IsHaveLinshiDaima ( cUnicodeSrc ) ) break ;
            IsNoVariant = 1 ;
            break ;
         default :
            if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED && IsHaveLinshiDaima ( cUnicodeSrc ) ) break ;
            IsNoVariant = 1 ;
            break ;
      }
   }


   // p̌ÂR[hy[WȂǂ̏
   if ( nDstWinCodePage != nDstCodePage && nDstCodePage != CP_CHINESE_HKSCS && nDstCodePage != CP_EUC_CHINESE_TRADITIONAL && nDstCodePage != CP_KOREAN_JOHAB && IsMultiByteCodePage ( nDstWinCodePage ) ) {

      if ( ! IsBetweenSameCodePage && cUnicodeSrc >= CJK_RADICALS_START && ! IsNoVariant ) {

         unsigned long dwConversionModeTmp = dwConversionMode ;
         if ( nDstCodePage == CP_EUC_JAPANESE_MS ) dwConversionModeTmp |= CONVMODE_EUC_HOJO ;
         if ( nDstCodePage == CP_MAC_CHINESE_SIMPLIFIED ) dwConversionModeTmp |= CONVMODE_NO_GBK ;
         if ( GetPriorChar ( cUnicodeSrc, & cUnicodeResult, nSrcCodePage, nDstWinCodePage, nSrcWinCodePage, nDstWinCodePage, dwConversionModeTmp, dwAdditionalMode ) == EOF ) return EOF ;

         cMbcharResult = UnicodeToChar ( cUnicodeResult, nDstCodePage, dwConversionMode ) ;
         if ( cMbcharResult == EOF ) return EOF ;

         if ( cMbcharResult && IsRoundTrip ) {
            cMbcharFirstHit = UnicodeToChar ( cUnicodeSrc, nDstCodePage, dwConversionMode ) ;
            if ( cMbcharFirstHit == EOF ) return EOF ;
         }
      }
   }



   // ʏ̏
   if ( ! cMbcharResult ) {


      // GB13000Վ̏
      if ( nSrcCodePage == CP_UNICODE && nDstCodePage == CP_CHINESE_SIMPLIFIED ) {
         if ( dwAdditionalMode & ADDITIONAL_MODE_LINSHIDAIMA_TO_GBK ) {
            int cNonLinshiDaima = GetNonLinshiDaima ( cUnicodeSrc ) ;
            if ( cNonLinshiDaima ) cUnicodeSrc = cNonLinshiDaima ;
         }
         // GB18030ɂՎcĂ
         else if ( dwConversionMode & CONVMODE_OUTPUT_GB18030 ) {
            int cNonLinshiDaima = GetNonLinshiDaima ( cUnicodeSrc ) ;
            if ( cNonLinshiDaima && IsLinshiDaimaGb18030 ( cUnicodeSrc ) ) cUnicodeSrc = cNonLinshiDaima ;
         }
      }

      // ݊
      int cUnicodeSrcNonCmpt = cUnicodeSrc ;
      if ( ! IsNoVariant ) {
         // CJKȊǑ݊
         if ( IsFourByteLetter ( cUnicodeSrc ) && ! IsCjkExCmpt ( cUnicodeSrc ) ) {
            int cNonCmpt = NormalizeUnicodeCompatibility ( cUnicodeSrc ) ;
            if ( cNonCmpt ) cUnicodeSrc = cNonCmpt ;
         }
         // CJǨ݊iƂŔr邽ߕۑj
         if ( IsCjkCmpt ( cUnicodeSrc ) || IsCjkExCmpt ( cUnicodeSrc ) ) {
            int cNonCmpt = NormalizeUnicodeCompatibility ( cUnicodeSrc ) ;
            if ( cNonCmpt ) cUnicodeSrcNonCmpt = cNonCmpt ;
         }
         // 
         else if ( IsCjkRadicals ( cUnicodeSrc ) ) {
            int cNonCmpt = CjkRadicalsToCjk ( cUnicodeSrc ) ;
            if ( cNonCmpt ) cUnicodeSrcNonCmpt = cNonCmpt ;
         }
      }

      // Dx𒲐邩
      int IsAdjustPriority = 0 ;
      int IsNoAdjustPriority = 0 ;
      if ( ! IsNoVariant ) {
         IsAdjustPriority = IsAdjustPrioritySrc ( cUnicodeSrcNonCmpt ) ;
         IsNoAdjustPriority = isascii ( cUnicodeSrcNonCmpt ) || IsBasicCyrillic ( cUnicodeSrcNonCmpt ) || IsBasicGreek ( cUnicodeSrcNonCmpt ) ;
      }

      // ɑ̎DȂ
      int IsToGbFanti = 0 ;
      int cUnicodeFromBig5 = 0 ;
      if ( ! IsNoVariant ) {
         IsToGbFanti = ( nDstCodePage == CP_CHINESE_SIMPLIFIED && dwAdditionalMode & ADDITIONAL_MODE_GBK_OUTPUT_FANTIZI && IsCjkAll ( cUnicodeSrc ) ) ;
         if ( IsToGbFanti && GetPriorChar ( cUnicodeSrcNonCmpt, & cUnicodeFromBig5, nSrcCodePage, CP_CHINESE_TRADITIONAL, nSrcWinCodePage, CP_CHINESE_TRADITIONAL, dwConversionMode, dwAdditionalMode ) == EOF ) return EOF ;
      }

      // JIS 78/83
      int cJisNewOrOld = 0 ;
      if ( nDstCodePage == CP_JAPANESE ) {
         int cResult = AdjustJisOldNew ( cUnicodeSrc, dwConversionMode, dwAdditionalMode ) ;
         if ( cResult ) cUnicodeSrc = cJisNewOrOld = cResult ;
      }


      int cUnicode = cUnicodeSrc ;
      int nCountCycle = 0 ;         // 0xFFFF 𒴂ďz
      int nPrioritySaved = 0 ;


      // Unichain 
      while ( nCountCycle < 2 ) {

         int cUnicodeTmp = cUnicode ;

         // GB13000Վ̏
         if ( nDstCodePage == CP_CHINESE_SIMPLIFIED ) {
            int cTmp = GetLinshiDaima ( cUnicodeTmp ) ;
            if ( cTmp ) cUnicodeTmp = cTmp ;
         }

         // ϊ
         int cMbchar = UnicodeToChar ( cUnicodeTmp, nDstCodePage, dwConversionMode ) ;
         if ( cMbchar == EOF ) return EOF ;

         // ŏɊO
         if ( IsUnicodePua ( cUnicode ) ) {
            // GB13000Վ̏
            if ( nDstCodePage == CP_CHINESE_SIMPLIFIED && IsLinshiDaima ( cUnicode ) ) {
               if ( ! ( dwAdditionalMode & ADDITIONAL_MODE_LINSHIDAIMA_TO_GBK ) ) cMbchar = 0 ;
            }
            // Ȍ
            else if ( ! ( dwConversionMode & CONVMODE_CONVERT_PUA ) ) { // Don't remove "else"!
               cMbchar = 0 ;
            }
         }
         // wuC
         if ( ( nDstCodePage == CP_ISO_HEBREW_VISUAL || nDstCodePage == CP_ISO_HEBREW_LOGICAL ) ) {
            if ( cUnicode == 0x200E ) cMbchar = 0xFD ;
            if ( cUnicode == 0x200F ) cMbchar = 0xFE ;
         }
         // HKSCS
         if ( nDstCodePage == CP_CHINESE_HKSCS && dwAdditionalMode & ( ADDITIONAL_MODE_HKSCS_1999 | ADDITIONAL_MODE_HKSCS_2001 | ADDITIONAL_MODE_HKSCS_2004 ) ) {
            int cUnicodePua = CharToUnicode ( cMbchar, CP_CHINESE_TRADITIONAL, dwConversionMode ) ;
            if ( IsUnicodePua ( cUnicodePua ) ) {
               if ( dwAdditionalMode & ADDITIONAL_MODE_HKSCS_1999 && ! IsHkscs1999 ( cUnicodePua ) ) cMbchar = 0 ;
               if ( dwAdditionalMode & ADDITIONAL_MODE_HKSCS_2001 && ! IsHkscs2001 ( cUnicodePua ) ) cMbchar = 0 ;
               if ( dwAdditionalMode & ADDITIONAL_MODE_HKSCS_2004 && ! IsHkscs2004 ( cUnicodePua ) ) cMbchar = 0 ;
            }
         }
         // GBX
         if ( nDstCodePage == CP_CHINESE_SIMPLIFIED && dwConversionMode & CONVMODE_OUTPUT_GB18030 ) {
            // 80h  FFh  GB18030 ł͖`
            if ( cMbchar == 0x80 || cMbchar == 0xFF ) cMbchar = 0 ;
            if ( ! cMbchar && ( ! IsUnicodePua ( cUnicode ) || dwConversionMode & CONVMODE_CONVERT_PUA ) ) {
               int cMbcharTmp = (int) UnicodeToGbx ( cUnicode ) ;
               if ( cMbcharTmp ) cMbchar = cMbcharTmp ;
            }
         }
         // EUC-JP⏕̏
         if ( dwConversionMode & CONVMODE_EUC_HOJO && nDstCodePage == CP_JAPANESE && ( cMbchar == 0 || cMbchar > 0xF500 ) ) {
            int cJisHojo = UnicodeToChar_JisHojo ( cUnicode, dwConversionMode ) ;
            if ( cJisHojo > 0 ) cMbchar = cJisHojo + 0x10000 ;    // ⏕̕ϊs̓G[ԂȂ
         }
         // MacintoshpVtgJIS
         if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_MAC && nDstCodePage == CP_JAPANESE ) {
            cMbchar = UnicodeToSjis_Mac ( cUnicode, cMbchar ) ;
         }
         // @ˑȂ̃VtgJIS
         if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR && nDstCodePage == CP_JAPANESE ) {
            cMbchar = UnicodeToSjis_Org ( cUnicode, cMbchar ) ;
         }
         // VtgJIS2004
         if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 && nDstCodePage == CP_JAPANESE ) {
            cMbchar = UnicodeToSjis_2004 ( cUnicode, cMbchar, NULL, dwConversionMode ) ;
            if ( cMbchar == EOF ) return EOF ;
         }


         // ~L̏
         if ( dwConversionMode & CONVMODE_YEN_NO_BACKSLASH ) {
            if ( nDstCodePage == CP_JAPANESE ) {
               if ( cUnicode == 0xA5 ) cMbchar = '\\' ;
               if ( dwAdditionalMode & ( ADDITIONAL_MODE_JIS_OUTPUT_NO_VENDOR | ADDITIONAL_MODE_JIS_OUTPUT_2004 ) ) {
                  if ( cUnicode == 0x5C ) cMbchar = 0x815F ;
                  if ( cUnicode == 0xFF3C ) cMbchar = 0 ;      // L'_'
               }
            }
            if ( nDstCodePage == CP_KOREAN ) {
               if ( cUnicode == 0x20A9 ) cMbchar = '\\' ;
               if ( cUnicode == 0x5C ) cMbchar = 0xA1AC ;
               if ( cUnicode == 0xFF3C ) cMbchar = 0 ;      // L'_'
            }
            if ( nDstCodePage == CP_KOREAN_JOHAB ) {
               if ( cUnicode == 0x20A9 ) cMbchar = '\\' ;
               if ( cUnicode == 0x5C ) cMbchar = 0xD93C ;
               if ( cUnicode == 0xFF3C ) cMbchar = 0 ;      // L'_'
            }
         }
         // `_̏
         if ( dwAdditionalMode & ADDITIONAL_MODE_OVERLINE_NO_TILDE ) {
            if ( nDstCodePage == CP_JAPANESE ) {
               if ( cUnicode == 0x203E ) cMbchar = '~' ;
               if ( cUnicode == 0x7E ) cMbchar = 0 ;
            }
         }
         // ~Lƃ`_ďC
         if ( nDstCodePage == CP_JAPANESE && ! cMbchar ) {
            if ( cUnicode == 0x5C && dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_MAC ) {
               if ( Sjis2uni_Mac_SingleByte ( 0x80, 0 ) == 0x5C ) cMbchar = 0x80 ;
            }
            if ( cUnicode == 0x7E && dwConversionMode & CONVMODE_EUC_HOJO ) {
               int cJisHojo = UnicodeToChar_JisHojo ( cUnicode, dwConversionMode ) ;
               if ( cJisHojo > 0 ) cMbchar = cJisHojo + 0x10000 ;    // ⏕̕ϊs̓G[ԂȂ
            }
         }


         // D揇ʂ̐ݒ
         if ( cMbchar ) {

            int nPriority = 30000 ;

            if ( nDstCodePage == CP_JAPANESE ) {
               if ( isascii ( cMbchar ) || cMbchar >= 0x8140 && cMbchar <= 0x988F ) nPriority = 30000 ;
               else if ( cMbchar >= 0x80 && cMbchar <= 0xFF ) nPriority = 20000 ;
               else if ( IsFourByteLetter ( cMbchar ) ) {
                  if ( dwConversionMode & CONVMODE_EUC_HOJO ) nPriority = 5000 ;
                  else                                        nPriority = 0 ;
               }
               else nPriority = 10000 ;
               if ( nPriority && cJisNewOrOld && cUnicode == cJisNewOrOld ) nPriority = 40000 ;
            }
            else if ( nDstCodePage == CP_CHINESE_SIMPLIFIED ) {
               if ( IsToGbFanti ) {
                  if ( cUnicode == cUnicodeFromBig5 ) nPriority = 30000 ;
                  else if ( ! ( dwConversionMode & CONVMODE_OUTPUT_GB18030 ) && IsFourByteLetter ( cMbchar ) ) nPriority = 0 ;
                  else if ( IsUniFantizi ( cUnicode ) ) nPriority = 20000 ;
                  else nPriority = 6000 ;
               }
               else {
                  if ( IsGb2312 ( cMbchar ) ) nPriority = 30000 ;
                  else if ( dwConversionMode & CONVMODE_NO_GBK ) nPriority = 0 ;
                  else if ( cMbchar >= 0xA1A1 && cMbchar <= 0xF7FF && ( cMbchar & 0xFF ) >= 0xA1 ) nPriority = 30000 ;  // GB2312͈̔͂̋KiO̕
                  else if ( ! ( dwConversionMode & CONVMODE_OUTPUT_GB18030 ) && IsFourByteLetter ( cMbchar ) ) nPriority = 0 ;
                  else if ( ! IsCjkAll ( cUnicode ) ) nPriority = 20000 ;
                  else if ( IsUniJiantizi ( cUnicode ) ) nPriority = 14000 ;
                  else nPriority = 6000 ;
               }
               if ( dwConversionMode & CONVMODE_OUTPUT_GB18030 && IsCjkRadicals ( cUnicode ) ) nPriority >>= 2 ;
            }
            else if ( nDstCodePage == CP_CHINESE_TRADITIONAL || nDstCodePage == CP_CHINESE_HKSCS ) {
               if ( isascii ( cMbchar ) || cMbchar >= 0xA140 && cMbchar <= 0xC67F ) nPriority = 30000 ;
               else nPriority = 10000 ;
            }
            else if ( nDstCodePage == CP_EUC_CHINESE_TRADITIONAL ) {
               if ( ! IsFourByteLetter ( cMbchar ) ) nPriority = 40000 ;
               else if ( ( (unsigned) cMbchar >> 16 ) == 0x8EA2 ) nPriority = 18000 ;
               else if ( ( (unsigned) cMbchar >> 16 ) == 0x8EA3 ) nPriority = 8000 ;
               else nPriority = 3000 ;
            }
            else if ( nDstCodePage == CP_KOREAN ) {
               if ( IsKsX1001 ( cMbchar ) ) nPriority = 30000 ;
               else if ( dwAdditionalMode & ADDITIONAL_MODE_KS_NO_UHC ) nPriority = 0 ;
               else nPriority = 6000 ;
            }

            // D揇ʂ𒲐
            if ( nPriority && ! IsNoVariant ) {
               // First Fit D
               if ( cUnicode == cUnicodeSrc || cUnicode == cUnicodeSrcNonCmpt ) nPriority *= 2 ;
               // ͌`SɈv̂̂ݏo
               else if ( IsCjkRadicals ( cUnicodeSrc ) ) nPriority = 0 ;
               // 
               if ( ( cUnicode != cUnicodeSrc || ! IsBetweenSameCodePage || IsToGbFanti ) && ! IsNoAdjustPriority ) {
                  int cTmp = 0 ;
                  if ( IsCjkCmpt ( cUnicode ) || IsCjkExCmpt ( cUnicode ) ) cTmp = NormalizeUnicodeCompatibility ( cUnicode ) ;
                  if ( ! cTmp ) cTmp = cUnicode ;
                  else          nPriority >>= 2 ;
                  if ( IsAdjustPriority || IsAdjustPriorityDst ( cTmp ) ) {
                     if ( IsCjkAll ( cTmp ) ) nPriority = AdjustCjkPriority ( nPriority, cTmp, cUnicodeSrcNonCmpt, nSrcWinCodePage, nDstWinCodePage, IsToGbFanti ) ;
                     else                     nPriority = AdjustPriority ( nPriority, cTmp, cUnicodeSrcNonCmpt, nSrcWinCodePage, nDstWinCodePage ) ;
                  }
               }
            }

            // ʂۑ
            if ( nPriority > nPrioritySaved || nPriority && nPriority == nPrioritySaved && (unsigned) cMbchar < (unsigned) cMbcharResult ) {
               cMbcharResult = cMbchar ;
               cUnicodeResult = cUnicode ;
               nPrioritySaved = nPriority ;
            }
         }

         // First Fit ۑ
         if ( cUnicode == cUnicodeSrc ) {
            cMbcharFirstHit = cMbcharResult ;
            if ( IsNoVariant ) break ;
            if ( IsRoundTrip && nSrcCodePage == CP_UNICODE ) break ;
            if ( cMbcharResult ) {
               if ( IsNoAdjustPriority ) break ;
               if ( IsBetweenSameCodePage && ! IsToGbFanti ) break ;
            }
         }

         // ̒l擾
         int cNextUnicode = GetNextUnichain ( cUnicode ) ;
         if ( cNextUnicode == 0 || cNextUnicode == cUnicodeSrc ) break ;
         if ( cUnicode >= cNextUnicode ) nCountCycle ++ ;
         cUnicode = cNextUnicode ;
      }
   }



   if ( cMbcharResult && IsRoundTrip && nSrcCodePage != CP_UNICODE ) {

      int cUnicodeRev = 0 ;

      GetPriorChar ( cUnicodeResult, & cUnicodeRev, nDstCodePage, nSrcCodePage, nDstWinCodePage, nSrcWinCodePage, dwConversionMode, dwAdditionalMode ) ;
      if ( cUnicodeRev == cUnicodeSrc ) {
         if ( pUnicodeDst ) *pUnicodeDst = cUnicodeResult ;
         return cMbcharResult ;
      }

      if ( cMbcharFirstHit ) {
         GetPriorChar ( cUnicodeSrc, & cUnicodeRev, nDstCodePage, nSrcCodePage, nDstWinCodePage, nSrcWinCodePage, dwConversionMode, dwAdditionalMode ) ;
         if ( cUnicodeRev == cUnicodeSrc ) {
            if ( pUnicodeDst ) *pUnicodeDst = cUnicodeSrc ;
            return cMbcharFirstHit ;
         }
      }

      return 0 ;
   }


   if ( pUnicodeDst ) *pUnicodeDst = cUnicodeResult ;
   return cMbcharResult ;
}



// JIS  78/83 𒲐
static int AdjustJisOldNew ( int cUnicode, unsigned long dwConversionMode, unsigned long dwAdditionalMode ) {

   // ˒\ ͕̋̂⏕ło͕s
   // q ͕̋̂⏕ŏo͉ɂ炸AJIS2004ł͕ۂƂ

   if ( 1 ) {

      static const wchar_t szNew [] = L"ahЌz{xGٓvǓOUĖy" ;
      static const wchar_t szOld [] = L"y}|˚融A增wM" ;

      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_NO_JIS83_NEW_STYLE ) {
         const wchar_t *szResult = wcschr ( szNew, cUnicode ) ;
         if ( szResult ) return *( szOld + ( szResult - szNew ) ) ;
      }

      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_NO_JIS83_OLD_STYLE ) {
         const wchar_t *szResult = wcschr ( szOld, cUnicode ) ;
         if ( szResult ) return *( szNew + ( szResult - szOld ) ) ;
      }
   }

   if ( dwConversionMode & CONVMODE_EUC_HOJO ) {

      static const wchar_t szNew [] = L"qƏJӏݑ~͓U^Xj˗X" ;
      static const wchar_t szOld [] = L"\x555E\x7130\x9DD7\x5699\x4FE0\x8EC0\x7E6B\x9E7C\x9EB4\x5C62\x7E61\x8523\x91AC\x6414\x7626\x6451\x5861\x985A\x79B1\x7006\x56CA\x6F51\x91B1\x9830\x9EB5\x840A\x881F\x6522" ;

      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_NO_JIS83_NEW_STYLE ) {
         const wchar_t *szResult = wcschr ( szNew, cUnicode ) ;
         if ( szResult ) return *( szOld + ( szResult - szNew ) ) ;
      }

      if ( dwAdditionalMode & ADDITIONAL_MODE_JIS_NO_JIS83_OLD_STYLE ) {
         const wchar_t *szResult = wcschr ( szOld, cUnicode ) ;
         if ( szResult ) return *( szNew + ( szResult - szOld ) ) ;
      }
   }

   return 0 ;
}



////////////////////////////////////////////
//         RTF o̓Tu[eB         //
////////////////////////////////////////////



static int UnicodeToCharset ( int cUnicode, int nIndex, int *pIndex, int nSrcCodePage, int IsForVista ) ;
static int UnicodeToBidirect ( int cUnicode ) ;
static int GetUserDefinedFont ( int cUnicode, int nIndex, int *pIndex, RTFCONVDATA *pData ) ;
static int GetUserDefinedFontCharset ( int nFontNumber, RTFCONVDATA *pData ) ;
static int GetUserDefinedLangId ( int cUnicode, int nIndex, int *pIndex, RTFCONVDATA *pData ) ;
#define DEFAULT_LANGID        MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US )


// ChꓙɎgtHgԍ_uȂKȐ
#define INIT_SPECIAL_FONT_NUMBER    322
#define GB18030_FONT_NUMBER         ( INIT_SPECIAL_FONT_NUMBER + 0 )


// [Uڍאݒ肵tHg̔ԍ̏l
// ̔ԍ{ Unicode Block Index ỸtHgԍ
#define INIT_USER_DEFINED_FONT_NUMBER  400



// Putc ̉֐ (RTFt@C̏o͂)
//  0 Ԃ
// G[ StreamOut ɃG[lۑ
// t@C̖ŕK C  EOF w肵ČĂяo
static int PutcRtf ( int C, int cUnicode, int nSrcCodePage, const char *szUnknown, RTFCONVDATA *pData ) {

   STREAM *StreamOut =  pData->StreamOut ;
   unsigned long dwConversionMode = pData->dwConversionMode ;
   unsigned long dwAdditionalMode = pData->dwAdditionalMode ;


   // t@C̍Ō
   if ( C == EOF ) {
      if ( pData->wr->ReverseBuffer ) {
         wchar32_t *szBuffer = GetStoredCharsW32 ( pData->wr->ReverseBuffer ) ;
         if ( *szBuffer ) {
            pData->wr->IsReverseOnFlushBuffer = 0 ;
            pData->wr->IsBidiFlushBuffer = 1 ;
            for ( ; *szBuffer ; szBuffer ++ ) {
               PutcRtf ( 0, *szBuffer, CP_UNICODE, "?", pData ) ;
            }
            pData->wr->IsBidiFlushBuffer = 0 ;
            ClearStoredCharsW32 ( pData->wr->ReverseBuffer ) ;
         }
      }
      if ( pData->wr->nFontPrev >= 0 ) StreamWrite ( BRACE_END, StreamOut ) ;    // tHg̏I

      FreeStoredCharsW32 ( pData->wr->ReverseBuffer ) ;
      return 0 ;
   }


   // BOM
   if ( cUnicode == BYTE_ORDER_MARK ) return 0 ;


   // 䕶̏
   if ( cUnicode < 0x20 && cUnicode != L'\n' && cUnicode != L'\t' ) return 0 ;


   // E獶H
   int IsReverse ;
   int nBidi = 0 ;
   if ( pData->wr->IsBidiFlushBuffer ) IsReverse = pData->wr->IsReverseOnFlushBuffer ;
   else {
      nBidi = UnicodeToBidirect ( cUnicode ) ;
      IsReverse = pData->wr->IsReversePrev ;
      if ( nBidi > 0 ) IsReverse = 0 ;
      if ( nBidi < 0 ) IsReverse = 1 ;
      if ( IsReverse && ! nBidi ) {
         pData->wr->ReverseBuffer = StoreCharsW32 ( cUnicode, pData->wr->ReverseBuffer ) ;
         if ( pData->wr->ReverseBuffer == STOREDCHARS_ERROR ) {
            if ( StreamOut ) *(int*) GetStreamValue ( StreamOut ) = CONV_ERROR_MEMORY ;
         }
         return 0 ;
      }
   }

   // ܂ro
   if ( pData->wr->ReverseBuffer ) {
      wchar32_t *szBuffer = GetStoredCharsW32 ( pData->wr->ReverseBuffer ) ;
      if ( *szBuffer && ! pData->wr->IsBidiFlushBuffer ) {
         pData->wr->IsReverseOnFlushBuffer = IsReverse ;
         pData->wr->IsBidiFlushBuffer = 1 ;
         for ( ; *szBuffer ; szBuffer ++ ) {
            PutcRtf ( 0, *szBuffer, CP_UNICODE, "?", pData ) ;
         }
         pData->wr->IsBidiFlushBuffer = 0 ;
         ClearStoredCharsW32 ( pData->wr->ReverseBuffer ) ;
      }
   }


   // ̓R[hy[W
   int nSrcCodePageForRtf ;
   if ( nSrcCodePage != CP_UNICODE ) nSrcCodePageForRtf = pData->nSrcWinCodePage ;
   else {
      if ( cUnicode >= 0x2018 && cUnicode <= 0x201F ) {
         if ( cUnicode & 1 ) pData->wr->IsInsideQuote -- ;
         else                pData->wr->IsInsideQuote ++ ;
      }
      if ( cUnicode == '\n' ) pData->wr->IsInsideQuote = 0 ;
      if ( ! pData->wr->IsInsideQuote && ! pData->wr->IsBackgroundMultiByte ) pData->wr->IsBackgroundMultiByte = ( cUnicode >= 0x3000 ) ;
      if ( cUnicode >= 0x3000 || pData->wr->IsBackgroundMultiByte ) nSrcCodePageForRtf = pData->wr->nDefaultCodePage ;
      else                                                          nSrcCodePageForRtf = CP_ANSI_EUROPEAN ;
   }


   int nUnicodeBlock = -1 ;


   // tHg̎w
   int nFont = GetUserDefinedFont ( cUnicode, nUnicodeBlock, & nUnicodeBlock, pData ) ;
   if ( nFont < 0 ) {
      // PoCg̃tHg͕ʂɐݒ
      if ( isascii ( cUnicode ) && pData->wr->nFontPrev >= 0 && pData->wr->nFontPrev < INIT_SPECIAL_FONT_NUMBER && pData->wr->nFontPrev != THAI_CHARSET ) nFont = pData->wr->nFontPrev ;
      else {

         // ȃtHg nFont ͕ZbgԍƓi̕ϊj
         nFont = UnicodeToCharset ( cUnicode, nUnicodeBlock, & nUnicodeBlock, nSrcCodePageForRtf, ( dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_VISTA ) ) ;
         if ( nFont < 0 && nSrcCodePage == CP_UNICODE && ( cUnicode & 0xF000 ) == 0x2000 && ! pData->wr->IsBackgroundMultiByte ) {
            nSrcCodePageForRtf = pData->wr->nDefaultCodePage ;
            nFont = UnicodeToCharset ( cUnicode, nUnicodeBlock, & nUnicodeBlock, nSrcCodePageForRtf, ( dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_VISTA ) ) ;
         }

         // tHg̒
         if ( dwConversionMode & CONVMODE_USE_LOCAL_FONT && nFont != HANGUL_CHARSET ) {
            if ( IsMultiByteCodePage ( nSrcCodePageForRtf ) ) nFont = CodePageToCharset ( nSrcCodePageForRtf ) ;
            else                                              nFont = CodePageToCharset ( pData->wr->nDefaultCodePage ) ;
         }
         else if ( isascii ( cUnicode ) ) {
            if ( IsMultiByteCodePage ( nSrcCodePageForRtf ) ) nFont = CodePageToCharset ( nSrcCodePageForRtf ) ;
         }
         else if ( IsUnicodePua ( cUnicode ) ) {
            if ( nSrcCodePageForRtf == CP_CHINESE_SIMPLIFIED && IsLinshiDaima ( cUnicode ) ) {
               if ( dwAdditionalMode & ADDITIONAL_MODE_GBK_TO_LINSHIDAIMA ) nFont = CodePageToCharset ( nSrcCodePageForRtf ) ;
            }
            else if ( dwConversionMode & CONVMODE_CONVERT_PUA ) { // Don't remove "else"!
               if ( IsMultiByteCodePage ( nSrcCodePageForRtf ) ) nFont = CodePageToCharset ( nSrcCodePageForRtf ) ;
               else                                              nFont = CodePageToCharset ( pData->wr->nDefaultCodePage ) ;
            }
         }

         // ȃtHg
         if ( nFont < 0 ) {
            nFont = 0 ;
            if ( IsCjkExA ( cUnicode ) || IsCjkExB ( cUnicode ) ) {
               nFont = GB18030_FONT_NUMBER ;
               if ( dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_VISTA ) {
                  if ( nSrcCodePageForRtf == CP_CHINESE_TRADITIONAL ) nFont = CHINESEBIG5_CHARSET ;
                  else                                                nFont = GB2312_CHARSET ;
               }
               else if ( ! IsFourByteLetter ( cUnicode ) && IsHaveLinshiDaima ( cUnicode ) ) {
                  nFont = GB2312_CHARSET ;
               }
            }
            if ( IsCjkExC ( cUnicode ) ) {
               if ( dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_VISTA ) {
                  if ( nSrcCodePageForRtf == CP_CHINESE_TRADITIONAL ) nFont = CHINESEBIG5_CHARSET ;
                  else                                                nFont = GB2312_CHARSET ;
               }
            }
         }
      }
   }

   // PoCg̃tHgw
   int nLowByteFont = GetUserDefinedFont ( 0, UB_BASIC_LATIN, NULL, pData ) ;
   if ( nLowByteFont < 0 ) {
      nLowByteFont = ANSI_CHARSET ;
      if ( dwConversionMode & CONVMODE_USE_LOCAL_FONT ) nLowByteFont = nFont ;
      if ( nFont >= INIT_SPECIAL_FONT_NUMBER || nFont == THAI_CHARSET ) nLowByteFont = nFont ;
   }


   // 
   int nLang = GetUserDefinedLangId ( cUnicode, nUnicodeBlock, & nUnicodeBlock, pData ) ;
   int nLangFe = 0 ;
   if ( ! nLang ) {
      // Lނ͑Ǒp
      if ( pData->wr->nLangPrev && ! nBidi ) { nLang = pData->wr->nLangPrev ; nLangFe = pData->wr->nLangFePrev ; }
      else {
         if ( ! nLang && isascii ( cUnicode ) && IsMultiByteCodePage ( nSrcCodePageForRtf ) ) nLang = CharsetToLangId ( CodePageToCharset ( nSrcCodePageForRtf ) ) ;
         if ( ! nLang && nFont < INIT_SPECIAL_FONT_NUMBER && ( nFont != ANSI_CHARSET || cUnicode < 0x0370 ) ) {   // 0x0370 Greek Start
            nLang = CharsetToLangId ( nFont ) ;
            if ( ( nSrcCodePage == CP_CYRILLIC_KOI8U || nSrcCodePage == CP_MAC_UKRAINIAN ) && nFont == RUSSIAN_CHARSET ) nLang = MAKELANGID ( LANG_UKRAINIAN, SUBLANG_DEFAULT ) ;
            if ( nSrcCodePage == CP_CHINESE_HKSCS && nFont == CHINESEBIG5_CHARSET ) nLang = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_HONGKONG ) ;
            if ( nSrcCodePage == CP_PERSIAN_ISIRI && nFont == ARABIC_CHARSET ) nLang = MAKELANGID ( LANG_FARSI, SUBLANG_DEFAULT ) ;
         }
         if ( ! nLang ) {
            if ( cUnicode == 0x0964 || cUnicode == 0x0965 ) nLang = pData->wr->nLangPrev ;   // Indic Danda
         }
         if ( ! nLang ) {
            nLang = UnicodeToLangId ( cUnicode, nUnicodeBlock, & nUnicodeBlock ) ;
            if ( nSrcCodePage == CP_PERSIAN_ISIRI && nLang == MAKELANGID ( LANG_ARABIC, SUBLANG_DEFAULT ) ) nLang = MAKELANGID ( LANG_FARSI, SUBLANG_DEFAULT ) ;
            if ( nSrcCodePage == CP_ISCII_ASSAMESE && nLang == MAKELANGID ( LANG_BENGALI, SUBLANG_DEFAULT ) ) nLang = MAKELANGID ( LANG_ASSAMESE, SUBLANG_DEFAULT ) ;
         }
         if ( ! nLang ) {
            if ( nFont == GB18030_FONT_NUMBER ) nLang = CharsetToLangId ( GB2312_CHARSET ) ;
            else                                nLang = DEFAULT_LANGID ;
         }
      }
   }
   // AWAp̌w
   if ( ! nLangFe ) {
      if ( IsLangIdCjk ( nLang ) ) nLangFe = nLang ;
      if ( nLangFe ) nLang = DEFAULT_LANGID ;
   }


   // tHg̏o
   if ( pData->wr->nFontPrev != nFont || pData->wr->nLowByteFontPrev != nLowByteFont || pData->wr->nLangPrev != nLang || pData->wr->nLangFePrev != nLangFe || pData->wr->IsReversePrev != IsReverse ) {

      // ÂtHg̏I
      if ( pData->wr->nFontPrev >= 0 ) StreamWrite ( BRACE_END, StreamOut ) ;

      // VtHg̊Jn
      StreamWrite ( BRACE_START, StreamOut ) ;

      // EE̐؂ւ
      if ( pData->wr->IsReversePrev && ! IsReverse ) StreamWriteString ( "\\ltrch ", StreamOut ) ;
      if ( IsReverse ) StreamWriteString ( "\\rtlch ", StreamOut ) ;

      // tHg̊Jn
      if ( nLangFe ) StreamWriteFormat ( StreamOut, "\\lang%d\\langfe%d ", (short) nLang, (short) nLangFe ) ;
      else           StreamWriteFormat ( StreamOut, "\\lang%d ", (short) nLang ) ;

      if ( nFont >= INIT_SPECIAL_FONT_NUMBER ) StreamWriteFormat ( StreamOut, "\\fs%d\\loch\\af%d\\hich\\af%d\\dbch\\f%d ", (short) pData->wr->nFontSize, (short) nFont, (short) nFont, (short) nFont ) ;
      else if ( IsMultiByteCharset ( nFont ) ) StreamWriteFormat ( StreamOut, "\\fs%d\\loch\\af%d\\hich\\af%d\\dbch\\f%d ", (short) pData->wr->nFontSize, (short) nLowByteFont, (short) nLowByteFont, (short) nFont ) ;
      else                                     StreamWriteFormat ( StreamOut, "\\fs%d\\loch\\af%d\\hich\\f%d ",             (short) pData->wr->nFontSize, (short) nLowByteFont, (short) nFont ) ;

      pData->wr->nFontPrev = nFont ;
      pData->wr->nLowByteFontPrev = nLowByteFont ;
      pData->wr->nLangPrev = nLang ;
      pData->wr->nLangFePrev = nLangFe ;
      pData->wr->IsReversePrev = IsReverse ;
   }


   // ̕Zbgɕϊ
   int nDstCodePage ;
   if ( nFont == THAI_CHARSET )                       nDstCodePage = CharsetToCodePage ( ANSI_CHARSET ) ;
   else if ( nFont < INIT_SPECIAL_FONT_NUMBER )       nDstCodePage = CharsetToCodePage ( nFont ) ;
   else if ( nFont >= INIT_USER_DEFINED_FONT_NUMBER ) nDstCodePage = CharsetToCodePage ( GetUserDefinedFontCharset ( nFont, pData ) ) ;
   else                                               nDstCodePage = 0 ;

   // ϊ
   int cMbchar ;
   if ( nDstCodePage && nDstCodePage != CP_SYMBOL ) {
      cMbchar = UnicodeToChar ( cUnicode, nDstCodePage, dwConversionMode ) ;
      if ( cMbchar == EOF ) {
         if ( StreamOut ) *(int*) GetStreamValue ( StreamOut ) = CONV_ERROR_CODEPAGE ;
         return 0 ;
      }
      if ( IsFourByteLetter ( cMbchar ) ) cMbchar = 0 ;
   }
   else {
      if ( isascii ( cUnicode ) ) cMbchar = cUnicode ;
      else                        cMbchar = 0 ;
   }


   //  RTF pɃGR[hďo

   // L̏o
   if ( cUnicode < 0x100 || ! cMbchar ) {
      for ( int nCount = 0 ; nCount < RTF_ESCAPE_LIST_NELEM ; nCount ++ ) {
         if ( cUnicode == ( RtfEscapeList [ nCount ] ).nCode ) {
            if ( cUnicode == '\n' ) StreamWriteString ( "\r\n", StreamOut ) ;
            StreamWriteFormat ( StreamOut, "\\%s", ( RtfEscapeList [ nCount ] ).szName ) ;
            if ( isalpha ( *( RtfEscapeList [ nCount ] ).szName ) ) StreamWriteString ( " ", StreamOut ) ;
            return 0 ;
         }
      }
   }

   // ϊłȂꍇ Unicode ŏo
   if ( cMbchar == 0 ) {
      if ( IsFourByteLetter ( cUnicode ) ) {
         if ( (unsigned) cUnicode > UTF16_MAX ) return PutUnmappableChar ( cUnicode, szUnknown, pData ) ;
         cUnicode = MAKE_UNICODE_SURROGATE ( cUnicode ) ;
         StreamWriteFormat ( StreamOut, "\\u%d", (short) ( cUnicode >> 16 ) ) ;
         StreamWriteFormat ( StreamOut, szCharFormat, '?' ) ;
      }
      StreamWriteFormat ( StreamOut, "\\u%d", (short) cUnicode ) ;
      StreamWriteFormat ( StreamOut, szCharFormat, '?' ) ;
      return 0 ;
   }

   // QoCg̏o
   if ( cMbchar >= 0x100 ) {
      StreamWriteFormat ( StreamOut, szCharFormat, cMbchar >> 8 ) ;
      StreamWriteFormat ( StreamOut, szCharFormat, cMbchar & 0xFF ) ;
      return 0 ;
   }

   // PoCgAXL[̏o
   if ( ! isascii ( cMbchar ) ) {
      StreamWriteFormat ( StreamOut, szCharFormat, cMbchar ) ;
      return 0 ;
   }

   // AXL[̏o
   if ( ! iscntrl ( cMbchar ) ) {
      StreamWrite ( cMbchar, StreamOut ) ;
      return 0 ;
   }

   return 0 ;
}



// Unicode K؂ȕZbgԂ
// ŶȂ -1 Ԃ
static int UnicodeToCharset ( int cUnicode, int nIndex, int *pIndex, int nSrcCodePage, int IsForVista ) {

   if ( cUnicode ) {

      unsigned long nBit = GetUnicodeCharsetBit ( cUnicode ) ;
      if ( nBit != (unsigned long) -1 ) {

         if ( cUnicode >= 0x2000 ) {
            if ( nSrcCodePage == CP_JAPANESE && nBit & UNICS_SJIS )              return SHIFTJIS_CHARSET ;
            if ( nSrcCodePage == CP_KOREAN && nBit & UNICS_KS )                  return HANGUL_CHARSET ;
            if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED && nBit & UNICS_GB )      return GB2312_CHARSET ;
            if ( nSrcCodePage == CP_CHINESE_TRADITIONAL && nBit & UNICS_BIG5 )   return CHINESEBIG5_CHARSET ;
         }
         if ( ! isascii ( cUnicode ) ) {
            if ( nBit & UNICS_CYRILLIC && nSrcCodePage == CP_ANSI_CYRILLIC )        return RUSSIAN_CHARSET ;
            if ( nBit & UNICS_GREEK && nSrcCodePage == CP_ANSI_GREEK )              return GREEK_CHARSET ;
            if ( nBit & UNICS_TURKISH && nSrcCodePage == CP_ANSI_TURKISH )          return TURKISH_CHARSET ;
            if ( nBit & UNICS_BALTIC && nSrcCodePage == CP_ANSI_BALTIC )            return BALTIC_CHARSET ;
            if ( nBit & UNICS_CENTRAL && nSrcCodePage == CP_ANSI_CENTRAL_EUROPEAN ) return EASTEUROPE_CHARSET ;
         }
         if ( 1 ) {
            if ( nBit & UNICS_EUROPEAN )  return ANSI_CHARSET ;
            if ( nBit & UNICS_CENTRAL )   return EASTEUROPE_CHARSET ;
            if ( nBit & UNICS_BALTIC )    return BALTIC_CHARSET ;
            if ( nBit & UNICS_TURKISH )   return TURKISH_CHARSET ;
            if ( nBit & UNICS_GREEK )     return GREEK_CHARSET ;
            if ( nBit & UNICS_CYRILLIC )  return RUSSIAN_CHARSET ;
            if ( IsForVista ) {
               if ( nBit & UNICS_EUROPEAN_VISTA )  return ANSI_CHARSET ;
               if ( nBit & UNICS_GREEK_VISTA )     return GREEK_CHARSET ;
               if ( nBit & UNICS_CYRILLIC_VISTA )  return RUSSIAN_CHARSET ;
            }
         }
         if ( cUnicode < 0x2000 ) {
            if ( nSrcCodePage == CP_JAPANESE && nBit & UNICS_SJIS )              return SHIFTJIS_CHARSET ;
            if ( nSrcCodePage == CP_KOREAN && nBit & UNICS_KS )                  return HANGUL_CHARSET ;
            if ( nSrcCodePage == CP_CHINESE_SIMPLIFIED && nBit & UNICS_GB )      return GB2312_CHARSET ;
            if ( nSrcCodePage == CP_CHINESE_TRADITIONAL && nBit & UNICS_BIG5 )   return CHINESEBIG5_CHARSET ;
         }

         return -1 ;
      }
   }


   if ( nIndex < 0 ) nIndex = GetUnicodeBlock ( cUnicode, 0, 0, NULL, NULL, NULL ) ;

   if ( nIndex >= 0 ) {
      if ( pIndex ) *pIndex = nIndex ;

      if ( nIndex == UB_HALFWIDTH_AND_FULLWIDTH_FORMS ) {
         if ( cUnicode >= 0xFF61 && cUnicode <= 0xFF9F ) return SHIFTJIS_CHARSET ;  // pJi
         if ( cUnicode == 0xFFE6 )                       return HANGUL_CHARSET ;    // SpEHL
      }
      if ( nIndex == UB_CJK_SYMBOLS_AND_PUNCTUATION ) {
         // hB
         if ( cUnicode >= 0x3021 && cUnicode <= 0x3029 || cUnicode >= 0x3038 && cUnicode <= 0x303A ) {
            if ( nSrcCodePage == CP_CHINESE_TRADITIONAL ) return CHINESEBIG5_CHARSET ;
            else                                          return GB2312_CHARSET ;
         }
      }

      int nCharset = UnicodeBlockToCharset ( nIndex ) ;

      if ( nCharset == UNICS_CJK_CHARSET ) {
         if ( nSrcCodePage != CP_UNICODE ) return CodePageToCharset ( nSrcCodePage ) ;
         else                              return -1 ;
      }
      if ( nCharset == UNICS_CHINESE_CHARSET ) {
         if ( nSrcCodePage == CP_CHINESE_TRADITIONAL ) return CHINESEBIG5_CHARSET ;
         else                                          return GB2312_CHARSET ;
      }
      if ( nCharset != UNICS_CHARSET_UNDEFINED ) return nCharset ;
   }

   return -1 ;
}



// Unicode oASY擾
// Ȃ 1 AtȂ -1 AȂ 0 Ԃ
static int UnicodeToBidirect ( int cUnicode ) {

   if ( isascii ( cUnicode ) ) {
      if ( isalpha ( cUnicode ) || iscntrl ( cUnicode ) ) return 1 ;
      else                                                return 0 ; // punct, space, digit
   }

   if ( /* cUnicode >= 0x0080 && */ cUnicode <= 0x04FF ) {
      if ( cUnicode >= 0x00A0 && cUnicode <= 0x00BF ) return 0 ;  // punct
      if ( cUnicode == 0x00D7 ) return 0 ;   // punct
      if ( cUnicode == 0x00F7 ) return 0 ;   // punct
      if ( cUnicode >= 0x02B0 /* && cUnicode <= 0x02FF ) return 0 ;  // Spacing Modifier Letters
      if ( cUnicode >= 0x0300 */ && cUnicode <= 0x036F ) return 0 ;  // Combining Diacritical Marks
      return 1 ;
   }

   if ( /* cUnicode >= 0x0500 && */ cUnicode <= 0x30FF ) {

      if ( /* cUnicode >= 0x0500 && */ cUnicode <= 0x08FF ) {
         if ( cUnicode >= 0x0660 && cUnicode <= 0x066D ) return 0 ;  // Arabic-Indic Digits
         if ( cUnicode >= 0x06F0 && cUnicode <= 0x06F9 ) return 0 ;  // Extended Arabic-Indic Digits

         if ( cUnicode >= 0x0590 /* && cUnicode <= 0x05FF ) return -1 ; // Hebrew Basic
         if ( cUnicode >= 0x0600    && cUnicode <= 0x06FF ) return -1 ; // Arabic Basic
         if ( cUnicode >= 0x0700    && cUnicode <= 0x074F ) return -1 ; // Syriac
         if ( cUnicode >= 0x0750    && cUnicode <= 0x077F ) return -1 ; // Arabic Supplement
         if ( cUnicode >= 0x0780    && cUnicode <= 0x07BF ) return -1 ; // Thaana
         if ( cUnicode >= 0x07C0 */ && cUnicode <= 0x07FF ) return -1 ; // NKo
         return 1 ;
      }

      if ( /* cUnicode >= 0x0900 && */ cUnicode <= 0x1FFF ) {
         if ( cUnicode == 0x0964 || cUnicode == 0x0965 ) return 0 ;  // Indic Danda
         return 1 ;
      }

      if ( /* cUnicode >= 0x2000 && */ cUnicode <= 0x21FF ) {
         if ( /* cUnicode >= 0x2000 && */ cUnicode <= 0x200D ) return 0 ;  // Spaces
         if ( cUnicode == 0x200E ) return 1 ;      //    &#lrm;
         if ( cUnicode == 0x200F ) return -1 ;     //    &#rlm;
         if ( /* cUnicode >= 0x2010 && */ cUnicode <= 0x206F ) return 0 ;  // General Punctuation
         if ( /* cUnicode >= 0x2070 && */ cUnicode <= 0x209F ) return 0 ;  // Subscripts and Superscripts
         if ( cUnicode == 0x20AA ) return 1 ;                           // Currency Symbols
         if ( cUnicode >= 0x20A0 && cUnicode <= 0x20AC ) return 0 ;  // Currency Symbols
         if ( cUnicode >= 0x20D0 && cUnicode <= 0x20FF ) return 0 ;  // Combining Diacritical Marks for Symbols
         if ( cUnicode >= 0x2190 && cUnicode <= 0x21FF ) return 0 ;  // Arrows
         return 1 ;
      }

      if ( /* cUnicode >= 0x2200 && */ cUnicode <= 0x2FFF ) return 1 ;

      if ( cUnicode >= 0x3000 && cUnicode <= 0x3004 ) return 0 ;  // CJK Symbols and Punctuation
      if ( cUnicode >= 0x3008 && cUnicode <= 0x3020 ) return 0 ;  // CJK Symbols and Punctuation

      return 1 ;
   }

   if ( /* cUnicode >= 0x3100 && */ cUnicode <= 0xFAFF ) {
      return 1 ;
   }

   if ( /* cUnicode >= 0xFB00 && */ cUnicode <= 0xFFFF ) {
      if ( cUnicode >= 0xFB19 /* && cUnicode <= 0xFB4F ) return -1 ; // Hebrew Presentation Forms
      if ( cUnicode >= 0xFB50 */ && cUnicode <= 0xFDFF ) return -1 ; // Arabic Presentation Forms-A
      if ( cUnicode >= 0xFE70    && cUnicode <= 0xFEFE ) return -1 ; // Arabic Presentation Forms-B
      return 1 ;
   }

   if ( /* cUnicode >= 0xFFFF && */ cUnicode <= 0x10FFFF ) {
      if ( cUnicode >= 0x10800 && cUnicode <= 0x1083F ) return -1 ;  // Cypriot Syllabary
      if ( cUnicode >= 0x10900 && cUnicode <= 0x1091F ) return -1 ;  // Phoenician
      if ( cUnicode >= 0x10A00 && cUnicode <= 0x10A5F ) return -1 ;  // Kharoshthi
      if ( cUnicode >= 0xE0000 && cUnicode <= 0xE007F ) return 0 ;   // Tags
      return 1 ;
   }

   return 1 ;
}



////////////////////////////////////////////
//      t@C̐擪Ɩo        //
////////////////////////////////////////////



static int WriteFontTable ( RTFCONVDATA *pData ) ;



// t@C擪o
static int WriteFileStart ( RTFCONVDATA *pData ) {

   switch ( pData->nDstCodePage ) {
      // for RTF
      case 0 :
         StreamWriteString ( szRtfStartString, pData->StreamOut ) ;
         WriteFontTable ( pData ) ;
         break ;
      // for TXT
      case CP_ISO_KOREAN :
         StreamWriteString ( "\x1B$)C", pData->StreamOut ) ;
         break ;
      case CP_UTF16 :
      case CP_UTF16_BE :
      case CP_UTF32 :
      case CP_UTF32_BE :
      case CP_UTF8 :
         Putc ( BYTE_ORDER_MARK, CP_UNICODE, pData ) ;
         break ;
   }

   return CONV_SUCCESS ;
}



// t@Co
static int WriteFileEnd ( RTFCONVDATA *pData ) {
   if ( ! pData->nDstCodePage ) StreamWriteString ( szRtfEndString, pData->StreamOut ) ;
   return CONV_SUCCESS ;
}



static char *EncodeFontName ( const char *szSrc, char *szDst, int nCodePage ) ;



// tHgo
static int WriteFontTable ( RTFCONVDATA *pData ) {

   STREAM *StreamOut = pData->StreamOut ;

   char szBuffer [ MAX_PATH * 2 ] ;
   int nFontNumber ;
   const char *szFontSuffix ;
   static const char szFontSuffixNull [] = "" ;


   // tHg
   const char *szDefaultJapaneseFont            = DEFAULT_JAPANESE_FONT ;
   const char *szDefaultChineseSimplifiedFont   = DEFAULT_CHINESE_SIMPLIFIED_FONT ;
   const char *szDefaultChineseTraditionalFont  = DEFAULT_CHINESE_TRADITIONAL_FONT ;
   const char *szDefaultKoreanFont              = DEFAULT_KOREAN_FONT ;
   const char *szDefaultGb18030Font             = DEFAULT_GB18030_FONT ;

   if ( nOsCodePage == CP_JAPANESE ) {
      const char *szTmp = GetLocalizedFontName ( SHIFTJIS_CHARSET, szDefaultJapaneseFont, NULL ) ;
      if ( szTmp ) szDefaultJapaneseFont = szTmp ;
   }
   else if ( nOsCodePage == CP_CHINESE_SIMPLIFIED ) {
      const char *szTmp = GetLocalizedFontName ( GB2312_CHARSET, szDefaultChineseSimplifiedFont, NULL ) ;
      if ( szTmp ) szDefaultChineseSimplifiedFont = szTmp ;
      const char *szTmpx = GetLocalizedFontName ( GB2312_CHARSET, szDefaultGb18030Font, NULL ) ;
      if ( szTmpx ) szDefaultGb18030Font = szTmpx ;
   }
   else if ( nOsCodePage == CP_CHINESE_TRADITIONAL ) {
      const char *szTmp = GetLocalizedFontName ( CHINESEBIG5_CHARSET, szDefaultChineseTraditionalFont, NULL ) ;
      if ( szTmp ) szDefaultChineseTraditionalFont = szTmp ;
   }
   else if ( nOsCodePage == CP_KOREAN ) {
      const char *szTmp = GetLocalizedFontName ( HANGUL_CHARSET, szDefaultKoreanFont, NULL ) ;
      if ( szTmp ) szDefaultKoreanFont = szTmp ;
   }


   RTFFONTINFO FontInfo = { 0 } ;
   if ( pData->wr->pFontInfo ) FontInfo = *pData->wr->pFontInfo ;

   if ( ! FontInfo.pEuropean           || ! *FontInfo.pEuropean )             FontInfo.pEuropean = DEFAULT_EUROPEAN_FONT ;
   if ( ! FontInfo.pJapanese           || ! *FontInfo.pJapanese )             FontInfo.pJapanese = szDefaultJapaneseFont ;
   if ( ! FontInfo.pChineseSimplified  || ! *FontInfo.pChineseSimplified )    FontInfo.pChineseSimplified = szDefaultChineseSimplifiedFont ;
   if ( ! FontInfo.pChineseTraditional || ! *FontInfo.pChineseTraditional )   FontInfo.pChineseTraditional = szDefaultChineseTraditionalFont ;
   if ( ! FontInfo.pKorean             || ! *FontInfo.pKorean )               FontInfo.pKorean = szDefaultKoreanFont ;
   if ( ! FontInfo.pThai               || ! *FontInfo.pThai )                 FontInfo.pThai = DEFAULT_THAI_FONT ;


   // tHg
   int nGbxFontCharset = FontNameToCharset ( szDefaultGb18030Font, 0 ) ;


   StreamWrite ( BRACE_START, StreamOut ) ;
   StreamWriteString ( "\\fonttbl", StreamOut ) ;

   // Charset 0 (EUROPEAN)
   nFontNumber = ANSI_CHARSET ;
   szFontSuffix = szFontSuffixNull ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_EUROPEAN ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 128 (JAPANESE)
   nFontNumber = SHIFTJIS_CHARSET ;
   szFontSuffix = szFontSuffixNull ;
   EncodeFontName ( FontInfo.pJapanese, szBuffer, CP_JAPANESE ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 129 (KOREAN)
   nFontNumber = HANGUL_CHARSET ;
   szFontSuffix = szFontSuffixNull ;
   EncodeFontName ( FontInfo.pKorean, szBuffer, CP_KOREAN ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 134 (GB)
   nFontNumber = GB2312_CHARSET ;
   szFontSuffix = szFontSuffixNull ;
   EncodeFontName ( FontInfo.pChineseSimplified, szBuffer, CP_CHINESE_SIMPLIFIED ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 136 (BIG5)
   nFontNumber = CHINESEBIG5_CHARSET ;
   szFontSuffix = szFontSuffixNull ;
   EncodeFontName ( FontInfo.pChineseTraditional, szBuffer, CP_CHINESE_TRADITIONAL ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 161 (GREEK)
   nFontNumber = GREEK_CHARSET ;
   szFontSuffix = szFontSuffixGreek ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_GREEK ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 162 (TURKISH)
   nFontNumber = TURKISH_CHARSET ;
   szFontSuffix = szFontSuffixTurkish ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_TURKISH ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 177 (HEBREW)
   nFontNumber = HEBREW_CHARSET ;
   szFontSuffix = szFontSuffixHebrew ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_HEBREW ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 178 (ARABIC)
   nFontNumber = ARABIC_CHARSET ;
   szFontSuffix = szFontSuffixArabic ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_ARABIC ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 186 (BALTIC)
   nFontNumber = BALTIC_CHARSET ;
   szFontSuffix = szFontSuffixBaltic ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_BALTIC ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 204 (CYRILLIC)
   nFontNumber = RUSSIAN_CHARSET ;
   szFontSuffix = szFontSuffixCyrillic ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_CYRILLIC ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // Charset 222 (uses "Tahoma" Charset ANSI_CHARSET)
   nFontNumber = THAI_CHARSET ;
   szFontSuffix = szFontSuffixNull ;
   EncodeFontName ( FontInfo.pThai, szBuffer, CP_ANSI_EUROPEAN ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, ANSI_CHARSET, szBuffer, szFontSuffix ) ;

   // Charset 238 (CENTRAL EUROPEAN)
   nFontNumber = EASTEUROPE_CHARSET ;
   szFontSuffix = szFontSuffixCentralEuropean ;
   EncodeFontName ( FontInfo.pEuropean, szBuffer, CP_ANSI_CENTRAL_EUROPEAN ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nFontNumber, szBuffer, szFontSuffix ) ;

   // ȉ͕ϑI

   // FontNumber 322 (uses "Simsun (Founder Extended)" )
   if ( ! ( pData->dwAdditionalMode & ADDITIONAL_MODE_RTFFONT_VISTA ) ) {
      nFontNumber = GB18030_FONT_NUMBER ;
      szFontSuffix = szFontSuffixNull ;
      EncodeFontName ( szDefaultGb18030Font, szBuffer, CharsetToCodePage ( nGbxFontCharset ) ) ;
      StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nGbxFontCharset, szBuffer, szFontSuffix ) ;
   }

   // [Uݒ肵tHgԍ
   if ( pData->wr->FontNameArray ) {

      for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {

         const char *szFontName = pData->wr->FontNameArray [ nIndex ] ;
         if ( ! szFontName ) continue ;

         if ( nFontNumber >= pData->wr->FontNumberArray [ nIndex ] ) continue ;

         nFontNumber = pData->wr->FontNumberArray [ nIndex ] ;
         int nCharset = pData->wr->CharsetArray [ nFontNumber - INIT_USER_DEFINED_FONT_NUMBER ] ;

         szFontSuffix = szFontSuffixNull ;
         EncodeFontName ( szFontName, szBuffer, CharsetToCodePage ( nCharset ) ) ;
         StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, nCharset, szBuffer, szFontSuffix ) ;
      }
   }

   StreamWrite ( BRACE_END, StreamOut ) ;
   StreamWriteString ( "\r\n", StreamOut ) ;

   return 0 ;
}



static char *EncodeHexaDecimal ( const char *szSrc, char *szDst, int nCodePage ) ;



// QoCg̃tHg16i\Lɕϊ
// [JȃtHgɂ͉pꖼt
static char *EncodeFontName ( const char *szSrc, char *szDst, int nCodePage ) {

   char szTmp [ FONTNAME_MAX ] ;
   if ( strlen ( szSrc ) >= FONTNAME_MAX ) {
      strlcpy ( szTmp, szSrc, FONTNAME_MAX ) ;
      szSrc = szTmp ;
   }

   EncodeHexaDecimal ( szSrc, szDst, nCodePage ) ;

   if ( ! IsAsciiString ( szSrc ) ) {
      const char *szEnglishName = GetEnglishFontName ( CodePageToCharset ( nCodePage ), szSrc, NULL ) ;
      if ( szEnglishName ) {
         sprintf ( strend ( szDst ), "%c\\*\\falt %s%c", BRACE_START, szEnglishName, BRACE_END ) ;
         szSrc = szEnglishName ;
      }
   }

   const char *szAltName = NULL ;
   if ( ! stricmp ( szSrc, "SimSun-18030" ) || ! stricmp ( szSrc, "Simsun (Founder Extended)" ) ) szAltName = "SimSun" ;
   if ( ! stricmp ( szSrc, "NSimSun-18030" ) ) szAltName = "NSimSun" ;
   if ( szAltName ) sprintf ( strend ( szDst ), "%c\\*\\falt %s%c", BRACE_START, szAltName, BRACE_END ) ;

   return szDst ;
}



// tHg當Zbg𐄑
// RTF̃tHge[uŕZbgw肳ĂȂƂɎg
static int FontNameToCharset ( const char *szFontName, int nDefaultCharset ) {

   int nCharset ;

   // European Additional
   if ( ! strtailicmp ( szFontName, szFontSuffixGreek ) ) return GREEK_CHARSET ;
   if ( ! strtailicmp ( szFontName, szFontSuffixBaltic ) ) return BALTIC_CHARSET ;
   if ( ! strtailicmp ( szFontName, szFontSuffixTurkish ) ) return TURKISH_CHARSET ;
   if ( ! strtailicmp ( szFontName, szFontSuffixCyrillic ) ) return RUSSIAN_CHARSET ;
   if ( ! strtailicmp ( szFontName, szFontSuffixCentralEuropean ) ) return EASTEUROPE_CHARSET ;
   if ( ! strtailicmp ( szFontName, szFontSuffixHebrew ) ) return HEBREW_CHARSET ;
   if ( ! strtailicmp ( szFontName, szFontSuffixArabic ) ) return ARABIC_CHARSET ;

   // East Asian
   if ( GetLocalizedFontName ( -1, szFontName, & nCharset ) ) return nCharset ;
   if ( GetEnglishFontName ( -1, szFontName, & nCharset ) ) return nCharset ;
   if ( ! stricmp ( szFontName, "MS Song" ) ) return GB2312_CHARSET ;
   if ( ! stricmp ( szFontName, "MS Hei" ) ) return GB2312_CHARSET ;
   if ( ! stricmp ( szFontName, "MS UI Gothic" ) ) return SHIFTJIS_CHARSET ;

   if ( ! IsAsciiString ( szFontName ) ) {
      if ( IsMultiByteCodePage ( nOsCodePage ) ) return CodePageToCharset ( nOsCodePage ) ;
      else                                       return SHIFTJIS_CHARSET ;
   }

   // Symbol
   int nCodePage = FontNameToPseudoCodePageA ( CP_SYMBOL, szFontName ) ;
   if ( nCodePage & CP_DEFINE_PSEUDOCODE ) return SYMBOL_CHARSET ;

   if ( nDefaultCharset ) return nDefaultCharset ;
   return ANSI_CHARSET ;
}



////////////////////////////////////////////
//      [U[ɂtHg̐ݒ      //
////////////////////////////////////////////



// [Uݒ肵tHg̔ԍ擾
// nIndex Ȃ cUnicode AȊOȂ nIndex 狁߂
// CfbNX pIndex ɕۑ
// s -1 Ԃ
static int GetUserDefinedFont ( int cUnicode, int nIndex, int *pIndex, RTFCONVDATA *pData ) {

   if ( pData->wr->FontNameArray ) {

      if ( nIndex < 0 ) nIndex = GetUnicodeBlock ( cUnicode, 0, 0, NULL, NULL, NULL ) ;

      if ( nIndex >= 0 ) {
         if ( pIndex ) *pIndex = nIndex ;
         const char *szFontName = pData->wr->FontNameArray [ nIndex ] ;
         if ( szFontName ) return pData->wr->FontNumberArray [ nIndex ] ;
      }
   }

   return -1 ;
}



// [Uݒ肵tHg̕Zbg擾
// s 0 Ԃ
static int GetUserDefinedFontCharset ( int nFontNumber, RTFCONVDATA *pData ) {

   if ( pData->wr->CharsetArray ) {
      return pData->wr->CharsetArray [ nFontNumber - INIT_USER_DEFINED_FONT_NUMBER ] ;
   }

   return 0 ;
}



//  0 As 0 ȊOԂ
static int AllocFontArray ( RTFCONVDATA *pData, const RTFCONVEXINFO *pInfo ) {

   if ( pInfo->pFontExInfo && pInfo->nNumberOfFontExInfo ) {

      if ( pInfo->nNumberOfFontExInfo >= INT_MAX ) return 1 ;

      pData->wr->FontNameArray = (const char**) malloc ( UNICODE_BLOCK_NELEM * sizeof(char*) ) ;
      if ( ! pData->wr->FontNameArray ) return 1 ;
      memzero ( pData->wr->FontNameArray, UNICODE_BLOCK_NELEM * sizeof(char*) ) ;

      pData->wr->FontNumberArray = (unsigned short*) malloc ( UNICODE_BLOCK_NELEM * sizeof(unsigned short) ) ;
      if ( ! pData->wr->FontNumberArray ) return 1 ;
      memzero ( pData->wr->FontNumberArray, UNICODE_BLOCK_NELEM * sizeof(unsigned short) ) ;

      pData->wr->CharsetArray = (unsigned short*) malloc ( UNICODE_BLOCK_NELEM * sizeof(unsigned short) ) ;
      if ( ! pData->wr->CharsetArray ) return 1 ;
      memzero ( pData->wr->CharsetArray, UNICODE_BLOCK_NELEM * sizeof(unsigned short) ) ;


      for ( int nCount = 0 ; nCount < (int) pInfo->nNumberOfFontExInfo ; nCount ++ ) {

         const RTFFONTEXINFO *pFontExInfo = pInfo->pFontExInfo + nCount ;

         if ( pFontExInfo->C == -1 ) {
            for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
               pData->wr->FontNameArray [ nIndex ] = pFontExInfo->pFontName ;
            }
         }
         else {
            int nIndex = GetUnicodeBlock ( pFontExInfo->C, 0, 0, NULL, NULL, NULL ) ;
            if ( nIndex >= 0 ) {
               pData->wr->FontNameArray [ nIndex ] = pFontExInfo->pFontName ;
            }
         }
      }


      for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {

         const char *szFontName = pData->wr->FontNameArray [ nIndex ] ;
         if ( ! szFontName ) continue ;

         int IsFontNameFound = 0 ;
         for ( int nCount = 0 ; nCount < nIndex ; nCount ++ ) {
            const char *szTmp = pData->wr->FontNameArray [ nCount ] ;
            if ( szTmp && ! strcmp ( szTmp, szFontName ) ) {
               pData->wr->FontNumberArray [ nIndex ] = pData->wr->FontNumberArray [ nCount ] ;
               IsFontNameFound = 1 ;
               break ;
            }
         }
         if ( IsFontNameFound ) continue ;

         int nFontNumber = INIT_USER_DEFINED_FONT_NUMBER + nIndex ;
         pData->wr->FontNumberArray [ nIndex ] = nFontNumber ;

         int nCharset = FontNameToCharset ( szFontName, 0 ) ;
         if ( nCharset == SYMBOL_CHARSET ) nCharset = ANSI_CHARSET ;
         pData->wr->CharsetArray [ nFontNumber - INIT_USER_DEFINED_FONT_NUMBER ] = nCharset ;
      }
   }

   return 0 ;
}



//  0 As 0 ȊOԂ
static int FreeFontArray ( RTFCONVDATA *pData ) {

   free ( pData->wr->FontNameArray ) ;
   free ( pData->wr->FontNumberArray ) ;
   free ( pData->wr->CharsetArray ) ;

   return 0 ;
}



////////////////////////////////////////////
//     [U[ɂ錾ꎯʎq̐ݒ     //
////////////////////////////////////////////



// [Uݒ肵ꎯʎq擾
// s 0 Ԃ
static int GetUserDefinedLangId ( int cUnicode, int nIndex, int *pIndex, RTFCONVDATA *pData ) {

   if ( pData->wr->LangIdArray ) {

      if ( nIndex < 0 ) nIndex = GetUnicodeBlock ( cUnicode, 0, 0, NULL, NULL, NULL ) ;

      if ( nIndex >= 0 ) {
         if ( pIndex ) *pIndex = nIndex ;
         return pData->wr->LangIdArray [ nIndex ] ;
      }
   }

   return 0 ;
}



//  0 As 0 ȊOԂ
static int AllocLangIdArray ( RTFCONVDATA *pData, const RTFCONVEXINFO *pInfo ) {

   if ( pInfo->pLangIdInfo && pInfo->nNumberOfLangIdInfo ) {

      if ( pInfo->nNumberOfLangIdInfo >= INT_MAX ) return 1 ;

      pData->wr->LangIdArray = (unsigned short*) malloc ( UNICODE_BLOCK_NELEM * sizeof(unsigned short) ) ;
      if ( ! pData->wr->LangIdArray ) return 1 ;
      memzero ( pData->wr->LangIdArray, UNICODE_BLOCK_NELEM * sizeof(unsigned short) ) ;


      for ( int nCount = 0 ; nCount < (int) pInfo->nNumberOfLangIdInfo ; nCount ++ ) {

         const RTFLANGIDINFO *pLangIdInfo = pInfo->pLangIdInfo + nCount ;

         if ( pLangIdInfo->C == -1 ) {
            for ( int nIndex = 0 ; nIndex < UNICODE_BLOCK_NELEM ; nIndex ++ ) {
               pData->wr->LangIdArray [ nIndex ] = pLangIdInfo->nLangId ;
            }
         }
         else {
            int nIndex = GetUnicodeBlock ( pLangIdInfo->C, 0, 0, NULL, NULL, NULL ) ;
            if ( nIndex >= 0 ) {
               pData->wr->LangIdArray [ nIndex ] = pLangIdInfo->nLangId ;
            }
         }
      }

   }

   return 0 ;
}



//  0 As 0 ȊOԂ
static int FreeLangIdArray ( RTFCONVDATA *pData ) {

   free ( pData->wr->LangIdArray ) ;

   return 0 ;
}



////////////////////////////////////////////
//          ̍̏          //
////////////////////////////////////////////



// Qȏ̑gݍ킹Ԃ
// s NULL Ԃ
static const wchar_t *GetDecomposedForm ( int cUnicode ) {

   const wchar_t *szResult ;

   szResult = DecomposeUnicodeCharacter ( cUnicode ) ;
   if ( szResult ) return szResult ;

   szResult = DecomposeUnicodePresentationForms ( cUnicode ) ;
   if ( szResult ) return szResult ;

   return NULL ;
}



////////////////////////////////////////////
//             {̏               //
////////////////////////////////////////////



// VtgJIS̋mdb@ˑR[hVhal@ˑR[hɕϊ
static int SjisNec2ibm ( int cSjis ) {

   const unsigned short *pTable = SjisNec2ibmTable ( cSjis >> 8 ) ;
   if ( pTable && ( cSjis & 0xFF ) >= 0x40 ) {
      int cResult = pTable [ ( cSjis & 0xFF ) - 0x40 ] ;
      if ( cResult ) return cResult ;
   }

   return cSjis ;
}



// VtgJIS̐Vhal@ˑR[hmdb@ˑR[hɕϊ
static int SjisIbm2nec ( int cSjis ) {

   const unsigned short *pTable = SjisIbm2necTable ( cSjis >> 8 ) ;
   if ( pTable && ( cSjis & 0xFF ) >= 0x40 ) {
      int cResult = pTable [ ( cSjis & 0xFF ) - 0x40 ] ;
      if ( cResult ) return cResult ;
   }

   return cSjis ;
}



// Sp̋Lނ𔼊pɕϊifor Unicodej
// ϊłȂ 0 Ԃ
static int ZenSymbolToHanSymbol ( int cUnicode ) {

   int cResult = NormalizeUnicodeWidthForms ( cUnicode ) ;
   if ( ! ( cResult >= 0x3000 && cResult <= 0x318F ) ) {
      if ( ! ( isascii ( cResult ) && isalnum ( cResult ) ) ) return cResult ;
      else                                                    return 0 ;
   }

   return 0 ;
}



// Sp̃[}E𔼊pɕϊifor Unicodej
// ϊłȂ 0 Ԃ
static int ZenAlphaToHanAlpha ( int cUnicode ) {

   int cResult = NormalizeUnicodeWidthForms ( cUnicode ) ;
   if ( ! ( cResult >= 0x3000 && cResult <= 0x318F ) ) {
      if ( isascii ( cResult ) && isalnum ( cResult ) ) return cResult ;
      else                                              return 0 ;
   }

   return 0 ;
}



// pJiSpɕϊ
// ϊłȂ 0 Ԃ
static int HanKanaToZenKana ( int cUnicode ) {

   int cResult = NormalizeUnicodeWidthForms ( cUnicode ) ;
   if ( cResult >= 0x3000 && cResult <= 0x318F ) return cResult ;

   return 0 ;
}



// SpJiLtSpJiɕϊ
// ϊłȂ 0 Ԃ
static int ZenKanaToDakuon ( int cUnicode ) {

   static const wchar_t szHanKanaDakuon [] = L"JLNPRTVXZ\^`cegnqtwzE" ;
   static const wchar_t szZenKanaDakuon [] = L"KMOQSUWY[]_adfhorux{" ;

   const wchar_t *szString = wcschr ( szHanKanaDakuon, cUnicode ) ;
   if ( szString ) return szZenKanaDakuon [ szString - szHanKanaDakuon ] ;

   return 0 ;
}



// SpJi𔼑LtSpJiɕϊ
// ϊłȂ 0 Ԃ
static int ZenKanaToHanDakuon ( int cUnicode ) {

   static const wchar_t szHanKanaHanDakuon [] = L"nqtwz" ;
   static const wchar_t szZenKanaHanDakuon [] = L"psvy|" ;

   const wchar_t *szString = wcschr ( szHanKanaHanDakuon, cUnicode ) ;
   if ( szString ) return szZenKanaHanDakuon [ szString - szHanKanaHanDakuon ] ;

   return 0 ;
}



////////////////////////////////////////////
//        {̏ ([R[h)       //
////////////////////////////////////////////



// Chinese Writer ȑ̎ GB ɕϊ
static int Cw2gb ( int C ) {

   const unsigned short *pTable = Cw2gbTable ( C >> 8 ) ;
   if ( pTable && ( C & 0xFF ) >= 0x40 ) return pTable [ ( C & 0xFF ) - 0x40 ] ;

   int cGb = jmstojis ( SjisIbm2nec ( C ) ) ;
   if ( cGb ) cGb |= 0x8080 ;
   return cGb ;
}



// Chinese Writer ɑ̎ GB ɕϊ
static int Kg2gb ( int C ) {

   const unsigned short *pTable = Kg2gbTable ( C >> 8 ) ;
   if ( pTable && ( C & 0xFF ) >= 0x40 ) return pTable [ ( C & 0xFF ) - 0x40 ] ;

   int cGb = jmstojis ( SjisIbm2nec ( C ) ) ;
   if ( cGb ) cGb |= 0x8080 ;
   return cGb ;
}



// Korean Writer  KS ɕϊ
static int Kw2ks ( int C ) {

   switch ( C >> 8 ) {
      case 0x95 :
         if ( C >= 0x9540 && C <= 0x957E ) C += ( 0x829F - 0x9540 ) ;
         if ( C >= 0x9580 && C <= 0x959E ) C += ( 0x82DE - 0x9580 ) ;
         break ;
      case 0x98 :
         if      ( C >= 0x9840 && C <= 0x989E ) C = 0x8140 ;   // Ȃ
         break ;
      case 0xEA :
         if      ( C >= 0xEA9F && C <= 0xEAA4 ) C = 0x8140 ;   // Ȃ
         break ;
      case 0xED :
      case 0xEE :
         C = 0x8140 ;   // Ȃ
         break ;
      case 0xF0 :
      case 0xF1 :
      case 0xF2 :
         if      ( C >= 0xF040 && C <= 0xF09E ) C += ( 0x9840 - 0xF040 ) ;
         else if ( C >= 0xF09F && C <= 0xF2FC ) C += ( 0xEA9F - 0xF09F ) ;
         break ;
      case 0xF3 :
      case 0xF4 :
      case 0xF5 :
         if      ( C >= 0xF340 && C <= 0xF598 ) C += ( 0xED40 - 0xF340 ) ;
         else if ( C >= 0xF59F && C <= 0xF5E6 ) C += ( 0x819F - 0xF59F ) ;
         else if ( C >= 0xF5EF && C <= 0xF5FC ) C += ( 0x8340 - 0xF5EF ) ;
         break ;
      case 0xF6 :
      case 0xF7 :
         if      ( C >= 0xF640 && C <= 0xF79E ) C += ( 0x8440 - 0xF640 ) ;
         break ;
      case 0xFA :
      case 0xFB :
      case 0xFC :
         if      ( C >= 0xFA4A && C <= 0xFA54 ) ;  // through
         else if ( C >= 0xFA58 && C <= 0xFA5B ) ;  // through
         else     C = 0x8140 ;   // Ȃ
         break ;
   }

   int cKs = jmstojis ( SjisIbm2nec ( C ) ) ;
   if ( cKs ) cKs |= 0x8080 ;

   return cKs ;
}



// cWnn  Chinese Writer ɕϊ
static int Cwn2cw ( int C ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cwn2cwTable ( C >> 8 ) ;
      if ( pTable && ( C & 0xFF ) >= 0x40 ) return pTable [ ( C & 0xFF ) - 0x40 ] ;
   }
   else if ( ! isascii ( C ) ) {
      if      ( C == (unsigned char) '' ) return 0x20 ;
//    else if ( C == (unsigned char) '' ) return 0x814A ;  // J
//    else if ( C == (unsigned char) '' ) return 0x814B ;  // K
   }

   return C ;
}



// kWnn  KS ɕϊ
static int Kwn2ks ( int C ) {

   switch ( C >> 8 ) {
      case 0x95 :
         if ( C >= 0x9540 && C <= 0x959E ) C += ( 0x9840 - 0x9540 ) ;
         break ;
      case 0xF0 :
         if      ( C >= 0xF040 && C <= 0xF07E ) C += ( 0x819F - 0xF040 ) ;
         else if ( C >= 0xF080 && C <= 0xF085 ) C += ( 0x81DE - 0xF080 ) ;
         else if ( C >= 0xF086 && C <= 0xF09E ) C += ( 0x869F - 0xF086 ) ;
         else if ( C >= 0xF09F && C <= 0xF0DD ) C += ( 0x8240 - 0xF09F ) ;
         else if ( C >= 0xF0DE && C <= 0xF0FC ) C += ( 0x8280 - 0xF0DE ) ;
         break ;
      case 0xF1 :
         if      ( C >= 0xF140 && C <= 0xF17E ) C += ( 0x829F - 0xF140 ) ;
         else if ( C >= 0xF180 && C <= 0xF19E ) C += ( 0x82DE - 0xF180 ) ;
         else if ( C >= 0xF19F && C <= 0xF1DE ) C += ( 0x8340 - 0xF19F ) ;
         else if ( C >= 0xF1DF && C <= 0xF1FC ) C += ( 0x8381 - 0xF1DF ) ;
         break ;
      case 0xF2 :
         if      ( C >= 0xF240 && C <= 0xF27E ) C += ( 0x839F - 0xF240 ) ;
         else if ( C >= 0xF280 && C <= 0xF284 ) C += ( 0x83DE - 0xF280 ) ;
         else if ( C >= 0xF285 && C <= 0xF28C ) C += ( 0x86B8 - 0xF285 ) ;
         else if ( C >= 0xF28D && C <= 0xF29E ) C += ( 0x86CF - 0xF28D ) ;
         else if ( C >= 0xF29F && C <= 0xF2DD ) C += ( 0x8440 - 0xF29F ) ;
         else if ( C >= 0xF2DE && C <= 0xF2ED ) C += ( 0x8480 - 0xF2DE ) ;
         else if ( C >= 0xF2EE && C <= 0xF2FC ) C += ( 0x86E1 - 0xF2EE ) ;
         break ;
      case 0xF3 :
         if      ( C >= 0xF340 && C <= 0xF37E ) C += ( 0x849F - 0xF340 ) ;
         else if ( C >= 0xF380 && C <= 0xF39E ) C += ( 0x84DE - 0xF380 ) ;
         else if ( C >= 0xF39F && C <= 0xF3DD ) C += ( 0x8540 - 0xF39F ) ;
         else if ( C >= 0xF3DE && C <= 0xF3FC ) C += ( 0x8580 - 0xF3DE ) ;
         break ;
      case 0xF4 :
         if      ( C >= 0xF440 && C <= 0xF47E ) C += ( 0x859F - 0xF440 ) ;
         else if ( C >= 0xF480 && C <= 0xF49E ) C += ( 0x85DE - 0xF480 ) ;
         else if ( C >= 0xF49F && C <= 0xF4DD ) C += ( 0x8640 - 0xF49F ) ;
         else if ( C >= 0xF4DE && C <= 0xF4FC ) C += ( 0x8680 - 0xF4DE ) ;
         break ;
      case 0xF5 :
      case 0xF6 :
      case 0xF7 :
      case 0xF8 :
      case 0xF9 :
         int cKwLow = C & 0xFF ;
         if      ( cKwLow >= 0x40 && cKwLow <= 0x7E ) C += ( 0xEA9F - 0xF540 ) ;
         else if ( cKwLow >= 0x80 && cKwLow <= 0x9E ) C += ( 0xEADE - 0xF580 ) ;
         else if ( cKwLow >= 0x9F && cKwLow <= 0xDD ) C += ( 0xEB40 - 0xF59F ) ;
         else if ( cKwLow >= 0xDE && cKwLow <= 0xFF ) C += ( 0xEB80 - 0xF5DE ) ;
         break ;
   }

   int cKs = jmstojis ( SjisIbm2nec ( C ) ) ;
   if ( cKs ) cKs |= 0x8080 ;

   return cKs ;
}



////////////////////////////////////////////
//         {̏ (@ˑ)        //
////////////////////////////////////////////



// @ˑȂ̃VtgJISϊ
static int SjisToUnicode_Org ( int cSjis, int cUnicode ) {

   if ( cSjis < 0x100 ) return Sjis2uni_Org_SingleByte ( cSjis, cUnicode ) ;

   const unsigned short *pTable = Sjis2uniTable_Org ( cSjis >> 8 ) ;
   if ( pTable && ( cSjis & 0xFF ) >= 0x40 ) {
      int cResult = pTable [ ( cSjis & 0xFF ) - 0x40 ] ;
      if ( cResult ) return cResult ;
   }

   if ( cUnicode ) {
      if      ( cSjis >= 0x8500 && cSjis <= 0x889E ) cUnicode = 0 ;
      else if ( cSjis >= 0xEB00 && cSjis <= 0xEFFF ) cUnicode = 0 ;
      else if ( cSjis >= 0xFA00 && cSjis <= 0xFCFF ) cUnicode = 0 ;
   }

   return cUnicode ;
}



// @ˑȂ̃VtgJISϊ
static int UnicodeToSjis_Org ( int cUnicode, int cSjis ) {

   if ( cSjis > 0 && cSjis < 0x100 ) {
      if ( cUnicode == Sjis2uni_Org_SingleByte ( cSjis, cUnicode ) ) return cSjis ;
      cSjis = 0 ;
   }

   const unsigned short *pTable = Uni2sjisTable_Org ( cUnicode >> 8 ) ;
   if ( pTable ) {
      int cResult = pTable [ cUnicode & 0xFF ] ;
      if ( cResult ) return cResult ;
   }

   if ( cSjis ) {
      if      ( cSjis >= 0x8500 && cSjis <= 0x889E ) cSjis = 0 ;
      else if ( cSjis >= 0xEB00 && cSjis <= 0xEFFF ) cSjis = 0 ;
      else if ( cSjis >= 0xFA00 && cSjis <= 0xFCFF ) cSjis = 0 ;
   }

   if ( cSjis ) {
      const unsigned short *pTable = Sjis2uniTable_Org ( cSjis >> 8 ) ;
      if ( pTable && pTable [ ( cSjis & 0xFF ) - 0x40 ] ) cSjis = 0 ;
   }

   return cSjis ;
}



// Macintosh p̃VtgJISϊ
// PɕϊȂApszUnicode  NULL  Unicode ̕Ԃ
// QȏɕϊȂAւ̃|C^ pszUnicode ɓ 0 Ԃ
static int SjisToUnicode_Mac ( int cSjis, int cUnicode, const wchar_t **pszUnicode ) {

   *pszUnicode = NULL ;

   if ( cSjis < 0x100 ) return Sjis2uni_Mac_SingleByte ( cSjis, cUnicode ) ;

   const unsigned short *pTable = Sjis2uniTable_Mac ( cSjis >> 8 ) ;
   if ( pTable && ( cSjis & 0xFF ) >= 0x40 ) {
      int cResult = pTable [ ( cSjis & 0xFF ) - 0x40 ] ;
      if ( cResult == WEOF ) {
         *pszUnicode = Sjis2uni_Mac_Composition ( cSjis ) ;
         return 0 ;
      }
      if ( cResult ) return cResult ;
   }

   if ( cUnicode ) {
      if      ( cSjis >= 0x8500 && cSjis <= 0x889E ) cUnicode = 0 ;
      else if ( cSjis >= 0xEB00 && cSjis <= 0xEFFF ) cUnicode = 0 ;
      else if ( cSjis >= 0xFA00 && cSjis <= 0xFCFF ) cUnicode = 0 ;
   }

   return cUnicode ;
}



// Macintosh p̃VtgJISϊ
static int UnicodeToSjis_Mac ( int cUnicode, int cSjis ) {

   if ( cSjis > 0 && cSjis < 0x100 ) {
      if ( cUnicode == Sjis2uni_Mac_SingleByte ( cSjis, cUnicode ) ) return cSjis ;
      cSjis = 0 ;
   }

   const unsigned short *pTable = Uni2sjisTable_Mac ( cUnicode >> 8 ) ;
   if ( pTable ) {
      int cResult = pTable [ cUnicode & 0xFF ] ;
      if ( cResult ) return cResult ;
   }

   if ( cSjis ) {
      if      ( cSjis >= 0x8500 && cSjis <= 0x889E ) cSjis = 0 ;
      else if ( cSjis >= 0xEB00 && cSjis <= 0xEFFF ) cSjis = 0 ;
      else if ( cSjis >= 0xFA00 && cSjis <= 0xFCFF ) cSjis = 0 ;
   }

   if ( cSjis ) {
      const unsigned short *pTable = Sjis2uniTable_Mac ( cSjis >> 8 ) ;
      if ( pTable && pTable [ ( cSjis & 0xFF ) - 0x40 ] ) cSjis = 0 ;
   }

   return cSjis ;
}



////////////////////////////////////////////
//          {̏ (JIS2004)        //
////////////////////////////////////////////



// JIS 2004 ̂Qʂ Shift JIS ɕϊ
// ϊłȂ 0 Ԃ
static int jistojms2004 ( int c ) {

   int high = c >> 8 ;
   int low = c & 0xFF ;

   if ( high < 0x21 || high > 0x7E || low < 0x21 || low > 0x7E ) return 0 ;

   if ( high & 1 ) {
      if ( low < 0x60 ) low += 0x1F ;
      else              low += 0x20 ;
   }
   else {
      low += 0x7E ;
   }

   if ( high >= 0x6E ) {
      high = ( high + 0x17B ) >> 1 ;
   }
   else {
      switch ( high ) {
         case 0x21 : high = 0xF0 ; break ;
         case 0x23 : high = 0xF1 ; break ;
         case 0x24 : high = 0xF1 ; break ;
         case 0x25 : high = 0xF2 ; break ;
         case 0x28 : high = 0xF0 ; break ;
         case 0x2C : high = 0xF2 ; break ;
         case 0x2D : high = 0xF3 ; break ;
         case 0x2E : high = 0xF3 ; break ;
         case 0x2F : high = 0xF4 ; break ;
         default : return 0 ;
      }
   }

   return ( high << 8 ) | low ;
}



// Shift JIS  JIS 2004 ̂Qʂɕϊ
// ϊłȂ 0 Ԃ
static int jmstojis2004 ( int c ) {

   int high = c >> 8 ;
   int low = c & 0xFF ;

   if ( high & ~ 0xFF || ! ismbblead_sjis ( high ) || ! ismbbtrail_sjis ( low ) ) return 0 ;

   if ( low < 0x9F ) {
      if ( high >= 0xF5 ) high = ( high << 1 ) - 0x17B ;
      else {
         switch ( high ) {
            case 0xF0 : high = 0x21 ; break ;
            case 0xF1 : high = 0x23 ; break ;
            case 0xF2 : high = 0x25 ; break ;
            case 0xF3 : high = 0x2D ; break ;
            case 0xF4 : high = 0x2F ; break ;
            default : return 0 ;
         }
      }
   }
   else {
      if ( high >= 0xF4 ) high = ( high << 1 ) - 0x17B + 1 ;
      else {
         switch ( high ) {
            case 0xF0 : high = 0x28 ; break ;
            case 0xF1 : high = 0x24 ; break ;
            case 0xF2 : high = 0x2C ; break ;
            case 0xF3 : high = 0x2E ; break ;
            default : return 0 ;
         }
      }
   }

   if ( low < 0x7F )      low -= 0x1F ;
   else if ( low < 0x9F ) low -= 0x20 ;
   else                   low -= 0x7E ;

   return ( high << 8 ) | low ;
}



// JIS2004 ̏̕
// K؂ȕΕԂ
// ŏ 0 AŏȂ EOF Ԃ
// t@C̖ EOF w肵ČĂяo
static int EncodeSjis2004Composition ( int C, int cUnicode, int nSrcCodePage, RTFCONVDATA *pData ) {

   if ( pData->wt->IsAvoidLoop ) return EOF ;

   if ( C == EOF ) {
      if ( pData->wt->U1 ) {
         pData->wt->IsAvoidLoop = 1 ;
         PutcTxt ( pData->wt->C1, pData->wt->U1, nSrcCodePage, ( pData->wt->C1 > 0x100 ) ? "??" : "?", pData ) ;
         pData->wt->U1 = 0 ;
         pData->wt->IsAvoidLoop = 0 ;
      }
      return 0 ;
   }


   if ( pData->wt->U1 ) {

      wchar_t szBuffer [] = { pData->wt->U1, cUnicode, 0 } ;

      int cResult = UnicodeToSjis_2004 ( 0, 0, szBuffer, pData->dwConversionMode ) ;
      if ( cResult == EOF ) {
         pData->nError = CONV_ERROR_CODEPAGE ;
         return 0 ;
      }
      if ( cResult ) {
         pData->wt->U1 = 0 ;
         return cResult ;
      }
      else {
         pData->wt->IsAvoidLoop = 1 ;
         PutcTxt ( pData->wt->C1, pData->wt->U1, nSrcCodePage, ( pData->wt->C1 > 0x100 ) ? "??" : "?", pData ) ;
         pData->wt->U1 = 0 ;
         pData->wt->IsAvoidLoop = 0 ;
      }
   }


   wchar_t szBuffer [] = { cUnicode, 0 } ;

   int nResult = UnicodeToSjis_2004 ( 0, 0, szBuffer, pData->dwConversionMode ) ;
   if ( nResult == EOF ) {
      pData->nError = CONV_ERROR_CODEPAGE ;
      return 0 ;
   }
   if ( nResult ) {
      pData->wt->U1 = cUnicode ;
      pData->wt->C1 = C ;
      return 0 ;
   }

   return EOF ;
}



////////////////////////////////////////////
//            {̏ (EUC)          //
////////////////////////////////////////////



// EUC-JP  SJIS 
// ⏕ + 0x10000 ŕԂ
// ϊłȂ 0 Ԃ
static int eucjp2sjis ( int C, int Is2004 ) {

   int C1 = ( C >> 24 ) & 0xFF ;
   int C2 = ( C >> 16 ) & 0xFF ;
   int C3 = ( C >>  8 ) & 0xFF ;
   int C4 = ( C >>  0 ) & 0xFF ;

   if ( C1 ) return 0 ;

   // ⏕
   if ( C2 == 0x8F ) {
      if ( C3 < 0x80 || C4 < 0x80 ) return 0 ;
      int cResult = 0 ;
      if ( Is2004 ) cResult = jistojms2004 ( ( ( C3 << 8 ) | C4 ) & 0x7F7F ) ;
      if ( ! cResult ) cResult = ( ( ( C3 << 8 ) | C4 ) & 0x7F7F ) + 0x10000 ;   // ⏕ + 0x10000 
      return cResult ;
   }

   if ( C2 ) return 0 ;

   // pJi
   if ( C3 == 0x8E && C4 >= 0x80 ) return C4 ;

   // 
   if ( C3 >= 0xA1 && C4 >= 0xA1 ) return jistojms ( ( ( C3 << 8 ) | C4 ) & 0x7F7F ) ;

   if ( C3 ) return 0 ;

   // ASCII
   if ( C4 < 0x80 ) return C4 ;

   return 0 ;
}



// SJIS  EUC-JP 
// ⏕ + 0x10000 Ŏw
// ϊłȂ 0 Ԃ
static int sjis2eucjp ( int C, int Is2004 ) {

   int C1 = ( C >> 24 ) & 0xFF ;
   int C2 = ( C >> 16 ) & 0xFF ;
   int C3 = ( C >>  8 ) & 0xFF ;
   int C4 = ( C >>  0 ) & 0xFF ;

   if ( C1 ) return 0 ;

   // ⏕
   if ( C2 == 1 ) return ( ( ( C3 << 8 ) | C4 ) | 0x8080 ) + 0x8F0000 ;

   if ( C2 ) return 0 ;

   // 
   if ( C3 ) {
      if ( Is2004 ) {
         int cResult = jmstojis2004 ( C ) ;
         if ( cResult ) return ( cResult | 0x8080 ) + 0x8F0000 ;
      }
      int cResult = jmstojis ( SjisIbm2nec ( C ) ) ;
      if ( cResult ) return cResult | 0x8080 ;
   }

   if ( C3 ) return 0 ;

   // pJi
   if ( C4 >= 0x80 ) return C4 + 0x8E00 ;

   // ASCII
   if ( C4 < 0x80 ) return C4 ;

   return 0 ;
}



////////////////////////////////////////////
//             ̏               //
////////////////////////////////////////////



// GB2312 ̕Ȃ 0 ȊOԂ
static int IsGb2312 ( int cGb ) {

   if ( isascii ( cGb ) ) return 1 ;

   if ( cGb >= 0xA1A1 && cGb <= 0xF7FF && ( cGb & 0xFF ) >= 0xA1 ) {
      if ( cGb >= 0xA2A1 && cGb <= 0xA2AA ) return 0 ;   // GBK
      if ( cGb >= 0xA6D9 && cGb <= 0xA6F5 ) return 0 ;   // GBK
      if ( cGb == 0xA2E3 ) return 0 ;                    // GB18030
      return 1 ;
   }

   return 0 ;
}



////////////////////////////////////////////
//             ؍̏               //
////////////////////////////////////////////



// KSX1001 ̕Ȃ 0 ȊOԂ
static int IsKsX1001 ( int cKs ) {

   if ( isascii ( cKs ) ) return 1 ;

   if ( cKs >= 0xA1A1 && cKs <= 0xFDFF && ( cKs & 0xFF ) >= 0xA1 ) {
      if ( cKs == 0xA2E6 ) return 0 ;  // [
      if ( cKs == 0xA2E7 ) return 0 ;  // W
      return 1 ;
   }

   return 0 ;
}



////////////////////////////////////////////
//           wuC̏             //
////////////////////////////////////////////



static int HebrewPunctToMirror ( int C ) ;
static int IsHebrewDigitSymbol ( int C ) ;


// wuC̕tɂ
// VisualLogical  ŝƂ IsLineEndOfVtl  1 ɂ
// IsLineEndOfVtl  1 ̂Ƃ FEh 邱Ƃ̂ŁA
//  obt@[̒ +1 ̗]TKvB
//  0 As 0 ȊOԂ
static int HebrewReverseString ( char *szSrcBuffer, int IsLineEndOfVtl ) {

   char *szDstBuffer = (char*) malloc ( strlen ( szSrcBuffer ) + 2 ) ;
   if ( ! szDstBuffer ) return 1 ;

   strrev ( szSrcBuffer ) ;

   char *pDst = szDstBuffer ;
   char *pStart = szSrcBuffer ;
   while ( (unsigned char) *pStart < 0xE0 && ! isdigit ( (unsigned char) *pStart ) ) pStart ++ ;

   if ( IsLineEndOfVtl && isdigit ( (unsigned char) *pStart ) ) {
      *pDst ++ = (char) 0xFE ;
   }

   char *pCurrent ;

   for ( pCurrent = pStart ; *pCurrent ; pCurrent ++ ) {

      *pCurrent = HebrewPunctToMirror ( (unsigned char) *pCurrent ) ;

      if ( isdigit ( (unsigned char) *pCurrent ) || IsHebrewDigitSymbol ( *pCurrent ) ) {

         char *pDigitStart = pCurrent ;
         char *pDigitEnd = pCurrent ;

         while ( isdigit ( (unsigned char) *pDigitStart ) || IsHebrewDigitSymbol ( *pDigitStart ) || isdigit ( (unsigned char) *( pDigitStart + 1 ) ) && ( *pDigitStart == '.' || *pDigitStart == ',' || *pDigitStart == '/' ) ) pDigitStart ++ ;

         for ( pCurrent = pDigitStart - 1 ; pCurrent >= pDigitEnd ; pCurrent -- ) {
            *pDst ++ = *pCurrent ;
         }

         pCurrent = pDigitStart - 1 ;
      }
      else {
         *pDst ++ = *pCurrent ;
      }
   }

   for ( pCurrent = pStart - 1 ; pCurrent >= szSrcBuffer ; pCurrent -- ) {
      *pDst ++ = *pCurrent ;
   }

   *pDst = 0 ;

   strcpy ( szSrcBuffer, szDstBuffer ) ;

   free ( szDstBuffer ) ;
   return 0 ;
}



// wuC̋Lt
static int HebrewPunctToMirror ( int C ) {
   switch ( C ) {
      case '(' : return ')' ;
      case ')' : return '(' ;
      case '<' : return '>' ;
      case '>' : return '<' ;
      case '[' : return ']' ;
      case ']' : return '[' ;
      case '{' : return '}' ;
      case '}' : return '{' ;
      case 0xAB : return 0xBB ;  // <<
      case 0xBB : return 0xAB ;  // >>
   }
   return C ;
}



// wuCŐƂƂɈʒuς镶
static int IsHebrewDigitSymbol ( int C ) {

   switch ( C & 0xFF ) {
      case '#' :
      case '$' :
      case '%' :
      case 0xA2 :    // 
      case 0xA3 :    // 
      case 0xA5 :    // 
      case 0xB1 :    // }
      case 0xB2 :    // <sup>2
      case 0xB3 :    // <sup>3
      case 0xB9 :    // <sup>1
         return 1 ;
      default :
         return 0 ;
   }
}



////////////////////////////////////////////
//              ISCIȈ               //
////////////////////////////////////////////



//  1 ܂ 2 Ԃ
// KPȏ
// t@C̖ C2  EOF w肵ČĂяo
// G[Ȃ 0 Ԃ
static int DecodeIscii ( int C1, int C2, RTFCONVDATA *pData ) {

   static const int HALANT = 0xE8 ;    // Vowel Omission Sign
   static const int NUKTA  = 0xE9 ;    // Diacritic Mark
   static const int DANDA  = 0xEA ;    // End of Sentence
   static const int INV    = 0xD9 ;
   static const int ATR    = 0xEF ;

   static const int ZWNJ   = 0x200C ;  // ZERO WIDTH NON-JOINER
   static const int ZWJ    = 0x200D ;  // ZERO WIDTH JOINER


   int nSrcCodePage = pData->nSrcCodePage ;
   if ( pData->rt->nIsciiCodePage ) nSrcCodePage = pData->rt->nIsciiCodePage ;


   if ( C1 == HALANT ) {
      if ( C2 == HALANT ) {   // HALANT + HALANT = HALANT + ZWNJ
         Putc ( C1, nSrcCodePage, pData ) ;
         Putc ( ZWNJ, CP_UNICODE, pData ) ;
         return 2 ;
      }
      if ( C2 == NUKTA ) {    // HALANT + NUKTA = HALANT + ZWJ
         Putc ( C1, nSrcCodePage, pData ) ;
         Putc ( ZWJ, CP_UNICODE, pData ) ;
         return 2 ;
      }
   }

   if ( C2 == NUKTA ) {
      // CONSONANT + NUKTA = SPECIAL CONSONANT
      // CHANDRABINDU + NUKTA = OM
      int cResult = GetNuktaChar ( C1, nSrcCodePage ) ;
      if ( cResult ) {
         Putc ( cResult, CP_UNICODE, pData ) ;
         return 2 ;
      }
   }

   if ( C1 == DANDA ) {
      if ( C2 == DANDA ) {    // DANDA + DANDA = DOUBLE DANDA
         Putc ( 0x0965, CP_UNICODE, pData ) ;
         return 2 ;
      }
   }

   if ( C1 == INV ) {
      if ( C2 == HALANT ) {
         Putc ( 0x20, CP_UNICODE, pData ) ;  // SPACE
         return 1 ;
      }
      else {
         Putc ( ZWJ, CP_UNICODE, pData ) ;
         return 1 ;
      }
   }

   if ( C1 == ATR ) {

      switch ( C2 ) {
         case 0x40 :    pData->rt->nIsciiCodePage = 0 ; break ;   // default
         case 0x41 :    pData->rt->nIsciiCodePage = 0 ; break ;   // roman
         case 0x42 :    pData->rt->nIsciiCodePage = CP_ISCII_DEVANAGARI ; break ;
         case 0x43 :    pData->rt->nIsciiCodePage = CP_ISCII_BENGALI ; break ;
         case 0x44 :    pData->rt->nIsciiCodePage = CP_ISCII_TAMIL ; break ;
         case 0x45 :    pData->rt->nIsciiCodePage = CP_ISCII_TELUGU ; break ;
         case 0x46 :    pData->rt->nIsciiCodePage = CP_ISCII_ASSAMESE ; break ;
         case 0x47 :    pData->rt->nIsciiCodePage = CP_ISCII_ORIYA ; break ;
         case 0x48 :    pData->rt->nIsciiCodePage = CP_ISCII_KANNADA ; break ;
         case 0x49 :    pData->rt->nIsciiCodePage = CP_ISCII_MALAYALAM ; break ;
         case 0x4A :    pData->rt->nIsciiCodePage = CP_ISCII_GUJARATHI ; break ;
         case 0x4B :    pData->rt->nIsciiCodePage = CP_ISCII_PANJABI ; break ;
         case 0x71 :    pData->rt->nIsciiCodePage = 0 ; break ;   // Arabic
         case 0x72 :    pData->rt->nIsciiCodePage = 0 ; break ;   // Persian
         case 0x73 :    pData->rt->nIsciiCodePage = 0 ; break ;   // Urdu
         case 0x74 :    pData->rt->nIsciiCodePage = 0 ; break ;   // Sindhi
         case 0x75 :    pData->rt->nIsciiCodePage = 0 ; break ;   // Kashmiri
         case 0x76 :    pData->rt->nIsciiCodePage = 0 ; break ;   // Pushto
      }

      return 2 ;
   }

   if ( C1 == '\n' || C1 == '\r' ) pData->rt->nIsciiCodePage = 0 ; // default

   Putc ( C1, nSrcCodePage, pData ) ;
   return 1 ;
}



static int GetIsciiLangAttr ( int nCodePage ) ;



//  0 ȊOԂ
static int EncodeIscii ( int C, int cUnicode, RTFCONVDATA *pData ) {

   static const int HALANT = 0xE8 ;    // Vowel Omission Sign
   static const int NUKTA  = 0xE9 ;    // Diacritic Mark
   static const int DANDA  = 0xEA ;    // End of Sentence
   static const int INV    = 0xD9 ;
   static const int ATR    = 0xEF ;

   static const int ZWJ    = 0x200D ;  // ZERO WIDTH JOINER
   static const int ZWNJ   = 0x200C ;  // ZERO WIDTH NON-JOINER


   STREAM *StreamOut = pData->StreamOut ;


   if ( pData->wt->IsPrevHalant ) {
      switch ( cUnicode ) {
         case ZWNJ : C = HALANT ; break ;    // HALANT + HALANT = HALANT + ZWNJ
         case ZWJ :  C = NUKTA ; break ;     // HALANT + NUKTA = HALANT + ZWJ
      }
      pData->wt->IsPrevHalant = 0 ;
   }
   else if ( C == HALANT ) pData->wt->IsPrevHalant = 1 ;

   if ( ! C && cUnicode == ZWJ ) {
      pData->wt->IsPrevZwj = 1 ;
      return 1 ;
   }
   if ( pData->wt->IsPrevZwj ) {
      if ( C == NUKTA ) StreamWrite ( INV, StreamOut ) ;
      pData->wt->IsPrevZwj = 0 ;
   }

   if ( ! C ) {
      // DANDA + DANDA = DOUBLE DANDA
      if ( cUnicode == 0x0965 ) {
         StreamWrite ( DANDA, StreamOut ) ;
         StreamWrite ( DANDA, StreamOut ) ;
         return 1 ;
      }
      // CONSONANT + NUKTA = SPECIAL CONSONANT
      // CHANDRABINDU + NUKTA = OM
      int cResult = GetNonNuktaChar ( cUnicode, pData->nDstCodePage ) ;
      if ( cResult ) {
         StreamWrite ( cResult, StreamOut ) ;
         StreamWrite ( NUKTA, StreamOut ) ;
         return 1 ;
      }
   }

   if ( C || ! C && ! cUnicode ) {
      if ( pData->wt->nIsciiCodePage && C != HALANT && C != NUKTA ) {
         StreamWrite ( ATR, StreamOut ) ;
         StreamWrite ( GetIsciiLangAttr ( 0 ), StreamOut ) ;
         pData->wt->nIsciiCodePage = 0 ;
      }
   }
   else {
      if ( pData->wt->nIsciiCodePage ) {
         C = UnicodeToChar ( cUnicode, pData->wt->nIsciiCodePage, pData->dwConversionMode ) ;
         if ( C == EOF ) goto CAUGHT_ERROR ;
      }
      if ( ! C ) {
         for ( int nCodePage = CP_ISCII_DEVANAGARI ; nCodePage <= CP_ISCII_PANJABI ; nCodePage ++ ) {
            C = UnicodeToChar ( cUnicode, nCodePage, pData->dwConversionMode ) ;
         // if ( C == EOF ) goto CAUGHT_ERROR ;
            if ( C > 0 ) {
               if ( pData->wt->nIsciiCodePage != nCodePage ) {
                  pData->wt->nIsciiCodePage = nCodePage ;
                  StreamWrite ( ATR, StreamOut ) ;
                  StreamWrite ( GetIsciiLangAttr ( nCodePage ), StreamOut ) ;
               }
               break ;
            }
            // CONSONANT + NUKTA = SPECIAL CONSONANT
            // CHANDRABINDU + NUKTA = OM
            C = GetNonNuktaChar ( cUnicode, nCodePage ) ;
            if ( C > 0 ) {
               if ( pData->wt->nIsciiCodePage != nCodePage ) {
                  pData->wt->nIsciiCodePage = nCodePage ;
                  StreamWrite ( ATR, StreamOut ) ;
                  StreamWrite ( GetIsciiLangAttr ( nCodePage ), StreamOut ) ;
               }
               StreamWrite ( C, StreamOut ) ;
               StreamWrite ( NUKTA, StreamOut ) ;
               return 1 ;
            }
         }
      }

      if ( C <= 0 ) return 0 ;
      if ( C == HALANT ) pData->wt->IsPrevHalant = 1 ;
   }


   StreamWrite ( C, StreamOut ) ;
   return 1 ;


   CAUGHT_ERROR :
   if ( StreamOut ) pData->nError = CONV_ERROR_CODEPAGE ;
   return 1 ;
}



static int GetIsciiLangAttr ( int nCodePage ) {

   switch ( nCodePage ) {
      case CP_ISCII_DEVANAGARI : return 0x42 ;
      case CP_ISCII_BENGALI :    return 0x43 ;
      case CP_ISCII_TAMIL :      return 0x44 ;
      case CP_ISCII_TELUGU :     return 0x45 ;
      case CP_ISCII_ASSAMESE :   return 0x46 ;
      case CP_ISCII_ORIYA :      return 0x47 ;
      case CP_ISCII_KANNADA :    return 0x48 ;
      case CP_ISCII_MALAYALAM :  return 0x49 ;
      case CP_ISCII_GUJARATHI :  return 0x4A ;
      case CP_ISCII_PANJABI :    return 0x4B ;
      default : return 0x40 ;
   }
}



////////////////////////////////////////////
//          TSCII/TAB/TAM ̏          //
////////////////////////////////////////////



//  1 ܂ 2 ܂ 3 Ԃ
// KPȏ
// t@C̖ C3  EOF w肵ČĂяoA C2  EOF w肵ČĂяo
// G[Ȃ 0 Ԃ
static int DecodeTamil ( int C1, int C2, int C3, RTFCONVDATA *pData ) {

   int nSrcCodePage = pData->nSrcCodePage ;

   char szSrc [] = {
      ( (unsigned) C1 > 0xFF ) ? 0 : C1,
      ( (unsigned) C2 > 0xFF ) ? 0 : C2,
      ( (unsigned) C3 > 0xFF ) ? 0 : C3,
   0 } ;


   if ( nSrcCodePage == CP_TAMIL_TAB || nSrcCodePage == CP_TAMIL_TAM ) {
      for ( int nCount = 0 ; szSrc [ nCount ] ; nCount ++ ) if ( (unsigned char) szSrc [ nCount ] == 0xCA ) szSrc [ nCount ] = (unsigned char) 0xA0 ;
   }

   const TAMIL_EQUIVALENT_TABLE *pTable = TamilToUnicode ( szSrc, nSrcCodePage ) ;
   if ( pTable ) {

      Putws ( pTable->szUnicode, pData ) ;
//    if ( *( wcsend ( pTable->szUnicode ) - 1 ) == 0x0BCD ) Putc ( 0x200C, CP_UNICODE, pData ) ;

      const char *szDst = NULL ;
      switch ( nSrcCodePage ) {
         case CP_TAMIL_TSCII :   szDst = pTable->szTscii ; break ;
         case CP_TAMIL_TAB :     szDst = pTable->szTab ; break ;
         case CP_TAMIL_TAM :     szDst = pTable->szTam ; break ;
      }

      if ( szDst ) return (int) strlen ( szDst ) ;
      else         return 0 ;
   }

   Putc ( C1, nSrcCodePage, pData ) ;
   return 1 ;
}



//  0 ȊOԂ
// t@C̖ EOF w肵ČĂяo
static int EncodeTamil ( int C, int cUnicode, RTFCONVDATA *pData ) {

   STREAM *StreamOut = pData->StreamOut ;
   int C5 = C, U5 = cUnicode ;
   size_t nLength = 0 ;

   static const int ZWNJ   = 0x200C ;  // ZERO WIDTH NON-JOINER

   if ( pData->wt->IsAvoidLoop ) return 0 ;


   if ( pData->wt->U1 != EOF ) {

      wchar_t szSrc [] = {
         ( (unsigned) pData->wt->U1 > 0xFFFF ) ? 0 : pData->wt->U1,
         ( (unsigned) pData->wt->U2 > 0xFFFF ) ? 0 : pData->wt->U2,
         ( (unsigned) pData->wt->U3 > 0xFFFF ) ? 0 : pData->wt->U3,
         ( (unsigned) pData->wt->U4 > 0xFFFF ) ? 0 : pData->wt->U4,
         ( (unsigned) U5 > 0xFFFF ) ? 0 : U5,
      0 } ;

      if ( pData->nDstCodePage == CP_TAMIL_TSCII ) {
         for ( int nCount = 0 ; szSrc [ nCount ] ; nCount ++ ) if ( szSrc [ nCount ] == 0xA0 ) szSrc [ nCount ] = 0x20 ;
      }

      static const wchar_t szOldSri [] = L"\x0BB8\x0BCD\x0BB0\x0BC0" ;
      if ( ! wcsheadcmp ( szSrc, szOldSri ) ) szSrc [ 0 ] = 0x0BB6 ;

      const TAMIL_EQUIVALENT_TABLE *pTable = UnicodeToTamil ( szSrc ) ;
      if ( pTable ) {

         const char *szDst = NULL ;
         switch ( pData->nDstCodePage ) {
            case CP_TAMIL_TSCII :   szDst = pTable->szTscii ; break ;
            case CP_TAMIL_TAB :     szDst = pTable->szTab ; break ;
            case CP_TAMIL_TAM :     szDst = pTable->szTam ; break ;
         }

         if ( szDst ) {
            nLength = wcslen ( pTable->szUnicode ) ;
            if ( pTable->szUnicode [ nLength - 1 ] == 0x0BCD && szSrc [ nLength ] == ZWNJ ) nLength ++ ;
            StreamWriteString ( szDst, StreamOut ) ;
         }
      }

      if ( ! nLength ) {
         if ( pData->wt->C1 || ! pData->wt->U1 ) StreamWrite ( pData->wt->C1, StreamOut ) ;
         else {
            pData->wt->IsAvoidLoop = 1 ;
            Putc ( pData->wt->U1, CP_UNICODE, pData ) ;
            pData->wt->IsAvoidLoop = 0 ;
         }
      }
   }


   if ( ! nLength ) nLength = 1 ;

   switch ( nLength ) {
      case 1 :    pData->wt->U1 = pData->wt->U2 ;  pData->wt->U2 = pData->wt->U3 ;  pData->wt->U3 = pData->wt->U4 ;  pData->wt->U4 = U5 ; break ;
      case 2 :    pData->wt->U1 = EOF ;            pData->wt->U2 = pData->wt->U3 ;  pData->wt->U3 = pData->wt->U4 ;  pData->wt->U4 = U5 ; break ;
      case 3 :    pData->wt->U1 = EOF ;            pData->wt->U2 = EOF ;            pData->wt->U3 = pData->wt->U4 ;  pData->wt->U4 = U5 ; break ;
      case 4 :    pData->wt->U1 = EOF ;            pData->wt->U2 = EOF ;            pData->wt->U3 = EOF ;            pData->wt->U4 = U5 ; break ;
      case 5 :    pData->wt->U1 = EOF ;            pData->wt->U2 = EOF ;            pData->wt->U3 = EOF ;            pData->wt->U4 = EOF ;   break ;
   }
   switch ( nLength ) {
      case 1 :    pData->wt->C1 = pData->wt->C2 ;  pData->wt->C2 = pData->wt->C3 ;  pData->wt->C3 = pData->wt->C4 ;  pData->wt->C4 = C5 ; break ;
      case 2 :    pData->wt->C1 = EOF ;            pData->wt->C2 = pData->wt->C3 ;  pData->wt->C3 = pData->wt->C4 ;  pData->wt->C4 = C5 ; break ;
      case 3 :    pData->wt->C1 = EOF ;            pData->wt->C2 = EOF ;            pData->wt->C3 = pData->wt->C4 ;  pData->wt->C4 = C5 ; break ;
      case 4 :    pData->wt->C1 = EOF ;            pData->wt->C2 = EOF ;            pData->wt->C3 = EOF ;            pData->wt->C4 = C5 ; break ;
      case 5 :    pData->wt->C1 = EOF ;            pData->wt->C2 = EOF ;            pData->wt->C3 = EOF ;            pData->wt->C4 = EOF ;   break ;
   }

   if ( C == EOF ) {
      EncodeTamil ( 0, EOF, pData ) ;
      EncodeTamil ( 0, EOF, pData ) ;
      EncodeTamil ( 0, EOF, pData ) ;
   }

   return (int) nLength ;
}



////////////////////////////////////////////
//            xgȉ            //
////////////////////////////////////////////



//  0 ȊOԂ
// G[Ȃ 0 Ԃ
static int DecodeVietnameseVni ( int C, RTFCONVDATA *pData ) {

   const wchar_t *szString = GetVniToUnicodeString ( C ) ;
   if ( szString ) {
      Putws ( szString, pData ) ;
      return 1 ;
   }

   return 0 ;
}



//  1 ܂ 2 ܂ 3 Ԃ
// KPȏ
// t@C̖ C3  EOF w肵ČĂяoA C2  EOF w肵ČĂяo
// G[Ȃ 0 Ԃ
static int DecodeVietnameseViqr ( int C1, int C2, int C3, RTFCONVDATA *pData ) {

   static const int VietnameseMode = 0 ;
   static const int EnglishMode = 1 ;
   static const int LiteralMode = 2 ;
   static const int FixedLiteralMode = 0x40 ;
   static const int ExchangeMode = 0x80 ;
   static const int CtrlA = 1 ;

   if ( pData->rt->nViqrMode == FixedLiteralMode ) {
      Putc ( C1, pData->nSrcCodePage, pData ) ;
      return 1 ;
   }
   if ( C1 == CtrlA ) {
      pData->rt->nViqrMode = FixedLiteralMode ;
      return 1 ;
   }


   if ( C1 == '\\' ) {
      switch ( C2 ) {
         case 'V' : case 'v' :   pData->rt->nViqrMode = VietnameseMode ; return 2 ;
         case 'M' : case 'm' :   pData->rt->nViqrMode = EnglishMode ; return 2 ;
         case 'L' : case 'l' :   pData->rt->nViqrMode = LiteralMode ; return 2 ;
      }
      if ( ! isascii ( C2 ) || ! isalpha ( C2 ) ) {
         Putc ( C2, pData->nSrcCodePage, pData ) ;
         return 2 ;
      }
      pData->rt->nViqrMode |= ExchangeMode ;
      return 1 ;
   }


   int nMode = pData->rt->nViqrMode & ~ ExchangeMode ;
   if ( pData->rt->nViqrMode == ( VietnameseMode | ExchangeMode ) ) nMode = EnglishMode ;
   if ( pData->rt->nViqrMode == ( EnglishMode | ExchangeMode ) ) nMode = VietnameseMode ;
   pData->rt->nViqrMode &= ~ ExchangeMode ;


   if ( nMode == EnglishMode || nMode == LiteralMode ) {
      Putc ( C1, pData->nSrcCodePage, pData ) ;
      return 1 ;
   }


   char szSrc [] = {
      ( (unsigned) C1 > 0xFF ) ? 0 : C1,
      ( (unsigned) C2 > 0xFF ) ? 0 : C2,
      ( (unsigned) C3 > 0xFF ) ? 0 : C3,
   0 } ;

   size_t nLength = 0 ;

   const VIET_EQUIVALENT_TABLE *pTable = ViqrStringToUnicode ( szSrc, & nLength ) ;
   if ( pTable ) {

      if ( pTable->szUnicode ) Putws ( pTable->szUnicode, pData ) ;
      else                     Putc ( pTable->cUnicode, CP_UNICODE, pData ) ;

      return (int) nLength ;
   }


   Putc ( C1, pData->nSrcCodePage, pData ) ;
   return 1 ;
}



//  0 ȊOԂ
// t@C̖ EOF w肵ČĂяo
static int EncodeVietnamese ( int cUnicode, RTFCONVDATA *pData ) {

   int U3 = cUnicode ;
   size_t nLength = 0 ;

   if ( pData->wt->IsAvoidLoop ) return 0 ;


   if ( pData->wt->U1 != EOF ) {

      wchar_t szSrc [] = {
         ( (unsigned) pData->wt->U1 > 0xFFFF ) ? 0 : pData->wt->U1,
         ( (unsigned) pData->wt->U2 < 0x0300 || (unsigned) pData->wt->U2 > 0x036F ) ? 0 : pData->wt->U2,
         ( (unsigned) U3 < 0x0300 || (unsigned) U3 > 0x036F ) ? 0 : U3,
      0 } ;

      for ( int nCount = 1 ; szSrc [ nCount ] ; nCount ++ ) {
         if ( szSrc [ nCount ] == 0x0340 ) szSrc [ nCount ] = 0x0300 ;
         if ( szSrc [ nCount ] == 0x0341 ) szSrc [ nCount ] = 0x0301 ;
      }

      const VIET_EQUIVALENT_TABLE *pTable = UnicodeToViet ( szSrc, & nLength ) ;
      if ( pTable ) {

         switch ( pData->nDstCodePage ) {

            case CP_ANSI_VIETNAMESE :

               pData->wt->IsAvoidLoop = 1 ;
               if ( pTable->szUnicode ) Putws ( pTable->szUnicode, pData ) ;
               else                     Putc ( pTable->cUnicode, CP_UNICODE, pData ) ;
               pData->wt->IsAvoidLoop = 0 ;
               break ;

            case CP_VIETNAMESE_TCVN :
            case CP_VIETNAMESE_VISCII :
            case CP_VIETNAMESE_VPS :

               pData->wt->IsAvoidLoop = 1 ;
               Putc ( pTable->cUnicode, CP_UNICODE, pData ) ;
               pData->wt->IsAvoidLoop = 0 ;
               break ;

            case CP_VIETNAMESE_VNI :

               pData->wt->IsAvoidLoop = 1 ;
               if ( pTable->szVni ) StreamWriteString ( pTable->szVni, pData->StreamOut ) ;
               else                 Putc ( pTable->cUnicode, CP_UNICODE, pData ) ;
               pData->wt->IsAvoidLoop = 0 ;
               break ;

            case CP_VIETNAMESE_VIQR :
               pData->wt->IsAvoidLoop = 1 ;
               if ( pTable->szViqr ) StreamWriteString ( pTable->szViqr, pData->StreamOut ) ;
               else                  Putc ( pTable->cUnicode, CP_UNICODE, pData ) ;
               pData->wt->IsAvoidLoop = 0 ;
               break ;

            case CP_VIETNAMESE_TCVN3 :

               pData->wt->IsAvoidLoop = 1 ;
               Putc ( GetUnicodeForTcvn3 ( pTable->cUnicode, pTable->szUnicode ), CP_UNICODE, pData ) ;
               pData->wt->IsAvoidLoop = 0 ;
               break ;

            case CP_VIETNAMESE_TCVN2 :

               if ( GetUnicodeForTcvn2 ( pTable->cUnicode, pTable->szUnicode ) ) {
                  pData->wt->IsAvoidLoop = 1 ;
                  Putc ( pTable->cUnicode, CP_UNICODE, pData ) ;
                  pData->wt->IsAvoidLoop = 0 ;
               }
               else {
                  pData->wt->IsAvoidLoop = 1 ;
                  Putws ( pTable->szUnicode, pData ) ;
                  pData->wt->IsAvoidLoop = 0 ;
               }
               break ;
         }
      }
      else {

         if ( pData->nDstCodePage == CP_VIETNAMESE_VIQR && IsViqrDiacritics ( pData->wt->U1 ) ) {
            StreamWrite ( '\\', pData->StreamOut ) ;
            StreamWrite ( pData->wt->U1, pData->StreamOut ) ;
         }
         else {
            pData->wt->IsAvoidLoop = 1 ;
            Putc ( pData->wt->U1, CP_UNICODE, pData ) ;
            pData->wt->IsAvoidLoop = 0 ;
         }
      }
   }

   if ( ! nLength ) nLength = 1 ;

   switch ( nLength ) {
      case 1 :    pData->wt->U1 = pData->wt->U2 ;  pData->wt->U2 = U3 ; break ;
      case 2 :    pData->wt->U1 = EOF ;            pData->wt->U2 = U3 ; break ;
      case 3 :    pData->wt->U1 = EOF ;            pData->wt->U2 = EOF ; break ;
   }

   if ( cUnicode == EOF ) {
      EncodeVietnamese ( 0, pData ) ;
   }

   return (int) nLength ;
}



////////////////////////////////////////////
//               UTF8 ̏              //
////////////////////////////////////////////



// pData w肳Ă΁AC ɂPڂw
// łȂ΁AGR[hŜ C Ɏw
// G[ EOF Ԃ
// XJ[lԂ
static int DecodeUtf8 ( int C, RTFCONVDATA *pData ) {

   STREAM *StreamIn = ( pData ) ? pData->StreamIn : NULL ;

   int cUtf8 ;

   if ( ! C ) return 0 ;

   if ( ! StreamIn ) {
      cUtf8 = C ;
      while ( ! ( cUtf8 >> 24 ) ) cUtf8 <<= 8 ;
      C = ( cUtf8 >> 24 ) & 0xFF ;
      cUtf8 <<= 8 ;
   }

   unsigned char Bit = 0x80 ;
   while ( Bit & C ) Bit >>= 1 ;

   if ( Bit <= 1 || Bit == 0x40 ) return EOF ;

   int cMbchar = cMbchar = C & ( Bit - 1 ) ;

   for ( ; Bit & ( 0x40 - 1 ) ; Bit <<= 1 ) {
      if ( StreamIn ) C = StreamRead ( StreamIn ) ;
      else {
         C = ( cUtf8 >> 24 ) & 0xFF ;
         cUtf8 <<= 8 ;
      }
      cMbchar = ( cMbchar << 6 ) + ( C & ( 0x40 - 1 ) ) ;
   }

   if ( ! C || C == EOF ) return EOF ;

   return cMbchar ;
}



// pData w肳Ă΁AXg[ɏo͂ 0 Ԃ
// łȂ΁AGR[hԂ
// G[ EOF Ԃ
static int EncodeUtf8 ( int cUnicode, RTFCONVDATA *pData ) {

   STREAM *StreamOut = ( pData ) ? pData->StreamOut : NULL ;

   int cResult = 0 ;

   int Byte = 1 ;
   unsigned char Bit = 0x80 ;

   for ( int Mask = 0x80, Shift = 4 ; Mask && ( cUnicode & ~ ( Mask - 1 ) ) ; Mask <<= Shift, Shift = 5 ) {
      Byte ++ ;
      Bit = (char) Bit >> 1 ;
   }

   if ( Byte <= 1 ) Bit <<= 1 ;

   for ( ; Byte ; Byte -- ) {
      int C = ( ( cUnicode >> ( ( Byte - 1 ) * 6 ) ) & ( ( Bit >> 1 ) - 1 ) ) | Bit ;
      if ( StreamOut ) StreamWrite ( C, StreamOut ) ;
      else {
         if ( cResult >> 24 ) return EOF ;
         cResult = C + ( cResult << 8 ) ;
      }
      Bit = 0x80 ;
   }

   return cResult ;
}



////////////////////////////////////////////
//               UTF7 ̏              //
////////////////////////////////////////////



static int IsBase64Direct ( int C ) ;
static int GetBase64Index ( int C ) ;

static const char Base64CharTable [] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/" ;



// Ԃ
// ŏ EOF Ԃ
static int DecodeUtf7 ( int C, RTFCONVDATA *pData ) {

   STREAM *StreamIn = pData->StreamIn ;

   if ( pData->rt->nUtf7Shift ) {
      if ( C == '-' ) {
         pData->rt->nUtf7Shift = 0 ;
         return EOF ;
      }
   }
   else {
      if ( C == '+' ) {
         int C2 = StreamRead ( StreamIn ) ;
         if ( C2 == '-' ) return '+' ;
         else {
            StreamUnread ( C2, StreamIn ) ;
            pData->rt->nUtf7Shift = 1 ;
            return EOF ;
         }
      }
   }


   int C1, C2, C3, C4, C5, C6, C7, C8 ;
   int U ;

   switch ( pData->rt->nUtf7Shift ) {

      case 0 :

         U = C ;
         break ;

      case 1 :

         C1 = GetBase64Index ( C ) ;
         C2 = GetBase64Index ( StreamRead ( StreamIn ) ) ;
         C3 = GetBase64Index ( StreamRead ( StreamIn ) ) ;

         U = ( ( C1 << 10 ) | ( C2 << 4 ) | ( C3 >> 2 ) ) & 0xFFFF ;
         pData->rt->cUtf7Next = C3 ;
         pData->rt->nUtf7Shift = 2 ;
         break ;

      case 2 :

         C3 = pData->rt->cUtf7Next ;
         C4 = GetBase64Index ( C ) ;
         C5 = GetBase64Index ( StreamRead ( StreamIn ) ) ;
         C6 = GetBase64Index ( StreamRead ( StreamIn ) ) ;

         U = ( ( C3 << 14 ) | ( C4 << 8 ) | ( C5 << 2 ) | ( C6 >> 4 ) ) & 0xFFFF ;
         pData->rt->cUtf7Next = C6 ;
         pData->rt->nUtf7Shift = 3 ;
         break ;

      case 3 :

         C6 = pData->rt->cUtf7Next ;
         C7 = GetBase64Index ( C ) ;
         C8 = GetBase64Index ( StreamRead ( StreamIn ) ) ;

         U = ( ( C6 << 12 ) | ( C7 << 6 ) | ( C8 << 0 ) ) & 0xFFFF ;
         pData->rt->cUtf7Next = 0 ;
         pData->rt->nUtf7Shift = 1 ;
         break ;

      default :

         U = EOF ;
         break ;
   }

   return U ;
}



// t@C̖ EOF w肵ČĂяo
static int EncodeUtf7 ( int cUnicode, RTFCONVDATA *pData ) {

   STREAM *StreamOut = pData->StreamOut ;

   int U3 = cUnicode ;
   int nLength = 0 ;

   if ( pData->wt->U1 != EOF ) {

      if ( pData->wt->U1 == '+' ) {
         if ( pData->wt->nUtf7Shift ) {
            StreamWrite ( '-', StreamOut ) ;
            pData->wt->nUtf7Shift = 0 ;
         }
         StreamWrite ( '+', StreamOut ) ;
         StreamWrite ( '-', StreamOut ) ;
         nLength = 1 ;
      }
      else if ( IsBase64Direct ( pData->wt->U1 ) ) {
         if ( pData->wt->nUtf7Shift ) {
            StreamWrite ( '-', StreamOut ) ;
            pData->wt->nUtf7Shift = 0 ;
         }
         StreamWrite ( pData->wt->U1 & 0xFF, StreamOut ) ;
         nLength = 1 ;
      }
      else {

         if ( ! pData->wt->nUtf7Shift ) {
            StreamWrite ( '+', StreamOut ) ;
            pData->wt->nUtf7Shift = 1 ;
         }

         int C1, C2, C3, C4, C5, C6, C7, C8 ;

         C1 = ( pData->wt->U1 >> 10 ) & 0x3F ;
         C2 = ( pData->wt->U1 >> 4 ) & 0x3F ;
         C3 = ( pData->wt->U1 << 2 ) & 0x3F ;
         nLength = 1 ;
         if ( ! IsBase64Direct ( pData->wt->U2 ) && pData->wt->U2 >= 0 && pData->wt->U2 != '+' ) {
            C3 |= ( pData->wt->U2 >> 14 ) & 0x3F ;
            C4 = ( pData->wt->U2 >> 8 ) & 0x3F ;
            C5 = ( pData->wt->U2 >> 2 ) & 0x3F ;
            C6 = ( pData->wt->U2 << 4 ) & 0x3F ;
            nLength = 2 ;
            if ( ! IsBase64Direct ( U3 ) && U3 >= 0 && U3 != '+' ) {
               C6 |= ( U3 >> 12 ) & 0x3F ;
               C7 = ( U3 >> 6 ) & 0x3F ;
               C8 = ( U3 >> 0 ) & 0x3F ;
               nLength = 3 ;
            }
         }

         StreamWrite ( Base64CharTable [ C1 ], StreamOut ) ;
         StreamWrite ( Base64CharTable [ C2 ], StreamOut ) ;
         StreamWrite ( Base64CharTable [ C3 ], StreamOut ) ;
         if ( nLength >= 2 ) {
            StreamWrite ( Base64CharTable [ C4 ], StreamOut ) ;
            StreamWrite ( Base64CharTable [ C5 ], StreamOut ) ;
            StreamWrite ( Base64CharTable [ C6 ], StreamOut ) ;
            if ( nLength >= 3 ) {
               StreamWrite ( Base64CharTable [ C7 ], StreamOut ) ;
               StreamWrite ( Base64CharTable [ C8 ], StreamOut ) ;
            }
         }

      }
   }

   if ( ! nLength ) nLength = 1 ;

   switch ( nLength ) {
      case 1 :    pData->wt->U1 = pData->wt->U2 ;  pData->wt->U2 = U3 ; break ;
      case 2 :    pData->wt->U1 = EOF ;            pData->wt->U2 = U3 ; break ;
      case 3 :    pData->wt->U1 = EOF ;            pData->wt->U2 = EOF ;   break ;
   }

   if ( cUnicode == EOF ) {
      EncodeUtf7 ( 0, pData ) ;
      if ( pData->wt->nUtf7Shift ) StreamWrite ( '-', StreamOut ) ;
   }

   return 0 ;
}



static int IsBase64Direct ( int C ) {

   if ( isascii ( C ) ) {
      if ( isalpha ( C ) || isdigit ( C ) || isspace ( C ) ) return 1 ;
      if ( C ) {
         if ( strchr ( "'(),-./:?", C ) ) return 1 ;
//       if ( strchr ( "!\"#$%&*;<=>@[]^_`{|}", C ) ) return 1 ;
      }
   }

   return 0 ;
}



static int GetBase64Index ( int C ) {

   if ( C > 0 ) {
      const char *szTmp = strchr ( Base64CharTable, C ) ;
      if ( szTmp ) return (int) ( szTmp - Base64CharTable ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//         RTFCONV for CHARACTER          //
////////////////////////////////////////////



static int RtfconvCharInternal ( int C, RTFCONVDATA *pData, int *pError ) ;
static int exchange_endian_16 ( int C ) ;
static int exchange_endian_32 ( int C ) ;



// ϊ
// ϊ̕Ԃ
// s -1 Ԃ
int WINAPI RtfconvCharEx ( int C, int nSrcCodePage, int nDstCodePage, const RTFCONVEXINFO *pInfo ) {

   int nError = CONV_ERROR_PARAM ;
   int cResult ;

   if ( ! pInfo || pInfo->cbSize == sizeof(RTFCONVEXINFO) ) {

      RTFCONVDATA Data = { 0 } ;

      __try {

         // f[^̏
         if ( InitRtfconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }

         // s
         nError = CONV_SUCCESS ;
         cResult = RtfconvCharInternal ( C, & Data, & nError ) ;

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      if ( C != 0 && cResult == 0 ) cResult = -1 ;
   }

   // ̏o
   if ( SetRtfconvLastError ( nError ) ) return -1 ;
   return cResult ;
}



static int RtfconvCharInternal ( int C, RTFCONVDATA *pData, int *pError ) {

   int cResult ;

   if ( ! pData->nSrcCodePage || ! pData->nDstCodePage ) {
      *pError = CONV_ERROR_CODEPAGE ;
      return -1 ;
   }
   if ( pData->nSrcCodePage == CP_VIETNAMESE_VIQR || pData->nDstCodePage == CP_VIETNAMESE_VIQR ) {
      *pError = CONV_ERROR_CODEPAGE ;
      return -1 ;
   }

   int IsOnlyCr = ( ( pData->dwConversionMode & CONVMODE_OUTPUT_CR ) && ! ( pData->dwConversionMode & CONVMODE_OUTPUT_LF ) ) ? 1 : 0 ;

   // ̓R[hy[W̒
   switch ( pData->nSrcCodePage ) {
      case CP_UTF32_BE :
         C = exchange_endian_32 ( C ) ;
         // fall through
      case CP_UTF32 :
         pData->nSrcCodePage = CP_UNICODE ;
         break ;
      case CP_UTF16_BE :
         C = exchange_endian_16 ( C ) ;
         // fall through
      case CP_UTF16 :
         if ( IsFourByteLetter ( C ) ) {
            if ( IsUnicodeHighSurrogate ( ( C >> 16 ) & 0xFFFF ) && IsUnicodeLowSurrogate ( C & 0xFFFF ) ) C = MAKE_UNICODE_SCALAR ( ( C >> 16 ) & 0xFFFF, C & 0xFFFF ) ;
            else                                                                                           C = -1 ;
         }
         pData->nSrcCodePage = CP_UNICODE ;
         break ;
      case CP_UTF8 :
         C = DecodeUtf8 ( C, NULL ) ;
         pData->nSrcCodePage = CP_UNICODE ;
         break ;
      case CP_EUC_JAPANESE :
         int cSaved = C ;
         C = eucjp2sjis ( C, pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_INPUT_2004 ) ;
         if ( cSaved && ! C ) C = -1 ;
         pData->nSrcCodePage = CP_JAPANESE ;
         break ;
   }

   // o̓R[hy[W̒
   int IsExchangeEndian = 0 ;
   int IsUtf32 = 0 ;
   int IsUtf8 = 0 ;
   int IsEucJp = 0 ;

   switch ( pData->nDstCodePage ) {
      case CP_UTF32_BE :
         IsExchangeEndian = 1 ;
         // fall through
      case CP_UTF32 :
         IsUtf32 = 1 ;
         pData->nDstCodePage = CP_UNICODE ;
         break ;
      case CP_UTF16_BE :
         IsExchangeEndian = 1 ;
         // fall through
      case CP_UTF16 :
         pData->nDstCodePage = CP_UNICODE ;
         break ;
      case CP_UTF8 :
         IsUtf8 = 1 ;
         pData->nDstCodePage = CP_UNICODE ;
         break ;
      case CP_EUC_JAPANESE :
         IsEucJp = 1 ;
         pData->nDstCodePage = CP_JAPANESE ;
         break ;
   }

   if ( pData->nSrcPseudoCodePage != pData->nSrcCodePage || pData->nDstPseudoCodePage != pData->nDstCodePage ) {
      *pError = CONV_ERROR_CODEPAGE ;
      return -1 ;
   }

   if ( pData->nSrcCodePage != CP_UNICODE && pData->nSrcCodePage != CP_EUC_CHINESE_TRADITIONAL && C >= 0x100 && C <= 0xFFFF ) {
      if ( ! ismbblead_cp ( C >> 8, pData->nSrcCodePage ) || ! ismbbtrail_cp ( C & 0xFF, pData->nSrcCodePage ) ) C = -1 ;
   }


   // ̕ϊ
   if ( C == -1 ) cResult = 0 ;
   else           cResult = Putc ( C, pData->nSrcCodePage, pData ) ;

   if ( IsOnlyCr && cResult == '\n' ) cResult = '\r' ;

   if ( IsEucJp ) cResult = sjis2eucjp ( cResult, pData->dwAdditionalMode & ADDITIONAL_MODE_JIS_OUTPUT_2004 ) ;

   if ( pData->nDstCodePage == CP_UNICODE ) {
      if ( IsUtf32 ) {
         if ( IsExchangeEndian ) cResult = exchange_endian_32 ( cResult ) ;
      }
      else if ( IsUtf8 ) {
         cResult = EncodeUtf8 ( cResult, NULL ) ;
      }
      else {
         if ( IsFourByteLetter ( cResult ) ) {
            if ( (unsigned) cResult > UTF16_MAX ) cResult = 0 ;
            else                                  cResult = MAKE_UNICODE_SURROGATE ( cResult ) ;
         }
         if ( IsExchangeEndian ) cResult = exchange_endian_16 ( cResult ) ;
      }
   }

   return cResult ;
}



static int exchange_endian_16 ( int C ) {

   int C1 = ( C >> 24 ) & 0xFF ;
   int C2 = ( C >> 16 ) & 0xFF ;
   int C3 = ( C >>  8 ) & 0xFF ;
   int C4 = ( C >>  0 ) & 0xFF ;

   return ( C2 << 24 ) + ( C1 << 16 ) + ( C4 << 8 ) + C3 ;
}



static int exchange_endian_32 ( int C ) {

   int C1 = ( C >> 24 ) & 0xFF ;
   int C2 = ( C >> 16 ) & 0xFF ;
   int C3 = ( C >>  8 ) & 0xFF ;
   int C4 = ( C >>  0 ) & 0xFF ;

   return ( C4 << 24 ) + ( C3 << 16 ) + ( C2 << 8 ) + C1 ;
}



////////////////////////////////////////////
//                 TOOLS                  //
////////////////////////////////////////////



// AXL[16iɕϊ
// ϊ̕ԋp
static char *EncodeHexaDecimal ( const char *szSrc, char *szDst, int nCodePage ) {

   char *szDstSaved = szDst ;

   while ( *szSrc ) {
      if ( ismbblead_cp ( (unsigned char) *szSrc, nCodePage ) ) {
         sprintf ( szDst, "\\\'%02x\\\'%02x", (unsigned char) *szSrc, (unsigned char) *( szSrc + 1 ) ) ;
         szSrc ++ ;
         szSrc ++ ;
         szDst = strend ( szDst ) ;
         continue ;
      }
      if ( ! isascii ( *szSrc ) || iscntrl ( *szSrc ) || *szSrc == ESCAPE || *szSrc == BRACE_START || *szSrc == BRACE_END || *szSrc == SEMICOLON ) {
         sprintf ( szDst, "\\\'%02x", (unsigned char) *szSrc ) ;
         szSrc ++ ;
         szDst = strend ( szDst ) ;
         continue ;
      }
      *szDst = *szSrc ;
      szSrc ++ ;
      szDst ++ ;
      continue ;
   }
   *szDst = 0 ;

   return szDstSaved ;
}



// 񂪃AXL[ǂ
static int IsAsciiString ( const char *szString ) {
   while ( *szString ) {
      if ( ! isascii ( *szString ) ) return 0 ;
      szString ++ ;
   }
   return 1 ;
}



// P̃oCgTCY擾
static int GetCharSize ( int nCodePage ) {

   switch ( nCodePage ) {
      case CP_UTF16 :
      case CP_UTF16_BE :
         return (int) sizeof(wchar16_t) ;

      case CP_UTF32 :
      case CP_UTF32_BE :
         return (int) sizeof(wchar32_t) ;

      default :
         return (int) sizeof(char) ;
   }
}



// PƂĔr
// v 0 AvȂ 0 ȊOԂ
static int wcswordicmp ( const wchar_t *szString, const wchar_t *szKeyword ) {

   size_t nKeywordLength = wcslen ( szKeyword ) ;

   int nResult = wcsnicmp ( szString, szKeyword, nKeywordLength ) ;
   if ( ! nResult ) {
      const wchar_t *szEnd = szString + nKeywordLength ;
      if ( ! ( isascii ( *szEnd ) && isalnum ( *szEnd ) ) ) return 0 ;
      else                                                  return 1 ;
   }

   return nResult ;
}



////////////////////////////////////////////
//               CODEPAGES                //
////////////////////////////////////////////



// ̃R[hy[WoRĕϊȂÃR[hy[WԂ
static int GetPseudoCodePage ( int nCodePage ) {

   if ( nCodePage == CP_EUC_JAPANESE ) return CP_JAPANESE ;

   return GetModelessCodePage ( nCodePage ) ;
}



#define VALID_CODEPAGE_ARRAY_SIZE   0x100



// R[hy[ŴAVXeŗp\Ȃ̂
// pȂ 1 AsȂ 0 Ԃ
static int IsOsAvailableCodePage ( int nCodePage ) {

   static wchar_t ValidCodePageArray [ VALID_CODEPAGE_ARRAY_SIZE ] ;
   static wchar_t InvalidCodePageArray [ VALID_CODEPAGE_ARRAY_SIZE ] ;

   if ( nCodePage < 0x10 || nCodePage == CP_SYMBOL || nCodePage > 0xFFFF ) return 0 ;  // CP_ACP,CP_OEMCP,CP_MACCP,CP_THREAD_ACP,CP_SYMBOL
   if ( nCodePage >= CP_RTFCONV_SPECIFIC_MIN && nCodePage <= CP_RTFCONV_SPECIFIC_MAX ) return 0 ;

   if ( wcschr ( ValidCodePageArray, (wchar_t) nCodePage ) ) return 1 ;
   if ( wcschr ( InvalidCodePageArray, (wchar_t) nCodePage ) ) return 0 ;


   int IsValid = 0 ;

   if ( nCodePage == CP_CHINESE_HKSCS ) IsValid = 0 ;
   else if ( nCodePage == CP_CHINESE_TRADITIONAL ) IsValid = ! IsValidCodePageHkscs () ;
   else if ( IsValidCodePage ( nCodePage ) ) {
      if ( IsMultiByteCodePage ( GetWinCodePage ( nCodePage ) ) ) IsValid = 1 ;
      else if ( IsCodePageIscii ( nCodePage ) ) IsValid = 0 ;
      else {
         CPINFO CpInfo = { 0 } ;
         if ( GetCPInfo ( nCodePage, & CpInfo ) ) {
            if ( CpInfo.MaxCharSize == 1 && ! ismbcp ( nCodePage ) ) IsValid = 1 ;
            if ( CpInfo.MaxCharSize == 2 && ismbcp ( nCodePage ) ) IsValid = 1 ;
         }
      }
   }


   wchar_t *pArray ;
   if ( IsValid ) pArray = ValidCodePageArray ;
   else           pArray = InvalidCodePageArray ;

   EnterCriticalSection ( & AvailableCodePageSection ) ;
   __try {
      if ( ! wcschr ( pArray, (wchar_t) nCodePage ) ) {
         wchar_t *pArrayTail = wcsend ( pArray ) ;
         if ( pArrayTail - pArray >= VALID_CODEPAGE_ARRAY_SIZE - 1 ) pArrayTail = pArray ;
         pArrayTail [ 1 ] = 0 ;
         pArrayTail [ 0 ] = (wchar_t) nCodePage ;
      }
   } __finally {
      LeaveCriticalSection ( & AvailableCodePageSection ) ;
   }


   return IsValid ;
}



////////////////////////////////////////////
//             LOAD LIBRARY               //
////////////////////////////////////////////



#include "conv_loaddll.h"
#include "sbctbl.h"
#include "mbctbl.h"
#include "cnstbl.h"



static volatile long IsFreeTableDllRegistered ;
static void __cdecl FreeTableDll ( void ) ;


static HINSTANCE           hSbctbl ;
static SINGLEBYTETOUNICODE SingleByteToUnicode ;
static UNICODETOSINGLEBYTE UnicodeToSingleByte ;
static ISSBCVALIDCODEPAGE  IsSbcValidCodePage ;



// Sbctbl.dll [h
// s 0 ȊOԂ
static int LoadSbctbl ( void ) {

   static volatile int IsTried ;

   if ( ! IsTried ) {

      EnterCriticalSection ( & LoadSbctblSection ) ;
      __try {

         if ( ! IsTried ) {

            HINSTANCE hDll ;
            if ( IsNT () ) hDll = LoadDllW ( hInstance, L"sbctbl.dll" ) ;
            else           hDll = LoadDllA ( hInstance,  "sbctbl.dll" ) ;

            int IsError = 0 ;
            if ( GETPROCADDRESS ( hDll, SINGLEBYTETOUNICODE, SingleByteToUnicode ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, UNICODETOSINGLEBYTE, UnicodeToSingleByte ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, ISSBCVALIDCODEPAGE, IsSbcValidCodePage ) ) IsError = 1 ;

            if ( IsError ) {
               FreeLibrary ( hDll ) ;
            }
            else {
               if ( ! InterlockedExchange ( & IsFreeTableDllRegistered, TRUE ) ) atexit ( FreeTableDll ) ;
               hSbctbl = hDll ;
            }

            IsTried = 1 ;
         }

      } __finally {
         LeaveCriticalSection ( & LoadSbctblSection ) ;
      }
   }

   return ( hSbctbl ) ? 0 : 1 ;
}



static HINSTANCE              hMbctbl ;
static MULTIBYTETOUNICODE     MultiByteToUnicode ;
static UNICODETOMULTIBYTE     UnicodeToMultiByte ;
static JISHOJOTOUNICODE       JisHojoToUnicode ;
static UNICODETOJISHOJO       UnicodeToJisHojo ;
static SHIFTJIS2004TOUNICODE  ShiftJis2004ToUnicode ;
static UNICODETOSHIFTJIS2004  UnicodeToShiftJis2004 ;
static ISMBCVALIDCODEPAGE     IsMbcValidCodePage ;



// Mbctbl.dll [h
// s 0 ȊOԂ
static int LoadMbctbl ( void ) {

   static volatile int IsTried ;

   if ( ! IsTried ) {

      EnterCriticalSection ( & LoadMbctblSection ) ;
      __try {

         if ( ! IsTried ) {

            HINSTANCE hDll ;
            if ( IsNT () ) hDll = LoadDllW ( hInstance, L"mbctbl.dll" ) ;
            else           hDll = LoadDllA ( hInstance,  "mbctbl.dll" ) ;

            int IsError = 0 ;
            if ( GETPROCADDRESS ( hDll, MULTIBYTETOUNICODE, MultiByteToUnicode ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, UNICODETOMULTIBYTE, UnicodeToMultiByte ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, ISMBCVALIDCODEPAGE, IsMbcValidCodePage ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, JISHOJOTOUNICODE, JisHojoToUnicode ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, UNICODETOJISHOJO, UnicodeToJisHojo ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, SHIFTJIS2004TOUNICODE, ShiftJis2004ToUnicode ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, UNICODETOSHIFTJIS2004, UnicodeToShiftJis2004 ) ) IsError = 1 ;

            if ( IsError ) {
               FreeLibrary ( hDll ) ;
            }
            else {
               if ( ! InterlockedExchange ( & IsFreeTableDllRegistered, TRUE ) ) atexit ( FreeTableDll ) ;
               hMbctbl = hDll ;
            }

            IsTried = 1 ;
         }

      } __finally {
         LeaveCriticalSection ( & LoadMbctblSection ) ;
      }
   }

   return ( hMbctbl ) ? 0 : 1 ;
}



static HINSTANCE           hCnstbl ;
static CNS11643TOUNICODE   Cns11643ToUnicode ;
static UNICODETOCNS11643   UnicodeToCns11643 ;



// Cnstbl.dll [h
// s 0 ȊOԂ
static int LoadCnstbl ( void ) {

   static volatile int IsTried ;

   if ( ! IsTried ) {

      EnterCriticalSection ( & LoadCnstblSection ) ;
      __try {

         if ( ! IsTried ) {

            HINSTANCE hDll ;
            if ( IsNT () ) hDll = LoadDllW ( hInstance, L"cnstbl.dll" ) ;
            else           hDll = LoadDllA ( hInstance,  "cnstbl.dll" ) ;

            int IsError = 0 ;
            if ( GETPROCADDRESS ( hDll, CNS11643TOUNICODE, Cns11643ToUnicode ) ) IsError = 1 ;
            if ( GETPROCADDRESS ( hDll, UNICODETOCNS11643, UnicodeToCns11643 ) ) IsError = 1 ;

            if ( IsError ) {
               FreeLibrary ( hDll ) ;
            }
            else {
               if ( ! InterlockedExchange ( & IsFreeTableDllRegistered, TRUE ) ) atexit ( FreeTableDll ) ;
               hCnstbl = hDll ;
            }

            IsTried = 1 ;
         }

      } __finally {
         LeaveCriticalSection ( & LoadCnstblSection ) ;
      }
   }

   return ( hCnstbl ) ? 0 : 1 ;
}



// DLL 
static void __cdecl FreeTableDll ( void ) {
   if ( hSbctbl ) FreeLibrary ( hSbctbl ) ;
   if ( hMbctbl ) FreeLibrary ( hMbctbl ) ;
   if ( hCnstbl ) FreeLibrary ( hCnstbl ) ;
   hSbctbl = NULL ;
   hMbctbl = NULL ;
   hCnstbl = NULL ;
}



////////////////////////////////////////////
//           CONVERT CHARACTER            //
////////////////////////////////////////////



// Ȃ 0 Ԃ
// R[hy[W̃G[Ȃ EOF Ԃ
static int CharToUnicode ( int C, int nCodePage, unsigned long dwConversionMode ) {

   if ( nCodePage == CP_VIETNAMESE_VIQR ) nCodePage = CP_VIETNAMESE_VISCII ;

   if ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) {
      if ( ! IsOsAvailableCodePage ( nCodePage ) ) return EOF ;
      return ConvertCharacter ( C, nCodePage, CP_UNICODE ) ;
   }

   if ( ! ( dwConversionMode & CONVMODE_USE_DLL_TABLE ) && IsOsAvailableCodePage ( nCodePage ) ) {
      return ConvertCharacter ( C, nCodePage, CP_UNICODE ) ;
   }

   if ( nCodePage == CP_EUC_CHINESE_TRADITIONAL ) {
      if ( ! Cns11643ToUnicode && LoadCnstbl () ) return EOF ;
      return Cns11643ToUnicode ( C ) ;
   }

   if ( IsMbcDllCodePage ( nCodePage ) ) {
      if ( ! IsMbcValidCodePage && LoadMbctbl () ) return EOF ;
      if ( ! IsMbcValidCodePage ( nCodePage ) ) return EOF ;
      return MultiByteToUnicode ( C, nCodePage ) ;
   }
   else {
      if ( ! IsSbcValidCodePage && LoadSbctbl () ) return EOF ;
      if ( ! IsSbcValidCodePage ( nCodePage ) ) return EOF ;
      return SingleByteToUnicode ( C, nCodePage ) ;
   }
}



// Ȃ 0 Ԃ
// R[hy[W̃G[Ȃ EOF Ԃ
static int UnicodeToChar ( int C, int nCodePage, unsigned long dwConversionMode ) {

   if ( nCodePage == CP_VIETNAMESE_VIQR ) nCodePage = CP_VIETNAMESE_VISCII ;

   if ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) {
      if ( ! IsOsAvailableCodePage ( nCodePage ) ) return EOF ;
      int cResult = ConvertCharacter ( C, CP_UNICODE, nCodePage ) ;
      if ( cResult && C != ConvertCharacter ( cResult, nCodePage, CP_UNICODE ) ) cResult = 0 ;  // tϊŌ
      return cResult ;
   }

   if ( ! ( dwConversionMode & CONVMODE_USE_DLL_TABLE ) && IsOsAvailableCodePage ( nCodePage ) ) {
      int cResult = ConvertCharacter ( C, CP_UNICODE, nCodePage ) ;
      if ( cResult && C != ConvertCharacter ( cResult, nCodePage, CP_UNICODE ) ) cResult = 0 ;  // tϊŌ
      return cResult ;
   }

   if ( nCodePage == CP_EUC_CHINESE_TRADITIONAL ) {
      if ( ! UnicodeToCns11643 && LoadCnstbl () ) return EOF ;
      int cResult = UnicodeToCns11643 ( C ) ;
      if ( C != Cns11643ToUnicode ( cResult ) ) cResult = 0 ;  // tϊŌ
      return cResult ;
   }

   if ( IsMbcDllCodePage ( nCodePage ) ) {
      if ( ! IsMbcValidCodePage && LoadMbctbl () ) return EOF ;
      if ( ! IsMbcValidCodePage ( nCodePage ) ) return EOF ;
      int cResult = UnicodeToMultiByte ( C, nCodePage ) ;
      if ( C != MultiByteToUnicode ( cResult, nCodePage ) ) cResult = 0 ;  // tϊŌ
      return cResult ;
   }
   else {
      if ( ! IsSbcValidCodePage && LoadSbctbl () ) return EOF ;
      if ( ! IsSbcValidCodePage ( nCodePage ) ) return EOF ;
      int cResult = UnicodeToSingleByte ( C, nCodePage ) ;
      if ( cResult && C != SingleByteToUnicode ( cResult, nCodePage ) ) cResult = 0 ;  // tϊŌ
      return cResult ;
   }
}



static int IsAvailableCodePageWithConversionMode ( int nCodePage, unsigned long dwConversionMode ) {
   int C = UnicodeToChar ( 0, nCodePage, dwConversionMode ) ;
   if ( C == EOF ) return 0 ;
   else            return 1 ;
}



// VtgJIS2004  Unicode ɕϊ
// R[hy[W̃G[Ȃ EOF Ԃ
static int SjisToUnicode_2004 ( int cSjis, int cUnicode, const wchar_t **pszUnicode, unsigned long dwConversionMode ) {

   if ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) return EOF ;
   if ( ! IsMbcValidCodePage && LoadMbctbl () ) return EOF ;
   if ( ! ShiftJis2004ToUnicode ) return EOF ;

   return ShiftJis2004ToUnicode ( cSjis, cUnicode, pszUnicode ) ;
}



// Unicode  VtgJIS2004 ɕϊ
// R[hy[W̃G[Ȃ EOF Ԃ
static int UnicodeToSjis_2004 ( int cUnicode, int cSjis, const wchar_t *pszUnicode, unsigned long dwConversionMode ) {

   if ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) return EOF ;
   if ( ! IsMbcValidCodePage && LoadMbctbl () ) return EOF ;
   if ( ! UnicodeToShiftJis2004 ) return EOF ;

   return UnicodeToShiftJis2004 ( cUnicode, cSjis, pszUnicode ) ;
}



// ⏕ Unicode ɕϊ
// Ȃ 0 Ԃ
// R[hy[W̃G[Ȃ EOF Ԃ
static int CharToUnicode_JisHojo ( int C, unsigned long dwConversionMode ) {

   if ( ! ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) ) {
      if ( ! IsMbcValidCodePage && LoadMbctbl () ) goto CONVERT_BY_SYSTEM ;
      if ( ! JisHojoToUnicode ) goto CONVERT_BY_SYSTEM ;
      return JisHojoToUnicode ( C ) ;
   }

   CONVERT_BY_SYSTEM :

   const int nCodePage = CP_EUC_JAPANESE_MS ;

   if ( ! ( dwConversionMode & CONVMODE_USE_DLL_TABLE ) ) {
      if ( ! IsOsAvailableCodePage ( nCodePage ) ) return EOF ;

      C &= 0x7F7F ;
      if ( C >= 0x7500 ) return 0 ;
      if ( C == 0x2237 ) return 0x007E ;
      if ( C == 0x2271 ) return 0x2116 ;

      return ConvertCharacter ( C | 0x8000, nCodePage, CP_UNICODE ) ;
   }

   return EOF ;
}



// Unicode ⏕ɕϊ
// Ȃ 0 Ԃ
// R[hy[W̃G[Ȃ EOF Ԃ
static int UnicodeToChar_JisHojo ( int C, unsigned long dwConversionMode ) {

   if ( ! ( dwConversionMode & CONVMODE_USE_SYSTEM_TABLE ) ) {
      if ( ! IsMbcValidCodePage && LoadMbctbl () ) goto CONVERT_BY_SYSTEM ;
      if ( ! UnicodeToJisHojo ) goto CONVERT_BY_SYSTEM ;
      return UnicodeToJisHojo ( C ) ;
   }

   CONVERT_BY_SYSTEM :

   const int nCodePage = CP_EUC_JAPANESE_MS ;

   if ( ! ( dwConversionMode & CONVMODE_USE_DLL_TABLE ) ) {
      if ( ! IsOsAvailableCodePage ( nCodePage ) ) return EOF ;

      if ( IsUnicodePua ( C ) ) return 0 ;
      if ( C == 0x007E ) return 0x2237 | 0x8080 ;
      if ( C == 0x2116 ) return 0x2271 | 0x8080 ;

      int cResult = ConvertCharacter ( C, CP_UNICODE, nCodePage ) ;
      if ( ! ( cResult & 0x8000 ) || cResult & 0x80 ) return 0 ;

      return cResult | 0x8080 ;
   }

   return EOF ;
}



// sym2uni.h p
int CharToUnicode_external ( int C, int nCodePage, unsigned long dwConversionMode ) {
   return CharToUnicode ( C, nCodePage, dwConversionMode ) ;
}



// sym2uni.h p
int UnicodeToChar_external ( int C, int nCodePage, unsigned long dwConversionMode ) {
   return UnicodeToChar ( C, nCodePage, dwConversionMode ) ;
}



////////////////////////////////////////////
//            PRINTF ARGUMENTS            //
////////////////////////////////////////////



// printf tH[}bg̈̐ԂiUNICODEŁj
static int CountPrintfArgumentsW ( const wchar_t *szFormat ) {

   int nCount = 0 ;

   while ( *szFormat ) {

      if ( *szFormat == '%' ) {

         szFormat ++ ;

         if ( *szFormat && wcschr ( L"+- #", *szFormat ) ) szFormat ++ ;

         if ( *szFormat == '*' ) { szFormat ++ ; nCount ++ ; }
         else while ( isascii ( *szFormat ) && isdigit ( *szFormat ) ) szFormat ++ ;

         if ( *szFormat == '.' ) {
            szFormat ++ ;
            if ( *szFormat == '*' ) { szFormat ++ ; nCount ++ ; }
            else while ( isascii ( *szFormat ) && isdigit ( *szFormat ) ) szFormat ++ ;
         }

         if      ( ! wcsheadcmp ( szFormat, L"I32" ) ) szFormat += wcslen ( L"I32" ) ;
         else if ( ! wcsheadcmp ( szFormat, L"I64" ) ) szFormat += wcslen ( L"I64" ) ;
         else if ( *szFormat && wcschr ( L"hlI", *szFormat ) ) szFormat ++ ;

         if ( *szFormat && wcschr ( L"cCdiouxXeEfgGnpsS", *szFormat ) ) nCount ++ ;
      }

      szFormat ++ ;
   }

   return nCount ;
}



////////////////////////////////////////////
//    StoreCharsW() over BMP, with Attr   //
////////////////////////////////////////////



// obt@ɕi[ (TQ[gyAΉ)
static STOREDCHARSW *StoreCharsExW ( int C, STOREDCHARSW *StoredChars ) {

   if ( IsFourByteLetter ( C ) ) {
      C = MAKE_UNICODE_SURROGATE ( C ) ;
      StoredChars = StoreCharsW ( ( C >> 16 ) & 0xFFFF, StoredChars ) ;
      C &= 0xFFFF ;
   }

   return StoreCharsW ( C, StoredChars ) ;
}



// obt@ɕi[ (TQ[gyAΉ{ȇi[)
// s STOREDCHARS_ERROR Ԃ
static STOREDCHARSATTRW *StoreCharsAttrW ( int C, STOREDCHARSATTRW *StoredChars, unsigned long dwAttributes ) {

   if ( StoredChars == STOREDCHARS_ERROR ) return (STOREDCHARSATTRW*) STOREDCHARS_ERROR ;

   if ( ! StoredChars ) {
      StoredChars = (STOREDCHARSATTRW*) malloc ( sizeof(STOREDCHARSATTRW) ) ;
      if ( ! StoredChars ) return (STOREDCHARSATTRW*) STOREDCHARS_ERROR ;
      memzero ( StoredChars, sizeof(STOREDCHARSATTRW) ) ;
   }

   size_t dwBufferSize = ( StoredChars->StoredChars ) ? StoredChars->StoredChars->dwBufferSize : 0 ;
   size_t nLength = ( StoredChars->StoredChars ) ? StoredChars->StoredChars->szWrite - StoredChars->StoredChars->szStart : 0 ;
   StoredChars->StoredChars = StoreCharsExW ( C, StoredChars->StoredChars ) ;

   if ( dwBufferSize != StoredChars->StoredChars->dwBufferSize ) {
      unsigned long *NewAttribBuffer = (unsigned long*) realloc ( StoredChars->AttribBuffer, StoredChars->StoredChars->dwBufferSize * sizeof(unsigned long) ) ;
      if ( ! NewAttribBuffer ) {
         FreeStoredCharsAttrW ( StoredChars ) ;
         return (STOREDCHARSATTRW*) STOREDCHARS_ERROR ;
      }
      StoredChars->AttribBuffer = NewAttribBuffer ;
   }

   size_t nNewLength = StoredChars->StoredChars->szWrite - StoredChars->StoredChars->szStart ;
   for ( unsigned long *p = StoredChars->AttribBuffer + nLength ; p < StoredChars->AttribBuffer + nNewLength ; p ++ ) *p = dwAttributes ;

   return StoredChars ;
}



// obt@̐擪AhXԂ
static wchar_t *GetStoredCharsAttrW ( STOREDCHARSATTRW *StoredChars ) {
   if ( ! StoredChars || StoredChars == STOREDCHARS_ERROR ) return GetStoredCharsW ( NULL ) ;
   return GetStoredCharsW ( StoredChars->StoredChars ) ;
}



// obt@̕ɂ
static void ClearStoredCharsAttrW ( STOREDCHARSATTRW *StoredChars ) {
   if ( ! StoredChars || StoredChars == STOREDCHARS_ERROR ) return ;
   ClearStoredCharsW ( StoredChars->StoredChars ) ;
}



// obt@
static void FreeStoredCharsAttrW ( STOREDCHARSATTRW *StoredChars ) {
   if ( ! StoredChars || StoredChars == STOREDCHARS_ERROR ) return ;
   FreeStoredCharsW ( StoredChars->StoredChars ) ;
   free ( StoredChars->AttribBuffer ) ;
   free ( StoredChars ) ;
}



////////////////////////////////////////////
//          IS AVAILABLE CODEPAGE         //
////////////////////////////////////////////



// RTFCONV ϊ\ȃR[hy[W
// pȂ 1 AsȂ 0 Ԃ
int WINAPI IsRtfconvValidCodePage ( int nCodePage ) {

   RTFCONVDATA Data = { 0 } ;
   Data.nSrcCodePage = nCodePage ;
   Data.nDstCodePage = nCodePage ;

   if ( InitConversionMode ( & Data ) ) return 0 ;

   return 1 ;
}



////////////////////////////////////////////
//            GET LAST ERROR              //
////////////////////////////////////////////



// G[l擾
// G[l̓XbhƂɊǗ
int WINAPI GetRtfconvLastError ( void ) {
   return (int)(intptr_t) TlsGetValue ( nTlsIndex ) ;
}



// G[lݒ
// ݒ肳ꂽG[lԂ
int SetRtfconvLastError ( int nError ) {
   TlsSetValue ( nTlsIndex, (void*)(intptr_t) nError ) ;
   return nError ;
}



////////////////////////////////////////////
//                 ABORT                  //
////////////////////////////////////////////



// ~𖽂
int WINAPI AbortRtfconv ( void ) {
   SetAbortFlag () ;
   return 0 ;
}



////////////////////////////////////////////
//                VERSION                 //
////////////////////////////////////////////



// o[WԂ
int WINAPI GetDllVersion ( void ) {

   FILEVERSION FileVersion ;
   if ( GetFileVersion ( & FileVersion ) ) return 0 ;

   return TranslateVersionInfo ( & FileVersion ) ;
}



